SystemOrganization addCategory: #'GlorpTests-DatabaseTypes'!
SystemOrganization addCategory: #'GlorpTests-Models'!
SystemOrganization addCategory: #'GlorpTests-Database'!
SystemOrganization addCategory: #'GlorpTests-Licence'!
SystemOrganization addCategory: #'GlorpTests-CollectionTypeModels'!
SystemOrganization addCategory: #GlorpTests!
SystemOrganization addCategory: #'GlorpTests-Extras'!

GlorpSession subclass: #GlorpMockSession
	instanceVariableNames: 'rows'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpMockSession class methodsFor: 'instance creation' stamp: ''!
new

	^super new initialize.! !

!GlorpMockSession methodsFor: 'accessing' stamp: ''!
accessor
	^GlorpMockAccessor new.! !

!GlorpMockSession methodsFor: 'initialize' stamp: ''!
initialize

	super initialize.
	rows := OrderedCollection new.! !

!GlorpMockSession methodsFor: 'accessing' stamp: ''!
rows

	^rows.! !

!GlorpMockSession methodsFor: 'testing' stamp: ''!
useBinding
	^true.! !

!GlorpMockSession methodsFor: 'read/write' stamp: ''!
writeHomogeneousRows: aCollection
	aCollection do: [:each | self writeRow: each].
	^true.! !

!GlorpMockSession methodsFor: 'read/write' stamp: ''!
writeRow: aRow
	aRow shouldBeWritten ifFalse: [^true].
	aRow preWriteAssignSequencesUsing: self.
	rows add: aRow.
	aRow postWriteAssignSequencesUsing: self.
	^true.! !

AbstractReadQuery subclass: #GlorpQueryStub
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpQueryStub commentStamp: '<historical>' prior: 0!
A test class. This behaves like a query, but doesn't go to the database, it just returns whatever it was told to return on its creation, regardless of the parameters or session.

Instance Variables:
	result	<Object>	The thing we should return.

!

!GlorpQueryStub class methodsFor: 'instance creation' stamp: ''!
newQuery
	^self new.! !

!GlorpQueryStub methodsFor: 'executing' stamp: ''!
executeWithParameters: parameterArray in: aSession 

	aSession register: result.
	^result.! !

!GlorpQueryStub methodsFor: 'initialize' stamp: ''!
initialize

	super initialize.
	readsOneObject := true.! !

!GlorpQueryStub methodsFor: 'accessing' stamp: ''!
result
	^result! !

!GlorpQueryStub methodsFor: 'accessing' stamp: ''!
result: anObject
	result := anObject! !

TestAsserter subclass: #GlorpTestModelClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpTestModelClass commentStamp: '<historical>' prior: 0!
An abstract superclass providing convenience methods for domain objects to facilitate testing.!

GlorpTestModelClass subclass: #GlorpVideoCreditStatus
	instanceVariableNames: 'customer balanceOwing dateOwing comments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpVideoCreditStatus commentStamp: '<historical>' prior: 0!
There is a possibility of one of these for each customer in a Store. If there is no credit owing or owed, then there is no record. Otherwise the record exists and records the total amount. This is pretty unlikely as a model of credit, but suits our purposes by providing a 1-1 relationship that may or may not exist.

Instance Variables:
	balanceOwing	<Number>	
	comments	<String>	
	customer	<Customer>
	dateOwing	<Date>

!

!GlorpVideoCreditStatus methodsFor: 'testing' stamp: ''!
assertEqual: aCreditStatus

	self assert: customer name = aCreditStatus customer name.
	self assert: customer dateOfBirth = aCreditStatus customer dateOfBirth.
	self assert: balanceOwing = aCreditStatus balanceOwing.
	self assert: dateOwing = aCreditStatus dateOwing.
	self assert: comments = aCreditStatus comments.! !

!GlorpVideoCreditStatus methodsFor: 'As yet unclassified' stamp: ''!
balanceOwing
	^balanceOwing.! !

!GlorpVideoCreditStatus methodsFor: 'As yet unclassified' stamp: ''!
balanceOwing: aNumber
	balanceOwing := aNumber.! !

!GlorpVideoCreditStatus methodsFor: 'As yet unclassified' stamp: ''!
comments
	^comments.! !

!GlorpVideoCreditStatus methodsFor: 'As yet unclassified' stamp: ''!
comments: aString
	comments := aString.! !

!GlorpVideoCreditStatus methodsFor: 'As yet unclassified' stamp: ''!
customer
	^customer.! !

!GlorpVideoCreditStatus methodsFor: 'As yet unclassified' stamp: ''!
customer: aCustomer
	customer := aCustomer.! !

!GlorpVideoCreditStatus methodsFor: 'As yet unclassified' stamp: ''!
dateOwing
	^dateOwing.! !

!GlorpVideoCreditStatus methodsFor: 'As yet unclassified' stamp: ''!
dateOwing: aDate
	dateOwing := aDate.! !

GlorpTestModelClass subclass: #GlorpVideoCustomer
	instanceVariableNames: 'name dateOfBirth onlinePurchases inStorePurchases inStoreRentals store creditStatus'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpVideoCustomer class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpVideoCustomer methodsFor: 'comparing' stamp: ''!
<= aVideoCustomer
	^self name < aVideoCustomer name
	or: [self name = aVideoCustomer name
		and: [self dateOfBirth <= aVideoCustomer dateOfBirth]]! !

!GlorpVideoCustomer methodsFor: 'As yet unclassified' stamp: ''!
addInStorePurchase: aPurchase
	inStorePurchases add: aPurchase.
	aPurchase customer: self.! !

!GlorpVideoCustomer methodsFor: 'As yet unclassified' stamp: ''!
addOnlinePurchase: aPurchase
	onlinePurchases add: aPurchase.
	aPurchase customer: self.! !

!GlorpVideoCustomer methodsFor: 'As yet unclassified' stamp: ''!
addRental: aRental
	inStoreRentals add: aRental.
	aRental customer: self.! !

!GlorpVideoCustomer methodsFor: 'testing' stamp: ''!
assertEqual: aCustomer

	onlinePurchases assertEqual: aCustomer onlinePurchases.
	inStorePurchases assertEqual: aCustomer inStorePurchases.
	inStoreRentals assertEqual: aCustomer inStoreRentals.
	creditStatus assertEqual: aCustomer creditStatus.
	self assert: store id = aCustomer videoStore id.
	self assert: name = aCustomer name.
	self assert: dateOfBirth = aCustomer dateOfBirth.! !

!GlorpVideoCustomer methodsFor: 'accessing' stamp: ''!
creditStatus
	^creditStatus! !

!GlorpVideoCustomer methodsFor: 'accessing' stamp: ''!
creditStatus: aCreditStatus
	creditStatus := aCreditStatus.
	creditStatus isNil ifFalse: [creditStatus customer: self].! !

!GlorpVideoCustomer methodsFor: 'As yet unclassified' stamp: ''!
dateOfBirth
	^dateOfBirth.! !

!GlorpVideoCustomer methodsFor: 'As yet unclassified' stamp: ''!
dateOfBirth: aDate
	dateOfBirth := aDate.! !

!GlorpVideoCustomer methodsFor: 'accessing' stamp: ''!
inStorePurchases
	^inStorePurchases! !

!GlorpVideoCustomer methodsFor: 'accessing' stamp: ''!
inStorePurchases: anObject
	inStorePurchases := anObject! !

!GlorpVideoCustomer methodsFor: 'accessing' stamp: ''!
inStoreRentals
	^inStoreRentals.! !

!GlorpVideoCustomer methodsFor: 'initialize-release' stamp: ''!
initialize
	onlinePurchases := OrderedCollection new.
	inStorePurchases :=OrderedCollection new.
	inStoreRentals := OrderedCollection new.! !

!GlorpVideoCustomer methodsFor: 'As yet unclassified' stamp: ''!
name
	^name.! !

!GlorpVideoCustomer methodsFor: 'As yet unclassified' stamp: ''!
name: aString
	name := aString.! !

!GlorpVideoCustomer methodsFor: 'accessing' stamp: ''!
onlinePurchases
	^onlinePurchases! !

!GlorpVideoCustomer methodsFor: 'accessing' stamp: ''!
onlinePurchases: anObject
	onlinePurchases := anObject! !

!GlorpVideoCustomer methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: (name isNil ifTrue: [''] ifFalse: [name]).
	aStream nextPut: $).! !

!GlorpVideoCustomer methodsFor: 'As yet unclassified' stamp: ''!
purchases
	^inStorePurchases, onlinePurchases.! !

!GlorpVideoCustomer methodsFor: 'accessing' stamp: ''!
store: anObject
	store := anObject! !

!GlorpVideoCustomer methodsFor: 'accessing' stamp: ''!
videoStore
	^store! !

GlorpTestModelClass subclass: #GlorpVideoPurchase
	instanceVariableNames: 'id title price inStore customer store'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpVideoPurchase class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpVideoPurchase methodsFor: 'comparing' stamp: ''!
<= aVideoPurchase
	^self title <= aVideoPurchase title.! !

!GlorpVideoPurchase methodsFor: 'testing' stamp: ''!
assertEqual: aPurchase

	self assert: customer name = aPurchase customer name.
	self assert: customer dateOfBirth = aPurchase customer dateOfBirth.
	self assert: store id = aPurchase videoStore id.
	self assert: id = aPurchase id.
	self assert: title = aPurchase title.
	self assert: (price * 1000) rounded = (aPurchase price * 1000) rounded.
	self assert: inStore = aPurchase inStore.! !

!GlorpVideoPurchase methodsFor: 'As yet unclassified' stamp: ''!
customer
	^customer! !

!GlorpVideoPurchase methodsFor: 'As yet unclassified' stamp: ''!
customer: aCustomer
	customer := aCustomer.! !

!GlorpVideoPurchase methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpVideoPurchase methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpVideoPurchase methodsFor: 'accessing' stamp: ''!
inStore
	^inStore! !

!GlorpVideoPurchase methodsFor: 'accessing' stamp: ''!
inStore: anObject
	inStore := anObject! !

!GlorpVideoPurchase methodsFor: 'initialize-release' stamp: ''!
initialize! !

!GlorpVideoPurchase methodsFor: 'accessing' stamp: ''!
price
	^price! !

!GlorpVideoPurchase methodsFor: 'accessing' stamp: ''!
price: anObject
	price := anObject! !

!GlorpVideoPurchase methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self title.
	aStream nextPut: $).! !

!GlorpVideoPurchase methodsFor: 'As yet unclassified' stamp: ''!
store: aGlorpVideoStore
	store := aGlorpVideoStore.! !

!GlorpVideoPurchase methodsFor: 'accessing' stamp: ''!
title
	^title! !

!GlorpVideoPurchase methodsFor: 'accessing' stamp: ''!
title: aString
	title := aString! !

!GlorpVideoPurchase methodsFor: 'As yet unclassified' stamp: ''!
videoStore
	^store.! !

GlorpTestModelClass subclass: #GlorpVideoRental
	instanceVariableNames: 'id title dueDate price customer store'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpVideoRental class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpVideoRental methodsFor: 'comparing' stamp: ''!
<= aVideoRental
	^self title <= aVideoRental title.! !

!GlorpVideoRental methodsFor: 'testing' stamp: ''!
assertEqual: aRental

	self assert: customer name = aRental customer name.
	self assert: customer dateOfBirth = aRental customer dateOfBirth.
	self assert: id = aRental id.
	self assert: title = aRental title.
	self assert: price = aRental price.
	self assert: dueDate = aRental dueDate.! !

!GlorpVideoRental methodsFor: 'As yet unclassified' stamp: ''!
customer
	^customer.! !

!GlorpVideoRental methodsFor: 'As yet unclassified' stamp: ''!
customer: aCustomer
	customer := aCustomer.! !

!GlorpVideoRental methodsFor: 'accessing' stamp: ''!
dueDate
	^dueDate! !

!GlorpVideoRental methodsFor: 'accessing' stamp: ''!
dueDate: anObject
	dueDate := anObject! !

!GlorpVideoRental methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpVideoRental methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpVideoRental methodsFor: 'initialize-release' stamp: ''!
initialize
	"Initialize a newly created instance. This method must answer the receiver."

	" *** Edit the following to properly initialize instance variables ***"
	id := nil.
	title := nil.
	dueDate := nil.
	price := nil.
	" *** And replace this comment with additional initialization code *** "
	^self! !

!GlorpVideoRental methodsFor: 'accessing' stamp: ''!
price
	^price! !

!GlorpVideoRental methodsFor: 'accessing' stamp: ''!
price: anObject
	price := anObject! !

!GlorpVideoRental methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self title.
	aStream nextPut: $).! !

!GlorpVideoRental methodsFor: 'As yet unclassified' stamp: ''!
store: aGlorpVideoStore
	store := aGlorpVideoStore.! !

!GlorpVideoRental methodsFor: 'accessing' stamp: ''!
title
	^title! !

!GlorpVideoRental methodsFor: 'accessing' stamp: ''!
title: anObject
	title := anObject! !

GlorpTestModelClass subclass: #GlorpVideoStore
	instanceVariableNames: 'id name customers purchases rentals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpVideoStore class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpVideoStore methodsFor: 'testing' stamp: ''!
assertEqual: aStore
	self assert: id = aStore id.
	self assert: name = aStore name.

	customers assertEqual: aStore customers.
	purchases assertEqual: aStore purchases.! !

!GlorpVideoStore methodsFor: 'private' stamp: ''!
computeCollections
	customers do: [:each | each store: self].
	purchases := customers inject: #() into: [:sum :each | sum, each purchases].
	purchases do: [:each | each store: self].! !

!GlorpVideoStore methodsFor: 'accessing' stamp: ''!
customers
	^customers! !

!GlorpVideoStore methodsFor: 'accessing' stamp: ''!
customers: anObject
	customers := anObject! !

!GlorpVideoStore methodsFor: 'As yet unclassified' stamp: ''!
id
	^id.! !

!GlorpVideoStore methodsFor: 'As yet unclassified' stamp: ''!
id: anInteger
	id := anInteger.! !

!GlorpVideoStore methodsFor: 'initialize-release' stamp: ''!
initialize
	customers := OrderedCollection new.
	purchases := OrderedCollection new.! !

!GlorpVideoStore methodsFor: 'As yet unclassified' stamp: ''!
name
	^name.! !

!GlorpVideoStore methodsFor: 'As yet unclassified' stamp: ''!
name: aString
	name := aString.! !

!GlorpVideoStore methodsFor: 'accessing' stamp: ''!
purchases
	^purchases! !

!GlorpVideoStore methodsFor: 'accessing' stamp: ''!
purchases: aCollection
	purchases := aCollection! !

!Collection methodsFor: '*GlorpTests' stamp: ''!
assertEqual: aCollection
	"A convenience method for testing"
	self asSortedCollection with: aCollection asSortedCollection do: [:a :b |
		a assertEqual: b].! !

DatabaseAccessor subclass: #GlorpMockAccessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpMockAccessor methodsFor: 'executing' stamp: ''!
basicExecuteSQLString: aString
	^ReadStream on: #(#(3)).! !

!GlorpMockAccessor methodsFor: 'executing' stamp: ''!
executeCommand: aCommand returnCursor: aBoolean
	| result |
	result := #(#(3)).
	^aBoolean ifTrue: [result readStream] ifFalse: [result].! !

!GlorpMockAccessor methodsFor: 'executing' stamp: ''!
executeSQLString: aString
	^#((3)).! !

TestCase subclass: #GlorpBreadthFirstTopologicalSortTest
	instanceVariableNames: 'unsorted sorted groupA groupB groupC'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'support' stamp: ''!
assertSortOrder: aCollection
	| ids |
	ids := (sorted collect: [:each | each id]) asArray.
	self assert: ids = aCollection asArray.! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'data' stamp: ''!
data1
	| one two three |
	one := GlorpBreadthFirstTopologicalSortItem new id: 1.
	two := GlorpBreadthFirstTopologicalSortItem new id: 2.
	three := GlorpBreadthFirstTopologicalSortItem new id: 3.
	one relateTo: two.
	two relateTo: three.
	^Array with: one with: two with: three.! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'data' stamp: ''!
dataForGroupNamed: aString
	| one two three four |
	one := GlorpBreadthFirstTopologicalSortItem new id: aString, '1'.
	two := GlorpBreadthFirstTopologicalSortItem new id: aString, '2'.
	three := GlorpBreadthFirstTopologicalSortItem new id: aString, '3'.
	four := GlorpBreadthFirstTopologicalSortItem new id: aString, '4'.
	^Array with: one with: two with: three with: four.! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'data' stamp: ''!
dataForMedium
	"Data to ensure that things tend to stay together.  This is a simple grouping a1->b, b->c, b->a2, repeated, with some extra floating b's and c's."
	groupA := self dataForGroupNamed: 'a'.
	groupB := self dataForGroupNamed: 'b'.
	groupC := self dataForGroupNamed: 'c'.
	groupA first relateTo: groupB first.
	(groupA at: 3) relateTo: (groupB at: 3).
	groupB first relateTo: groupC first.
	(groupB at: 3) relateTo: (groupC at: 3).
	groupB first relateTo: (groupA at: 2).
	(groupB at: 3) relateTo: (groupA at: 4).
	^groupA, groupB, groupC.! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'data' stamp: ''!
dataForNoCyclesMedium
	"Data to ensure that things tend to stay together.  This is a simple grouping a->b, b->c repeated"
	groupA := self dataForGroupNamed: 'a'.
	groupB := self dataForGroupNamed: 'b'.
	groupC := self dataForGroupNamed: 'c'.
	1 to: 4 do: [:i |
		(groupA at: i) relateTo: (groupB at: i).
		(groupB at: i) relateTo: (groupC at: i)].
	^groupC, groupB, groupA.! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'data' stamp: ''!
dataForNoCyclesMediumPermuted
	"Data to ensure that things tend to stay together.  This is a simple grouping a->b, b->c repeated"
	groupA := self dataForGroupNamed: 'a'.
	groupB := self dataForGroupNamed: 'b'.
	groupC := self dataForGroupNamed: 'c'.
	1 to: 4 do: [:i |
		(groupA at: i) relateTo: (groupB at: i).
		(groupB at: i) relateTo: (groupC at: i)].
	^groupB, groupA, groupC.! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'support' stamp: ''!
sort
	| sorter |
	sorter := GlorpRowSorter new getChildrenVia: #relatedItems.
	sorted := sorter sort: unsorted.! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'support' stamp: ''!
sort: aCollection
	unsorted := aCollection.
	self sort.! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'tests' stamp: ''!
testBasicSort1
	self sort: self data1.
	self assertSortOrder: #(3 2 1).! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'tests' stamp: ''!
testBasicSort2
	"Try it with a different initial order. The sort is a convoluted way of writing the questionably-portable reverse"
	self sort: (self data1 asSortedCollection: [:a :b | a id >= b id]).
	self assertSortOrder: #(3 2 1).! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'tests' stamp: ''!
testBasicSortCompleteGraph
	| one two three |
	one := GlorpBreadthFirstTopologicalSortItem new id: 1.
	two := GlorpBreadthFirstTopologicalSortItem new id: 2.
	three := GlorpBreadthFirstTopologicalSortItem new id: 3.
	one relateTo: two.
	one relateTo: three.
	three relateTo: two.
	three relateTo: one.
	two relateTo: three.
	two relateTo: one.
	self sort: (Array with: one with: two with: three).
	self assert: sorted size = 3.! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'tests' stamp: ''!
testMedium1
	self sort: self dataForMedium.
	self assertSortOrder: #('c1' 'a2' 'b1' 'a1' 'c3' 'a4' 'b3' 'a3' 'b2' 'b4' 'c2' 'c4').! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'tests' stamp: ''!
testNoCyclesMedium
	self sort: self dataForNoCyclesMedium.
	self assertSortOrder: #('c1' 'c2' 'c3' 'c4' 'b1' 'b2' 'b3' 'b4' 'a1' 'a2' 'a3' 'a4').! !

!GlorpBreadthFirstTopologicalSortTest methodsFor: 'tests' stamp: ''!
testNoCyclesMediumPermuted
	self sort: self dataForNoCyclesMediumPermuted.
	self assertSortOrder: #('c1' 'b1' 'c2' 'b2' 'c3' 'b3' 'c4' 'b4' 'a1' 'a2' 'a3' 'a4').! !

TestCase subclass: #GlorpCommandTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpCommandTest methodsFor: 'As yet unclassified' stamp: ''!
testFieldsToUpdate

	| command system row old bookTable session |
	session := GlorpLockingDescriptorSystem sessionForLogin: (Login new database: OraclePlatform new).
	system := session system.
	bookTable := system tableNamed: 'BOOK'.
	row := DatabaseRow newForTable: bookTable.
	old := DatabaseRow newForTable: bookTable.
	old at: (bookTable fieldNamed: 'ID') put: 6.
	old at: (bookTable fieldNamed: 'TITLE') put: 'ABook'.
	old at: (bookTable fieldNamed: 'DESCRIPTION') put: 'Stuff'.
	old at: (bookTable fieldNamed: 'COPIES_IN_STOCK') put: 3.
	old at: (bookTable fieldNamed: 'VERSION') put: 2.

	row at: (bookTable fieldNamed: 'ID') put: 6.
	row at: (bookTable fieldNamed: 'TITLE') put: 'ABook'.
	row at: (bookTable fieldNamed: 'DESCRIPTION') put: 'Stuff'.
	row at: (bookTable fieldNamed: 'COPIES_IN_STOCK') put: 4.
	row at: (bookTable fieldNamed: 'VERSION') put: 2.
	row withAllFieldsIn: old.
	command := UpdateCommand forRows: (Array with: row) useBinding: false session: session.
	self assert: command shouldUpdateOnlyChangedFields.
	self assert: command singleRowBoundFields size = 2.
	self assert: command singleRowBindings isEmpty. 
	command useBinding: true.
	self deny: command shouldUpdateOnlyChangedFields.
	self assert: command singleRowBoundFields size = 5.
	"The lock field is bound at the end"
	self assert: command singleRowBindings size = 6.! !

!GlorpCommandTest methodsFor: 'tests' stamp: ''!
testSplittingInserts1
	| rows command subCommands |
	rows := Array with: DatabaseRow new.
	command := InsertCommand forRows: rows useBinding: false session: nil.
	subCommands := command subCommands.
	self assert: subCommands size = 1.
	self assert: subCommands first allRows size = 1.
	self assert: subCommands first allRows first == rows first.! !

!GlorpCommandTest methodsFor: 'tests' stamp: ''!
testSplittingInserts1000
	| rows command subCommands |
	rows := (Array new: 1000) atAllPut: DatabaseRow new.
	command := InsertCommand forRows: rows useBinding: false session: nil.
	subCommands := command subCommands.
	self assert: subCommands size = 1.
	self assert: subCommands first allRows size = 1000.
	self assert: subCommands first allRows first == rows first.! !

!GlorpCommandTest methodsFor: 'tests' stamp: ''!
testSplittingInserts1001
	| rows command subCommands |
	rows := (Array new: 1001) atAllPut: DatabaseRow new.
	rows at: rows size put: DatabaseRow new.
	command := InsertCommand forRows: rows useBinding: false session: nil.
	subCommands := command subCommands.
	self assert: subCommands size = 2.
	self assert: subCommands first allRows size = 1000.
	self assert: subCommands first allRows first == rows first.
	self assert: subCommands last allRows size = 1.
	self assert:subCommands last allRows first == rows last.! !

!GlorpCommandTest methodsFor: 'tests' stamp: ''!
testSplittingInserts1003
	| rows command subCommands |
	rows := (Array new: 1003) atAllPut: DatabaseRow new.
	rows at: rows size put: DatabaseRow new.
	command := InsertCommand forRows: rows useBinding: false session: nil.
	subCommands := command subCommands.
	self assert: subCommands size = 2.
	self assert: subCommands first allRows size = 1000.
	self assert: subCommands first allRows first == rows first.
	self assert: subCommands last allRows size = 3.
	self assert:subCommands last allRows last == rows last.
	self assert: subCommands last allRows first == rows first.! !

!GlorpCommandTest methodsFor: 'tests' stamp: ''!
testSplittingInserts2
	| rows command subCommands |
	rows := Array with: DatabaseRow new with: DatabaseRow new.
	command := InsertCommand forRows: rows useBinding: false session: nil.
	subCommands := command subCommands.
	self assert: subCommands size = 1.
	self assert: subCommands first allRows size = 2.
	self assert: subCommands first allRows first == rows first.
	self assert: subCommands first allRows last == rows last.! !

TestCase subclass: #GlorpCursoredCollectionTest
	instanceVariableNames: 'session stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpCursoredCollectionTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpSessionResource.! !

!GlorpCursoredCollectionTest methodsFor: 'support' stamp: ''!
customerStream
	| rows |
	rows := #( ( 1 'Alan' ) (2 'Bob')).
	^ReadStream on: rows.! !

!GlorpCursoredCollectionTest methodsFor: 'support' stamp: ''!
joinedStream
	| rows |
	rows := #( ( 1 'Alan' 101 101 '1st Avenue' 111) (2 'Bob' 102 102 '2nd Avenue' 222) ).
	^ReadStream on: rows.! !

!GlorpCursoredCollectionTest methodsFor: 'support' stamp: ''!
joinedStream2
	| rows |
	rows := #( (1 'Alan' 101 101 '1st Avenue' 111  ) (2 'Bob' 102 102 '2nd Avenue' 222  ) ).
	^ReadStream on: rows.! !

!GlorpCursoredCollectionTest methodsFor: 'support' stamp: ''!
joinedStreamToMany
	| rows |
	rows := #( (1 'Alan' 101  7 'root' 'foo.com' )  (1 'Alan' 101  8 'root' 'bar.com' )  ).
	^ReadStream on: rows.! !

!GlorpCursoredCollectionTest methodsFor: 'support' stamp: ''!
joinedStreamToMany3
	| rows |
	rows := #( 
			(1 'Alan' 1 '001' '001'  '123' 1 1 #CDN 1.5 'stuff' #CDN 1)  
			(1 'Alan' 1 '001' '001'  '123' 2 1  #USD 1.5 'stuff' #USD 1)
 			(1 'Alan' 2 '002' '002'  '234' 3 1  #CHF 1.5 'stuff' #CHF 1)
			(1 'Alan' 2 '002' '002'  '234' 4 1  #EUR 1.5 'stuff' #EUR 1)
 ).
	^ReadStream on: rows.! !

!GlorpCursoredCollectionTest methodsFor: 'support' stamp: ''!
mockStream
	| rows |
	rows := #( ( 1 'Alan' 101) (2 'Bob' 102) (3 'Carol' 103)).
	^ReadStream on: rows.! !

!GlorpCursoredCollectionTest methodsFor: 'support' stamp: ''!
mockStream2
	| rows |
	rows := #( ( 4 'David' 104) (5 'Evan' 105)).
	^ReadStream on: rows.! !

!GlorpCursoredCollectionTest methodsFor: 'support' stamp: ''!
setUp
	| query |
	super setUp.
	session := GlorpSessionResource current newSession.
	((session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #name) shouldProxy: false.
	query := Query readManyOf: GlorpPerson.
	query session: session.
	query prepare.
	stream := GlorpCursoredStream on: self mockStream for: (query sqlWith: nil) accessor: nil.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testBuilderIsolation
	"Make sure that we can have multiple streams open simultaneously against the same query"
	| stream2 person1 stream3 person4 person4a |
	stream2 := GlorpCursoredStream on: self mockStream2 for: stream command.
	stream3 := GlorpCursoredStream on: self mockStream2 for: stream command.
	person1 := stream next.
	person4 := stream2 next.
	person4a := stream3 next.
	self assert: person4 == person4a.
	self assert: person4 id = 4.
	self assert: person1 id = 1.
	self assert: stream next id = 2.
	self assert: person4a id = 4.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testCollect
	self assert: (stream collect: [:each | each name]) = #('Alan' 'Bob' 'Carol').! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testKnittingWithAlsoFetch
	| query person1 person2 |
	query := Query readManyOf: GlorpPerson.
	query session: session.
	query alsoFetch: [:each | each address].
	query prepare.
	stream := GlorpCursoredStream on: self joinedStream2 for: (query sqlWith: nil).
	person1 := stream next.
	self assert: session privateGetCache numberOfElements = 2.
	self assert: person1 address street = '1st Avenue'.
	person2 := stream next.
	self assert: session privateGetCache numberOfElements = 4.
	self assert: stream atEnd.
	self assert: person2 id = 2.
	self assert: person2 address street = '2nd Avenue'.
	self assert: session privateGetCache numberOfElements = 4.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testKnittingWithAlsoFetchDoubleToMany
	| query customer1 |
	query := Query readManyOf: GlorpCustomer.
	query session: session.
	query alsoFetch: [:each | each accounts].
	query alsoFetch: [:each | each transactions].
	query prepare.
	stream := GlorpCursoredStream on: self joinedStreamToMany3 for: (query sqlWith: #()).
	customer1 := stream next.
	self assert: session privateGetCache numberOfElements = 7.
	self assert: stream atEnd.
	self assert: customer1 id = 1.
	self assert: customer1 accounts size = 2.
	self assert: customer1 accounts first accountNumber accountNumber = '123'.
	self assert: customer1 accounts last accountNumber accountNumber = '234'.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testKnittingWithAlsoFetchToManyInSingleGroup
	| query person1 |
	query := Query readManyOf: GlorpPerson.
	query session: session.
	query alsoFetch: [:each | each emailAddresses].
	query prepare.
	stream := GlorpCursoredStream on: self joinedStreamToMany for: (query sqlWith: #()).
	person1 := stream next.
	self assert: session privateGetCache numberOfElements = 3.
	self assert: (person1 emailAddresses allSatisfy: [:each | each user = 'root']).
	self assert: person1 emailAddresses first ~= person1 emailAddresses last.
	self assert: stream atEnd.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testKnittingWithRetrieve
	| query tuple person1 address1 person2 address2 |
	query := Query readManyOf: GlorpPerson.
	query session: session.
	query retrieve: [:each | each].
	query retrieve: [:each | each address].
	query prepare.
	stream := GlorpCursoredStream on: self joinedStream for: (query sqlWith: nil).
	tuple := stream next.
	self assert: session privateGetCache numberOfElements = 2.
	person1 := tuple first.
	address1 := tuple last.
	self assert: address1 street = '1st Avenue'.
	self assert: person1 address == address1.
	tuple := stream next.
	self assert: stream atEnd.
	person2 := tuple first.
	address2 := tuple last.
	self assert: person2 id = 2.
	self assert: address2 street = '2nd Avenue'.
	self assert: person2 address == address2.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testNext
	| person |
	person := stream next.
	self assert: person class == GlorpPerson.
	self assert: person id = 1.
	self assert: person name = 'Alan'.
	self assert: session privateGetCache numberOfElements = 1.
	person := stream next.
	self assert: person id = 2.
	self assert: person name = 'Bob'.
	self assert: session privateGetCache numberOfElements = 2.
	stream next.
	self assert: stream atEnd.
	self assert: stream next = nil.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testNextColon
	| people |
	people := stream next: 2.
	self assert: people size = 2.
	self assert: people first id = 1.
	self assert: people first name = 'Alan'.
	self assert: session privateGetCache numberOfElements = 2.
	self assert: people last id = 2.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testPostFetchEvents
	| query customer |
	query := Query readManyOf: GlorpCustomer.
	query session: session.
	query prepare.
	stream := GlorpCursoredStream on: self customerStream for: (query sqlWith: #()).
	customer := stream next.
	self assert: customer seenPostFetch.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testReadEmpty
	stream := GlorpCursoredStream on: (ReadStream on: #()) for: (stream query sqlWith: #()).
	self assert: stream atEnd.
	self assert: stream next isNil.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testRegistration
	| person |
	session beginUnitOfWork.
	person := stream next.
	self assert: (session isRegistered: person).! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testRelease
	| person |
	stream next.
	stream release.
	self assert: stream atEnd.
	self assert: stream next isNil.! !

!GlorpCursoredCollectionTest methodsFor: 'tests' stamp: ''!
testUpToEnd
	| people |
	people := stream upToEnd.
	self assert: people size = 3.
	self assert: people first name = 'Alan'.
	self assert: people last name = 'Carol'.! !

TestCase subclass: #GlorpDescriptorValidationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDescriptorValidationTest methodsFor: 'testing' stamp: ''!
testJoinDirection
	| descriptor table1 table2 mapping system descriptor2 |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	descriptor2 := system addEmptyDescriptorFor: GlorpVideoStore.
	table1 := DatabaseTable named: 'Table1'.
	table1 createFieldNamed: 'one' type: nil.
	table2 := DatabaseTable named: 'Table2'.
	table2 createFieldNamed: 'two' type: nil.
	descriptor addTable: table1.
	descriptor2  addTable: table2.
	mapping := descriptor newMapping: OneToOneMapping.
	mapping attributeName: #accounts.
	mapping referenceClass: GlorpVideoStore.
	mapping join: (Join from: (table1 fieldNamed: 'one') to: (table2 fieldNamed: 'two')).
	mapping validate.

	mapping := descriptor newMapping: OneToOneMapping.
	mapping attributeName: #bar.
	mapping referenceClass: GlorpVideoStore.
	mapping join: (Join from: (table2 fieldNamed: 'two') to: (table1 fieldNamed: 'one')).
	self should: [mapping validate] raise: Dialect error.! !

!GlorpDescriptorValidationTest methodsFor: 'testing' stamp: ''!
testValidateDirectInstvarAccessExisting
	| descriptor table mapping system field |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	table := DatabaseTable named: 'Table1'.
	field := table createFieldNamed: 'one' type: OraclePlatform new integer.
	descriptor addTable: table.
	mapping := descriptor newMapping: DirectMapping.
	mapping from: #id to: field.
	mapping validate! !

!GlorpDescriptorValidationTest methodsFor: 'testing' stamp: ''!
testValidateDirectInstvarAccessNonExisting
	| descriptor table mapping system field |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	table := DatabaseTable named: 'Table1'.
	field := table createFieldNamed: 'one' type: OraclePlatform new integer.
	descriptor addTable: table.
	mapping := descriptor newMapping: DirectMapping.
	mapping from: #bar to: field.
	self should: [mapping validate] raise: Dialect error! !

!GlorpDescriptorValidationTest methodsFor: 'testing' stamp: ''!
testValidateSelectorAccessExisting
	| descriptor table mapping system field |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	table := DatabaseTable named: 'Table1'.
	field := table createFieldNamed: 'one' type: OraclePlatform new integer.
	descriptor addTable: table.
	mapping := descriptor newMapping: DirectMapping.
	mapping from: #id to: field.
	mapping attribute useDirectAccess: false.
	mapping validate! !

!GlorpDescriptorValidationTest methodsFor: 'testing' stamp: ''!
testValidateSelectorAccessNonExisting
	| descriptor table mapping system field |
	system := DynamicDescriptorSystem new.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	table := DatabaseTable named: 'Table1'.
	field := table createFieldNamed: 'one' type: OraclePlatform new integer.
	descriptor addTable: table.
	mapping := descriptor newMapping: DirectMapping.
	mapping from: #foo to: field.
	mapping attribute useDirectAccess: false.
	self should: [mapping validate] raise: Dialect error! !

!GlorpDescriptorValidationTest methodsFor: 'testing' stamp: 'GuillermoPolito 4/26/2011 04:38'!
testValidatingExistingDescriptors
	| session |
	session := GlorpSessionResource current newSession.
	GlorpTestDescriptorSystem allSubclasses do: [:each |
		session system: (each forPlatform: (OraclePlatform new)).
		session system validate].! !

TestCase subclass: #GlorpFilteringQueryTest
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpFilteringQueryTest methodsFor: 'support' stamp: ''!
setUp! !

!GlorpFilteringQueryTest methodsFor: 'tests' stamp: ''!
testSimple
	| query result |
	query := FilteringQuery on: #( 1 2 3 4 5 6 7 8) filter: [:each :parameters | each odd].
	result := GlorpSession new execute: query.
	self assert: result = #(1 3 5 7).! !

TestCase subclass: #GlorpMainBackendTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Extras'!

!GlorpMainBackendTest class methodsFor: 'as yet unclassified' stamp: 'GuillermoPolito 4/24/2011 23:56'!
glorpBackendFacility
	^ self subclassResponsibility ! !

!GlorpMainBackendTest methodsFor: 'as yet unclassified' stamp: 'GuillermoPolito 4/24/2011 23:57'!
testSetToBaseTestAFacility
	(Smalltalk globals at: #DBXGlorpBaseTest) ifNotNil: [ :c | c facility: self class glorpBackendFacility ]! !

TestCase subclass: #GlorpNoProxyCollectionReadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpNoProxyCollectionReadTest methodsFor: 'tests' stamp: ''!
testReadCollectionWithoutProxies
	"before running this test change RelationshipMapping>>initialize shouldProxy := false"

	| persons session |
	session := GlorpSessionResource current newSession.
	session platform supportsMultipleOpenCursors ifFalse: [^self].
	(session system descriptorFor: GlorpPerson) mappings do: [:each | (each respondsTo: #shouldProxy: ) 
		ifTrue: [each shouldProxy: false]].
	session beginTransaction.
	[session beginUnitOfWork.
	session register: GlorpPerson example1.
	session register: GlorpPerson example2.
	session commitUnitOfWork.
	session reset.
	persons := session readManyOf: GlorpPerson.
	self assert: persons size = 2] ensure: [session rollbackTransaction].! !

TestCase subclass: #GlorpTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

GlorpTestCase subclass: #GlorpAttributeModelTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpAttributeModelTest methodsFor: 'tests' stamp: ''!
testDictionaryTypes
	| attributeModel obj classModel dict model |
	dict := Dictionary new at: #one put: 'two'; yourself.
	obj := 1@dict.

	classModel := GlorpClassModel for: Point.
	attributeModel := classModel newAttributeNamed: #y dictionaryFrom: Symbol to: String.
	model := classModel attributeNamed: #y.
	self assert: model == attributeModel.
	self assert: model keyType == Symbol.
	self assert: model type == String.

	self assert: dict == (attributeModel getValueFrom: obj).
	attributeModel setValueIn: obj to: nil.
	self assert: nil == (attributeModel getValueFrom: obj).! !

!GlorpAttributeModelTest methodsFor: 'tests' stamp: ''!
testGetSetAssociation
	| attributeModel obj classModel |
	obj := nil->2.

	classModel := GlorpClassModel for: Association.
	attributeModel := classModel newAttributeNamed: #value.

	attributeModel useDirectAccess: true.
	self assert: 2 == (attributeModel getValueFrom: obj).
	attributeModel setValueIn: obj to: 3.
	self assert: 3 == (attributeModel getValueFrom: obj).

	attributeModel useDirectAccess: false.
	self assert: 3 == (attributeModel getValueFrom: obj).
	attributeModel setValueIn: obj to: 2.
	self assert: 2 == (attributeModel getValueFrom: obj).! !

!GlorpAttributeModelTest methodsFor: 'tests' stamp: ''!
testGetSetObject
	| attributeModel obj classModel |
	obj := (GlorpObjectWithNoAccessors new)
								extremelyPrivateValue: 'Glorp';
								yourself.

	classModel := GlorpClassModel for: GlorpObjectWithNoAccessors.
	attributeModel := classModel newAttributeNamed: #alue.

	attributeModel useDirectAccess: true.
	self assert: 'Glorp' = (attributeModel getValueFrom: obj).
	attributeModel setValueIn: obj to: 'GLORP'.
	self assert: 'GLORP' = (attributeModel getValueFrom: obj).

	attributeModel useDirectAccess: false.
	self should: [attributeModel getValueFrom: obj] raise: Dialect error.
	self should: [attributeModel setValueIn: obj to: 'Glorp'] raise: Dialect error.! !

!GlorpAttributeModelTest methodsFor: 'tests' stamp: ''!
testPreserveAttributes
	| attributeModel classModel descriptor mapping table field system |
	system := DynamicDescriptorSystem new.
	classModel := system addClassModelFor: Association.
	attributeModel := classModel newAttributeNamed: #key.
	descriptor := Descriptor new.
	descriptor classModel: classModel.
	table := DatabaseTable named: 'FRED'.
	field := table createFieldNamed: 'BARNEY' type: OraclePlatform new int4.
	mapping := DirectMapping from: #key type: Object to: field.
	self assert: mapping attributeAccessor useDirectAccess.
	self assert: attributeModel useDirectAccess.

	mapping attributeAccessor useDirectAccess: false.
	self assert: mapping attribute type == Object.
	descriptor addMapping: mapping.
	self assert: mapping attribute == attributeModel.
	self assert: mapping attribute == mapping attributeAccessor.
	self assert: attributeModel type == Object.
	self deny: attributeModel useDirectAccess.! !

GlorpTestCase subclass: #GlorpBasicMappingTest
	instanceVariableNames: 'mapping person'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpBasicMappingTest methodsFor: 'support' stamp: ''!
setUp

	super setUp.
	mapping := DirectMapping new.
	mapping attributeName: #id.
	mapping attribute classModel describedClass: GlorpPerson.
	person := GlorpPerson example1.! !

!GlorpBasicMappingTest methodsFor: 'tests' stamp: ''!
testGet
	
	self assert: (mapping getValueFrom: person) = 1! !

!GlorpBasicMappingTest methodsFor: 'tests' stamp: ''!
testSet
	
	mapping setValueIn: person to: 2.
	self assert: person id = 2.
	self assert: (mapping getValueFrom: person) = 2.! !

GlorpTestCase subclass: #GlorpConstantMappingTest
	instanceVariableNames: 'mappingToClass mappingToRow mappingToSession slot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpConstantMappingTest methodsFor: 'support' stamp: ''!
setUp
	| descriptor |
	super setUp.
	mappingToClass := ConstantMapping new
		attributeName: #slot;
		constantValue: 34.
	descriptor := Descriptor new.
	descriptor classModel: (GlorpClassModel for: self class).
	descriptor addMapping: mappingToClass.
	mappingToRow := ConstantMapping new.
	mappingToSession := ConstantMapping new! !

!GlorpConstantMappingTest methodsFor: 'tests' stamp: ''!
testConstantInClass
	slot := nil.
	mappingToClass mapObject: self inElementBuilder: nil.
	self assert: slot = 34.! !

!GlorpConstantMappingTest methodsFor: 'tests' stamp: ''!
testConstantInClassDoesNotWriteToRow

	"Would raise an exception if it tried to write into nil"
	mappingToClass mapFromObject: self intoRowsIn: nil.! !

!GlorpConstantMappingTest methodsFor: 'tests' stamp: ''!
testGetValue

	slot := nil.
	self assert: (mappingToClass getValueFrom: self) = 34.! !

!GlorpConstantMappingTest methodsFor: 'tests' stamp: ''!
testSessionValue
	mappingToClass constantValueIsSession.
	self assert: (mappingToClass constantValueIn: 38)== 38.! !

GlorpTestCase subclass: #GlorpConstantValueInRelationshipTest
	instanceVariableNames: 'session system'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpConstantValueInRelationshipTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource.! !

!GlorpConstantValueInRelationshipTest methodsFor: 'support' stamp: ''!
idsFor: aCollection

	^(aCollection collect: [:each | each id]) asSortedCollection asArray.! !

!GlorpConstantValueInRelationshipTest methodsFor: 'support' stamp: ''!
sampleWorker

	| worker job3 |
	worker := GlorpWorker new.
	worker id: 1234.
	worker name: 'Some Worker'.
	worker pendingJobs add: (GlorpJob new id: 1; description: 'job 1').
	worker pendingJobs add: (GlorpJob new id: 2; description: 'job 2').
	worker finishedJobs add: (job3 := GlorpJob new id: 3; description: 'job 3').
	worker finishedJobs add: (GlorpJob new id: 4; description: 'job 4').
	worker priorityJobs add: job3.
	^worker.! !

!GlorpConstantValueInRelationshipTest methodsFor: 'support' stamp: ''!
setUp
	system := GlorpWorkerDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database.
	session := GlorpSessionResource current newSession.
	session system: system.! !

!GlorpConstantValueInRelationshipTest methodsFor: 'tests' stamp: ''!
testRead

	| worker |
	[session beginTransaction.
	self writeTestData.
	worker := session execute: (Query returningOneOf: GlorpWorker where: [:each | each id = 1234]).

	self assert: (self idsFor: worker pendingJobs) = #(1 2).
	self assert: (self idsFor: worker finishedJobs) = #(3 4).
	self assert: (self idsFor: worker priorityJobs) = #(3).

	] ensure: [session rollbackTransaction].! !

!GlorpConstantValueInRelationshipTest methodsFor: 'tests' stamp: ''!
testReadWithJoin
	| worker |
	[session beginTransaction.
	self writeTestData.
	worker := session
		execute:
			(Query returningOneOf: GlorpWorker where: [:each | each pendingJobs id = 1]).
	self assert: worker id = 1234] ensure: [session rollbackTransaction].! !

!GlorpConstantValueInRelationshipTest methodsFor: 'tests' stamp: ''!
testWrite

	| worker sampleWorker |
	[session beginTransaction.
	session beginUnitOfWork.
	sampleWorker := self sampleWorker.
	session register: sampleWorker.
	session commitUnitOfWork.
	session reset.
	worker := session execute: (Query returningOneOf: GlorpWorker where: [:each | each id = 1234]).

	self assert: (self idsFor: worker pendingJobs) = #(1 2).
	self assert: (self idsFor: worker finishedJobs) = #(3 4).
	self assert: (self idsFor: worker priorityJobs) = #(3).

	] ensure: [session rollbackTransaction].! !

!GlorpConstantValueInRelationshipTest methodsFor: 'support' stamp: ''!
writeTestData

	session writeRow: system exampleWorkerRow.
	session writeRow: (system exampleJobRow: 1 finished: false).
	session writeRow: (system exampleJobRow: 2 finished: false).
	session writeRow: (system exampleJobRow: 3 finished: true).
	session writeRow: (system exampleJobRow: 4 finished: true).
	session writeRow: system exampleLinkRow1.
	session writeRow: system exampleLinkRow2.! !

GlorpTestCase subclass: #GlorpCoreExtensionsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpCoreExtensionsTest methodsFor: 'tests' stamp: ''!
testGlorpIsCollection
	self deny: Object new glorpIsCollection.
	self assert: Collection new glorpIsCollection! !

GlorpTestCase subclass: #GlorpDatabaseAccessorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDatabaseAccessorTest methodsFor: 'tests' stamp: ''!
testLoggingSwitch
	| currentSetting accessor |
	currentSetting := DatabaseAccessor loggingEnabled.
	accessor := DatabaseAccessor new.
	[DatabaseAccessor loggingEnabled: true.
	self assert: accessor logging.
	DatabaseAccessor loggingEnabled: false.
	self deny: accessor logging.
	accessor logging: true.
	self assert: accessor logging.
	accessor logging: false.
	self deny: accessor logging.
	DatabaseAccessor loggingEnabled: true.
	self deny: accessor logging] ensure: [DatabaseAccessor loggingEnabled: currentSetting].! !

GlorpTestCase subclass: #GlorpDatabaseBasedTest
	instanceVariableNames: 'system'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

GlorpDatabaseBasedTest subclass: #GlorpAdHocMappingTest
	instanceVariableNames: 'mapping person descriptor table money rowMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpAdHocMappingTest methodsFor: 'tests' stamp: ''!
moneyNegatingMappingTo: amountField in: aDescriptor 
	mapping := aDescriptor newMapping: AdHocMapping.
	mapping
		forAttribute: #amount
		fromDb: [:row :elementBuilder :context | (elementBuilder valueOfField: (context translateField: amountField) in: row) negated]
		toDb: [:rows :attribute :attributeRows | (rows at: table) at: amountField put: attribute negated]
		mappingFields: (Array with: amountField).
	^mapping.! !

!GlorpAdHocMappingTest methodsFor: 'tests' stamp: ''!
testNegateMappingRead
	| amountField inputRow builder |
	descriptor := system descriptorFor: GlorpMoney.
	table := system tableNamed: 'MONEY_IMAGINARY_TABLE'.
	amountField := table fieldNamed: 'AMOUNT'.
	mapping := self moneyNegatingMappingTo: amountField in: descriptor.
	descriptor addMapping: mapping.
	money := GlorpMoney basicNew.
	inputRow := #('US' 1).
	builder := ElementBuilder new.
	builder row: inputRow.
	mapping mapObject: money inElementBuilder: builder.
	self assert: money amount = -1.! !

!GlorpAdHocMappingTest methodsFor: 'tests' stamp: ''!
testNegateMappingWrite
	| amountField outputRow |
	descriptor := system descriptorFor: GlorpMoney.
	table := system tableNamed: 'MONEY_IMAGINARY_TABLE'.
	amountField := table fieldNamed: 'AMOUNT'.
	mapping := self moneyNegatingMappingTo: amountField in: descriptor.
	descriptor table: table.
	descriptor addMapping: mapping.
	money := GlorpMoney forAmount: 3.
	rowMap := RowMap new.
	mapping mapFromObject: money intoRowsIn: rowMap.
	outputRow := rowMap rowForTable: table withKey: money.
	self assert: (outputRow at: (table fieldNamed: 'AMOUNT')) = -3! !

!GlorpAdHocMappingTest methodsFor: 'tests' stamp: ''!
testSplitMappingRead
	| inputRow builder |
	money := GlorpCompressedMoney basicNew.
	mapping := (system descriptorFor: GlorpCompressedMoney)
		mappingForAttributeNamed: #array.
	inputRow := #(432 'US' 1).
	builder := ElementBuilder new.
	builder row: inputRow.
	mapping mapObject: money inElementBuilder: builder.
	self assert: money amount = 1.
	self assert: money currency = 'US'.! !

!GlorpAdHocMappingTest methodsFor: 'tests' stamp: ''!
testSplitMappingWrite
	| outputRow |
	money := GlorpCompressedMoney currency: 'DM' amount: 99.
	mapping := (system descriptorFor: GlorpCompressedMoney) 
				mappingForAttributeNamed: #array.
	rowMap := RowMap new.
	mapping mapFromObject: money intoRowsIn: rowMap.
	table := mapping descriptor primaryTable.
	outputRow := rowMap rowForTable: table withKey: money.
	self assert: (outputRow at: (table fieldNamed: 'AMOUNT')) = 99.
	self assert: (outputRow at: (table fieldNamed: 'CURRENCY_NAME')) = 'DM'! !

GlorpDatabaseBasedTest subclass: #GlorpCacheTest
	instanceVariableNames: 'cache session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpCacheTest class methodsFor: 'Accessing' stamp: ''!
resources
	^Array with: GlorpSessionResource! !

!GlorpCacheTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	system cachePolicy: CachePolicy new.
	session := GlorpSessionResource current newSession.
	session system: system.
	cache := session privateGetCache.! !

!GlorpCacheTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session reset.
	session := nil.! !

!GlorpCacheTest methodsFor: 'tests' stamp: ''!
testDuplicates 
	| c1 c2 |
	c1 := GlorpCustomer example1.
	c2 := GlorpCustomer example1.
	cache at: 3 insert: c1.
	cache at: 3 insert: c2.
	self assert: (cache lookupClass: GlorpCustomer key: 3) = c1.! !

!GlorpCacheTest methodsFor: 'tests' stamp: ''!
testDuplicatesDifferentClasses

	| cust trans |
	cust := GlorpCustomer example1.
	trans := GlorpBankTransaction example1.
	cache at: 3 insert: cust.
	cache at: 3 insert: trans.
	self assert: (cache lookupClass: GlorpCustomer key: 3) = cust.
	self assert: (cache lookupClass: GlorpBankTransaction key: 3) = trans.! !

!GlorpCacheTest methodsFor: 'tests' stamp: ''!
testInsert
	| customer |
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self assert: (cache lookupClass: GlorpCustomer key: 3) == customer! !

!GlorpCacheTest methodsFor: 'tests' stamp: ''!
testManualExpireAll
	| customer customer2 readBackCustomer |
	system cachePolicy: TimedExpiryCachePolicy new.
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self deny: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	cache expireAll.
	readBackCustomer := session readOneOf: GlorpCustomer where: [:each | each id = 3].
	self assert: readBackCustomer isNil.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	customer2 := GlorpCustomer new.
	cache at: 3 insert: customer2.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == customer2.! !

!GlorpCacheTest methodsFor: 'tests' stamp: ''!
testManualExpiry
	| customer customer2 readBackCustomer |
	system cachePolicy: TimedExpiryCachePolicy new.
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self deny: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	cache expireClass: GlorpCustomer key: 3 ifAbsent: [self error: 'wrong'].
	readBackCustomer := session readOneOf: GlorpCustomer where: [:each | each id = 3].
	self assert: readBackCustomer isNil.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	customer2 := GlorpCustomer new.
	cache at: 3 insert: customer2.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == customer2.! !

!GlorpCacheTest methodsFor: 'tests' stamp: ''!
testRemove
	| customer |
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self assert: (cache lookupClass: GlorpCustomer key: 3) == customer.
	cache removeClass: GlorpCustomer key: 3 ifAbsent: [ self signalFailure: 'Item was not in cache.' ].
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: []) == nil.! !

GlorpCacheTest subclass: #GlorpTimedExpiryCacheTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpTimedExpiryCacheTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	self setUpExpiryWithRealDelay.! !

!GlorpTimedExpiryCacheTest methodsFor: 'support' stamp: ''!
setUpExpiryWithRealDelay
	(cache session descriptorFor: GlorpCustomer) cachePolicy: (TimedExpiryCachePolicy new timeout: 1).
	(cache session descriptorFor: GlorpBankTransaction) cachePolicy: (TimedExpiryCachePolicy new timeout: 1).! !

!GlorpTimedExpiryCacheTest methodsFor: 'support' stamp: ''!
setUpExpiryWithZeroDelay
	(cache session descriptorFor: GlorpCustomer) cachePolicy: (TimedExpiryCachePolicy new timeout: 0).
	(cache session descriptorFor: GlorpBankTransaction) cachePolicy: (TimedExpiryCachePolicy new timeout: 0).! !

!GlorpTimedExpiryCacheTest methodsFor: 'support' stamp: ''!
setUpForExpiryActionOf: aSymbol
	(cache session descriptorFor: GlorpCustomer) cachePolicy expiryAction: aSymbol.
	(cache session descriptorFor: GlorpBankTransaction) cachePolicy expiryAction: aSymbol.! !

!GlorpTimedExpiryCacheTest methodsFor: 'support' stamp: ''!
setUpForNotify
	self setUpForExpiryActionOf: #notify.! !

!GlorpTimedExpiryCacheTest methodsFor: 'support' stamp: ''!
setUpForNotifyAndRemove
	self setUpForExpiryActionOf: #notifyAndRemove.! !

!GlorpTimedExpiryCacheTest methodsFor: 'support' stamp: ''!
setUpForRefresh
	self setUpForExpiryActionOf: #refresh.! !

!GlorpTimedExpiryCacheTest methodsFor: 'tests' stamp: ''!
testExpiryReturningNilWithRealDelay
	"test that objects expire with a non-zero delay time."
	| customer customer2 |
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self deny: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	(Delay forSeconds: 2) wait.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	customer2 := GlorpCustomer new.
	cache at: 3 insert: customer2.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == customer2.! !

!GlorpTimedExpiryCacheTest methodsFor: 'tests' stamp: ''!
testNotExpiredAfterRefresh
	| customer |
	self setUpForRefresh.
	session accessor beginTransaction.
	[session accessor executeSQLStringNoResult: 'INSERT INTO GR_CUSTOMER VALUES (3,''Fred Flintstone'')'.
	customer := session execute: (Query returningOneOf: GlorpCustomer where: [:each | each id = 3]).
	(Delay forSeconds: 2) wait.
	self assert: (cache hasExpired: customer).
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == customer.
	self deny: (cache hasExpired: customer)]
		ensure: [session accessor rollbackTransaction].! !

!GlorpTimedExpiryCacheTest methodsFor: 'tests' stamp: ''!
testNotify
	| customer |
	self setUpExpiryWithZeroDelay.
	self setUpForNotify.
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self deny: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	self assert: customer seenExpiry.! !

!GlorpTimedExpiryCacheTest methodsFor: 'tests' stamp: ''!
testNotifyAndRemove
	| customer |
	self setUpExpiryWithZeroDelay.
	self setUpForNotifyAndRemove.
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	self assert: customer seenExpiry = true.! !

!GlorpTimedExpiryCacheTest methodsFor: 'tests' stamp: ''!
testRegisteredObjectsDoNotExpire
	| customer |
	self setUpExpiryWithZeroDelay.
	self setUpForNotifyAndRemove.
	customer := GlorpCustomer example1.
	customer id: 3.
	cache at: 3 insert: customer.
	session beginUnitOfWork.
	session register: customer.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == customer.
	self deny: customer seenExpiry.! !

GlorpCacheTest subclass: #GlorpTimedStrongCacheTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
setUp
	super setUp.
	self setUpExpiryWithRealDelay.! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
setUpExpiryWithRealDelay
	(cache session descriptorFor: GlorpCustomer) cachePolicy: (TimedStrongCachePolicy new timeout: 1).
	(cache session descriptorFor: GlorpBankTransaction) cachePolicy: (TimedStrongCachePolicy new timeout: 1).! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
setUpExpiryWithZeroDelay
	(cache session descriptorFor: GlorpCustomer) cachePolicy: (TimedStrongCachePolicy new timeout: 0).
	(cache session descriptorFor: GlorpBankTransaction) cachePolicy: (TimedStrongCachePolicy new timeout: 0).! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
setUpForExpiryActionOf: aSymbol
	(cache session descriptorFor: GlorpCustomer) cachePolicy expiryAction: aSymbol.
	(cache session descriptorFor: GlorpBankTransaction) cachePolicy expiryAction: aSymbol.! !

!GlorpTimedStrongCacheTest methodsFor: 'support' stamp: ''!
setUpForNotify
	self setUpForExpiryActionOf: #notify.! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
setUpForNotifyAndRemove
	self setUpForExpiryActionOf: #notifyAndRemove.! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
setUpForRefresh
	self setUpForExpiryActionOf: #refresh.! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
testExpiryReturningNilWithRealDelay
	"test that objects expire with a non-zero delay time."
	| customer customer2 |
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self deny: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	(Delay forSeconds: 2) wait.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	customer2 := GlorpCustomer new.
	cache at: 3 insert: customer2.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == customer2.! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
testNotExpiredAfterRefresh
	| customer |
	self setUpForRefresh.
	session accessor beginTransaction.
	[session accessor executeSQLStringNoResult: 'INSERT INTO GR_CUSTOMER VALUES (3,''Fred Flintstone'')'.
	customer := session execute: (Query returningOneOf: GlorpCustomer where: [:each | each id = 3]).
	(Delay forSeconds: 2) wait.
	self assert: (cache hasExpired: customer).
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == customer.
	self deny: (cache hasExpired: customer)]
		ensure: [session accessor rollbackTransaction].! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
testNotify
	| customer |
	self setUpExpiryWithZeroDelay.
	self setUpForNotify.
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self deny: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	self assert: customer seenExpiry.! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
testNotifyAndRemove
	| customer |
	self setUpExpiryWithZeroDelay.
	self setUpForNotifyAndRemove.
	customer := GlorpCustomer example1.
	cache at: 3 insert: customer.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == nil.
	self assert: customer seenExpiry = true.! !

!GlorpTimedStrongCacheTest methodsFor: 'tests' stamp: ''!
testRegisteredObjectsDoNotExpire
	| customer |
	self setUpExpiryWithZeroDelay.
	self setUpForNotifyAndRemove.
	customer := GlorpCustomer example1.
	customer id: 3.
	cache at: 3 insert: customer.
	session beginUnitOfWork.
	session register: customer.
	self assert: (cache lookupClass: GlorpCustomer key: 3 ifAbsent: [nil]) == customer.
	self deny: customer seenExpiry.! !

GlorpCacheTest subclass: #GlorpWeakCacheTest
	instanceVariableNames: 'mourned'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpWeakCacheTest class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpWeakCacheTest methodsFor: 'support' stamp: ''!
checkCacheExhaustivelyFor: aBlock
	"Check to make sure the cache satisfies the criteria. Since this relies on unpredictable finalization, do a full 10 garbage collect then wait iterations. Don't return early, because we're checking to see that finalization *doesn't* happen"
	self assert: (self doesCacheExhaustivelySatisfy: aBlock).! !

!GlorpWeakCacheTest methodsFor: 'support' stamp: ''!
checkCacheFor: aBlock
	"Check to make sure the cache satisfies the criteria. Since this relies on unpredictable finalization, do up to 10 garbage collect then wait iterations. If it's true before that, return early, but if it's not true at the end, fail"
	self assert: (self doesCacheSatisfy: aBlock).! !

!GlorpWeakCacheTest methodsFor: 'support' stamp: ''!
doesCacheExhaustivelySatisfy: aBlock
	| result |
	result := false.
	10
		timesRepeat:
			[Dialect garbageCollect.
			(Delay forMilliseconds: 100) wait.
			result := aBlock value].
	^result.! !

!GlorpWeakCacheTest methodsFor: 'support' stamp: ''!
doesCacheSatisfy: aBlock
	10
		timesRepeat:
			[Dialect garbageCollect.
			(Delay forMilliseconds: 100) wait.
			aBlock value ifTrue: [^true]].
	^false.! !

!GlorpWeakCacheTest methodsFor: 'initializing' stamp: ''!
initialize! !

!GlorpWeakCacheTest methodsFor: 'support' stamp: ''!
mournKeyOf: anEphemeron
	mourned := true.! !

!GlorpWeakCacheTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	system cachePolicy: WeakVWCachePolicy new.
	mourned := false.! !

!GlorpWeakCacheTest methodsFor: 'tests' stamp: ''!
testEphemeralValue
	| value ephemeron |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	value := Object new.
	ephemeron := (Dialect smalltalkAt: #EphemeralValue) key: 'abc' value: value.
	ephemeron manager: self.
	Dialect garbageCollect.
	value := nil.
	self should: [10 timesRepeat: [
		mourned ifFalse: [Dialect garbageCollect.
		(Delay forMilliseconds: 100) wait]].
		mourned].! !

!GlorpWeakCacheTest methodsFor: 'tests' stamp: ''!
testEphemeralValueDictionary
	| value dict done |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	value := Object new.
	dict := WeakVWCachePolicy new dictionaryClass new.
	dict at: 'abc' put: value.
	Dialect garbageCollect.
	value := nil.
	done := false.
	self should:
		[10	timesRepeat:
			[done ifFalse: [
				Dialect garbageCollect.
				(Delay forMilliseconds: 100) wait.
				done := dict size = 0]].
			 done].! !

!GlorpWeakCacheTest methodsFor: 'tests' stamp: ''!
testEphemeron
	| value ephemeron |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	value := Object new.
	ephemeron := (Dialect smalltalkAt: #Ephemeron) key: value value: 'abc'.
	ephemeron manager: self.
	Dialect garbageCollect.
	value := nil.
	self should: [10 timesRepeat: [
		mourned ifFalse: [Dialect garbageCollect.
		(Delay forMilliseconds: 100) wait]].
		mourned].! !

!GlorpWeakCacheTest methodsFor: 'tests' stamp: ''!
testEphemeronDictionary
	| value dict done |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	value := Object new.
	dict := (Dialect smalltalkAt: #EphemeronDictionary) new.
	dict at: value put: 'abc'.
	Dialect garbageCollect.
	value := nil.
	done := false.
	self should:
		[10	timesRepeat:
			[done ifFalse: [
				Dialect garbageCollect.
				(Delay forMilliseconds: 100) wait.
				done := dict size = 0]].
			 done].! !

!GlorpWeakCacheTest methodsFor: 'tests' stamp: ''!
testLargeWeakDictionary
	| elements size customers newElements |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	system cachePolicy numberOfElements: 1.
	size := 2 raisedTo: 14.
	customers := OrderedCollection new: size.
	1 to: size do: [:i | | cust |
		cust := GlorpCustomer new.
		customers add: cust.
		cache at: i insert: cust].
	elements := cache numberOfElements.
	self assert: elements = (2 raisedTo: 14).
	customers := nil.
	Dialect garbageCollect.
	(Delay forSeconds: 2) wait.
	Dialect garbageCollect.
	(Delay forSeconds: 1) wait.
	newElements := cache numberOfElements.
	self assert: newElements = 1.! !

!GlorpWeakCacheTest methodsFor: 'tests' stamp: ''!
testUnreferencedExcessObjectsAreRemoved
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	system cachePolicy numberOfElements: 1.
	cache at: 3 insert: GlorpCustomer new.
	cache at: 4 insert: GlorpCustomer new.
	self checkCacheFor: [(cache containsObjectForClass: GlorpCustomer key: 3) not].
	self assert: (cache containsObjectForClass: GlorpCustomer key: 4).! !

!GlorpWeakCacheTest methodsFor: 'tests' stamp: ''!
testUnreferencedObjectsAreRemoved

	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	system cachePolicy numberOfElements: 0.
	cache at: 3 insert: GlorpCustomer new.
	self
		checkCacheFor: [(cache containsObjectForClass: GlorpCustomer key: 3) not].! !

!GlorpWeakCacheTest methodsFor: 'tests' stamp: ''!
testUnreferencedObjectsAreRemovedInTheRightOrder
	| customer |
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	system cachePolicy numberOfElements: 1.
	cache at: 3 insert: GlorpCustomer new.
	cache at: 4 insert: GlorpCustomer new.
	customer := cache lookupClass: GlorpCustomer key: 3.
	self deny: customer isNil.
	cache at: 3 insert: customer.
	self checkCacheFor: [(cache containsObjectForClass: GlorpCustomer key: 4) not].
	self assert: (cache containsObjectForClass: GlorpCustomer key: 3).! !

!GlorpWeakCacheTest methodsFor: 'tests' stamp: ''!
testUnreferencedObjectsNotRemovedDueToExtraReferences
	Dialect isVWWithNameSpaces ifFalse: [ ^self ].
	cache at: 3 insert: GlorpCustomer new.
	self checkCacheExhaustivelyFor: [(cache containsObjectForClass: GlorpCustomer key: 3)].! !

GlorpDatabaseBasedTest subclass: #GlorpCommitOrderTest
	instanceVariableNames: 't1 t2 t3 t1id t2id t3id platform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpCommitOrderTest methodsFor: 'support' stamp: ''!
setUp

	super setUp.
	platform := system platform.
	t1 := DatabaseTable new name: 'T1'.
	t1id := (t1 createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	t2 := DatabaseTable new name: 'T2'.
	t2id := (t2 createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	t3 := DatabaseTable new name: 'T3'.
	t3id := (t3 createFieldNamed: 'ID' type: platform int4) bePrimaryKey.! !

!GlorpCommitOrderTest methodsFor: 'tests' stamp: ''!
testCommitOrder

	| sorter |
	sorter := TableSorter for: (Array with: (system tableNamed: 'BANK_TRANS') with: (system tableNamed: 'GR_CUSTOMER')).
	self assert: sorter sort first name = 'GR_CUSTOMER'.! !

!GlorpCommitOrderTest methodsFor: 'tests' stamp: ''!
testCommitOrder2
	"Test for a cycle between t1 and t2 with t3 also pointing to both. Order of t1, t2 is indeterminate, but t3 should be last"

	| table1 table1id table2 table2id t2fk t3fk t3fk2 sorter t1fk |
	table1 := DatabaseTable new name: 'T1'.
	table1id := (table1 createFieldNamed: 'ID' type: platform inMemorySequence) bePrimaryKey.
	table2 := DatabaseTable new name: 'T2'.
	table2id := (table2 createFieldNamed: 'ID' type: platform inMemorySequence) bePrimaryKey.

	t1fk := table1 createFieldNamed: 'T2_ID' type: platform int4.
	table1 addForeignKeyFrom: t1fk to: table2id.
	t2fk := table2 createFieldNamed: 'T1_ID' type: platform int4.
	table2 addForeignKeyFrom: t2fk to: table1id.

	t3 := DatabaseTable new name: 'T3'.
	t3fk := t3 createFieldNamed: 'T2_ID' type: platform int4.
	t3 addForeignKeyFrom: t3fk to: table2id.
	t3fk2 := t3 createFieldNamed: 'T1_ID' type: platform int4.
	t3 addForeignKeyFrom: t3fk2 to: table1id.

	sorter := TableSorter for: (Array with: t3 with: table2 with: table1).
	self assert: sorter sort last name = 'T3'.! !

!GlorpCommitOrderTest methodsFor: 'tests' stamp: ''!
testCommitOrderNonSequencedFieldsDontCount

	"Test for a cycle between t1 and t2 with t3 also pointing to both, but with nothing sequenced. Order should be completely indeterminate. We rely on the topological sort being predictable and depending on the insert order so that if we feed objects with no dependencies in in different orders we should get different results."

	| t1fk t2fk t3fk t3fk2 sorter sorter2 |
	t1fk := t1 createFieldNamed: 'T2_ID' type: platform int4.
	t1 addForeignKeyFrom: t1fk to: (t2id).
	t2fk := t2 createFieldNamed: 'T1_ID' type: platform int4.
	t2 addForeignKeyFrom: t2fk to: (t1id).

	t3 := DatabaseTable new name: 'T3'.
	t3fk := t3 createFieldNamed: 'T2_ID' type: platform int4.
	t3 addForeignKeyFrom: t3fk to: (t2id).
	t3fk2 := t3 createFieldNamed: 'T1_ID' type: platform int4.
	t3 addForeignKeyFrom: t3fk2 to: (t1id).

	sorter := TableSorter for: (Array with: t3 with: t2 with: t1).
	sorter2 := TableSorter for: (Array with: t1 with: t2 with: t3).
	self assert: sorter sort first ~= sorter2 sort first.! !

!GlorpDatabaseBasedTest methodsFor: 'support' stamp: ''!
assert: aBrackettedString equalsIgnoringOuterBrackets: anotherString
	self assert: aBrackettedString =
		(anotherString first = $(
			ifTrue: [anotherString]
			ifFalse: ['(', anotherString, ')'])! !

!GlorpDatabaseBasedTest methodsFor: 'support' stamp: ''!
setUp
	system := GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database.
	system session: GlorpMockSession new.
	system session system: system.! !

GlorpDatabaseBasedTest subclass: #GlorpDescriptorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDescriptorTest methodsFor: 'support' stamp: ''!
errorSignal

	^Dialect error.! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testAllClassesAndNames
	| identity1 identity2 identity3 |
	system flushAllClasses.
	identity1 := system allClasses.
	identity2 := system allClasses.
	system flushAllClasses.
	identity3 := system allClasses.
	self assert: identity1 == identity2.
	self assert: identity1 ~~ identity3.
	self should: [system allClassNames] raise: Dialect error.! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testAllMappingsForField
	| descriptor mappings |
	descriptor := system descriptorFor: GlorpCustomer.
	mappings := descriptor 
				allMappingsForField: ((system tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
	self assert: mappings first attributeName = #id! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testBuildBankTransactionAndDependentsFromRow
	| transactionDescriptor object row moneyDescriptor money1 table translations session builder |
	session := GlorpMockSession new.
	session system: system.
	transactionDescriptor := system descriptorFor: GlorpBankTransaction.
	object := GlorpBankTransaction new.
	row := #(99 nil 'CDN' 98 'service charge' 'USD' 97).
	builder := ObjectBuilder new.
	builder row: row.
	transactionDescriptor populateObject: object inBuilder: builder.
	self assert: object id = 99.
	moneyDescriptor := system descriptorFor: GlorpMoney.
	money1 := GlorpMoney new.
	table := system tableNamed: 'MONEY_IMAGINARY_TABLE'.
	translations := IdentityDictionary new.
	translations at: (table fieldNamed: 'CURRENCY') put: 3.
	translations at: (table fieldNamed: 'AMOUNT') put: 4.
	builder := ElementBuilder new fieldTranslations: translations; row: row.
	moneyDescriptor
		populateObject: money1
		inBuilder: builder.
	self assert: money1 amount = 98.
	self assert: money1 currency = #CDN.! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testBuildPersonFromRow
	| descriptor object address session builder |
	session := GlorpMockSession new.
	session system: system.
	address := GlorpAddress new.
	session cacheAt: 127 put: address.
	descriptor := system descriptorFor: GlorpPerson.
	system tableNamed: 'PERSON'.
	object := GlorpPerson new.
	builder := ObjectBuilder new.
	builder row: #(456 'Ralph' 127).
	descriptor populateObject: object inBuilder: builder.
	self assert: object class = GlorpPerson.
	self assert: object id = 456.
	self assert: object name = 'Ralph'.
	self assert: object address getValue == address.! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testClassLookup

	self assert: (Dialect smalltalkAt: 'Object') == Object.

	Dialect isVWWithNameSpaces ifTrue: [
		"Sneaky meta-tricks to avoid things that won't compile in non-VW dialects"
		self assert: (Dialect smalltalkAt: 'GlorpTestNamespace.GlorpTestClassInNamespace') == (Compiler evaluate: 'GlorpTestNamespace.GlorpTestClassInNamespace').
		self assert: (Dialect smalltalkAt: 'GlorpTestNamespace.GlorpTestClassInNamespace') notNil].! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testDescriptorIdentity

	| descriptor |
	descriptor := system descriptorFor: GlorpCustomer.
	self assert: descriptor == (system descriptorFor: GlorpCustomer).! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testDescriptorWithNamespace

	| descriptor testCaseClass |
	Dialect isVWWithNameSpaces ifFalse: [^self].
	system := GlorpDescriptorSystemWithNamespaces new.
	testCaseClass := 'GlorpTestNamespace.GlorpTestClassInNamespace' asQualifiedReference value.
	descriptor := system descriptorFor: testCaseClass.
	self assert: descriptor describedClass == testCaseClass.! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testMappedFields

	| descriptor |
	descriptor := system descriptorFor: GlorpBankTransaction.
	self assert: (descriptor mappedFields) = (descriptor table fields).! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testMappingForField
	| descriptor mapping |
	descriptor := system descriptorFor: GlorpCustomer.
	mapping := descriptor 
				directMappingForField: ((system tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
	self assert: mapping attributeName = #id! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testPrimaryKeyExpressionFor
	| descriptor trans exp |
	descriptor := system descriptorFor: GlorpBankTransaction.
	trans := GlorpBankTransaction new.
	trans id: 42.
	exp := descriptor
		primaryKeyExpressionFor: trans
		basedOn: BaseExpression new
		relation: #=.
	self assert: exp relation = #=.
	self assert: exp rightChild value = 42.! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testPrimaryKeyExpressionForFailing
	| descriptor trans |
	descriptor := system descriptorFor: GlorpBankTransaction.
	trans := GlorpCustomer new.
	self
		should:
			[descriptor
				primaryKeyExpressionFor: trans
				basedOn: BaseExpression new
				relation: #=]
		raise: self errorSignal.! !

!GlorpDescriptorTest methodsFor: 'tests' stamp: ''!
testPrimaryKeyExpressionForWithCompositeKey

	self unfinished.! !

GlorpDatabaseBasedTest subclass: #GlorpExpressionJoiningTest
	instanceVariableNames: 'source target base'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
join: exp 
	^exp asExpressionJoiningSource: source toTarget: target! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
resultOfJoiningFieldFor: aTable toExpressionBuiltOn: anotherTable
	| exp table |
	base descriptor: (system descriptorFor: GlorpCustomer).
	exp := FieldExpression forField: (aTable fieldNamed: 'ID')
				basedOn: BaseExpression new.
	table := base getTable: anotherTable.

	^exp asExpressionJoiningSource: base toTarget: table.! !

!GlorpExpressionJoiningTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	source := BaseExpression new.
	target := source get: #relation.
	base := BaseExpression new.! !

!GlorpExpressionJoiningTest methodsFor: 'support' stamp: ''!
tearDown

	source := nil.
	target := nil.
	base := nil.
	system := nil.! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testBase
	| result |
	result := self join: base. 
	self assert: result == source.! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testConstant
	| exp |
	exp := ConstantExpression for: 42.
	self assert: (self join: exp) == exp! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testField
	| exp  result |
	exp := FieldExpression forField: (DatabaseField named: 'test' type: system platform int4)
				basedOn: base.
	result := self join: exp.
	self assert: result base == source.
	self assert: result field == exp field! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testFieldBuiltOnDifferentTable
	| result custTable |
	custTable := system tableNamed: 'GR_CUSTOMER'.
	result := self 
		resultOfJoiningFieldFor: custTable
		toExpressionBuiltOn: custTable.
	self assert: result base == (base getTable: custTable).
	self assert: result field == ((system tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testFieldBuiltOnSameTable
	| exp result base2 table custTable |
	custTable := system tableNamed: 'GR_CUSTOMER'.
	base2 := BaseExpression new.
	base2 descriptor: (system descriptorFor: GlorpCustomer).
	table := base2 getTable: custTable.
	exp := FieldExpression forField: (custTable fieldNamed: 'ID')
				basedOn: base.
	result := exp asExpressionJoiningSource: base2 toTarget: table.
	self assert: result base == table.
	self assert: result field == exp field.! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testMapping
	| result exp |
	exp := base get: #foo.
	result := self join: exp. 
	self assert: result base == source.
	self assert: result name = #foo.! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testParameter
	| result exp table field |
	table := DatabaseTable named: 'T'.	
	field := DatabaseField named: 'F' type: system platform int4.
	table addField: field.
	exp := base getParameter: field.
	result := self join: exp.
	self assert: result base == source.
	self assert: result class == FieldExpression.
	self assert: result field == field.! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testRelation
	| result exp |
	exp := [:a | a foo = 3] asGlorpExpressionOn: base.
	result := self join: exp. 

	self assert: result class == RelationExpression.
	self assert: result rightChild == exp rightChild.
	self assert: result leftChild base == source.! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testRelation2
	| result exp field | 
	field := DatabaseField named: 'fred' type: system platform int4.
	exp := [:a | a foo = field] asGlorpExpressionOn: base.
	result := self join: exp. 
	self assert: result class == RelationExpression.
	self assert: result rightChild class == FieldExpression.
	self assert: result rightChild field == field.
	self assert: result leftChild base == source.! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testSelfJoinWithPrimaryKeyExpression
	"This tests a join of a class to itself, in this case customers who have other customers associated with them. Useful for hierarchies"
	| pkExpression field result |
	field := (system tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID'.
	pkExpression := Join 
		from: field
		to: field.
	result := self join: pkExpression.
	self assert: result leftChild basicField == field.
	self assert: result rightChild basicField == field.
	self assert: result rightChild base name = #relation.! !

!GlorpExpressionJoiningTest methodsFor: 'tests' stamp: ''!
testTable
	| result exp table |
	table := DatabaseTable named: 'T'.
	exp := base getTable: table.
	result := self join: exp. 
	self assert: result base == target.
	self assert: result table == table.! !

GlorpDatabaseBasedTest subclass: #GlorpExpressionTableAliasingTest
	instanceVariableNames: 'exp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpExpressionTableAliasingTest methodsFor: 'support' stamp: ''!
helpTestBasicAliasing: aTable
	self deny: exp hasTableAliases.
	exp assignTableAliasesStartingAt: 1 prefix: 's1t'.
	self assert: exp hasTableAliases.
	self assert: exp tableAliases size = 1.
	self assert: (exp tableAliases at: aTable) name = 's1t1'.! !

!GlorpExpressionTableAliasingTest methodsFor: 'tests' stamp: ''!
testBase
	exp := BaseExpression new.
	exp descriptor: (system descriptorFor: GlorpCustomer).
	self helpTestBasicAliasing: (system tableNamed: 'GR_CUSTOMER').! !

!GlorpExpressionTableAliasingTest methodsFor: 'tests' stamp: ''!
testMapping
	| base |
	base := BaseExpression new.
	base descriptor: (system descriptorFor: GlorpCustomer).
	exp := base get: 'transactions'.
	self helpTestBasicAliasing: (system tableNamed: 'BANK_TRANS').! !

!GlorpExpressionTableAliasingTest methodsFor: 'tests' stamp: ''!
testTable
	| base transTable |
	base := BaseExpression new.
	base descriptor: (system descriptorFor: GlorpCustomer).
	transTable := system tableNamed: 'BANK_TRANS'.
	exp := base getTable: transTable.
	self helpTestBasicAliasing: transTable! !

!GlorpExpressionTableAliasingTest methodsFor: 'tests' stamp: ''!
testTableSameAsBase
	| base custTable |
	base := BaseExpression new.
	base descriptor: (system descriptorFor: GlorpCustomer).
	custTable := system tableNamed: 'GR_CUSTOMER'.
	exp := base getTable: custTable.
	self deny: exp hasTableAliases.
	exp assignTableAliasesStartingAt: 1 prefix: ''.
	base assignTableAliasesStartingAt: 42 prefix: nil.
	self deny: exp hasTableAliases.
	self assert: (exp aliasedTableFor: custTable) name = 't42'.! !

GlorpDatabaseBasedTest subclass: #GlorpExpressionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpExpressionTest methodsFor: 'support' stamp: ''!
assertIdentityOf: aBlock and: anotherBlock
	
 	| base |
	base := BaseExpression new.
	self assert: (aBlock asGlorpExpressionOn: base) == (anotherBlock asGlorpExpressionOn: base)! !

!GlorpExpressionTest methodsFor: 'support' stamp: ''!
denyIdentityOf: aBlock and: anotherBlock
 
	| base |
	base := BaseExpression new.
	self deny: (aBlock asGlorpExpressionOn: base) == (anotherBlock asGlorpExpressionOn: base)! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testAndOperation
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | (a firstName = fred) & (a firstName ~= fred)] asGlorpExpressionOn: base.
	self assert: expression class == RelationExpression.
	self assert: expression relation = #AND.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testAndOperation2
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | (a firstName = fred) AND: (a firstName ~= fred)] asGlorpExpressionOn: base.
	self assert: expression class == RelationExpression.
	self assert: expression relation = #AND.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testAnySatisfyExists

	| expression base |
	system useJoinsForAnySatisfy: false.
	base := BaseExpression new.
	base descriptor: (system descriptorFor: GlorpCustomer).
	expression := [:a | a items anySatisfy: [:each | each id = 7]] asGlorpExpressionOn: base.
	self assert: expression class == SubSelectExpression.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testAnySatisfyJoin

	| expression base |
	system useJoinsForAnySatisfy: true.
	base := BaseExpression new.
	base descriptor: (system descriptorFor: GlorpCustomer).
	expression := [:a | a items anySatisfy: [:each | each id = 7]] asGlorpExpressionOn: base.
	self assert: expression class == CollectionExpression.
	self assert: expression leftChild == expression rightChild leftChild base.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testAnySatisfyPrintExists
	| expression |
	system useJoinsForAnySatisfy: false.
	expression := [:cust | cust transactions anySatisfy: [:each | each id = 7]] 
				asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	self assert: (' EXISTS (SELECT s1t1.ID<n> FROM BANK_TRANS s1t1<n> WHERE *s1t1.ID = ?* AND *s1t2.ID = s1t1.OWNER_ID)*' expandMacros match: expression sqlString).! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testAnySatisfyPrintJoin
	| expression |
	system useJoinsForAnySatisfy: true.
	expression := [:cust | cust transactions anySatisfy: [:each | each id = 7]] 
				asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	self assert: '(BANK_TRANS.ID = 7)' equalsIgnoringOuterBrackets: expression sqlString.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testBetweenAnd
	| expression base |
	base := BaseExpression new.
	expression := [:a | a between: 3 and: 4] asGlorpExpressionOn: base.
	self assert: expression class == InfixFunction.
	self assert: expression name = 'BETWEEN AND'.
	self assert: (expression arguments collect: [:each | each value]) = (Array with: 3 with: 4).! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testEqualityOperation
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | a firstName = fred] asGlorpExpressionOn: base.
	self assert: expression leftChild == (base get: #firstName).
	self assert: expression rightChild class == ConstantExpression.
	self assert: expression rightChild value == fred.
	self assert: expression relation == #=! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testFindingMapping
	| base baseDescriptor |
	baseDescriptor := system descriptorFor: GlorpBankTransaction.
	base := BaseExpression new descriptor: baseDescriptor.
	self assert: (base get: #serviceCharge) mapping == (baseDescriptor mappingForAttributeNamed: #serviceCharge).
	self assert: (base get: #serviceCharge) sourceDescriptor == (system descriptorFor: GlorpBankTransaction).
	self assert: (base get: #serviceCharge) descriptor == (system descriptorFor: GlorpServiceCharge)! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testGetField
	| expression |
	expression := [:cust | cust getField: 'ID'] asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	self assert: expression sqlString = 'GR_CUSTOMER.ID'.
	self assert: expression class = FieldExpression.
	self assert: expression field table = (system tableNamed: 'GR_CUSTOMER').! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testGetFieldAmbiguous
	| |
	self 
		should: [[:pass | pass getField: 'ID']
			asGlorpExpressionForDescriptor: (system descriptorFor: GlorpPassenger)]
		raise: Dialect error.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testGetFieldFromSecondaryTable
	| expression |
	expression := [:pass | pass getField: 'POINTS']
		asGlorpExpressionForDescriptor: (system descriptorFor: GlorpPassenger).
	self assert: expression sqlString = 'FREQUENT_FLYER.POINTS'.
	self assert: expression class = FieldExpression.
	self assert: expression field table = (system tableNamed: 'FREQUENT_FLYER').! !

!GlorpExpressionTest methodsFor: 'As yet unclassified' stamp: ''!
testIsNullPrint

	| expression stream |
	expression := [:cust | cust id = nil]
		asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	stream := DatabaseCommand new session: system session.
	expression printSQLOn: stream withParameters: Dictionary new.
	self assert: '(GR_CUSTOMER.ID IS NULL)' equalsIgnoringOuterBrackets: expression sqlString.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testJoinOperation
	| userExpression base expression addressTable personTable query field1 field2 join |
	addressTable := system tableNamed: 'GR_ADDRESS'.
	personTable := system tableNamed: 'PERSON'.
	base := BaseExpression new.
	base descriptor: (system descriptorFor: GlorpPerson).
	userExpression := [:aPerson | aPerson address number = 12] 
				asGlorpExpressionOn: base.
	query := SimpleQuery returningOneOf: GlorpPerson where: userExpression.
	query session: (GlorpSession new system: system).
	self assert: (userExpression additionalExpressionsIn: query) size = 1.
	query prepare.
	expression := query whereClause.
	self assert: query joins size = 1.
	join := query joins first.
	self
		assert: expression == userExpression;
		assert: expression relation == #=.
	field1 := join leftChild field.
	self assert: field1 table parent == personTable.
	self assert: field1 name = 'ADDRESS_ID'.
	field2 := join rightChild field.
	self assert: field2 table parent == addressTable.
	self assert: field2 name = 'ID'.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testMappingBase

	| base |
	base := BaseExpression new.
	self assert: (base get: #someAttribute) base == base! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testMappingExpressionIdentity
 
	self assertIdentityOf: [:a | a someAttribute] and: [:a | a someAttribute].! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testMappingExpressionIdentity2

	self assertIdentityOf: [:a | a perform: #someAttribute] and: [:a | a someAttribute].! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testMappingExpressionIdentity3

	self assertIdentityOf: [:a | a get: #someAttribute] and: [:a | a someAttribute].! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testMappingExpressionIdentity4

	self denyIdentityOf: [:a | a get: #someAttribute] and: [:a | a someOtherAttribute].! !

!GlorpExpressionTest methodsFor: 'As yet unclassified' stamp: ''!
testNotNullPrint

	| expression stream |
	expression := [:cust | cust id ~= nil]
		asGlorpExpressionForDescriptor: (system descriptorFor: GlorpCustomer).
	stream := DatabaseCommand new session: system session.
	expression printSQLOn: stream withParameters: Dictionary new.
	self assert: '(GR_CUSTOMER.ID IS NOT NULL)' equalsIgnoringOuterBrackets: stream contents.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testOrOperation
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | (a firstName = fred) | (a firstName ~= fred)] asGlorpExpressionOn: base.
	self assert: expression class == RelationExpression.
	self assert: expression relation = #OR.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testOrOperation2
	| expression fred base |
	fred := 'Fred'.
	base := BaseExpression new.
	expression := [:a | (a firstName = fred) OR: (a firstName ~= fred)] asGlorpExpressionOn: base.
	self assert: expression class == RelationExpression.
	self assert: expression relation = #OR.! !

!GlorpExpressionTest methodsFor: 'tests' stamp: ''!
testTwoLevelMappingExpressionIdentity

	self assertIdentityOf: [:a | a someAttribute someAttribute] and: [:a | a someAttribute someAttribute].
	self denyIdentityOf: [:a | a someAttribute someAttribute] and: [:a | a someAttribute].! !

GlorpDatabaseBasedTest subclass: #GlorpJoinTest
	instanceVariableNames: 'expression compoundExpression'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpJoinTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	expression := Join 
		from: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'ID')
		to: ((system tableNamed: 'CUSTOMER_ACCT_LINK') fieldNamed: 'ACCT_ID').

	compoundExpression := Join
		from: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'ID')
		to: ((system tableNamed: 'PERSON') fieldNamed: 'NAME').
	compoundExpression
		addSource: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'BANK_CODE')
		target: ((system tableNamed: 'PERSON') fieldNamed: 'ADDRESS_ID').! !

!GlorpJoinTest methodsFor: 'tests' stamp: ''!
testAsExpressionCompound
	
	| e personTable accountTable |
	personTable := system tableNamed: 'PERSON'.
	accountTable := system tableNamed: 'BANK_ACCT'.

	e := compoundExpression asGeneralGlorpExpression.
	self assert: (e isKindOf: RelationExpression).
	self assert: e relation == #AND.
	self assert: (e leftChild isKindOf: RelationExpression).
	self assert: (e leftChild relation == #=).
	self assert: (e leftChild leftChild isKindOf: FieldExpression).
	self assert: e leftChild leftChild field == (personTable fieldNamed: 'NAME').
	self assert: (e leftChild rightChild isKindOf: ParameterExpression).
	self assert: e leftChild rightChild field == (accountTable fieldNamed: 'ID').


	self assert: (e rightChild isKindOf: RelationExpression).
	self assert: (e rightChild relation == #=).
	self assert: (e rightChild leftChild isKindOf: FieldExpression).
	self assert: e rightChild leftChild field == (personTable fieldNamed: 'ADDRESS_ID').
	self assert: (e rightChild rightChild isKindOf: ParameterExpression).
	self assert: e rightChild rightChild field = (accountTable fieldNamed: 'BANK_CODE').! !

!GlorpJoinTest methodsFor: 'tests' stamp: ''!
testAsExpressionSingle
	
	| e field param |
	e := expression asGeneralGlorpExpression.
	self assert: (e isKindOf: RelationExpression).
	self assert: e relation == #=.
	field := e leftChild.
	self assert: (field isKindOf: FieldExpression).
	self assert: field field == ((system tableNamed: 'CUSTOMER_ACCT_LINK') fieldNamed: 'ACCT_ID').
	param := e rightChild.
	self assert: (param isKindOf: ParameterExpression).
	self assert: param field == ((system tableNamed: 'BANK_ACCT') fieldNamed: 'ID').! !

!GlorpJoinTest methodsFor: 'tests' stamp: ''!
testCompoundSQLPrinting
	| stream params|
	stream := WriteStream on: (String new: 100).
	params := Dictionary new.
	params at: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'ID') put: 1.
	params at: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'BANK_CODE') put: 3.
	compoundExpression printSQLOn: stream withParameters: params.
	self assert: stream contents = 'PERSON.NAME = 1 AND PERSON.ADDRESS_ID = 3'.! !

!GlorpJoinTest methodsFor: 'tests' stamp: ''!
testCreation

	self assert: expression allSourceFields size = 1.
	self assert: expression allSourceFields first == ((system tableNamed: 'BANK_ACCT') fieldNamed: 'ID').! !

!GlorpJoinTest methodsFor: 'tests' stamp: ''!
testParameterCount

	self assert: expression numberOfParameters = 1.
	self assert: compoundExpression numberOfParameters = 2.! !

!GlorpJoinTest methodsFor: 'tests' stamp: ''!
testSQLPrinting
	| stream params |
	stream := WriteStream on: (String new: 100).
	params := Dictionary new.
	params at: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'ID') put: 'abc'.
	expression printSQLOn: stream withParameters: params.
	self assert: stream contents = 'CUSTOMER_ACCT_LINK.ACCT_ID = ''abc'''! !

GlorpDatabaseBasedTest subclass: #GlorpMappingTest
	instanceVariableNames: 'rowMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
helperForMergedOneToOneReversingWriteOrder: aBoolean 
	| account accountTable row |
	account := GlorpBankExampleSystem new objectNumber: 1 ofClass: GlorpBankAccount.
	accountTable := system tableNamed: 'BANK_ACCT'.
	aBoolean 
		ifTrue: 
			[self write: account.
			self write: account accountNumber]
		ifFalse: 
			[self write: account accountNumber.
			self write: account].
	self assert: (rowMap includesRowForTable: accountTable withKey: account).
	self assert: (rowMap includesRowForTable: accountTable
				withKey: account accountNumber).
	row := self rowFor: account.
	self assert: (row at: (accountTable fieldNamed: 'ID')) = account id.
	(Array with: row with: (self rowFor: account accountNumber)) do: 
			[:each | 
			self assert: (each at: (accountTable fieldNamed: 'BANK_CODE')) 
						= account accountNumber bankCode.
			self assert: (each at: (accountTable fieldNamed: 'BRANCH_NO')) 
						= account accountNumber branchNumber.
			self assert: (each at: (accountTable fieldNamed: 'ACCT_NO')) 
						= account accountNumber accountNumber].
	self assert: (rowMap numberOfEntriesForTable: accountTable) = 2! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
helperForNestedMergedOneToOneReversingWriteOrder: aBoolean 
	| trans transTable moneyTable row fieldNames fieldValues |
	trans := GlorpBankExampleSystem new objectNumber: 1
				ofClass: GlorpBankTransaction.
	transTable := system tableNamed: 'BANK_TRANS'.
	moneyTable := system tableNamed: 'MONEY_IMAGINARY_TABLE'.
	aBoolean 
		ifTrue: 
			[self write: trans.
			self write: trans amount.
			self write: trans serviceCharge.
			self write: trans serviceCharge amount]
		ifFalse: 
			[self write: trans serviceCharge amount.
			self write: trans serviceCharge.
			self write: trans amount.
			self write: trans].
	self 
		assert: (rowMap rowForTable: transTable withKey: trans) shouldBeWritten.
	self 
		assert: (rowMap rowForTable: transTable withKey: trans serviceCharge) 
				shouldBeWritten not.
	self 
		assert: (rowMap rowForTable: moneyTable withKey: trans amount) 
				shouldBeWritten not.
	self 
		assert: (rowMap rowForTable: moneyTable withKey: trans serviceCharge amount) 
				shouldBeWritten not.
	row := self rowFor: trans.
	self assert: (row at: (transTable fieldNamed: 'ID')) = trans id.
	fieldNames := #('AMT_CURR' 'AMT_AMT' 'SRVC_DESC' 'SRVC_AMT_CURR' 'SRVC_AMT_AMT').
	fieldValues := (Array 
				with: trans amount currency asString
				with: trans amount amount
				with: trans serviceCharge description) 
					, (Array with: trans serviceCharge amount currency asString
							with: trans serviceCharge amount amount).
	fieldNames with: fieldValues
		do: [:fieldName :value | self assert: (row at: (transTable fieldNamed: fieldName)) = value].
	self assert: (rowMap numberOfEntriesForTable: transTable) = 2.
	self assert: (rowMap numberOfEntriesForTable: moneyTable) = 2! !

!GlorpMappingTest methodsFor: 'support' stamp: ''!
rowFor: anObject
	| descriptor |
	descriptor := system descriptorFor: anObject.
	descriptor isNil ifTrue: [^nil].

	^rowMap rowForTable: descriptor table withKey: anObject.! !

!GlorpMappingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	rowMap := RowMap new.! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testManyToMany

	| customer customerTable accountTable linkTable linkRow |
	customer := GlorpBankExampleSystem new objectNumber: 1 ofClass: GlorpCustomer.
	rowMap := RowMap new.
	customerTable := system tableNamed: 'GR_CUSTOMER'.
	accountTable := system tableNamed: 'BANK_ACCT'.
	linkTable := system tableNamed: 'CUSTOMER_ACCT_LINK'.

	self write: customer.
	customer accounts do: [:each | 
		self write: each].

	self assert: (rowMap includesRowForTable: customerTable withKey: customer).
	customer accounts do: [:each |
		self assert: (rowMap includesRowForTable: accountTable withKey: each).
		self assert: (rowMap includesRowForTable: linkTable withKey: (RowMapKey new key1: customer; key2: each))].

	customer accounts do: [:each | | rowMapKey |
		self assert: ((self rowFor: each) at: (accountTable fieldNamed: 'ID')) = each id.
		rowMapKey := RowMapKey new key1: customer; key2: each.
		linkRow := rowMap rowForTable: linkTable withKey: rowMapKey.
		self assert: (linkRow at: (linkTable fieldNamed: 'ACCT_ID')) = each id.
		self assert: (linkRow at: (linkTable fieldNamed: 'CUSTOMER_ID')) = customer id.
		].
	self assert: ((self rowFor: customer) at: (customerTable fieldNamed: 'ID')) = customer id.
	self assert: ((rowMap numberOfEntriesForTable: linkTable) = 2).
	self assert: ((rowMap numberOfEntriesForTable: customerTable) = 1).! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testMergedOneToOne
	self helperForMergedOneToOneReversingWriteOrder: false! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testMergedOneToOneReversingWrites
	self helperForMergedOneToOneReversingWriteOrder: true! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testMissingDescriptor
	self assert: (system descriptorFor: nil) isNil.
	self assert: (system descriptorFor: UndefinedObject) isNil.
	self assert: (system descriptorFor: 3) isNil! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testMultipleTableCreation
	
	| descriptor table passenger table2 row1 row2 |
	descriptor := system descriptorFor: GlorpPassenger.
	passenger := GlorpPassenger example1.
	rowMap := RowMap new.
	table := system existingTableNamed: 'PASSENGER'.
	table2 := system existingTableNamed: 'FREQUENT_FLYER'.
	descriptor createRowsFor: passenger in: rowMap.
	self assert: (rowMap includesRowForTable: table withKey: passenger).
	self assert: (rowMap includesRowForTable: table2 withKey: passenger).
	row1 := rowMap rowForTable: table withKey: passenger.
	self assert: (row1 at: (table fieldNamed: 'ID'))
			= passenger id.
	self assert: (row1 at: (table fieldNamed: 'NAME'))
			= passenger name.
	row2 := rowMap rowForTable: table2 withKey: passenger.
	self assert: (row2 at: (table2 fieldNamed: 'ID'))
			= passenger id.
	self assert: (row2 at: (table2 fieldNamed: 'POINTS')) = passenger frequentFlyerPoints.
	self assert: rowMap numberOfEntries = 3! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testMultipleTableCreationWithOneMissing
	
	| descriptor table passenger table2 row1 |
	descriptor := system descriptorFor: GlorpPassenger.
	passenger := GlorpPassenger example1.
	passenger frequentFlyerPoints: nil.
	passenger airline: nil.
	rowMap := RowMap new.
	table := system existingTableNamed: 'PASSENGER'.
	table2 := system existingTableNamed: 'FREQUENT_FLYER'.
	descriptor createRowsFor: passenger in: rowMap.
	self assert: (rowMap includesRowForTable: table withKey: passenger).
	self deny: (rowMap includesRowForTable: table2 withKey: passenger).
	row1 := rowMap rowForTable: table withKey: passenger.
	self assert: (row1 at: (table fieldNamed: 'ID'))
			= passenger id.
	self assert: (row1 at: (table fieldNamed: 'NAME'))
			= passenger name.
	self assert: rowMap numberOfEntries = 1.! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testNestedMergedOneToOne
	self helperForNestedMergedOneToOneReversingWriteOrder: false! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testNestedMergedOneToOneReversingWriteOrder
	self helperForNestedMergedOneToOneReversingWriteOrder: true.! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testNilOneToOne
	
	| person  personTable addressTable |
	person := GlorpPerson example1.
	person address: nil.
	self write: person.
	self write: person address.
	personTable := system existingTableNamed: 'PERSON'.
	addressTable := system existingTableNamed: 'GR_ADDRESS'.

	self assert: (rowMap includesRowForTable: personTable withKey: person).
	self deny: (rowMap includesRowForTable: addressTable withKey: person address).

	self assert: rowMap numberOfEntries = 1.! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testOneToMany

	| customer customerTable transactionTable |
	customer := GlorpCustomer example1.
	rowMap := RowMap new.
	customerTable := system tableNamed: 'GR_CUSTOMER'.
	transactionTable := system tableNamed: 'BANK_TRANS'.
	self write: customer.
	customer transactions do: [:each |
		self write: each].

	self assert: (rowMap includesRowForTable: customerTable withKey: customer).
	customer transactions do: [:each |
		self assert: (rowMap includesRowForTable: transactionTable withKey: each)].

	customer transactions do: [:each |
		self assert: ((self rowFor: each) at: (transactionTable fieldNamed: 'OWNER_ID')) = customer id].
	self assert: ((self rowFor: customer) at: (customerTable fieldNamed: 'ID')) = customer id.! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testOneToOne
	
	| person  personTable addressTable |
	person := GlorpPerson example1.
	self write: person.
	self write: person address.
	personTable := system existingTableNamed: 'PERSON'.
	addressTable := system existingTableNamed: 'GR_ADDRESS'.

	self assert: (rowMap includesRowForTable: personTable withKey: person).
	self assert: (rowMap includesRowForTable: addressTable withKey: person address).

	self assert: ((self rowFor: person address) at: (addressTable fieldNamed: 'ID')) = person address id.
	self assert: ((self rowFor: person) at: (personTable fieldNamed: 'ADDRESS_ID')) = person address id.
	self assert: rowMap numberOfEntries = 2.! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testOneToOneWithProxy

	| person  personTable addressTable proxy stub |
	person := GlorpPerson example1.
	proxy := Proxy new.
	proxy session: GlorpSession new.
	stub := GlorpQueryStub returningOneOf: GlorpAddress where: [:address | address id = 1].
	stub result: person address.
	proxy query: stub.
	person address: proxy.
	self deny: person address isInstantiated.

	self write: person.

	personTable := system existingTableNamed: 'PERSON'.
	addressTable := system existingTableNamed: 'GR_ADDRESS'.

	self assert: (rowMap includesRowForTable: personTable withKey: person).
	self deny: (rowMap includesRowForTable: addressTable withKey: person address).

	self deny: ((self rowFor: person) hasValueFor: (personTable fieldNamed: 'ADDRESS_ID')).
	self assert: rowMap numberOfEntries = 1.! !

!GlorpMappingTest methodsFor: 'tests' stamp: ''!
testRowCreation

	| descriptor person row table  |
	descriptor := system descriptorFor: GlorpPerson.
	person := GlorpPerson example1.
	rowMap := RowMap new.
	table := system existingTableNamed: 'PERSON'.
	descriptor createRowsFor: person in: rowMap.
	self assert: (rowMap includesRowForTable: table withKey: person).
	row := rowMap rowForTable: table withKey: person.
	self assert: (row at: (table fieldNamed: 'ID')) = person id.
	self assert: (row at: (table fieldNamed: 'NAME')) = person name.
	self assert: rowMap numberOfEntries = 2.! !

!GlorpMappingTest methodsFor: 'support' stamp: ''!
write: anObject

	| descriptor |
	descriptor := system descriptorFor: anObject. 
	descriptor isNil ifTrue: [^self].
	descriptor createRowsFor: anObject in: rowMap.! !

GlorpDatabaseBasedTest subclass: #GlorpPrimaryKeyExpressionWithConstantTest
	instanceVariableNames: 'expression compoundExpression'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpPrimaryKeyExpressionWithConstantTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	expression := Join 
		from: 7
		to: ((system tableNamed: 'CUSTOMER_ACCT_LINK') fieldNamed: 'ACCT_ID').

	compoundExpression := Join
		from: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'ID')
		to: ((system tableNamed: 'PERSON') fieldNamed: 'NAME').
	compoundExpression
		addSource: 'B'
		target: ((system tableNamed: 'PERSON') fieldNamed: 'ADDRESS_ID').! !

!GlorpPrimaryKeyExpressionWithConstantTest methodsFor: 'tests' stamp: ''!
testAsExpressionCompound
	
	| e personTable |
	personTable := system tableNamed: 'PERSON'.

	e := compoundExpression asGeneralGlorpExpression.
	self assert: (e rightChild isKindOf: RelationExpression).
	self assert: (e rightChild relation == #=).
	self assert: (e rightChild leftChild isKindOf: FieldExpression).
	self assert: e rightChild leftChild field == (personTable fieldNamed: 'ADDRESS_ID').
	self assert: (e rightChild rightChild isKindOf: ConstantExpression).
	self assert: e rightChild rightChild value = 'B'.! !

!GlorpPrimaryKeyExpressionWithConstantTest methodsFor: 'tests' stamp: ''!
testAsExpressionSingle
	
	| e field param |
	e := expression asGeneralGlorpExpression.
	self assert: (e isKindOf: RelationExpression).
	self assert: e relation == #=.
	field := e leftChild.
	self assert: (field isKindOf: FieldExpression).
	self assert: field field == ((system tableNamed: 'CUSTOMER_ACCT_LINK') fieldNamed: 'ACCT_ID').
	param := e rightChild.
	self assert: (param isKindOf: ConstantExpression).
	self assert: param value = 7.! !

!GlorpPrimaryKeyExpressionWithConstantTest methodsFor: 'tests' stamp: ''!
testCompoundSQLPrinting
	| stream params|
	stream := WriteStream on: (String new: 100).
	params := Dictionary new.
	params at: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'ID') put: 1.
	compoundExpression printSQLOn: stream withParameters: params.
	self assert: stream contents = 'PERSON.NAME = 1 AND PERSON.ADDRESS_ID = ''B'''.! !

!GlorpPrimaryKeyExpressionWithConstantTest methodsFor: 'tests' stamp: ''!
testParameterCount

	self assert: expression numberOfParameters = 1.
	self assert: compoundExpression numberOfParameters = 2.! !

!GlorpPrimaryKeyExpressionWithConstantTest methodsFor: 'tests' stamp: ''!
testSQLPrinting
	| stream params |
	stream := WriteStream on: (String new: 100).
	params := Dictionary new.
	expression printSQLOn: stream withParameters: params.
	self assert: stream contents = 'CUSTOMER_ACCT_LINK.ACCT_ID = 7'! !

GlorpDatabaseBasedTest subclass: #GlorpQueryCopyingTest
	instanceVariableNames: 'query expression elementBuilder session newQuery'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpQueryCopyingTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpSessionResource.! !

!GlorpQueryCopyingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	query := Query returningManyOf: GlorpCustomer.
	query whereClause: [:each | each accounts anySatisfy: [:foo | foo id = 12]].
	session := GlorpSessionResource current newSession.
	query session: session.! !

!GlorpQueryCopyingTest methodsFor: 'As yet unclassified' stamp: ''!
tableAliasesPresentFor: aQuery
	aQuery whereClause detect: [:each | each hasTableAliases] ifNone: [^false].
	^true.! !

!GlorpQueryCopyingTest methodsFor: 'tests' stamp: ''!
testExpressionTableAliases
	query prepare.
	newQuery := query copy.
	self assert: (self tableAliasesPresentFor: query).
	self deny: (self tableAliasesPresentFor: newQuery).! !

!GlorpQueryCopyingTest methodsFor: 'tests' stamp: ''!
testPreparedness
	self deny: query isPrepared.
	query prepare.
	newQuery := query copy.
	self assert: query isPrepared.
	self deny: newQuery isPrepared.! !

GlorpDatabaseBasedTest subclass: #GlorpQueryTableAliasingTest
	instanceVariableNames: 'query expression elementBuilder session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpQueryTableAliasingTest methodsFor: 'As yet unclassified' stamp: ''!
helpTestPrintingWhereClause: anExpression
	| command |
	query 
		initResultClass: GlorpCustomer
		whereClause: expression
		singleObject: true.
	query setupTracing.
	query computeFields.
	command := GlorpNullCommand useBinding: false session: session.
	anExpression	
		printSQLOn: command
		withParameters: Dictionary new.
	^command sqlString.! !

!GlorpQueryTableAliasingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	query := SimpleQuery new.
	expression := BaseExpression new.
	expression descriptor: (system descriptorFor: GlorpCustomer).
	(expression descriptor mappingForAttributeNamed: #name) shouldProxy: false.
	elementBuilder := ObjectBuilder for: expression in: query.
	expression aliasTable: (system tableNamed: 'GR_CUSTOMER') to: 't1'.
	session := GlorpSession new.
	session system: system.
	query session: system session.! !

!GlorpQueryTableAliasingTest methodsFor: 'tests' stamp: ''!
testAliasWithEmbeddedMapping

	self unfinished.! !

!GlorpQueryTableAliasingTest methodsFor: 'tests' stamp: ''!
testBuildingObject

	| customer |
	elementBuilder instance: GlorpCustomer new.
	elementBuilder requiresPopulating: true.
	elementBuilder buildObjectFrom: #(12 'Name').
	customer := elementBuilder instance.
	self assert: customer class == GlorpCustomer.
	self assert: customer id = 12.
	self assert: customer name = 'Name'.! !

!GlorpQueryTableAliasingTest methodsFor: 'tests' stamp: ''!
testElementBuilderFields
	elementBuilder fieldsForSelectStatement 
		do: [:each | self assert: each table name = 't1']! !

!GlorpQueryTableAliasingTest methodsFor: 'tests' stamp: ''!
testExpressionTableAlias
	| fields |
	fields := expression translateFields: expression descriptor mappedFields.
	fields do: [:each | self assert: each table name = 't1']! !

!GlorpQueryTableAliasingTest methodsFor: 'tests' stamp: ''!
testQueryPrintingFields
	| stream |
	query 
		initResultClass: GlorpCustomer
		whereClause: expression
		singleObject: true.
	query limit: nil.
	query setupTracing.
	query computeFields.
	stream := String new writeStream.
	query printSelectFieldsOn: stream.
	self assert: stream contents = 't1.ID, t1.NAME'! !

!GlorpQueryTableAliasingTest methodsFor: 'tests' stamp: ''!
testQueryPrintingSimpleWhereClause
	| string |
	string := self helpTestPrintingWhereClause: ((expression get: #name) get: #= withArguments: #('Fred')).
	self assert: '(t1.NAME = ''Fred'')' equalsIgnoringOuterBrackets: string.! !

!GlorpQueryTableAliasingTest methodsFor: 'tests' stamp: ''!
testQueryPrintingTables
	| stream string |
	query 
		initResultClass: GlorpCustomer
		whereClause: expression
		singleObject: true.
	query setupTracing.
	query computeFields.
	stream := String new writeStream.
	query printTablesOn: stream.
	string := stream contents.
	self assert: string = '<n> FROM GR_CUSTOMER t1' expandMacros.! !

GlorpDatabaseBasedTest subclass: #GlorpRowDifferencingTest
	instanceVariableNames: 'session currentObject currentObjectRowMap correspondenceMap differenceMap mementoObject mementoObjectRowMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpRowDifferencingTest methodsFor: 'support' stamp: ''!
addRowsFor: object to: rowMap 
	| descriptor |
	descriptor := system descriptorFor: object class.
	descriptor createRowsFor: object in: rowMap.! !

!GlorpRowDifferencingTest methodsFor: 'tests' stamp: ''!
computeDifference
	currentObjectRowMap := self generateRowMapFor: currentObject.
	correspondenceMap := self correspond: currentObject to: mementoObject.
	mementoObjectRowMap := self generateMementoRowMapFor: mementoObject withCorrespondenceMap: correspondenceMap.
	differenceMap := currentObjectRowMap differenceFrom: mementoObjectRowMap.! !

!GlorpRowDifferencingTest methodsFor: 'support' stamp: ''!
correspond: person1 to: person2 
	| correspondanceMap |
	correspondanceMap := IdentityDictionary new.
	correspondanceMap at: person1 put: person2.
	person1 address notNil ifTrue: 
		[correspondanceMap at: person1 address put: person2 address.
		"Now fix it up so this actually looks like a real memento"
		person2 address: person1 address].
	^correspondanceMap! !

!GlorpRowDifferencingTest methodsFor: 'support' stamp: ''!
generateMementoRowMapFor: person withCorrespondenceMap: aDictionary
	| rowMap |
	rowMap := RowMapForMementos withCorrespondenceMap: aDictionary.
	self addRowsFor: person to: rowMap.
	(person address notNil and: [(aDictionary at: person address ifAbsent: [nil]) notNil]) ifTrue: [self addRowsFor: (aDictionary at: person address) to: rowMap].
	^rowMap! !

!GlorpRowDifferencingTest methodsFor: 'support' stamp: ''!
generateRowMapFor: person 
	| rowMap |
	rowMap := RowMap new.
	self addRowsFor: person to: rowMap.
	person address notNil ifTrue: [self addRowsFor: person address to: rowMap].
	^rowMap! !

!GlorpRowDifferencingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	system := session system.
	session beginUnitOfWork.! !

!GlorpRowDifferencingTest methodsFor: 'tests' stamp: ''!
testDifferenceFromAnotherObject
	currentObject := GlorpPerson example1.
	mementoObject := GlorpPerson example2.
	self computeDifference.
	self assert: differenceMap numberOfEntries = 2.
	self 
		assert: (differenceMap 
				numberOfEntriesForTable: (system tableNamed: 'PERSON')) = 1.
	self 
		assert: (differenceMap 
				numberOfEntriesForTable: (system tableNamed: 'GR_ADDRESS')) = 1! !

!GlorpRowDifferencingTest methodsFor: 'tests' stamp: ''!
testDifferenceFromSameObjectWithAddedComponent
"	Commenting these out because I think the setup is just wrong.
currentObject := Person example1.
	mementoObject := Person example1WithNoAddress.
	self computeDifference.
	self assert: differenceMap numberOfEntries = 2.
	self 
		assert: (differenceMap 
				numberOfEntriesForTable: (system tableNamed: 'PERSON')) = 1.
	self 
		assert: (differenceMap 
				numberOfEntriesForTable: (system tableNamed: 'GR_ADDRESS')) = 1"! !

!GlorpRowDifferencingTest methodsFor: 'tests' stamp: ''!
testDifferenceFromSameObjectWithChangedAttribute
	currentObject := GlorpPerson example1.
	mementoObject := GlorpPerson example1WithDifferentName.
	self computeDifference.
	self assert: differenceMap numberOfEntries = 1.
	self 
		assert: (differenceMap 
				numberOfEntriesForTable: (system tableNamed: 'PERSON')) = 1! !

!GlorpRowDifferencingTest methodsFor: 'tests' stamp: ''!
testDifferenceFromSameObjectWithChangedComponent
	"Commenting these out because I think the setup is just wrong"
	"currentObject := Person example1.
	mementoObject := Person example1WithDifferentAddress.

	currentObjectRowMap := self generateRowMapFor: currentObject."

	"Before changes occur, all original objects are registered with the unit of work.  
		To mimic that, the original person's address needs to be added to current (after changes) "
"	self addRowsFor: mementoObject address to: currentObjectRowMap.
	correspondenceMap := self correspond: currentObject to: mementoObject.
	correspondenceMap at: mementoObject address put: mementoObject address.
	correspondenceMap removeKey: currentObject address.
	mementoObjectRowMap := self generateMementoRowMapFor: mementoObject withCorrespondenceMap: correspondenceMap.
	differenceMap := currentObjectRowMap differenceFrom: mementoObjectRowMap.
	self assert: differenceMap numberOfEntries = 2.
	self 
		assert: (differenceMap 
				numberOfEntriesForTable: (system tableNamed: 'PERSON')) = 1.
	self 
		assert: (differenceMap 
				numberOfEntriesForTable: (system tableNamed: 'GR_ADDRESS')) = 1"! !

!GlorpRowDifferencingTest methodsFor: 'tests' stamp: ''!
testDifferenceFromSameObjectWithChangedComponentAttribute
	currentObject := GlorpPerson example1.
	mementoObject := GlorpPerson example1WithChangedAddress.
	self computeDifference.
	self assert: differenceMap numberOfEntries = 1.
	self 
		assert: (differenceMap 
				numberOfEntriesForTable: (system tableNamed: 'GR_ADDRESS')) = 1! !

!GlorpRowDifferencingTest methodsFor: 'tests' stamp: ''!
testDifferenceFromSameObjectWithDeletedComponent
	"Commenting these out because I think the setup is just wrong"
"	currentObject := Person example1WithNoAddress.
	mementoObject := Person example1.
	self computeDifference.
	self assert: differenceMap numberOfEntries = 1.
	self 
		assert: (differenceMap 
				numberOfEntriesForTable: (system tableNamed: 'PERSON')) = 1"! !

!GlorpRowDifferencingTest methodsFor: 'tests' stamp: ''!
testEquality
	| addressRow1 addressRow2 |
	addressRow1 := session system exampleAddressRow.
	addressRow2 := session system exampleAddressRowWithDifferentStreet.
	self assert: (addressRow1 equals: addressRow1).
	self assert: (addressRow1 equals: addressRow2) not.! !

!GlorpRowDifferencingTest methodsFor: 'tests' stamp: ''!
testNoDifference2
	| |
	currentObject := GlorpPerson example1.
	mementoObject := GlorpPerson example1.
	self computeDifference.
	self assert: differenceMap numberOfEntries = 0.! !

GlorpDatabaseBasedTest subclass: #GlorpSessionBasedTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

GlorpSessionBasedTest subclass: #GlorpCompositeKeyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpCompositeKeyTest methodsFor: 'setup' stamp: 'GuillermoPolito 5/8/2011 00:37'!
setUp
	super setUp.
	system := (GlorpCompositeKeyDescriptorSystem forPlatform: session platform).
	session system: system.
	session createTables.
	session beginTransaction.
	session transact: [session register: system example1].
	session reset.! !

!GlorpCompositeKeyTest methodsFor: 'setup' stamp: 'GuillermoPolito 5/8/2011 00:38'!
tearDown
	session rollbackTransaction.
	session resetPermissions.
	session dropTables.
	super tearDown.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testAlsoFetch
	| users |
	users := session readManyOf: GlorpUser.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testBasicReadFolder
	| folders |
	folders := (session readManyOf: GlorpFolder) asSortedCollection: [:a :b | a name <= b name].
	self assert: folders size = 3.
	self assert: folders first name = 'One'.
	self assert: folders last name = 'Two'.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testBasicReadMessage
	| messages |
	messages := (session readManyOf: GlorpMessage) asSortedCollection: [:a :b | a subject <= b subject].
	self assert: messages size = 3.
	self assert: messages first subject = 'also goes in folder 1'.
	self assert: (messages at: 2) subject = 'goes in folder 1'.
	self assert: messages last subject = 'goes in folder 2'.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testBasicReadUser
	| users |
	users := (session readManyOf: GlorpUser) asSortedCollection: [:a :b | a id <= b id].
	self assert: users size = 2.
	self assert: users first name = 'User One'.
	self assert: users last name = 'User Two'.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testFaulting
	| users user1 sortedFolders user2 |
	users := session readManyOf: GlorpUser.
	user1 := users detect: [:each | each id = 1].
	self assert: user1 folders size = 2.
	sortedFolders := user1 folders asSortedCollection: [:a :b | a name < b name].
	self assert: sortedFolders first name = 'One'.
	user2 := users detect: [:each | each id = 2].
	self assert: user2 folders first messages first subject = 'goes in folder 2'.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testFolderIdentity
	| folders |
	folders := session readManyOf: GlorpFolder.
	folders do: [:each | | individualFolders individualFolder otherIndividualFolder|
		individualFolders := session readManyOf: GlorpFolder where: [:eachFolder | eachFolder = each].
		self assert: individualFolders size = 1.
		individualFolder := individualFolders first.
		otherIndividualFolder := session readOneOf: GlorpFolder where: [:eachFolder | eachFolder = each].
		self assert: individualFolder == each.
		self assert: otherIndividualFolder == individualFolder].! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testMessageIdentity
	| messages |
	messages := session readManyOf: GlorpMessage.
	messages do: [:each | | individualMessages individualMessage otherIndividualMessage|
		individualMessages := session readManyOf: GlorpMessage where: [:eachMessage | eachMessage = each].
		self assert: individualMessages size = 1.
		individualMessage := individualMessages first.
		otherIndividualMessage := session readOneOf: GlorpMessage where: [:eachMessage | eachMessage = each].
		self assert: individualMessage == each.
		self assert: otherIndividualMessage == individualMessage].! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testReadWithCacheHitFromExpression
	| message |
	session reset.
	message := session readOneOf: GlorpMessage where: [:each | each subject = 'goes in folder 1'].
	self assert: message folder name = 'One'.
	session accessor permitNothing.
	session readOneOf: GlorpFolder where: [:each | each user id = 1 & (each name = 'One')].! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testReadWithCacheHitFromJoin
	| message |
	session reset.
	message := session readOneOf: GlorpMessage where: [:each | each subject = 'goes in folder 1'].
	session readOneOf: GlorpFolder where: [:each | each user id = 1 & (each name = 'One')].
	session accessor permitNothing.
	self assert: message folder name = 'One'.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testReadWithCacheHitFromParameterizedExpression
	| message query |
	session reset.
	message := session readOneOf: GlorpMessage where: [:each | each subject = 'goes in folder 1'].
	self assert: message folder name = 'One'.
	session accessor permitNothing.
	query := Query readOneOf: GlorpFolder where: [:each | each user id = (each parameter: 1)  & (each name = (each parameter: 2))].
	query executeWithParameters: #(1 'One') in: session.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testReadWithJoins
	| users |
	users := session readManyOf: GlorpUser where: [:each | each folders anySatisfyJoin: [:eachFolder |
		eachFolder messages anySatisfyJoin: [:eachMessage | eachMessage subject = 'goes in folder 1']]].
	self assert: users size = 1.
	self assert: users first id = 1.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testReadWithSubselect
	| users |
	users := session readManyOf: GlorpUser where: [:each | each folders anySatisfyExists: [:eachFolder |
		eachFolder messages anySatisfyExists: [:eachMessage | eachMessage subject = 'goes in folder 1']]].
	self assert: users size = 1.
	self assert: users first id = 1.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testReadWithSubselectAndINClause
	| users |
	self should: [
		users := session readManyOf: GlorpUser where: [:each | 
			each folders anySatisfyExists: [:eachFolder |
				| q |
				q := Query read: GlorpFolder.
				eachFolder in: q]]]
		raise: GlorpInvalidExpressionError.! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testUserIdentity
	| users |
	users := session readManyOf: GlorpUser.
	users
		do:
			[:each | 
			| individualUser otherIndividualUser individualUsers |
			individualUsers := session
				readManyOf: GlorpUser
				where: [:eachIndividual | eachIndividual id = each id].
			self assert: individualUsers size = 1.
			individualUser := individualUsers first.
			otherIndividualUser := session
				readOneOf: GlorpUser
				where: [:eachIndividual | eachIndividual id = each id].
			self assert: individualUser == each.
			self assert: otherIndividualUser == individualUser].! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testUserIdentityViaMessage
	| users |
	users := session readManyOf: GlorpUser.
	users
		do:
			[:each | 
			each folders
				do:
					[:eachFolder | 
					eachFolder messages
						do: [:eachMessage | 
							self assert: eachMessage folder yourSelf == eachFolder.
							self assert: eachMessage user yourSelf == each]]].! !

!GlorpCompositeKeyTest methodsFor: 'tests- basic reading' stamp: ''!
testUserIdentityViaMessageInUnitOfWork
	session beginUnitOfWork.
	[self testUserIdentityViaMessage] ensure: [session rollbackUnitOfWork].! !

GlorpSessionBasedTest subclass: #GlorpConditionalMappingTest
	instanceVariableNames: 'mapping descriptor mapping1 mapping2 mapping3 newMapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpConditionalMappingTest commentStamp: '<historical>' prior: 0!
This tests the conditional mapping basic API without going to the database.
!

!GlorpConditionalMappingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	descriptor := system descriptorFor: GlorpPassenger.
	mapping := descriptor newMapping: ConditionalMapping.
	mapping conditionalField: (descriptor primaryTable fieldNamed: 'ID').
	mapping if: #testIDLessThan5: then: (mapping1 := DirectMapping from: #blah to: (descriptor primaryTable fieldNamed: 'ID')).
	mapping if: #testIDMoreThan5: then: (mapping2 := DirectMapping from: #bletch to: (descriptor primaryTable fieldNamed: 'NAME')).
	mapping otherwise: (mapping3 := DirectMapping from: #bazoomer to: (descriptor primaryTable fieldNamed: 'ID')).
	mapping conditionalMethod: #id.! !

!GlorpConditionalMappingTest methodsFor: 'support' stamp: ''!
setUpNew
	"Set up using the newer protocol"
	newMapping := descriptor newMapping: ConditionalMapping.
	newMapping
		forField: (descriptor primaryTable fieldNamed: 'ID')
		attribute: #id
		if: [:value | value = 1]
		useMapping: (mapping1 := DirectMapping from: #blah to: (descriptor primaryTable fieldNamed: 'ID')).
	newMapping if: #testIDMoreThan5: then: (mapping2 := DirectMapping from: #bletch to: (descriptor primaryTable fieldNamed: 'NAME')).
	newMapping otherwise: (mapping3 := DirectMapping from: #bazoomer to: (descriptor primaryTable fieldNamed: 'ID')).! !

!GlorpConditionalMappingTest methodsFor: 'tests' stamp: ''!
testApplicableMappingForReadNewProtocol

	| builder |
	self setUpNew.
	builder := ElementBuilder for: ([:x | x] asGlorpExpressionForDescriptor: descriptor) in: SimpleQuery new.
	self assert: (newMapping applicableMappingForRow: #(1 foo nil nil) in: builder) == mapping1.
	self assert: (newMapping applicableMappingForRow: #(6 foo nil nil) in: builder) == mapping2.
	self assert: (newMapping applicableMappingForRow: #(5 foo nil nil) in: builder) == mapping3.! !

!GlorpConditionalMappingTest methodsFor: 'tests' stamp: ''!
testApplicableMappingForReadOldProtocol

	| builder |
	builder := ElementBuilder for: ([:x | x] asGlorpExpressionForDescriptor: descriptor) in: SimpleQuery new.
	self assert: (mapping applicableMappingForRow: #(1 foo nil nil) in: builder) == mapping1.
	self assert: (mapping applicableMappingForRow: #(6 foo nil nil) in: builder) == mapping2.
	self assert: (mapping applicableMappingForRow: #(5 foo nil nil) in: builder) == mapping3.! !

!GlorpConditionalMappingTest methodsFor: 'tests' stamp: ''!
testApplicableMappingForWriteNewProtocol

	| passenger |
	self setUpNew.
	passenger := GlorpPassenger new.
	passenger id: 1.
	self assert: (newMapping applicableMappingForObject: passenger) == mapping1.
	passenger id: 6.
	self assert: (newMapping applicableMappingForObject: passenger) == mapping2.
	passenger id: 5.
	self assert: (newMapping applicableMappingForObject: passenger) == mapping3.! !

!GlorpConditionalMappingTest methodsFor: 'tests' stamp: ''!
testApplicableMappingForWriteOldProtocol

	| passenger |
	passenger := GlorpPassenger new.
	passenger id: 1.
	self assert: (mapping applicableMappingForObject: passenger) == mapping1.
	passenger id: 6.
	self assert: (mapping applicableMappingForObject: passenger) == mapping2.
	passenger id: 5.
	self assert: (mapping applicableMappingForObject: passenger) == mapping3.! !

!GlorpConditionalMappingTest methodsFor: 'tests' stamp: ''!
testDescriptor

	self assert: mapping1 descriptor == descriptor.
	self assert: mapping2 descriptor == descriptor.
	self assert: mapping3 descriptor == descriptor.! !

!GlorpConditionalMappingTest methodsFor: 'tests' stamp: ''!
testMappedFieldsOldProtocol

	self assert: mapping mappedFields asSortedCollection = descriptor primaryTable fields asSortedCollection.

	mapping := descriptor newMapping: ConditionalMapping.
	self assert: mapping mappedFields isEmpty.

	mapping conditionalField: (descriptor primaryTable fieldNamed: 'ID').
	self assert: mapping mappedFields size = 1.	

	mapping if: #testIDLessThan5: then: (mapping1 := DirectMapping from: #blah to: (descriptor primaryTable fieldNamed: 'NAME')).
	self assert: mapping mappedFields asSortedCollection = descriptor primaryTable fields asSortedCollection.! !

GlorpSessionBasedTest subclass: #GlorpCursoredCollectionDBTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpCursoredCollectionDBTest methodsFor: 'support' stamp: ''!
setUp
	session := GlorpSessionResource current newSession.
	session beginTransaction.
	self writePersonRows.! !

!GlorpCursoredCollectionDBTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.! !

!GlorpCursoredCollectionDBTest methodsFor: 'tests' stamp: ''!
testParallelStreams
	| query person query2 stream1 stream2 address person2 address2 |
	session platform supportsMultipleOpenCursors ifFalse: [^self].
	query := Query returningManyOf: GlorpPerson.
	query orderBy: [:each | each id].
	query collectionType: GlorpCursoredStream.
	query2 := Query returningManyOf: GlorpAddress.
	query2 orderBy: [:each | each id].
	query2 collectionType: GlorpCursoredStream.
	stream1 := session execute: query.
	self assert: session privateGetCache numberOfElements = 0.
	stream2 := session execute: query2.
	person := stream1 next.
	self assert: session privateGetCache numberOfElements = 1.
	self assert: person id = 86.
	address := stream2 next.
	self assert: address id = 1.
	self assert: address street = 'Alpha'.
	self assert: session privateGetCache numberOfElements = 2.
	person2 := stream1 next.
	address2 := stream2 next.
	self assert: person2 class == GlorpPerson.
	self assert: address2 class == GlorpAddress.
	self assert: address2 street = 'Beta'.

"SQLite allows multiple processes to have the database file open at once, and multiple processes can read the database at once.  Thus we let that platform return true to #supportsMultipleOpenCursors.  When a process wants to write, it must lock the entire database file for the duration of its update (normally a few milliseconds, during which other processes wait on the writer to finish, then continue).  However this database locking is therefore dependent on the OS file-system and can show issues.  This test, after passing all its assertions, has been seen to fail in tearDown's rollback of the transaction due to 'database is locked'.  The presumption is that both cursors rollback and so appear like competing writes even though neither is really.   Releasing the cursors ends contention and lets the rollback succeed."
	session platform class == SQLite3Platform ifTrue:
		[stream1 release. stream1 := nil.
		stream2 release. stream2 := nil].! !

!GlorpCursoredCollectionDBTest methodsFor: 'tests' stamp: ''!
testReadAsStream
	| query stream person restOfPeople |
	query := Query returningManyOf: GlorpPerson.
	query orderBy: [:each | each id].
	query collectionType: GlorpCursoredStream.
	stream := session execute: query.
	self assert: session privateGetCache numberOfElements = 0.
	person := stream next.
	self assert: session privateGetCache numberOfElements = 1.
	self assert: person id = 86.
	restOfPeople := stream upToEnd.
	self assert: restOfPeople size = 2.
	self assert: session privateGetCache numberOfElements = 3.! !

!GlorpCursoredCollectionDBTest methodsFor: 'tests' stamp: ''!
testReadEmptyStream
	| query stream person |
	query := Query returningManyOf: GlorpPerson where: [:each | each id = 999].
	query collectionType: GlorpCursoredStream.
	stream := session execute: query.
	self assert: session privateGetCache numberOfElements = 0.
	person := stream next.
	self assert: session privateGetCache numberOfElements = 0.
	self assert: person isNil.
	self assert: stream atEnd.! !

!GlorpCursoredCollectionDBTest methodsFor: 'tests' stamp: ''!
testReadLiteralEmptyStream
	| query stream person |
	query := Query returningManyOf: GlorpPerson where: false.
	query collectionType: GlorpCursoredStream.
	stream := session execute: query.
	self assert: session privateGetCache numberOfElements = 0.
	person := stream next.
	self assert: session privateGetCache numberOfElements = 0.
	self assert: person isNil.
	self assert: stream atEnd.! !

!GlorpCursoredCollectionDBTest methodsFor: 'support' stamp: ''!
writePersonRows
	session writeRow: session system exampleAddressRowForOrdering1.
	session writeRow: session system exampleAddressRowForOrdering2.
	session writeRow: session system exampleAddressRowForOrdering3.
	session writeRow: session system examplePersonRowForOrdering1.
	session writeRow: session system examplePersonRowForOrdering2.
	session writeRow: session system examplePersonRowForOrdering3.! !

GlorpSessionBasedTest subclass: #GlorpDatabaseSequenceTest
	instanceVariableNames: 'sequence table row2 row1'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpDatabaseSequenceTest methodsFor: 'tests' stamp: ''!
assignSequences
	row1 preWriteAssignSequencesUsing: session.
	row1 postWriteAssignSequencesUsing: session.
	row2 preWriteAssignSequencesUsing: session.
	row2 postWriteAssignSequencesUsing: session.! !

!GlorpDatabaseSequenceTest methodsFor: 'tests' stamp: ''!
row1Value
	^row1 atFieldNamed: 'ID'.! !

!GlorpDatabaseSequenceTest methodsFor: 'tests' stamp: ''!
row2Value
	^row2 atFieldNamed: 'ID'.! !

!GlorpDatabaseSequenceTest methodsFor: 'setup' stamp: ''!
setUp
	super setUp.
	table := session system tableNamed: 'BANK_TRANS'.
	sequence := (table fieldNamed: 'ID') type sequence.
	row1 := DatabaseRow newForTable: table.
	row2 := DatabaseRow newForTable: table.! !

!GlorpDatabaseSequenceTest methodsFor: 'tests' stamp: ''!
testGetNextValue
	session platform usesIdentityColumns ifTrue: [^self].
	self assignSequences.
	self assert: self row2Value = (self row1Value + 1).! !

!GlorpDatabaseSequenceTest methodsFor: 'tests' stamp: ''!
testReserveEmpty
	session platform usesIdentityColumns ifTrue: [^self].
	sequence reserveSequenceNumbers: 2 in: session for: table.
	session := nil.
	self assignSequences.
	self assert: self row1Value + 1 = self row2Value.! !

!GlorpDatabaseSequenceTest methodsFor: 'tests' stamp: ''!
testReserveUsingTable
	session platform usesIdentityColumns ifTrue: [^self].
	[session beginTransaction.
	self writeTransactionRow: 65.
	self writeTransactionRow: 66.
	self writeTransactionRow: 67.
	sequence
		reserveSequenceNumbers: 7
		in: session
		for: table] ensure: [session rollbackTransaction].
	session := nil.
	self assignSequences.
	self assert: self row1Value + 1 = self row2Value.! !

!GlorpDatabaseSequenceTest methodsFor: 'tests' stamp: ''!
testReservedNumberAllocation
	session := nil.
	sequence reservedNumbers add: 23; add: 24.
	self assignSequences.
	self assert: self row1Value = 23.
	self assert: self row2Value = 24.! !

!GlorpDatabaseSequenceTest methodsFor: 'tests' stamp: ''!
writeTransactionRow: anInteger
	| tempRow |
	tempRow := DatabaseRow newForTable: table.
	session platform usesIdentityColumns ifFalse: [
		tempRow atFieldNamed: 'ID' put: anInteger].
	tempRow atFieldNamed: 'AMT_CURR' put: 'CDN'.
	session writeRow: tempRow.! !

GlorpSessionBasedTest subclass: #GlorpDictionaryMappingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDictionaryMappingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session system: (GlorpEncyclopediaDescriptorSystem forPlatform: session platform).
	session beginTransaction.! !

!GlorpDictionaryMappingTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testCasesToWrite

	"How des a dictionary mapping relate to 1:many vs many:many.
	dictionary of strings to strings
	dictionary of strings to objects
	dictionary of objects to objects
	keys always have to be related to values somehow, because I can't extract the association otherwise. Both might also be associated to source.
    You should be able to use the topological sort to determine the create/delete order of tables as well"! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testCollectionKeyQuery

	| encyclopedias matches |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
"	matches := session read: GlorpEncyclopedia	 where: [:each | each entries anySatisfy: [:eachEntry |
		((eachEntry getTable: 'ENCYC_ENTRY_LINK') getField: 'REFERENCE_NO') = 'unique']]."
	matches := session read: GlorpEncyclopedia	 where: [:each | each entries anySatisfy: [:eachEntry |
		eachEntry key = 'unique']].
	self assert: matches size = 1.
	self assert: matches first name = 'The Canadian Encyclopedia'.
	matches do: [:each | self assert: (each entries anySatisfy: [:eachEntry | eachEntry id = 'three'])].
	matches do: [:each | self assert: each entries size = 3].! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testCollectionValueQuery

	| encyclopedias matches |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	matches := session read: GlorpEncyclopedia where: [:each | each entries anySatisfy: [:eachEntry |
		eachEntry value title = 'Vehicular Fish Decay']].
	self assert: matches size = 1.
	self assert: matches first name = 'The Canadian Encyclopedia'.
	matches do: [:each | self assert: (each entries anySatisfy: [:eachEntry | eachEntry id = 'three'])].
	matches do: [:each | self assert: each entries size = 3].! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testCollectionValueQueryWithImpliedValue

	| encyclopedias matches |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	matches := session read: GlorpEncyclopedia where: [:each | each entries anySatisfy: [:eachEntry |
		eachEntry title = 'Vehicular Fish Decay']].
	self assert: matches size = 1.
	self assert: matches first name = 'The Canadian Encyclopedia'.
	matches do: [:each | self assert: (each entries anySatisfy: [:eachEntry | eachEntry id = 'three'])].
	matches do: [:each | self assert: each entries size = 3].! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testCollectionValueQueryWithImpliedValueAndObjectEquality

	| encyclopedias matches entry |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	entry := session readOneOf: GlorpEncyclopediaEntry where: [:each | each title = 'Vehicular Fish Decay'].
	matches := session read: GlorpEncyclopedia where: [:each | each entries anySatisfy: [:eachEntry |
		eachEntry = entry]].
	self assert: matches size = 1.
	self assert: matches first name = 'The Canadian Encyclopedia'.
	matches do: [:each | self assert: (each entries anySatisfy: [:eachEntry | eachEntry id = 'three'])].
	matches do: [:each | self assert: each entries size = 3].! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference classes' stamp: ''!
testDeleteSimpleTypeOneToMany
	| publishers publisher |
	self assert: (session read: GlorpPublisher) isEmpty.
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titles: (publisher titles copyWithout: 'Mastering ENVY/Developer')].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: publisher titles size = 2.
	self assert: publisher titles asSortedCollection = (session system examplePublisher1 titles copyWithout: 'Mastering ENVY/Developer') asSortedCollection.! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testQueryWithDictionaryType

	| encyclopedias query dict linkTable entryTable refField linkTableField idField |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	linkTable := session system tableNamed: 'ENCYC_ENTRY_LINK'.
	entryTable := session system tableNamed: 'ENCYC_ENTRY'.
	refField := linkTable fieldNamed: 'REFERENCE_NO'.
	linkTableField := linkTable fieldNamed: 'ENCYC_ENTRY_ID'.
	idField := entryTable fieldNamed: 'ID'.
	query := Query readManyOf: GlorpEncyclopediaEntry.
	query where: [:each | ((each getTable: linkTable) getField: linkTableField) = ((each getTable: entryTable) getField: idField)].
	query retrieveDictionaryWithKey: [:each | (each getTable: linkTable) getField: refField] value: [:each | each].
	dict := session execute: query.
	self assert: dict class == Dictionary.
	self assert: dict size = 5.
	self assert: (dict at: 'abcde') id = 'two'.
	self assert: (dict at: 'abcde') == (dict at: 'edcba').! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testReadWithAlsoFetch
	| encyclopedias query readBackEncyclopedias |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	query := Query readManyOf: GlorpEncyclopedia.
	query orderBy: [:each | each name].
	query alsoFetch: [:each | each entries asOuterJoin].
	readBackEncyclopedias := session execute: query.
	session accessor denyReads.
	self verifyEncyclopedias: readBackEncyclopedias.! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testReadWithRetrieve
	| encyclopedias query readBackEncyclopedias |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	query := Query readManyOf: GlorpEncyclopedia.
	query orderBy: [:each | each name].
	query retrieve: [:each | each].
	query retrieve: [:each | each entries asOuterJoin].
	readBackEncyclopedias := session execute: query.
	session accessor denyReads.
	self assert: readBackEncyclopedias size = 6.
	self verifyEncyclopedias: ((readBackEncyclopedias collect: [:each | each first]) asSet asSortedCollection: [:a :b | a name <= b name]).
	readBackEncyclopedias do: [:eachArray |
		eachArray last key isNil 
			ifTrue: [self assert: eachArray first entries isEmpty]
			ifFalse: [self assert: (eachArray first entries at: eachArray last key) = eachArray last value]].! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testSimpleTypeDictionary
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each id = 1].
	publishers do: [:each | 
		each titlesWithNumberInStock keysAndValuesDo: [:eachTitle :eachNumber | 	
			self assert: eachTitle isString.
			self assert: eachNumber = 2]].
	self assert: publishers first titlesWithNumberInStock size = 3.
	self assert: publishers first titlesWithNumberInStock keys asSortedCollection = session system examplePublisher1 titles asSortedCollection.

"	publishers do: [:each | 
		each titlesWithNumberInStockLink keysAndValuesDo: [:eachTitle :eachNumber | 	
			self assert: eachTitle isString.
			self assert: eachNumber = 2]].
	self assert: publishers first titlesWithNumberInStockLink size = 3.
	self assert: publishers first titlesWithNumberInStockLink keys asSortedCollection = session system examplePublisher1 titles asSortedCollection."! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testSimpleTypeDictionaryComplexQuerying
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock sqlCount = 3].
	self assert: publishers size = 1.
	self assert: publishers first titlesWithNumberInStock size = 3.
	self assert: (publishers first titlesWithNumberInStock keys includes: 'Mastering ENVY/Developer').

"	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStockLink sqlCount > 0].
	publishers := publishers asSortedCollection: [:a :b | a name <= b name].
	self assert: publishers size = 2.
	self assert: publishers first titlesWithNumberInStock size = 3.
	self assert: (publishers first titlesWithNumberInStock keys includes: 'Mastering ENVY/Developer')."! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testSimpleTypeDictionaryQuerying
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.

	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock anySatisfy: [:x | x key like: 'Master%']].
	self assert: publishers size = 1.
	self assert: publishers first titlesWithNumberInStock size = 3.
	self assert: (publishers first titlesWithNumberInStock keys includes: 'Mastering ENVY/Developer').

"	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock anySatisfy: [:x | x value  = 2]].
	self assert: publishers size = 1.
	self assert: publishers first id = 1."! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testSimpleTypeDictionaryQueryingWithAlsoFetch
	| publishers publisherOne publisher2 q |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.

	q := Query read: GlorpPublisher where: [:each | each id < 3].
	q orderBy: [:each | each id].
	q alsoFetch: [:each | each titlesWithNumberInStock].
	publishers := session execute: q.
	session accessor denyReads.
	
	self assert: publishers asSet size = 2.
	publisherOne := publishers detect: [:each | each id = 1].
	self assert: publisherOne titlesWithNumberInStock size = 3.
	self assert: (publisherOne titlesWithNumberInStock includesKey: 'Mastering ENVY/Developer').
	self assert: (publisherOne titlesWithNumberInStock values asArray = #(2 2 2)).

	publisher2 := publishers detect: [:each | each id = 2].
	self assert: publisher2 titlesWithNumberInStock keys asArray first = 'A book'.
	self assert: (publisher2 titlesWithNumberInStock at: 'A book') = 200.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testSimpleTypeDictionaryQueryingWithFilteredReads
	| publishers publisherOne publisher2 |
	((session system descriptorFor: GlorpPublisher) mappingForAttributeNamed: #titlesWithNumberInStock) useFilteredRead.
	"((session system descriptorFor: GlorpPublisher) mappingForAttributeNamed: #titlesWithNumberInStockLink) useFilteredRead."

	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.

	publishers := session read: GlorpPublisher where: [:each | each id < 3].
	self assert: publishers size = 2.
	publisherOne := publishers detect: [:each | each id = 1].
	self assert: publisherOne titlesWithNumberInStock size = 3.
	self assert: (publisherOne titlesWithNumberInStock includesKey: 'Mastering ENVY/Developer').
	self assert: (publisherOne titlesWithNumberInStock values asArray = #(2 2 2)).
"	self assert: publisherOne titlesWithNumberInStockLink size = 3.
	self assert: (publisherOne titlesWithNumberInStockLink includesKey: 'Mastering ENVY/Developer').
	self assert: (publisherOne titlesWithNumberInStockLink values asArray = #(2 2 2)).
"
	session accessor denyReads.
	publisher2 := publishers detect: [:each | each id = 2].
	self assert: publisher2 titlesWithNumberInStock keys asArray first = 'A book'.
"	self assert: publisher2 titlesWithNumberInStockLink keys asArray first = 'A book'."
	self assert: (publisher2 titlesWithNumberInStock at: 'A book') = 200.
"	self assert: (publisher2 titlesWithNumberInStockLink at: 'A book') = 200."! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testSimpleTypeDictionaryQueryingWithRetrieve
	| publishers publisherOne publisher2 q actualPublishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.

	q := Query read: GlorpPublisher where: [:each | each id < 3].
	q retrieve: [:each | each].
	q retrieve: [:each | each titlesWithNumberInStock].
	publishers := session execute: q.
	
	actualPublishers := publishers collect: [:each | each first].

	self assert: actualPublishers asSet size = 2.
	publisherOne := actualPublishers detect: [:each | each id = 1].
	self assert: publisherOne titlesWithNumberInStock size = 3.
	self assert: (publisherOne titlesWithNumberInStock includesKey: 'Mastering ENVY/Developer').
	self assert: (publisherOne titlesWithNumberInStock values asArray = #(2 2 2)).

	publisher2 := actualPublishers detect: [:each | each id = 2].
	self assert: publisher2 titlesWithNumberInStock keys asArray first = 'A book'.
	self assert: (publisher2 titlesWithNumberInStock at: 'A book') = 200.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testSimpleTypeDictionaryRead
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock anySatisfy: [:eachTitleAssociation |
		eachTitleAssociation value = 2]].
	self assert: publishers size = 1.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testSimpleTypeDictionaryReadWithImpliedValue
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titlesWithNumberInStock anySatisfy: [:eachTitleAssociation |
		eachTitleAssociation = 2]].
	self assert: publishers size = 1.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference classes' stamp: ''!
testSimpleTypeOneToMany
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each id = 1].
	publishers do: [:each | 
		each titles do: [:eachTitle | self assert: eachTitle isString]].
	self assert: publishers first titles size = 3.
	self assert: publishers first titles asSortedCollection = session system examplePublisher1 titles asSortedCollection.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference classes' stamp: ''!
testSimpleTypeOneToManyComplexQuerying
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titles sqlCount = 3].
	self assert: publishers size = 1.
	self assert: publishers first titles size = 3.
	self assert: (publishers first titles includes: 'Mastering ENVY/Developer').! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference classes' stamp: ''!
testSimpleTypeOneToManyQuerying
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titles anySatisfy: [:x | x like: 'Master%' ]].
	self assert: publishers size = 1.
	self assert: (publishers first titles anySatisfy: [:each | each like: 'Master%']).! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference classes' stamp: ''!
testSimpleTypeOneToManyQueryingIncludes
	| publishers |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each titles includes: 'Mastering ENVY/Developer' ].
	self assert: publishers size = 1.
	self assert: (publishers first titles includes: 'Mastering ENVY/Developer').! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference classes' stamp: ''!
testSimpleTypeOneToManyQueryingWithFilteredReads
	| publishers publisherOne publisher2 |
	((session system descriptorFor: GlorpPublisher) mappingForAttributeNamed: #titles) useFilteredRead.
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publishers := session read: GlorpPublisher where: [:each | each id < 3].
	self assert: publishers size = 2.
	publisherOne := publishers detect: [:each | each id = 1].
	self assert: publisherOne titles size = 3.
	self assert: (publisherOne titles includes: 'Mastering ENVY/Developer').
	session accessor denyReads.
	publisher2 := publishers detect: [:each | each id = 2].
	self assert: publisher2 titles first = 'A book'.! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testUpdate

	| encyclopedias canadian allEntryRows allLinkRows |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	session accessor denyCommands: (Array with: DeleteCommand).
	session inUnitOfWorkDo: [
		| newEntry |
		canadian := session readOneOf: GlorpEncyclopedia where: [:each | each name like: 'The Ca%'].
		newEntry := GlorpEncyclopediaEntry new
			id: 'seven';
			title: 'The Latest Entry';
			text: 'This is all about the very newest stuff'.
		canadian entries at: 'unique' put: newEntry].
	self assert: (canadian entries at: 'abcde') id = 'two'.
	self assert: (canadian entries at: '12345') id = 'one'.
	self assert: (canadian entries at: 'unique') id = 'seven'.
	self assert: (canadian entries at: 'unique') title = 'The Latest Entry'.
	allEntryRows := session accessor executeSQLString: 'select * from ENCYC_ENTRY'.
	"We didn't actually delete the 'three' entry"
	self assert: allEntryRows size = 4.
	allLinkRows := session accessor executeSQLString: 'select * from ENCYC_ENTRY_LINK'.
	"But we should have only our 3 link table rows, plus 2 for the other encyclopedia"
	self assert: allLinkRows size = 5.! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testUpdateNonLinkTable
	"Test updating a dictionary where we don't have a link table, but the target table refers back to the containing object."
	| encyclopedias canadian bio newBio |
	encyclopedias := session system exampleEncyclopedias.
	bio := GlorpEncyclopediaBioEntry new title: 'Alan Kay'; text: 'A stub'.
	encyclopedias first biographicalEntries at: bio title put: bio.
	session transact: [session register: encyclopedias].
	session reset.
	session inUnitOfWorkDo: [
		canadian := session readOneOf: GlorpEncyclopedia where: [:each | each name like: 'The Ca%'].
		newBio := GlorpEncyclopediaBioEntry new
			title: 'Alan Kay';
			text: 'a bit more information'.
		canadian biographicalEntries at: 'Alan Kay' put: newBio].
	session reset.
	canadian := session readOneOf: GlorpEncyclopedia where: [:each | each name like: 'The Ca%'].
	self assert: canadian biographicalEntries size = 1.
	self assert: (canadian biographicalEntries at: 'Alan Kay') text = 'a bit more information'.
	self assert: (session count: GlorpEncyclopediaBioEntry) = 1.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testUpdateSimpleTypeDictionaryAdding
	| publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStock at: 'New Book' put: 1].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStock at: 'New Book') = 1.
	self assert: publisher titlesWithNumberInStock size = 4.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testUpdateSimpleTypeDictionaryDeleting
	| publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStock removeKey: 'Mastering ENVY/Developer'].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStock at: 'Mastering ENVY/Developer' ifAbsent: [999]) = 999.
	self assert: publisher titlesWithNumberInStock size = 2.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testUpdateSimpleTypeDictionaryLinkModifying
"	| publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStockLink at: 'Mastering ENVY/Developer' put: 123].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStockLink at: 'Mastering ENVY/Developer') = 123."
	"It can't find how to delete properly, because it doesn't know the primary key of the old record, and we don't have it recorded anywhere in the image"
	"self assert: (session accessor executeSQLString: 'SELECT * FROM PUBLISHER_TITLE2') size = 4.
	self assert: (session accessor executeSQLString: 'SELECT * FROM PUB_TITLE_LINK') size = 4."! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testUpdateSimpleTypeDictionaryLinkedAdding
"	| publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStockLink at: 'New Book' put: 1].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStockLink at: 'New Book') = 1.
	self assert: publisher titlesWithNumberInStockLink size = 4."! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testUpdateSimpleTypeDictionaryLinkedDeleting
"	| publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStockLink removeKey: 'Mastering ENVY/Developer'].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStockLink at: 'Mastering ENVY/Developer' ifAbsent: [999]) = 999.
	self assert: publisher titlesWithNumberInStockLink size = 2."! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference dictionary' stamp: ''!
testUpdateSimpleTypeDictionaryModifying
	| publishers publisher |
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titlesWithNumberInStock at: 'Mastering ENVY/Developer' put: 123].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: (publisher titlesWithNumberInStock at: 'Mastering ENVY/Developer') = 123.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-simple reference classes' stamp: ''!
testUpdateSimpleTypeOneToMany
	| publishers publisher |
	self assert: (session read: GlorpPublisher) isEmpty.
	publishers := session system examplePublishers.
	session modify: publishers in: [].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	session modify: publisher in: [
		publisher titles: (Array with: 'New Improved Book'), publisher titles].
	session reset.
	publisher := session readOneOf: GlorpPublisher where: [:each | each id = 1].
	self assert: publisher titles size = 4.
	self assert: publisher titles asSortedCollection = (session system examplePublisher1 titles , (Array with: 'New Improved Book')) asSortedCollection.! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
testWriteAndReadBack
	| encyclopedias query readBackEncyclopedias |
	encyclopedias := session system exampleEncyclopedias.
	session transact: [session register: encyclopedias].
	session reset.
	query := Query readManyOf: GlorpEncyclopedia.
	query orderBy: [:each | each name].
	readBackEncyclopedias := session execute: query.
	self verifyEncyclopedias: readBackEncyclopedias.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-key in different tables' stamp: ''!
testWriteAndReadBackKeyInLinkTable

	| query readBackPublishers publishers withEmps |
	publishers := session system examplePublishers.
	session transact: [session register: publishers].
	session reset.
	query := Query readManyOf: GlorpPublisher.
	query orderBy: [:each | each name].
	readBackPublishers := session execute: query.
	self assert: readBackPublishers size = 3.
	withEmps := readBackPublishers at: 2.
	self assert: withEmps name = 'Cambridge'.
	self assert: withEmps differentBunchOfEmployeesById size = 2.
	self assert: withEmps differentBunchOfEmployeesById keys asSortedCollection asArray = #(3 4).
	self assert: (withEmps differentBunchOfEmployeesById at: 3) name = 'Contractor 1'.
	self assert: (withEmps differentBunchOfEmployeesById at: 4) name = 'Contractor 2'.! !

!GlorpDictionaryMappingTest methodsFor: 'tests-key in different tables' stamp: ''!
testWriteAndReadBackKeyInTargetTable

	| query readBackPublishers publishers withEmps |
	publishers := session system examplePublishers.
	session transact: [session register: publishers].
	session reset.
	query := Query readManyOf: GlorpPublisher.
	query orderBy: [:each | each name].
	readBackPublishers := session execute: query.
	self assert: readBackPublishers size = 3.
	withEmps := readBackPublishers at: 2.
	self assert: withEmps name = 'Cambridge'.
	self assert: withEmps employees size = 2.
	self assert: withEmps employees keys asSortedCollection asArray = #('CEO' 'Editor').
	self assert: (withEmps employees at: 'CEO') name = 'The Boss'.
	self assert: (withEmps employees at: 'Editor') name = 'Somebody'.! !

!GlorpDictionaryMappingTest methodsFor: 'tests' stamp: ''!
verifyEncyclopedias: readBackEncyclopedias
	| plagiarists canadian |
	self assert: readBackEncyclopedias size = 3.
	canadian := readBackEncyclopedias first.
	self assert: canadian name = 'The Canadian Encyclopedia'.
	self assert: canadian entries size = 3.
	self assert: (canadian entries at: 'abcde') id = 'two'.
	self assert: (canadian entries at: '12345') id = 'one'.
	self assert: (canadian entries at: 'unique') id = 'three'.
	plagiarists := readBackEncyclopedias at: 2.
	self assert: plagiarists entries size = 2.
	self
		assert:
			(plagiarists entries at: '54321') = (canadian entries at: '12345').
	self
		assert:
			(plagiarists entries at: 'edcba') = (canadian entries at: 'abcde').
	self assert: readBackEncyclopedias last entries isEmpty.! !

GlorpSessionBasedTest subclass: #GlorpDirectMappingTest
	instanceVariableNames: 'mapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDirectMappingTest methodsFor: 'support' stamp: ''!
setUp
	| descriptor |
	super setUp.
	mapping := DirectMapping from: #id to: ((system tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
	descriptor := Descriptor new.
	descriptor classModel: (GlorpClassModel for: GlorpCustomer).
	descriptor addMapping: mapping.! !

!GlorpDirectMappingTest methodsFor: 'tests' stamp: ''!
testExpressionFor
	| cust exp |
	cust := GlorpCustomer new.
	cust id: 12.
	exp := mapping
		expressionFor: cust
		basedOn: BaseExpression new
		relation: #=.
	self assert: exp rightChild class == ConstantExpression.
	self assert: exp rightChild value = 12.
	self assert: exp relation = #=.
	self assert: exp leftChild class == MappingExpression.! !

!GlorpDirectMappingTest methodsFor: 'tests' stamp: ''!
testProxying
	| customers |
	session beginTransaction.
	[((system descriptorFor: GlorpPerson) mappingForAttributeNamed: #name) shouldProxy: true.
	session modify: GlorpPerson example1 in: [].
	session reset.
	customers := session read: GlorpPerson.
	self assert: customers first name isGlorpProxy.
	self deny: customers first name isInstantiated.
	self assert: customers first name = 'Zaphod Beeblebrox']
		ensure: [session rollbackTransaction].
	self deny: ((system descriptorFor: GlorpPerson) mappedFields includes: ((system tableNamed: 'PERSON') fieldNamed: 'NAME')).! !

GlorpSessionBasedTest subclass: #GlorpFilteringQueryDBTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpFilteringQueryDBTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session beginTransaction.! !

!GlorpFilteringQueryDBTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	session resetPermissions.
	super tearDown.! !

!GlorpFilteringQueryDBTest methodsFor: 'tests' stamp: ''!
testReadDictionary
	| entryMapping encyclopedias query readBackEncyclopedias canadian plagiarists |
	session system: (GlorpEncyclopediaDescriptorSystem forPlatform: session platform).
	system := session system.
	encyclopedias := system exampleEncyclopedias.
	entryMapping := (system descriptorFor: GlorpEncyclopedia) mappingForAttributeNamed: #entries.
	entryMapping useFilteredRead.
	session transact: [session register: encyclopedias].
	session reset.
	query := Query readManyOf: GlorpEncyclopedia.
	query orderBy: [:each | each name].
	readBackEncyclopedias := session execute: query.
	canadian := readBackEncyclopedias first.
	self assert: canadian entries size = 3.
	session accessor permitNothing.

	self assert: (canadian entries at: 'abcde') id = 'two'.
	self assert: (canadian entries at: '12345') id = 'one'.
	self assert: (canadian entries at: 'unique') id = 'three'.
	plagiarists := readBackEncyclopedias at: 2.
	self assert: plagiarists entries size = 2.
	self assert: (plagiarists entries at: '54321') = (canadian entries at: '12345').
	self assert: (plagiarists entries at: 'edcba') = (canadian entries at: 'abcde').
	self assert: readBackEncyclopedias last entries isEmpty.! !

!GlorpFilteringQueryDBTest methodsFor: 'tests' stamp: ''!
testReadOneToMany
	"This doesn't actually short-circuit interestingly, because of the shared holder"
	| accounts txMapping tx1 tx2 otherHolder |
	(GlorpReadingTest new session: session) write2CustomerRowsWith3Transactions.
	txMapping := (system descriptorFor: GlorpCustomer) mappingForAttributeNamed: #transactions.
	txMapping useFilteredRead.
	accounts := session execute: ((Query returningManyOf: GlorpBankAccount where: [:each | each id ~= 2]) orderBy: [:each | each id]).
	"Account 9874, 1 holder, 2 transactions"
	tx1 := accounts last accountHolders first transactions.
	"Account 6, 2 holders, 1 with 1 transaction, 1 shared with 9874" 
	otherHolder := accounts first accountHolders detect: [:each | each id = 28].
	tx2 := otherHolder transactions.
	self assert: (tx1 collect: [:each | each amount currency]) asSortedCollection asArray = #(#CDN #USD).
	self assert: (tx2 collect: [:each | each amount currency]) asSortedCollection asArray = #(EUR).! !

!GlorpFilteringQueryDBTest methodsFor: 'tests' stamp: ''!
testReadOneToMany2
	| accounts txMapping otherHolder holder |
	(GlorpReadingTest new session: session) write2CustomerRowsWith3Transactions.
	txMapping := (system descriptorFor: GlorpCustomer) mappingForAttributeNamed: #transactions.
	txMapping useFilteredRead.
	accounts := session execute: ((Query returningManyOf: GlorpBankAccount where: [:each | each id ~= 2]) orderBy: [:each | each id]).
	"Account 6, 2 holders, 1 with 1 transaction, 1 shared with 9874" 
	holder := accounts first accountHolders detect: [:each | each id = 28].
	otherHolder := accounts first accountHolders detect: [:each | each id = 27].
	self assert:  (holder transactions collect: [:each | each amount currency]) asSortedCollection asArray = #(#EUR).
	session accessor permitNothing.
	self assert: (otherHolder transactions collect: [:each | each amount currency]) asSortedCollection asArray = #(#CDN #USD).! !

!GlorpFilteringQueryDBTest methodsFor: 'tests' stamp: ''!
testReadOneToManyEmptyCriteria
	| accounts txMapping otherHolder holder |
	(GlorpReadingTest new session: session) write2CustomerRowsWith3Transactions.
	txMapping := (system descriptorFor: GlorpCustomer) mappingForAttributeNamed: #transactions.
	txMapping useFilteredRead.
	accounts := session execute: (Query returningManyOf: GlorpBankAccount).
	accounts := accounts asSortedCollection: [:a :b | a id <= b id].
	"Account 6, 2 holders, 1 with 1 transaction, 1 shared with 9874" 
	holder := accounts first accountHolders detect: [:each | each id = 28].
	otherHolder := accounts first accountHolders detect: [:each | each id = 27].
	self assert:  (holder transactions collect: [:each | each amount currency]) asSortedCollection asArray = #(#EUR).
	session accessor permitNothing.
	self assert: (otherHolder transactions collect: [:each | each amount currency]) asSortedCollection asArray = #(#CDN #USD).! !

!GlorpFilteringQueryDBTest methodsFor: 'tests' stamp: ''!
testReadOneToManyNoProxy
	"This doesn't actually short-circuit interestingly, because of the shared holder"
	| accounts txMapping tx1 tx2 otherHolder |
	(GlorpReadingTest new session: session) write2CustomerRowsWith3Transactions.
	txMapping := (system descriptorFor: GlorpCustomer) mappingForAttributeNamed: #transactions.
	txMapping shouldProxy: false.
	txMapping useFilteredRead.
	accounts := session execute: ((Query returningManyOf: GlorpBankAccount where: [:each | each id ~= 2]) orderBy: [:each | each id]).
	"Account 9874, 1 holder, 2 transactions"
	tx1 := accounts last accountHolders first transactions.
	"Account 6, 2 holders, 1 with 1 transaction, 1 shared with 9874" 
	otherHolder := accounts first accountHolders detect: [:each | each id = 28].
	tx2 := otherHolder transactions.
	self assert: (tx1 collect: [:each | each amount currency]) asSortedCollection asArray = #(#CDN #USD).
	self assert: (tx2 collect: [:each | each amount currency]) asSortedCollection asArray = #(EUR).! !

!GlorpFilteringQueryDBTest methodsFor: 'tests' stamp: ''!
testReadOneToOne
	| people addressMapping |
	session writeRow: session system exampleAddressRowForOrdering1.
	session writeRow: session system exampleAddressRowForOrdering2.
	session writeRow: session system exampleAddressRowForOrdering3.

	session writeRow: session system examplePersonRowForOrdering1.
	session writeRow: session system examplePersonRowForOrdering2.
	session writeRow: session system examplePersonRowForOrdering3.

	addressMapping := (system descriptorFor: GlorpPerson) mappingForAttributeNamed: #address.
	addressMapping useFilteredRead.
	people := session execute: (Query returningManyOf: GlorpPerson where: [:each | each id ~= 2]).
	people first address yourSelf.
	session accessor permitNothing.
	people := people asSortedCollection: [:a :b | a id <= b id].
	self assert: people first address id = 2.
	self assert: (people at: 2) address id = 3.
	self assert: (people at: 3) address id = 1.
	self assert: people size = 3.! !

!GlorpFilteringQueryDBTest methodsFor: 'tests' stamp: ''!
testTwoLevelRead
	"A one-many and a many-many"
	| accounts txMapping otherHolder holder accountMapping |
	(GlorpReadingTest new session: session) write2CustomerRowsWith3Transactions.
	txMapping := (system descriptorFor: GlorpCustomer) mappingForAttributeNamed: #transactions.
	txMapping useFilteredRead.
	accountMapping := (system descriptorFor: GlorpBankAccount) mappingForAttributeNamed: #accountHolders.
	accountMapping useFilteredRead.
	accounts := session execute: ((Query returningManyOf: GlorpBankAccount where: [:each | each id ~= 2]) orderBy: [:each | each id]).
	"Account 6, 2 holders, 1 with 1 transaction, 1 shared with 9874" 
	self assert: accounts first accountHolders size = 2.
	holder := accounts first accountHolders detect: [:each | each id = 28].
	holder transactions yourSelf.
	session accessor permitNothing.
	self assert: (accounts last accountHolders collect: [:each | each id]) asArray = #(27).
	otherHolder := accounts first accountHolders detect: [:each | each id = 27].
	self assert:  (holder transactions collect: [:each | each amount currency]) asSortedCollection asArray = #(#EUR).
	session accessor permitNothing.
	self assert: (otherHolder transactions collect: [:each | each amount currency]) asSortedCollection asArray = #(#CDN #USD).! !

GlorpSessionBasedTest subclass: #GlorpGettingPrimaryKeyFromMappingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpGettingPrimaryKeyFromMappingTest methodsFor: 'tests' stamp: ''!
createData
	"Create seed data"
	| doc attb att |
	doc := GlorpDocument new initialize.
	attb := GlorpAttachmentBytes new content: 'Hello, Alan'.
	att := (GlorpAttachment new)
				index: 1;
				mimeType: 'text/plain';
				attachmentBytes: attb.
	session modify: doc in: [doc attachments add: att].! !

!GlorpGettingPrimaryKeyFromMappingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	system := GlorpDocumentDescriptorSystem forPlatform: system platform.
	session system: system.
	session beginTransaction.! !

!GlorpGettingPrimaryKeyFromMappingTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.! !

!GlorpGettingPrimaryKeyFromMappingTest methodsFor: 'tests' stamp: ''!
testUnchangedInUnitOfWork
 
	| doc |
	self createData.
	session reset.
	doc := session readOneOf: GlorpDocument.
	"Something changed? Should not."
	session modify: doc in: [].
	doc := nil.
	session reset.

	doc := session readOneOf: GlorpDocument.
	self assert: doc attachments size = 1.
	doc attachments do: 
			[:each | 
			self assert: each attachmentBytes content yourSelf notNil].
	"The bug would cause this to write out the attachment over again, generating a primary key constraint error. Just in case that doesn't catch it, do a count"
	session accessor permitNothingDuring: [
		session modify: doc in: []].
	self assert: (session count: GlorpDocument) = 1.
	self assert: (session count: GlorpAttachment) = 1.
	self assert: (session count: GlorpAttachmentBytes) = 1.
	self assert: doc id notNil.
	((session accessor executeSQLString: 'SELECT * FROM ATTACHMENT') first do: [:each | self assert: each notNil]).
	((session accessor executeSQLString: 'SELECT * FROM ATTACHMENTBYTES') first do: [:each | self assert: each notNil])! !

GlorpSessionBasedTest subclass: #GlorpObjectComparisonJoinsTest
	instanceVariableNames: 'itinerary1 itinerary2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpObjectComparisonJoinsTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session system useJoinsForAnySatisfy: true.
	session beginTransaction.
	itinerary1 := GlorpItinerary example1.
	itinerary2 := GlorpItinerary example2.
	itinerary2 reservation passenger: nil.
	itinerary2 reservation passengers: #().

	session transact: [
		session register: itinerary1.
		session register: itinerary2].! !

!GlorpObjectComparisonJoinsTest methodsFor: 'support' stamp: ''!
tearDown
	
	session rollbackTransaction.
	itinerary1 := nil.
	itinerary2 := nil.
	super tearDown.! !

!GlorpObjectComparisonJoinsTest methodsFor: 'tests' stamp: ''!
testBasicEqualNil

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passenger isNIL].
	self assert: reservations size = 1.
	self assert: reservations first id = 7.! !

!GlorpObjectComparisonJoinsTest methodsFor: 'tests' stamp: ''!
testBasicNotNil

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passenger notNIL].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.! !

!GlorpObjectComparisonJoinsTest methodsFor: 'tests' stamp: ''!
testEqualNil

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger isNIL]].
	self assert: reservations size = 1.
	self assert: reservations first id = 7.! !

!GlorpObjectComparisonJoinsTest methodsFor: 'tests' stamp: ''!
testEqualToIdsInExpression
	"Just to verify that this works when not directly comparing objects"

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger id = each passenger id]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.! !

!GlorpObjectComparisonJoinsTest methodsFor: 'tests' stamp: ''!
testEqualToObjectInExpression
	"A somewhat contrived example of comparing two relationships that should end up with the same object, because they're actually mapped the same way. But will fail if the passenger is nil"

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger = each passenger]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.! !

!GlorpObjectComparisonJoinsTest methodsFor: 'tests' stamp: ''!
testEqualToObjectInMemory
	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger = itinerary1 reservation passenger]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.! !

!GlorpObjectComparisonJoinsTest methodsFor: 'tests' stamp: ''!
testNotEqualToObjectInExpression
	"A somewhat contrived example of comparing two relationships that should end up with the same object, because they're actually mapped the same way. But will fail if the passenger is nil, because nil ~= nil"

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger ~= each passenger]].
	self assert: reservations isEmpty.! !

!GlorpObjectComparisonJoinsTest methodsFor: 'tests' stamp: ''!
testNotEqualToObjectInMemory
	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger ~= (GlorpPassenger new id: 7)]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.! !

!GlorpObjectComparisonJoinsTest methodsFor: 'tests' stamp: ''!
testNotNil

	| reservations |
	reservations := session readManyOf: GlorpReservation where: [:each |
		each passengers anySatisfy: [:eachPassenger | eachPassenger notNIL]].
	self assert: reservations size = 1.
	self assert: reservations first id = 2.
	self assert: reservations first passenger id = 3.! !

GlorpObjectComparisonJoinsTest subclass: #GlorpObjectComparisonSubselectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpObjectComparisonSubselectTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session system useJoinsForAnySatisfy: false.! !

GlorpSessionBasedTest subclass: #GlorpObjectMappedToFunctionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpObjectMappedToFunctionTest methodsFor: 'support' stamp: ''!
setUp
	| accountRow3 |
	super setUp.
	session beginTransaction.
	system writeCustomer1Rows.
	accountRow3 := system exampleAccountRow3.
	session writeRow: accountRow3.! !

!GlorpObjectMappedToFunctionTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.! !

!GlorpObjectMappedToFunctionTest methodsFor: 'tests' stamp: ''!
testBasicRead
	| branches |
	branches := session readManyOf: GlorpBankBranch.
	self assert: branches size = 3.
	self assert: (branches collect: [:each | each branchNumber]) asSortedCollection = #(2 3 712) asSortedCollection.! !

!GlorpObjectMappedToFunctionTest methodsFor: 'tests' stamp: ''!
testCache
	| branches branch2 branch3 |
	branches := session readManyOf: GlorpBankBranch.
	branch2 := session readOneOf: GlorpBankBranch where: [:each | each branchNumber = 2].
	branch3 := session readOneOf: GlorpBankBranch where: [:each | each branchNumber = 3].
	self deny: branch2 == branch3.
	self assert: (branches includes: branch2).
	self assert: (branches includes: branch3).! !

!GlorpObjectMappedToFunctionTest methodsFor: 'tests' stamp: ''!
testNoWriteValidation
	| |
	(session system descriptorFor: GlorpBankBranch) mappings next readOnly: false.
	self should: [session system validate] raise: Dialect error.! !

!GlorpObjectMappedToFunctionTest methodsFor: 'tests' stamp: ''!
testReadQueryingAttributes
	| branches |
	branches := session readManyOf: GlorpBankBranch where: [:each |
		each accounts anySatisfy: [:eachAccount |
			eachAccount accountNumber accountNumber in: #( '3' '4')]].
	self assert: branches size = 2.
	self assert: branches asSet size = 2.! !

!GlorpObjectMappedToFunctionTest methodsFor: 'support' stamp: ''!
testRelationships
	| branch accounts |
	branch := session readOneOf: GlorpBankBranch where: [:each | each branchNumber = 2].
	accounts := branch accounts yourself.
	self assert: accounts size = 1.
	self assert: accounts first id = 9874.! !

GlorpSessionBasedTest subclass: #GlorpObjectMappedToImaginaryTableTest
	instanceVariableNames: 'example example2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: ''!
queryForComplexSubselectRead2: anInteger
	"This exercises doubly-nested subselects, which caused a problem in a user application with joining to the wrong table. The gist of the query is stores where more than anInteger different customers have made rentals"

	| q1 q2 q3 |
	q1 := Query
		readManyOf: GlorpVideoStore
		where: [:store | 
			q2 := Query
				count: GlorpVideoCustomer
				where: [:customer | 
					q3 := Query
						readManyOf: GlorpVideoRental
						where: [:rental | rental customer = customer AND: (rental store = store)].
					q3 retrieve: [:trade | trade customerName].
					customer name in: q3].
			(store getConstant: anInteger) <= q2].
	^q1.! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: ''!
queryForComplexSubselectRead: anInteger
	"This exercises doubly-nested subselects, which caused a problem in a user application with joining to the wrong table. The gist of the query is stores where more than anInteger different customers have made rentals"

	| q1 q2 q3 |
	q1 := Query
		readManyOf: GlorpVideoStore
		where: [:store | 
			q2 := Query
				count: GlorpVideoCustomer
				where: [:customer | 
					q3 := Query
						readManyOf: GlorpVideoRental
						where: [:rental | rental store = store].
					q3 retrieve: [:trade | trade customerName].
					customer name in: q3].
			(store getConstant: anInteger) <= q2].
	^q1.! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	system := GlorpVideoDescriptorSystem forPlatform: session platform.
	session system: system.
	session beginTransaction.
	example := system exampleStore1.
	example2 := system exampleStore2.
	session inUnitOfWorkDo: [session register: example. session register: example2.].
	session reset.! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: ''!
testComplexSubselectRead
	"This exercises doubly-nested subselects, which caused a problem in a user application with joining to the wrong table"

	| |
	self assert: (session execute: (self queryForComplexSubselectRead: 3)) size = 1.
	self assert: (session execute: (self queryForComplexSubselectRead: 4)) size = 0.! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: ''!
testComplexSubselectRead2
	"This exercises doubly-nested subselects, which caused a problem in a user application with joining to the wrong table"

	| |
	self assert: (session execute: (self queryForComplexSubselectRead2: 1)) size = 1.
	self assert: (session execute: (self queryForComplexSubselectRead2: 0)) size = 2.! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: ''!
testReadBackAllCustomer
	| customers |
	customers := session readManyOf: GlorpVideoCustomer.
	self assert: ((customers collect: [:each | each name]) asSortedCollection asArray) = #('Adrian' 'Bob' 'Carol' 'David' 'Eleanor' 'Francois' 'George' 'Hieronymous').! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: ''!
testReadBackEntireExample
	| store |
	store := session readOneOf: GlorpVideoStore where: [:each | each id = 1].
	example assertEqual: store.! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: ''!
testReadCustomerWhere
	| customers |
	customers := session readManyOf: GlorpVideoCustomer where: [:each | each name like: '%an%'].
	self assert: ((customers collect: [:each | each name]) asSortedCollection asArray) = #('Adrian'  'Eleanor' 'Francois').! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: ''!
testReadCustomerWhereWithJoin
	| customers |
	customers := session readManyOf: GlorpVideoCustomer where: [:each | each onlinePurchases anySatisfy: [:eachPurchase | eachPurchase price > 2]].
	self assert: ((customers collect: [:each | each name]) asSortedCollection asArray) = #('Adrian'  'George' ).! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: ''!
testSelectByBoolean

	| inStore notInStore inStoreImplied notInStoreImplied |
	inStore := session read: GlorpVideoPurchase where: [:each | each inStore = true].
	notInStore := session read: GlorpVideoPurchase where: [:each | each inStore = false].
	self assert: inStore size = 7.
	self assert: notInStore size = 3.
	inStoreImplied := session read: GlorpVideoPurchase where: [:each | each inStore].
	notInStoreImplied := session read: GlorpVideoPurchase where: [:each | each inStore not].
	self assert: inStoreImplied size = 7.
	self assert: notInStoreImplied size = 3.! !

!GlorpObjectMappedToImaginaryTableTest methodsFor: 'tests' stamp: 'GuillermoPolito 4/26/2011 04:57'!
testWrite
	"Write and validate that it basically got written correctly, in terms of number and keys of db rows"

	| storeRows storeRow purchaseRows purchaseKeys rentalRows rentalKeys |
	storeRows := session accessor
		executeSQLString: 'SELECT * FROM VIDEO_STORE'.
	storeRows := storeRows asSortedCollection: [:a :b | a first <= b first].
	self assert: storeRows size = 2.
	storeRow := storeRows first.
	self assert: storeRow size = 2.
	self assert: storeRow first = 1.
	self assert: storeRow last = 'Als Video'.
	storeRow := storeRows last.
	self assert: storeRow first = 2.
	self assert: storeRow last = 'Bonzai Video'.
	purchaseRows := session accessor
		executeSQLString: 'SELECT * FROM VIDEO_PURCHASE'.
	self assert: purchaseRows size = 10.
	purchaseKeys := (purchaseRows collect: [:each | each at: 1])
		asSortedCollection.
	self assert: purchaseKeys asArray = #(1 2 3 101 102 103 104 105 106 107).
	rentalRows := session accessor
		executeSQLString: 'SELECT * FROM VIDEO_RENTAL'.
	self assert: rentalRows size = 4.
	rentalKeys := (rentalRows collect: [:each | each at: 1])
		asSortedCollection.
	self assert: rentalKeys asArray = #(1001 1002 1003 1004).! !

GlorpSessionBasedTest subclass: #GlorpOptimisticLockingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpOptimisticLockingTest methodsFor: 'tests' stamp: ''!
changeABookFrom: books
	^session
		transact:
			[session register: books.
			books first copiesInStock: books first copiesInStock - 1].! !

!GlorpOptimisticLockingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session system: GlorpLockingDescriptorSystem new.
	session beginTransaction.
	session transact: [session register: session system example1].! !

!GlorpOptimisticLockingTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.! !

!GlorpOptimisticLockingTest methodsFor: 'tests' stamp: ''!
testSuccessfulUpdate
	| books versions newVersions |
	books := session readManyOf: GlorpBook orderBy: #id.
	versions := (session accessor executeSQLString: 'SELECT VERSION FROM BOOK ORDER BY ID') collect: [:each | each first].
	session modify: books in: [
		books do: [:each | each copiesInStock: (each copiesInStock * 2)]].
	newVersions := (session accessor executeSQLString: 'SELECT VERSION FROM BOOK ORDER BY ID') collect: [:each | each first].
	self assert: newVersions = (versions collect: [:each | each + 1]).! !

!GlorpOptimisticLockingTest methodsFor: 'tests' stamp: ''!
testSuccessfulUpdateOntoNullVersionField
	| books copies someBook |
	someBook := session readOneOf: GlorpBook.
	session accessor executeSQLString: 'UPDATE BOOK SET VERSION = NULL WHERE ID = ', someBook id printString. 
	session reset.
	books := session read: GlorpBook orderBy: #id.
		copies := books collect: [:each | each copiesInStock].
	session transact: [
		session register: books.
		books do: [:each | each copiesInStock: (each copiesInStock - 1)]].
	session reset.
	books := session readManyOf: GlorpBook orderBy: #id.
	self assert: (books collect: [:each | each copiesInStock]) = (copies collect: [:each | each - 1]).! !

!GlorpOptimisticLockingTest methodsFor: 'tests' stamp: ''!
testSuccessfulUpdateTwiceWithRefresh
	| book |
	book := session readOneOf: GlorpBook where: [:each | each title like: 'Mastering%'].
	session modify: book in: [
		book copiesInStock: 4].
	session refresh: book.
	self assert: book copiesInStock = 4.
	"Make sure we've properly updated the optimistic lock field"
	session modify: book in: [
		book copiesInStock: 7].
	session reset.
	book := session readOneOf: GlorpBook where: [:each | each title like: 'Mastering%'].
	self assert: book copiesInStock = 7.! !

!GlorpOptimisticLockingTest methodsFor: 'tests' stamp: ''!
testSuccessfulUpdateTwiceWithRefreshAndSaveAndContinue
	| book |
	session beginUnitOfWork.
	book := session readOneOf: GlorpBook where: [:each | each title like: 'Mastering%'].
	book copiesInStock: 4.
	session saveAndContinue.
	session refresh: book.
	self assert: book copiesInStock = 4.
	"Make sure we've properly updated the optimistic lock field"
	book copiesInStock: 7.
	session commitUnitOfWork.
	session reset.
	book := session readOneOf: GlorpBook where: [:each | each title like: 'Mastering%'].
	self assert: book copiesInStock = 7.! !

!GlorpOptimisticLockingTest methodsFor: 'tests' stamp: ''!
testUnsuccessfulUpdateProceedingAndFailing
	| books |
	books := session readManyOf: GlorpBook.
	session accessor
		executeSQLStringNoResult: 'UPDATE BOOK SET COPIES_IN_STOCK = 7, VERSION = 3'.
	self should: [[self changeABookFrom: books] on: GlorpWriteFailure do: [:ex | ex resume: false]] raise: GlorpTransactionFailure.
		
	session reset.
	books := session readManyOf: GlorpBook.
	self assert: (books collect: [:each | each copiesInStock]) = #( 7 7).! !

!GlorpOptimisticLockingTest methodsFor: 'tests' stamp: ''!
testUnsuccessfulUpdateProceedingAnyway
	| books |
	books := session readManyOf: GlorpBook.
	session accessor
		executeSQLStringNoResult: 'UPDATE BOOK SET COPIES_IN_STOCK = 7, VERSION = 3'.
	[self changeABookFrom: books] on: GlorpWriteFailure do: [:ex | ex resume: true].
		
	session reset.
	books := session readManyOf: GlorpBook.
	self assert: (books collect: [:each | each copiesInStock]) = #( 7 7).! !

!GlorpOptimisticLockingTest methodsFor: 'tests' stamp: ''!
testUnsuccessfulUpdateWithNoAction
	"This is a little quirky. SUnit's action in should:raise: is to exit the exception with true. So the thing just short circuits, and we don't get a transaction failure because we killed the whole unit of work. This is, in general, what will happen if we handle and don't resume the write failures."
	| books |
	books := session readManyOf: GlorpBook.
	session accessor
		executeSQLStringNoResult: 'UPDATE BOOK SET COPIES_IN_STOCK = 7, VERSION = 3'.
	self shouldnt: [
		self should: [self changeABookFrom: books] raise: GlorpWriteFailure]
		raise: GlorpTransactionFailure.
	session reset.
	books := session readManyOf: GlorpBook.
	self assert: (books collect: [:each | each copiesInStock]) = #(7 7).! !

GlorpSessionBasedTest subclass: #GlorpQueryUnionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpQueryUnionTest commentStamp: '<historical>' prior: 0!
This tests the ability to do a union operation on queries, returning results corresponding to multiple different queries.!

!GlorpQueryUnionTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session beginTransaction.
	self writeAddressOrderingRows.! !

!GlorpQueryUnionTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testBasicExcept
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].	
	union := query1 except: query2.
	result := session execute: union.
	self assert: result size = 1.! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testBasicExcept2
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id in: #(1 2)].	
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	union := query1 except: query2.
	result := session execute: union.
	self assert: result size = 1.
	self assert: result first id = 2.! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testBasicExcept3
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each street = 'Alpha'].	
	union := query1 except: query2.
	result := session execute: union.
	self assert: result size = 0.! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testBasicIntersect
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].	
	union := query1 intersect: query2.
	result := session execute: union.
	self assert: result size = 0.! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testBasicIntersect2
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id in: #(1 2)].	
	union := query1 intersect: query2.
	result := session execute: union.
	self assert: result size = 1.! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testBasicIntersect3
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each street = 'Alpha'].	
	union := query1 intersect: query2.
	result := session execute: union.
	self assert: result size = 1.! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testBasicUnion
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].	
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result size = 2.
	self assert: result first id = 1.
	self assert: result last id = 2.
	self assert: (result allSatisfy: [:each | each class == GlorpAddress]).! !

!GlorpQueryUnionTest methodsFor: 'tests-unit' stamp: ''!
testCompoundCommandCreate
	| query1 union command |
	query1 := Query returningManyOf: GlorpAddress.
	union := query1 unionAll: query1.
	union session: session.
	command := union sqlWith: #().
	self assert: command class == CompoundSelectCommand.! !

!GlorpQueryUnionTest methodsFor: 'tests-unit' stamp: ''!
testCreateCompoundQuery
	| query1 union |
	query1 := Query returningManyOf: GlorpAddress.
	union := query1 unionAll: query1.
	self assert: union class == CompoundQuery.! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testOrderByInUnion
	| query1 query2 union result |
	self writePeople.
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id > 1].
	query1 retrieve: #id.
	query2 := Query returningManyOf: GlorpPerson.
	query2 retrieve: [:each | each id].
	union := query1 unionAll: query2.
	union orderBy: [:each | each id descending].
	result := session execute: union.
	self assert: result = #(87 86 3 2).! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testThreeWayUnion
	| query1 query2 union result query3 |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].	
	query3 := Query returningManyOf: GlorpAddress where: [:each | each id = 3].	

	union := (query1 unionAll: query2) unionAll: query3.
	union orderBy: [:each | each id].
	result := session execute: union.
	self assert: result size = 3.
	self assert: result first id = 1.
	self assert: result last id = 3.
	self assert: (result allSatisfy: [:each | each class == GlorpAddress]).! !

!GlorpQueryUnionTest methodsFor: 'tests-unit' stamp: ''!
testUnionSQL
	"Use expandMacros to avoid platform-specific line-end conventions blocking string matching."

	| query1 union command |
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 1].
	union := query1 unionAll: query1.
	union session: session.
	command := union sqlWith: #().
	self assert: ('SELECT * FROM (*SELECT *<n> FROM GR_ADDRESS t1<n> WHERE *t1.ID = * UNION ALL *SELECT *<n> FROM GR_ADDRESS t1<n> WHERE *t1.ID = *' expandMacros match: command sqlString).! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testUnionWithEmpty
	| query1 query2 union result |
	query1 := Query returningManyOf: GlorpAddress where: false.
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].	
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result size = 1.
	self assert: result first id = 2.
	self assert: (result allSatisfy: [:each | each class == GlorpAddress]).! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testUnionWithParameters
	| query1 query2 union result field1 dict |
	field1 := (system tableNamed: 'GR_ADDRESS') fieldNamed: 'ID'.
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = (each parameter: field1)].
	query2 := Query returningManyOf: GlorpAddress where: [:each | each id = ((each parameter: field1) + 1)].	
	union := query1 unionAll: query2.
	union orderBy: [:each | each id].
	dict := Dictionary new at: field1 put: 2; yourself.
	result := union executeWithParameters: dict in: session.
	self assert: result size = 2.
	self assert: result first id = 2.
	self assert: result last id = 3.
	self assert: (result allSatisfy: [:each | each class == GlorpAddress]).! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testUnionWithRetrieveDisaparateResults
	| query1 query2 union result |
	self writePeople.
	query1 := Query returningManyOf: GlorpAddress.
	query1 retrieve: #id.
	query2 := Query returningManyOf: GlorpPerson.
	query2 retrieve: #id.
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result asSortedCollection asArray = #(1 2 3 86 87).! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testUnionWithRetrieveThoroughlyDisaparateResults
	"Fields from mismatched tables"
	| query1 query2 union result |
	self writePeople.
	query1 := Query returningManyOf: GlorpAddress.
	query1 retrieve: [:each | each street].
	query2 := Query returningManyOf: GlorpPerson.
	query2 retrieve: [:each | each name].
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result asSortedCollection asArray = #('Alpha' 'Beta' 'Gamma' 'person1' 'person2').! !

!GlorpQueryUnionTest methodsFor: 'tests-functional' stamp: ''!
testUnionWithRetrieveThoroughlyDisaparateResults2
	"The tables will get aliased differently, make sure that works"

	| query1 query2 union result |
	self writePeople.
	query1 := Query returningManyOf: GlorpAddress where: [:each | each id = 2].
	query1 retrieve: [:each | each street].
	query2 := Query returningManyOf: GlorpPerson where: [:each | each id = 87].
	query2 retrieve: [:each | each address street].
	union := query1 unionAll: query2.
	result := session execute: union.
	self assert: result asSortedCollection asArray = #('Beta' 'Gamma').! !

!GlorpQueryUnionTest methodsFor: 'support' stamp: ''!
writeAddressOrderingRows

	session writeRow: session system exampleAddressRowForOrdering1.
	session writeRow: session system exampleAddressRowForOrdering2.
	session writeRow: session system exampleAddressRowForOrdering3.! !

!GlorpQueryUnionTest methodsFor: 'support' stamp: ''!
writePeople

	session writeRow: session system examplePersonRowForOrdering1.
	session writeRow: session system examplePersonRowForOrdering2.! !

GlorpSessionBasedTest subclass: #GlorpSelectCommandTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpSelectCommandTest methodsFor: 'tests' stamp: ''!
testExecute
	| command dict result string |
	dict := #('abc' 'Smalltalk').
	string := session useBinding
		ifTrue: ['SELECT ID FROM STUFF WHERE THING=? OR THING = ?']
		ifFalse:
			['SELECT ID FROM STUFF WHERE THING=''abc'' OR THING = ''Smalltalk'''].
	command := SQLStringSelectCommand
		sqlString: string
		parameters: dict
		useBinding: session useBinding
		session: session.
	result := session accessor executeCommand: command.
	self assert: result size = 2.
	self assert: result first first = 12.
	self assert: result last first = 113141.! !

!GlorpSessionBasedTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpDatabaseLoginResource with: GlorpSessionResource with: GlorpDemoTablePopulatorResource.! !

!GlorpSessionBasedTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	system session: session.
	session system: system.! !

!GlorpSessionBasedTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session isNil ifFalse: [session reset. session resetPermissions].
	session := nil.! !

GlorpSessionBasedTest subclass: #GlorpTestUpdatingTimestampInRecord
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpTestUpdatingTimestampInRecord methodsFor: 'tests' stamp: 'GuillermoPolito 5/2/2011 09:38'!
testUpdating

	| record firstTime secondTime |
	session beginTransaction.
	[
	"First verify that when we insert, a timestamp is set"
	record := GlorpRecordWithUpdateTime new id: 12; name: 'thing'.
	session modify: record in: [ ].
	session reset.
	record := session readOneOf: GlorpRecordWithUpdateTime.
	self assert: record id = 12.
	firstTime := record updateTime asSeconds.
	self assert: (firstTime - Dialect timestampClass now asSeconds) < 2.
	"Now wait long enough for the timestamp to definitely change, and verify that if modify the object it changes"
	(Delay forSeconds: 2) wait.
	session modify: record in: [record name: 'different'].
	secondTime := record updateTime asSeconds.
	self deny: secondTime = firstTime.
	"Now verify that if we don't modify the object, we don't write the timestamp, or put it into the object"
	(Delay forSeconds: 2) wait.
	session modify: record in: [].
	self assert: record updateTime asSeconds = secondTime.
	session reset.
	record := session readOneOf: GlorpRecordWithUpdateTime.
	self assert: record updateTime asSeconds = secondTime.]
		ensure: [session rollbackTransaction].! !

!GlorpTestUpdatingTimestampInRecord methodsFor: 'tests' stamp: ''!
testUpdatingComment
	"Test that we update the unmapped comment field. We just put the printString of time now into the field, so most of the same sort of logic applies as in testUpdating."

	| record time time2 rows |
	session beginTransaction.
	[
	"First verify that when we insert, a comment is set"
	record := GlorpRecordWithUpdateTime new id: 12; name: 'thing'.
	time := Time now printString.
	session modify: record in: [ ].
	time2 := Time now printString.
	session reset.
	rows := session accessor executeSQLString: 'SELECT UPDATE_COMMENT from RECORD_WITH_UPDATE'.
	"Just in case the seconds flipped in that interval between us noting the time and the value being generated, check against before and after. I really can't see there being more than a one second gap in there"
	self assert: (rows first first = time or: [rows first first = time2]).

	"Now wait long enough for the time to definitely change, and verify that if modify the object it changes"
	(Delay forSeconds: 2) wait.
	record := session readOneOf: GlorpRecordWithUpdateTime.
	time := Time now printString.
	session modify: record in: [record name: 'different'].
	time2 := Time now printString.
	rows := session accessor executeSQLString: 'SELECT UPDATE_COMMENT from RECORD_WITH_UPDATE'.
	"Just in case the seconds flipped in that interval between us noting the time and the value being generated, check against before and after. I really can't see there being more than a one second gap in there"
	self assert: (rows first first = time or: [rows first first = time2]).

	"Now verify that if we don't modify the object, we don't write the comment, or put it into the object"
	(Delay forSeconds: 2) wait.
	session modify: record in: [].
	rows := session accessor executeSQLString: 'SELECT UPDATE_COMMENT from RECORD_WITH_UPDATE'.
	"Now verify that we still have the previous time value"
	self assert: (rows first first = time or: [rows first first = time2]).
]
		ensure: [session rollbackTransaction].! !

!GlorpTestUpdatingTimestampInRecord methodsFor: 'tests' stamp: ''!
testUpdatingOtherThing
	"Test that we update the mapped other thing field. We just put the printString of time now into the field, so most of the same sort of logic applies as in testUpdating."

	| record time time2 newThing |
	session beginTransaction.
	[
	"First verify that when we insert, a comment is set"
	record := GlorpRecordWithUpdateTime new id: 12; name: 'thing'.
	time := Time now printString.
	session modify: record in: [ ].
	time2 := Time now printString.
	session reset.
	newThing := session readOneOf: GlorpRecordWithUpdateTime.
	"Just in case the seconds flipped in that interval between us noting the time and the value being generated, check against before and after. I really can't see there being more than a one second gap in there"
	self assert: (newThing updateSomeOtherThing = time or: [newThing updateSomeOtherThing = time2]).

	"Now wait long enough for the time to definitely change, and verify that if modify the object it changes"
	(Delay forSeconds: 2) wait.
	session reset.
	record := session readOneOf: GlorpRecordWithUpdateTime.
	time := Time now printString.
	session modify: record in: [record name: 'different'].
	time2 := Time now printString.
	session reset.
	newThing := session readOneOf: GlorpRecordWithUpdateTime.
	self assert: (newThing updateSomeOtherThing = time or: [newThing updateSomeOtherThing = time2]).

	record := session readOneOf: GlorpRecordWithUpdateTime.
	"Now verify that if we don't modify the object, we don't write the comment, or put it into the object"
	(Delay forSeconds: 2) wait.
	session modify: record in: [].
	session reset.
	newThing := session readOneOf: GlorpRecordWithUpdateTime.
	self assert: (newThing updateSomeOtherThing = time or: [newThing updateSomeOtherThing = time2]).

]
		ensure: [session rollbackTransaction].! !

GlorpSessionBasedTest subclass: #GlorpVirtualCollectionBasicTest
	instanceVariableNames: 'vc'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpVirtualCollectionBasicTest methodsFor: 'support' stamp: ''!
realObjects
	"Get the real objects from a virtual collection without resorting to any of its mechanisms except do:, so we can validate against more complex things."
	^self realObjectsFrom: vc.! !

!GlorpVirtualCollectionBasicTest methodsFor: 'support' stamp: ''!
realObjectsFrom: aVirtualCollection
	"Get the real objects from a virtual collection without resorting to any of its mechanisms except do:, so we can validate against more complex things."
	| all |
	all := OrderedCollection new.
	aVirtualCollection do: [:each | all add: each].
	^all.! !

!GlorpVirtualCollectionBasicTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session beginTransaction.
	self writePersonRows.
	vc := session virtualCollectionOf: GlorpPerson.! !

!GlorpVirtualCollectionBasicTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.! !

!GlorpVirtualCollectionBasicTest methodsFor: 'tests' stamp: ''!
testCollect
	| ids |
	ids := vc collect: [:each | each id].
	
	self assert: ids size = 3.
	ids do: [:each | self assert: each isInteger].! !

!GlorpVirtualCollectionBasicTest methodsFor: 'tests' stamp: ''!
testCreation
	
	self assert: vc notNil.
	self should: [vc isKindOf: GlorpVirtualCollection].! !

!GlorpVirtualCollectionBasicTest methodsFor: 'tests' stamp: ''!
testDo

	| all |
	all := OrderedCollection new.
	vc do: [:each |
		self assert: (each isKindOf: GlorpPerson).
		all add: each.
		].
	self assert: all size = 3.
	self assert: all asSet size = 3.
	self assert: (all collect: [:each | each id]) asSortedCollection asArray = #(86 87 88).! !

!GlorpVirtualCollectionBasicTest methodsFor: 'tests' stamp: ''!
testInject
	| sumofIds |
	sumofIds := vc inject: 0 into: [:sum :each | sum + each id].
	self assert: sumofIds = (86 + 87 + 88).! !

!GlorpVirtualCollectionBasicTest methodsFor: 'tests' stamp: ''!
testIsEmpty
	| vc2 |
	self deny: vc isEmpty.
	vc2 := vc select: [:each | each id = 98].
	self assert: vc2 isEmpty.! !

!GlorpVirtualCollectionBasicTest methodsFor: 'tests' stamp: ''!
testReject
	| vc2 |
	vc2 := vc reject: [:each | each id > 87 ].
	self deny: vc isInstantiated.
	self deny: vc2 isInstantiated.
	self assert: vc2 size = 2.
	self deny: vc isInstantiated.
	self assert: vc size = 3.
	self assert: (self realObjectsFrom: vc2) size = 2.! !

!GlorpVirtualCollectionBasicTest methodsFor: 'tests' stamp: ''!
testSelect
	| vc2 |
	vc2 := vc select: [:each | each id <= 87 ].
	self deny: vc isInstantiated.
	self deny: vc2 isInstantiated.
	self assert: vc2 size = 2.
	self deny: vc isInstantiated.
	self assert: vc size = 3.
	self assert: (self realObjectsFrom: vc2) size = 2.! !

!GlorpVirtualCollectionBasicTest methodsFor: 'support' stamp: ''!
writePersonRows
	session writeRow: session system exampleAddressRowForOrdering1.
	session writeRow: session system exampleAddressRowForOrdering2.
	session writeRow: session system exampleAddressRowForOrdering3.
	session writeRow: session system examplePersonRowForOrdering1.
	session writeRow: session system examplePersonRowForOrdering2.
	session writeRow: session system examplePersonRowForOrdering3.! !

GlorpSessionBasedTest subclass: #TestCommandPermission
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testAlsoDeny
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyCommand: InsertCommand.
	session accessor alsoDenyCommand: DeleteCommand.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					session delete: person]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testDenyReads
	| person address |
	[session beginTransaction.
	self writeTestRows.
	person := session readOneOf: GlorpPerson where: nil.
	address := person address yourSelf.
	session accessor denyReads.
	self should: [session readOneOf: GlorpPerson where: nil] raise: GlorpIllegalCommand.
	session inUnitOfWorkDo: [
		session register: person.
		session delete: address.
		person address: (GlorpAddress new id: 7).
		person name: 'Quigley Quagmire']]
	ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testForbiddenDelete
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyCommand: DeleteCommand.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					session delete: person address.
					person address: nil]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testForbiddenDeleteAllowsUpdateAndInsert
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyCommand: DeleteCommand.
	person := session readOneOf: GlorpPerson where: nil.
	session inUnitOfWorkDo: [
		session register: person.
		person address: (GlorpAddress new id: 7).
		person name: 'Quigley Quagmire']]
	ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testInsertOnlyAllowsInsert
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor insertOnly.
	person := session readOneOf: GlorpPerson where: nil.
	session
		inUnitOfWorkDo:
			[session register: person.
			session register: (GlorpPerson new id: 4)]]
		ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testInsertOnlyForbidsDelete
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor insertOnly.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					session delete: person]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testInsertOnlyForbidsUpdate
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyWrites.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					person name: 'Quigley Quagmire']]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testPermitEverything
	session accessor permitEverything.
	[session beginTransaction.
	self writeTestRows.
	session inUnitOfWorkDo: [ | person |
		person := session readOneOf: GlorpPerson where: nil.
		session delete: person address.
		person address: (GlorpAddress new id: 7).
		person name: 'Quigley Quagmire']]
	ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testPermitNothing
	session accessor permitNothing.
	self should: [session readManyOf: GlorpPerson] raise: GlorpIllegalCommand.
	self
		should:
			[session accessor executeCommand: InsertCommand new]
		raise: GlorpIllegalCommand.
	self
		should:
			[session accessor executeCommand: UpdateCommand new]
		raise: GlorpIllegalCommand.
	self
		should:
			[session accessor executeCommand: DeleteCommand new]
		raise: GlorpIllegalCommand.
	self
		should:
			[session accessor
				executeCommand: SQLStringSelectCommand new]
		raise: GlorpIllegalCommand.! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testReadOnlyForbidsDelete
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyWrites.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					session delete: person]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testReadOnlyForbidsInsert
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyWrites.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					person address: (GlorpAddress new id: 7)]]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'tests' stamp: ''!
testReadOnlyForbidsUpdate
	| person |
	[session beginTransaction.
	self writeTestRows.
	session accessor denyWrites.
	person := session readOneOf: GlorpPerson where: nil.
	self
		should:
			[session
				inUnitOfWorkDo:
					[session register: person.
					person name: 'Quigley Quagmire']]
		raise: GlorpIllegalCommand] ensure: [session rollbackTransaction].! !

!TestCommandPermission methodsFor: 'support' stamp: ''!
writeTestRows
	| addressRow personRow |
	addressRow := session system exampleAddressRow.
	session writeRow: addressRow.
	personRow := session system examplePersonRow1.
	session writeRow: personRow.! !

GlorpDatabaseBasedTest subclass: #GlorpSessionTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpSessionTest methodsFor: 'support' stamp: ''!
addCustomerToCache
	| customer |
	customer := GlorpCustomer example1.
	customer id: 3.
	session cacheAt: 3 put: customer.
	^customer! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
helpTestUpdateWithExpiredExistingEntry: aCachePolicy
	| customer customer2 row table unitOfWork |
	(session system descriptorFor: GlorpCustomer) 
		cachePolicy: aCachePolicy.
	customer := self addCustomerToCache.
	customer2 := GlorpCustomer new.
	customer2 id: customer id.
	customer2 name: 'Barney Rubble'.
	row := DatabaseRow newForTable: (table := system tableNamed: 'GR_CUSTOMER').
	row at: (table fieldNamed: 'ID') put: customer id.
	unitOfWork := UnitOfWork new.
	unitOfWork session: session.
	"Since there's already an object there, this shouldn't do anything"
	unitOfWork updateSessionCacheFor: customer2 withRow: row.
	self assert: (session expiredInstanceOf: GlorpCustomer key: 3) == customer.! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
helpTestUpdateWithoutExpiredExistingEntry: aCachePolicy
	| customer2 row table unitOfWork |
	(session system descriptorFor: GlorpCustomer) 
		cachePolicy: (TimedExpiryCachePolicy new timeout: 0).
	customer2 := GlorpCustomer new.
	customer2 id: 3.
	customer2 name: 'Barney Rubble'.
	row := DatabaseRow newForTable: (table := system tableNamed: 'GR_CUSTOMER').
	row at: (table fieldNamed: 'ID') put: 3.
	unitOfWork := UnitOfWork new.
	unitOfWork session: session.
	unitOfWork updateSessionCacheFor: customer2 withRow: row.
	self assert: (session expiredInstanceOf: GlorpCustomer key: 3) == customer2.! !

!GlorpSessionTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSession new.
	session system: system! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testAddingDescriptors

	self assert: session system == system.
	self assert: system session == session.
	(session descriptorFor: GlorpAddress) session == session.! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testDuplicateDetection
	| customer customer2 |
	session beginUnitOfWork.
	customer := self addCustomerToCache.
	customer2 := customer copy.
	self should: [session register: customer2] raise: Dialect error.
	session rollbackUnitOfWork.! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testExecuteQuery
	| q result |
	q := GlorpQueryStub new result: 3.
	result := session execute: q.
	self assert: result = 3! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testHasExpired1
	| customer |
	customer := self addCustomerToCache.
	self deny: (session hasExpired: customer)! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testHasExpired2
	| customer |
	(session system descriptorFor: GlorpCustomer) 
		cachePolicy: (TimedExpiryCachePolicy new timeout: 0).
	customer := self addCustomerToCache.
	self assert: (session hasExpired: customer)! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testHasExpired3
	| customer |
	(session system descriptorFor: GlorpCustomer) 
		cachePolicy: (TimedExpiryCachePolicy new timeout: 0).
	customer := self addCustomerToCache.
	self assert: (session hasExpired: customer)! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testHasObjectOfClassExpired1

	self addCustomerToCache.
	self deny: (session hasObjectExpiredOfClass: GlorpCustomer withKey: 3).! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testHasObjectOfClassExpired2

	(session system descriptorFor: GlorpCustomer) 
		cachePolicy: (TimedExpiryCachePolicy new timeout: 0).
	self addCustomerToCache.
	self assert: (session hasObjectExpiredOfClass: GlorpCustomer withKey: 3).! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testSQLDeleteStringFor
	| row table string |
	table := session system tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'ID') put: 12.
	row at: (table fieldNamed: 'STREET') put: 'some street'.
	string := (DeleteCommand forRow: row useBinding: true session: session) sqlString.
	self assert: string = 'DELETE FROM GR_ADDRESS WHERE ID = ?'.
	string := (DeleteCommand forRow: row useBinding: false session: session) sqlString.
	self assert: string = 'DELETE FROM GR_ADDRESS WHERE ID = 12'.! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testUpdateWithExpiredExistingEntry
	self helpTestUpdateWithExpiredExistingEntry: (TimedExpiryCachePolicy new timeout: 0; expiryAction: #refresh).! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testUpdateWithExpiredExistingEntry2
	self helpTestUpdateWithExpiredExistingEntry: (TimedStrongCachePolicy new timeout: 0; expiryAction: #refresh).! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testUpdateWithoutExpiredExistingEntry
	self helpTestUpdateWithoutExpiredExistingEntry: (TimedExpiryCachePolicy new timeout: 0).! !

!GlorpSessionTest methodsFor: 'tests' stamp: ''!
testUpdateWithoutExpiredExistingEntry2
	self helpTestUpdateWithoutExpiredExistingEntry: (TimedStrongCachePolicy new timeout: 0).! !

GlorpDatabaseBasedTest subclass: #GlorpTableTest
	instanceVariableNames: 'descriptors dbPlatform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpTableTest methodsFor: 'support' stamp: ''!
keywordForConstraintsWhenDropping: aConstraint onPlatform: aDatabasePlatform
	"This method returns the term to test against independently of #writeConstraintToDrop:onStream:.  The logic is that ANSI uses 'DROP CONSTRAINT but MySQL uses
		'DROP PRIMARY KEY' for primary key constraints (and then says don't name the constraint name in the drop statement!!)
		'DROP FOREIGN KEY'  for foreign key constraints
		'DROP KEY' for unique constraints (it also supports DROP INDEX for this)"

	(aDatabasePlatform isKindOf: MySQLPlatform) ifFalse: [^'CONSTRAINT'].
	aConstraint isForeignKeyConstraint ifTrue: [^'FOREIGN KEY'].
	aConstraint isPrimaryKeyConstraint ifTrue: [^'PRIMARY KEY'].
	^'KEY'! !

!GlorpTableTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	descriptors := system allDescriptors.
	dbPlatform := system platform.
	GlorpInMemorySequenceDatabaseType reset! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testBasicSequencing
	| row |
	row := DatabaseRow newForTable: (system tableNamed: 'STUFF').
	row preWriteAssignSequencesUsing: nil.
	row postWriteAssignSequencesUsing: nil.
	self assert: (row at: ((system tableNamed: 'STUFF') fieldNamed: 'ID')) = 1.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testCircularFieldRefs

	| field table1 table2 |
	table1 := DatabaseTable named: 'BAR'.
	field := table1 createFieldNamed: 'FOO' type: dbPlatform int4.
	table2 := DatabaseTable named: 'BLETCH'.
	table1 addForeignKeyFrom: field to: (table2 createFieldNamed: 'FLIRP' type: dbPlatform int4).
	self assert: (table2 fieldNamed: 'FLIRP') = table1 foreignKeyConstraints first targetFields first.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testConstraintCreation
	| constraint dropWord |
	constraint := ForeignKeyConstraint
		sourceField: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'BANK_CODE')
		targetField: ((system tableNamed: 'PERSON') fieldNamed: 'ID').
	self assert: constraint creationString =
		( 'CONSTRAINT ', constraint name, ' FOREIGN KEY (BANK_CODE) REFERENCES PERSON (ID)').
	dropWord := self keywordForConstraintsWhenDropping: constraint onPlatform: dbPlatform.	
	dbPlatform supportsAlterTableForConstraints ifTrue:
		[self assert: constraint dropString =
			( 'ALTER TABLE BANK_ACCT DROP ', dropWord, ' ', constraint name)].
	constraint := ForeignKeyConstraint
		sourceField: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'BANK_CODE')
		targetField: ((system tableNamed: 'PERSON') fieldNamed: 'ID')
		suffixExpression: 'ON DELETE CASCADE'.
	self assert: constraint creationString = ('CONSTRAINT ', constraint name, ' FOREIGN KEY (BANK_CODE) REFERENCES PERSON (ID) ON DELETE CASCADE').
	dbPlatform supportsAlterTableForConstraints ifTrue:
		[self assert: constraint dropString = ('ALTER TABLE BANK_ACCT DROP ', dropWord, ' ',  constraint name)].! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testConstraintNonCreation

	| constraint |
	constraint := ForeignKeyConstraint 
		sourceField: ((system tableNamed: 'BANK_ACCT') fieldNamed: 'BANK_CODE')
		targetField: ((system tableNamed: 'PERSON') fieldNamed: 'ID').
	constraint shouldCreateInDatabase: false.
	self assert: constraint creationString = ''.
	self assert: constraint dropString = ''.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testFieldTable

	| field table |
	field := DatabaseField named: 'FOO' type: dbPlatform int4.
	table := DatabaseTable named: 'BAR'.
	table addField: field.
	self assert: (table fieldNamed: 'FOO') = field.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testForeignKeyFieldAPI
	"SQLite does not support foreign keys but it will parse them;  this test passes on SQLite."

	| table1 table2 fieldCustAddr fieldAddrId constraint |
	table1 := DatabaseTable named: 'Customer'.
	(table1 createFieldNamed: 'ID' type: dbPlatform int4) bePrimaryKey.
	fieldCustAddr := table1 createFieldNamed: 'ADDRESS' type: dbPlatform int4.
	table2 := DatabaseTable named: 'Address'.
	fieldAddrId := (table2 createFieldNamed: 'ID' type: dbPlatform int4) bePrimaryKey.
	table2 createFieldNamed: 'Street' type: dbPlatform varChar.

	constraint := table1 addForeignKeyFrom: fieldCustAddr toTable: table2.
	self assert: (table2 fieldNamed: 'ID') = table1 foreignKeyConstraints first targetFields first.
	self assert: constraint name =  'Customer_A_TO_Address_ID_REF1'.

	table1 := DatabaseTable named: 'Customer'.
	(table1 createFieldNamed: 'ID' type: dbPlatform int4) bePrimaryKey.
	fieldCustAddr := table1 createFieldNamed: 'ADDRESS' type: dbPlatform int4.
	table2 := DatabaseTable named: 'Address'.
	fieldAddrId := (table2 createFieldNamed: 'ID' type: dbPlatform int4) bePrimaryKey.
	table2 createFieldNamed: 'Street' type: dbPlatform varChar.

	table1 addForeignKeyFrom: fieldCustAddr toField: fieldAddrId.
	self assert: (table2 fieldNamed: 'ID') = table1 foreignKeyConstraints first targetFields first.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testPrimaryKeyFields

	| pkFields table |
	table := system tableNamed: 'BANK_TRANS'.
	pkFields := table primaryKeyFields.
	self assert: pkFields size = 1.
	self assert: (pkFields at: 1) == (table fieldNamed: 'ID').! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testPrimaryKeyFields2

	| table field |
	table := DatabaseTable new.
	field := (DatabaseField named: 'FRED' type: (dbPlatform varChar: 10))
		bePrimaryKey.
	table addField: field.
	self assert: table primaryKeyFields size = 1.
	self assert: (table primaryKeyFields at: 1) == field.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testPrimaryKeyFieldsNoPK

	| pkFields table |
	table := system tableNamed: 'CUSTOMER_ACCT_LINK'.
	pkFields := table primaryKeyFields.
	self assert: pkFields size = 0.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testPrintingWithParent

	| t t1 |
	t := system tableNamed: 'GR_CUSTOMER'.
	t1 := t copy.
	t1 parent: t.
	t1 name: 'foo'.
	self assert: t1 sqlTableName = 'GR_CUSTOMER foo'.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testPrintingWithoutParent

	| t t1 |
	t := system tableNamed: 'GR_CUSTOMER'.
	self assert: t sqlTableName = 'GR_CUSTOMER'.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testRowCreation

	| row |
	row := system examplePersonRow1.
	self assert: (row at: (row table fieldNamed: 'ID')) = 3.! !

!GlorpTableTest methodsFor: 'tests' stamp: ''!
testTwoSequences
	| row1 row2 table idField |
	table := system tableNamed: 'STUFF'.
	row1 := DatabaseRow newForTable: table.
	row2 := DatabaseRow newForTable: table.
	row1 preWriteAssignSequencesUsing: nil.
	row1 postWriteAssignSequencesUsing: nil.
	row2 preWriteAssignSequencesUsing: nil.
	row2 postWriteAssignSequencesUsing: nil.
	idField := table fieldNamed: 'ID'.
	self assert: (row1 at: idField) = 1.
	self assert: (row2 at: idField) = 2.! !

GlorpTestCase subclass: #GlorpDatabaseBasicTest
	instanceVariableNames: 'system'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDatabaseBasicTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource.! !

!GlorpDatabaseBasicTest methodsFor: 'accessing' stamp: ''!
accessor
	^self databaseLoginResource accessor! !

!GlorpDatabaseBasicTest methodsFor: 'support' stamp: ''!
databaseLoginResource
	^GlorpDatabaseLoginResource current.! !

!GlorpDatabaseBasicTest methodsFor: 'support' stamp: ''!
setUp

	super setUp.
	system := GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database.! !

!GlorpDatabaseBasicTest methodsFor: 'tests' stamp: ''!
testBeginTransactionWithCommit
	self assert: self accessor isInTransaction not.
	self accessor beginTransaction.
	self assert: self accessor isInTransaction.
	self accessor commitTransaction.
	self assert: self accessor isInTransaction not! !

!GlorpDatabaseBasicTest methodsFor: 'tests' stamp: ''!
testBeginTransactionWithRollback
	self assert: self accessor isInTransaction not.
	self accessor beginTransaction.
	self assert: self accessor isInTransaction.
	self accessor rollbackTransaction.
	self assert: self accessor isInTransaction not! !

!GlorpDatabaseBasicTest methodsFor: 'tests' stamp: ''!
testCreateTable
	| selectResult presentFlag |
	presentFlag := true.
	[[self accessor beginTransaction.
	self accessor
		executeSQLStringNoResult: 'CREATE TABLE GLORP_TEST_CREATE (ID varchar(4))'] 
		ensure: [self accessor commitTransaction].
	selectResult := self accessor
		executeSQLString: 'SELECT * FROM GLORP_TEST_CREATE'.
	self assert: selectResult isEmpty]
		ensure:
			[[self accessor
				beginTransaction;
				dropTableNamed: 'GLORP_TEST_CREATE'
					ifAbsent: [:ex |presentFlag := false. ex sunitExitWith: nil]]
				ensure: [self accessor commitTransaction]].
	self assert: presentFlag
		description: 'Tried to drop the created table but it was already absent'.! !

!GlorpDatabaseBasicTest methodsFor: 'tests' stamp: ''!
testDropMissingTable
	| absentFlag |
	absentFlag := false.
	[self accessor beginTransaction.
	self accessor 
		dropTableNamed: 'GLORP_TEST_DROP' 
		ifAbsent: [:ex  | absentFlag := true. ex sunitExitWith: nil]]
	ensure: [self accessor rollbackTransaction].
	self assert: absentFlag! !

!GlorpDatabaseBasicTest methodsFor: 'tests' stamp: ''!
testNameBinding
	| selectResult |
	"This is more of a low-level db functionality test than a glorp test, but it's convenient to be able ot write it here"
	Dialect isVisualWorks ifFalse: [^self].
	[self accessor beginTransaction.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (10,''abc'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (20,''def'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (30,''ghi'')'.
	selectResult := self accessor 
		executeSQLString: 'SELECT * FROM STUFF WHERE ID = :key OR ID = :value'
			binding: 10->30.

	self assert: selectResult size = 2]
		ensure: [self accessor rollbackTransaction].! !

!GlorpDatabaseBasicTest methodsFor: 'tests' stamp: ''!
testPositionalBinding
	| selectResult | 
	"This is more of a low-level db functionality test than a glorp test, but it's convenient to be able ot write it here"
	Dialect isVisualWorks ifFalse: [^self].
	[self accessor beginTransaction.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (10,''abc'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (20,''def'')'.
	self accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (30,''ghi'')'.
	selectResult := self accessor 
		executeSQLString: 'SELECT * FROM STUFF WHERE ID = :2 OR ID = :17'
			binding: #(nil 10 nil nil nil nil nil nil nil nil nil nil nil nil nil nil 20).

	self assert: selectResult size = 2]
		ensure: [self accessor rollbackTransaction].! !

!GlorpDatabaseBasicTest methodsFor: 'tests' stamp: ''!
testReadEmpty
	| results |
	results := self accessor
				executeSQLString: 'SELECT * FROM PERSON'.
	self assert: results size = 0! !

!GlorpDatabaseBasicTest methodsFor: 'tests' stamp: ''!
testReadStatement
	| results |
	results := self accessor
				executeSQLString: 'SELECT * FROM STUFF ORDER BY ID'.
	self assert: results size = 5.
	self assert: results first size = 2.
	self assert: results first last =  'abc'! !

!GlorpDatabaseBasicTest methodsFor: 'tests' stamp: ''!
testRollbackRemovesData
	"Just to make sure I'm not losing my mind"
	| numAddresses newNumAddresses|
	numAddresses := (self accessor executeSQLString: 'SELECT * FROM GR_ADDRESS') size.
	self accessor beginTransaction.
	self accessor executeSQLStringNoResult: 'INSERT INTO GR_ADDRESS (ID,STREET,HOUSE_NUM)  VALUES (111,''Main Street'',''77777'')'.
	newNumAddresses := (self accessor executeSQLString: 'SELECT * FROM GR_ADDRESS') size.
	self assert: newNumAddresses = (numAddresses + 1).
	self accessor rollbackTransaction.
	self assert: numAddresses = (self accessor executeSQLString: 'SELECT * FROM GR_ADDRESS') size.! !

GlorpTestCase subclass: #GlorpDatabaseLoginTest
	instanceVariableNames: 'login accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDatabaseLoginTest class methodsFor: 'resources' stamp: ''!
resources

	^Array with: GlorpDatabaseLoginResource.! !

!GlorpDatabaseLoginTest methodsFor: 'accessing' stamp: ''!
accessor

	^accessor.! !

!GlorpDatabaseLoginTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	login := GlorpDatabaseLoginResource defaultLogin copy.
	accessor := DatabaseAccessor forLogin: login.! !

!GlorpDatabaseLoginTest methodsFor: 'tests' stamp: ''!
testLogin
	self assert: self accessor isLoggedIn not.
	self accessor login.
	self assert: self accessor isLoggedIn.
	self accessor logout.
	self assert: self accessor isLoggedIn not! !

!GlorpDatabaseLoginTest methodsFor: 'tests' stamp: ''!
testSecureLoginDiscardsPassword
	| oldValue |
	self assert: self accessor isLoggedIn not.
	oldValue := self accessor currentLogin secure.
	[self accessor currentLogin secure: true.
	self accessor login.
	self assert: accessor currentLogin password isNil.
	self accessor logout]
		ensure: [self accessor currentLogin secure: oldValue].! !

!GlorpDatabaseLoginTest methodsFor: 'tests' stamp: ''!
testUnsecureLoginDoesNotDiscardPassword
	| oldValue |
	self assert: self accessor isLoggedIn not.
	oldValue := self accessor currentLogin secure.
	[self accessor currentLogin secure: false.
	self accessor login.
	self assert: accessor currentLogin password notNil.
	self accessor logout]
		ensure: [self accessor currentLogin secure: oldValue].! !

!GlorpDatabaseLoginTest methodsFor: 'tests' stamp: ''!
testUnsuccessfulLogin

	| anotherAccessor invalidLogin |
	"Some databases can't fail to log in."
	login database supportsAuthentication ifFalse: [^self].

	invalidLogin := GlorpDatabaseLoginResource defaultLogin copy.
	invalidLogin
		password: 'you will never ever guess this password';
		username: 'not a valid user name'.
	anotherAccessor := DatabaseAccessor forLogin: invalidLogin.

	self assert: anotherAccessor isLoggedIn not.
	anotherAccessor loginIfError: [:ex | ].
	Dialect isVisualAge		"The isLoggedIn is unreliable under VA, can return false positive"
		ifFalse: [self assert: anotherAccessor isLoggedIn not]
		ifTrue: [Transcript cr; show: self class name, '>>testUnsuccessfulLogin skips a trivial isLoggedIn check, saying it was unreliable under VA55, could return false positive, later versions not yet verified'].
	anotherAccessor logout.! !

GlorpTestCase subclass: #GlorpDatabasePlatformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
addToString: aString
	"Append a couple of special characters. The result will utf-8 encode to be 3 characters longer than its number of characters"
	^aString, (String with: (Character value: 16r2022) "bullet point" with: (Character value: 16r0131) "Turkish i with no dot")

"If PostgreSQL, or another DB returning true to measuresStringFieldWidthsInBytes, is provided in VA, this test must alter to
handle incompatible protocol between VW, which lacks Character>>asString and VA, in which String>>with:with: will not handle
double-byte entries.  The VA could be implemented as
	^aString, (Character value: 16r2022) asString, (Character value: 16r0131) asString
"! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/2/2011 08:09'!
helpTestPrintTimestamp: aTimestampString

	| aStream ts timestamp |
	ts := DatabasePlatform new readTimestamp: aTimestampString for: nil.
	aStream := WriteStream on: String new.
	DatabasePlatform new printDate: ts isoFormatOn: aStream.
	aStream nextPut: Character space.
	DatabasePlatform new printTime: ts 
		isoFormatOn: aStream 
		milliseconds: Dialect supportsMillisecondsInTimes.
	timestamp := aStream contents.
	self assert: timestamp = aTimestampString.! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
string20
	| platform |
	platform := Dialect isVisualAge
		ifTrue:
			[Transcript cr; show: self class name, '>>string20: testing Oracle, not PostgreSQL'.
			OraclePlatform new]
		ifFalse: [PostgreSQLPlatform new].
	platform characterEncoding: #'utf-8'.
	^platform varchar: 20.! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
testNumericCoercion

	| type platform num |
	type := GlorpNumericType new scale: 5; precision: 2.
	platform := DatabasePlatform new.
	num := Dialect coerceToDoublePrecisionFloat: (123456 / 100).
	self assert: (platform convertToNumber: num for: type) = num.
	num := Dialect coerceToDoublePrecisionFloat: (123450 / 100).
	self assert: (platform convertToNumber: num for: type) = num.
	num := Dialect coerceToDoublePrecisionFloat: (123450 / 1000).
	self assert: (platform convertToNumber: num for: type) = num.
	num := Dialect coerceToDoublePrecisionFloat: (123456 / 1000).
	self deny: (platform convertToNumber: num for: type) = num.! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
testPrintTimestamp

	| strings |
	strings := #( '2005-01-01 21:29:28.337'  '2005-01-01 01:02:00.037'  '2005-01-01 21:29:28.002'  '2005-01-01 21:29:28.001').
	strings do: [:each |
		self helpTestPrintTimestamp: each].! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
testReadTimestamp

	| ts |
	ts := DatabasePlatform new readTimestamp: '2003-03-03 15:29:28.337-05' for: nil.
	self assert: ts asSeconds = 3224158168.
	self assert: ([ts asMilliseconds = 3224158168337 ]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #asMilliseconds ]).

	ts := DatabasePlatform new readTimestamp: '2003-03-13 15:29:28.337-05' for: nil.
	self assert: ts asSeconds = 3225022168.
	self assert: ([ts asMilliseconds = 3225022168337]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #asMilliseconds ])! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
testReadTimestampNoMS

	| ts |
	ts := DatabasePlatform new readTimestamp: '2003-03-03 15:29:28-05' for: nil.
	self assert: ts year = 2003.
	self assert: ts monthIndex = 3.
	self assert: ts dayOfMonth = 3.
	self assert: ts hour = 15.
	self assert: ts minute = 29.
	self assert: ts second truncated = 28.
	self assert: ([ ts milliseconds = 0 ]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #milliseconds ])! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
testReadTimestampNoMSNoTZ

	| ts |
	ts := DatabasePlatform new readTimestamp: '2003-03-03 15:29:28' for: nil.
	self assert: ts year = 2003.
	self assert: ts monthIndex = 3.
	self assert: ts dayOfMonth = 3.
	self assert: ts hour = 15.
	self assert: ts minute = 29.
	self assert: ts second truncated = 28.
	self assert: ([ ts milliseconds = 0 ]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #milliseconds ])! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
testReadTimestampNoTZ

	| ts |
	ts := DatabasePlatform new readTimestamp: '1975-08-13 21:29:28.337' for: nil.
	self assert: ts year = 1975.
	self assert: ts monthIndex = 8.
	self assert: ts dayOfMonth = 13.
	self assert: ts hour = 21.
	self assert: ts minute = 29.
	self assert: ts second truncated = 28.
	self assert: ([ ts milliseconds =  337]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #milliseconds ])! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
testReadTimestampOverflowDays

	| ts |
	ts := DatabasePlatform new readTimestamp: '1975-08-13 21:29:28.337-05' for: nil.
	self assert: ts year = 1975.
	self assert: ts monthIndex = 8.
	self assert: ts dayOfMonth = 13.
	self assert: ts hour = 21.
	self assert: ts minute = 29.
	self assert: ts second truncated = 28.
	self assert: ([ ts milliseconds = 337 ]
		on: MessageNotUnderstood
		do: [ :mnu | mnu return: mnu message selector = #milliseconds ])! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
testTrimString1
	"Normal ascii"
	| type plat |
	type := self string20.
	plat := type platform.
	self assert: (plat trimString: 'abcdefghi' for: type) = 'abcdefghi'.
	self assert: (plat trimString: 'abcdefghijklmnopqrst' for: type) = 'abcdefghijklmnopqrst'.
	self assert: (plat trimString: 'abcdefghijklmnopqrstu' for: type) = 'abcdefghijklmnopqrst'.
	self assert: (plat trimString: 'abcdefghijklmnopqrstuvwxy' for: type) = 'abcdefghijklmnopqrst'.! !

!GlorpDatabasePlatformTest methodsFor: 'tests' stamp: ''!
testTrimString2
	"Some special characters"
	| type plat string |
	Dialect isVisualAge ifTrue: [Transcript cr; show: self class name; show: '>>testTrimString2 skipped in VA'. ^self].

	type := self string20.
	plat := type platform.

	string := self addToString: 'abcdefghi'.
	self assert: (plat trimString: string for: type) = string.
	string := self addToString: 'abcdefghijklmno'.
	self assert: (plat trimString: string for: type) = string.
	string := self addToString: 'abcdefghijklmnop'.
	self assert: (plat trimString: string for: type) = (string copyFrom: 1 to: string size -1).
	string := self addToString: 'abcdefghijklmnopq'.
	self assert: (plat trimString: string for: type) = (string copyFrom: 1 to: string size -1).
	string := (String new: 100000) atAllPut: $a.
	self assert: (plat trimString: string for: type) = 'aaaaaaaaaaaaaaaaaaaa'! !

GlorpTestCase subclass: #GlorpDatabaseSessionTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDatabaseSessionTest class methodsFor: 'resources' stamp: ''!
resources

	^Array with: GlorpSessionResource.! !

!GlorpDatabaseSessionTest methodsFor: 'support' stamp: ''!
setUp

	super setUp.
	session := GlorpSessionResource current newSession.! !

!GlorpDatabaseSessionTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session reset.
	session := nil.! !

!GlorpDatabaseSessionTest methodsFor: 'tests' stamp: 'GuillermoPolito 4/26/2011 04:57'!
testWriteRow
	| rowToWrite fields rowReadFromDatabase |
	rowToWrite := session system examplePersonRow2.
	
	[session beginTransaction.
	session writeRow: rowToWrite.
	rowReadFromDatabase := (session accessor
				executeSQLString: 'SELECT * FROM ' , rowToWrite table name) first.
	fields := rowToWrite table fields.
	(1 to: fields size) with: fields
		do: [:index :field | self assert: (rowReadFromDatabase at: index) = (rowToWrite at: field)]] 
			ensure: [session rollbackTransaction]! !

GlorpTestCase subclass: #GlorpDatabaseTypeDBTests
	instanceVariableNames: 'type stType connection session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpDatabaseTypeDBTests class methodsFor: 'accessing' stamp: ''!
resources
	^Array with: GlorpSessionResource! !

!GlorpDatabaseTypeDBTests methodsFor: 'accessing' stamp: ''!
platform
	^connection platform! !

!GlorpDatabaseTypeDBTests methodsFor: 'setup' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	connection := session accessor.! !

!GlorpDatabaseTypeDBTests methodsFor: 'setup' stamp: ''!
tearDown
	super tearDown.
	session reset.
	session := nil.! !

!GlorpDatabaseTypeDBTests methodsFor: 'infrastructure tests' stamp: ''!
testReadTime
	self platform readTime: '18:06:22.12' for: self platform time.! !

!GlorpDatabaseTypeDBTests methodsFor: 'tests' stamp: ''!
testSerial
	"type := PGSerialType instance.
	self assert: false.

	self assert: (type typeString = 'SERIAL')"! !

!GlorpDatabaseTypeDBTests methodsFor: 'tests' stamp: ''!
testTimeWithTimeZone

	| time |
	"
	type := PGTimeWithTimeZoneType instance.
	time := Dialect timestampNow.

	self assert: false. This needs some thought.
	self assert: (self helpTestType: type withValue: time).	
	self assert: (type typeString = 'TIME WITH TIME ZONE')"! !

!GlorpDatabaseTypeDBTests methodsFor: 'tests' stamp: ''!
testTypeParametersNotAliased

	| type2 type3 |
	type := (self platform) varchar.
	self assert: type width isNil.
	self assert: (type2 := self platform varChar: 5) width = 5.
	self assert: type width isNil.
	type3 := self platform varChar: 10.
	self assert: type3 width = 10.
	self assert: type2 width = 5.
	self assert: type width isNil.! !

!GlorpDatabaseTypeDBTests methodsFor: 'tests' stamp: ''!
testVarBinary
	"Needs doing"
	self needsWork: 'write the test'.! !

GlorpTestCase subclass: #GlorpDatabaseTypeIndividualDBTests
	instanceVariableNames: 'type stType connection session table'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

GlorpDatabaseTypeIndividualDBTests subclass: #AbstractLOBTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!AbstractLOBTest methodsFor: 'private' stamp: ''!
adjustForMaximumSize: aValue
	^aValue size > session platform maximumQueryValueSize 
		ifTrue: [(aValue copyFrom: 1 to: session platform maximumQueryValueSize - 2), '%']
		ifFalse: [aValue].! !

!AbstractLOBTest methodsFor: 'private' stamp: ''!
convertToTypeFrom: anObject
	"For SQL-Server, return the function which converts anObject's value to my type."
	
	^anObject
		ifNotNil:
			[FunctionExpression for: #convertTo: withArguments: (Array with: anObject asGlorpExpression) basedOn: type asGlorpExpression system: session system]! !

!AbstractLOBTest methodsFor: 'private' stamp: 'GuillermoPolito 4/26/2011 04:56'!
helpTestExplicitReadBackFor: dbOutValue
	"Read back the row with the given value, using an explicit where clause."
	| queryStream result compareClause|
	queryStream := SQLStringSelectCommand sqlString: nil parameters: #() useBinding: session useBinding session: session.
	queryStream nextPutAll: 'SELECT TEST,ID FROM TYPETESTS WHERE TEST'.
	compareClause := self platform isSQLServerPlatform ifTrue: [' = '] ifFalse: [' LIKE '].
	queryStream nextPutAll: (dbOutValue isNil ifTrue: [' IS '] ifFalse: [compareClause]).
	(dbOutValue notNil and: [type requiresCast]) ifTrue: [queryStream nextPutAll: ' CAST ('].
	(self platform isSQLServerPlatform and: [dbOutValue notNil and: [type requiresConvertToVarbinary]])
		ifTrue: [queryStream nextPutAll: ' CONVERT(varbinary(max),'].
	(queryStream canBind: dbOutValue to: type) 
		ifTrue: [
			queryStream parameters: (Array with: dbOutValue).
			queryStream nextPutAll: '?']
		ifFalse: [type print: dbOutValue on: queryStream].
	(dbOutValue notNil and: [type requiresCast]) ifTrue: [
		queryStream nextPutAll: ' AS '.
		queryStream nextPutAll: type queryType typeName.
		queryStream nextPutAll: ')'].
	(self platform isSQLServerPlatform and: [dbOutValue notNil and: [type requiresConvertToVarbinary]])
		ifTrue: [queryStream nextPutAll: ')'].
	result := connection executeCommand: queryStream.
	result isEmpty ifTrue: [^nil].
	^(result at: 1) at: 1.! !

!AbstractLOBTest methodsFor: 'private' stamp: ''!
helpTestValue: anObject compareModelWith: modelBlock compareWith: aBlock 
	| dbOutValue |
	dbOutValue := self adjustForMaximumSize: anObject.
	^super helpTestValue: dbOutValue compareModelWith: modelBlock compareWith: aBlock.! !

!AbstractLOBTest methodsFor: 'private' stamp: ''!
readBackModelWithValue: anObject
	"For SQL-Server, use CONVERT (which is easier than CAST for now), with = instead of LIKE."
	
	^self platform isSQLServerPlatform
		ifTrue:
			[| converted |
			converted := self convertToTypeFrom: anObject.
			session
				readOneOf: GlorpTypeTestsModelClass
				where: [:each | each test = converted]]
		ifFalse:
			[session
				readOneOf: GlorpTypeTestsModelClass
				where: [:each | each test like: anObject]]

	"^session
		readOneOf: GlorpTypeTestsModelClass
		where: [:each | each test like: (Cast value: anObject as: type queryType)]."! !

AbstractLOBTest subclass: #GlorpBlobTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpBlobTest methodsFor: 'tests' stamp: ''!
adjustForMaximumSize: anObject
	^anObject.! !

!GlorpBlobTest methodsFor: 'tests' stamp: ''!
blob255
	^ByteArray withAll: #(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255)! !

!GlorpBlobTest methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform blob.! !

!GlorpBlobTest methodsFor: 'tests' stamp: ''!
largeBlob
	"Make a significant sized blob. 64k+ a bit should do"
	| blobStream |
	blobStream := WriteStream on: (ByteArray new: 10000).
	(((2 raisedTo: 16) + 100.0) / 255.0) ceiling timesRepeat: [
		blobStream nextPutAll: self blob255].
	^blobStream contents.! !

!GlorpBlobTest methodsFor: 'tests' stamp: ''!
mediumlargeBlob
	"Make a Blob just under 32K in size"
	"self new mediumlargeBlob"
	| blobStream |
	blobStream := WriteStream on: (ByteArray new: 10000).
	(((2 raisedTo: 15) - 600.0) / 255.0) ceiling timesRepeat: [
		blobStream nextPutAll: self blob255].
	^blobStream contents.! !

!GlorpBlobTest methodsFor: 'tests' stamp: ''!
testBlob
	self helpTestValue: nil.
	self helpTestValue: (ByteArray withAll: #(1 2 3 4 255)).
	stType := String.
	self helpTestValue: 'abcdef'.! !

!GlorpBlobTest methodsFor: 'tests' stamp: ''!
testBlobWithSubstring
	"Test our ability to query with substring against a blob. This is severely limited in Postgresql, and may be limited in others."
	| result system |
	session platform supportsStringQueryOnBlobs ifFalse: [^self knownFailure].
	stType := String.
	self inTransactionDo: [
		system := self systemFor: table.
		session system: system.
		session inUnitOfWorkDo: [
			| model |
			model := GlorpTypeTestsModelClass new test: 'hgfedcbazyx'.
			session register: model].
		session reset.
		result := session read: GlorpTypeTestsModelClass where: [:each | (each test copyFrom: 1 to: 3) = 'hgf'].
		self assert: result size = 1.
		result := session read: GlorpTypeTestsModelClass where: [:each | (each test copyFrom: 1 to: 3) = 'abc'].
		self assert: result size = 0.
		result := session read: GlorpTypeTestsModelClass where: [:each | (each test copyFrom: 4 to: 6) = 'edc'].
		self assert: result size = 1.
		result := session read: GlorpTypeTestsModelClass where: [:each | (each test copyFrom: 7 to: 9) = 'baz'].
		self assert: result size = 1.

		result := session read: GlorpTypeTestsModelClass where: [:each | (each test copyFrom: 1 to: 6) = 'hgfedc'].
		self assert: result size = 1].! !

!GlorpBlobTest methodsFor: 'tests' stamp: ''!
testDefaultBlobPrinting
	"Test out default (SQL-92 syntax) byte array printing"
	| stream |
	stream := WriteStream on: String new.
	type := type copy.
	type platform: DatabasePlatform new.
	type print: (ByteArray with: 0) on: stream.
	self assert: stream contents = 'X''00'''.
	stream reset.
	type print: (ByteArray with: 16rFF) on: stream.
	self assert: stream contents = 'X''FF'''.
	stream reset.
	type print: (ByteArray with: 16r47) on: stream.
	self assert: stream contents = 'X''47'''.
	stream reset.
	type print: (ByteArray withAll: #(1 2 16rFE)) asByteArray on: stream.
	self assert: stream contents = 'X''0102FE'''.! !

!GlorpBlobTest methodsFor: 'tests' stamp: ''!
testLargeBlob
	| blob |
	blob := self mediumlargeBlob.
	(session platform useBinding not and: [blob size > session platform maximumQueryValueSize])
		ifTrue: [Transcript cr; show: '******* Skipping large unbound blob test']
		ifFalse: [self helpTestValue: blob].
	blob := self largeBlob.
	(session platform useBinding not and: [blob size > session platform maximumQueryValueSize])
		ifFalse: [self helpTestValue: blob].! !

AbstractLOBTest subclass: #GlorpClobTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpClobTest methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform clob.! !

!GlorpClobTest methodsFor: 'tests' stamp: ''!
longString
	"Make a significant sized stringlob. 64k+ a bit should do"
	| stream |
	stream := WriteStream on: (String new: 10000).
	(((2 raisedTo: 16) + 100.0) / 255.0) ceiling timesRepeat: [
		stream nextPutAll: self string255].
	^stream contents.! !

!GlorpClobTest methodsFor: 'tests' stamp: ''!
string255
	^'abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstu'.! !

!GlorpClobTest methodsFor: 'tests' stamp: ''!
testClob
	self helpTestValue: nil.
	self helpTestValue: 'abcdef'.! !

!GlorpClobTest methodsFor: 'tests' stamp: ''!
testLargeClob
	"The long test here not only fails, it messes up the driver so that other tests fail"
	(session platform class == AccessPlatform and: [session useBinding not])
		ifTrue: [^self].
	self helpTestValue: self longString.! !

AbstractLOBTest subclass: #GlorpTextTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpTextTest methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform text! !

!GlorpTextTest methodsFor: 'tests' stamp: ''!
longString
	"Make a significant sized stringlob. 64k+ a bit should do"
	| stream |
	stream := WriteStream on: (String new: 10000).
	(((2 raisedTo: 16) + 100.0) / 255.0) ceiling timesRepeat: [
		stream nextPutAll: self string255].
	^stream contents.! !

!GlorpTextTest methodsFor: 'tests' stamp: ''!
string255
	^'abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstu'.! !

!GlorpTextTest methodsFor: 'tests' stamp: ''!
testLargeText
	"For Access, the long test here not only fails, it messes up the driver so that other tests fail"
	(session platform class == AccessPlatform and: [session useBinding not])
		ifTrue: [^self].
	self helpTestValue: self longString.! !

!GlorpTextTest methodsFor: 'tests' stamp: ''!
testSymbol
	"Test that we can store a Symbol as text. This is particularly interesting because then we have to go through a non-empty type converter, and those need to take into account that text type, although a string type, probably doesn't have a maximum width."
	stType := Symbol.
	self helpTestValue: #'ASymbolStoredAsText'.! !

!GlorpTextTest methodsFor: 'tests' stamp: ''!
testText
"	self helpTestValue: nil."
	self helpTestValue: 'Now is the time for all good squeakers to come to the aid of the mouse'.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpBooleanTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpBooleanTest methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform boolean.! !

!GlorpBooleanTest methodsFor: 'tests' stamp: ''!
testBoolean
	stType := Boolean.
	self helpTestValue: nil compareWith: [:read :original |
		self platform usesNullForFalse ifTrue: [read = false] ifFalse: [read = nil]].
	self helpTestValue: true.
	self helpTestValue: false.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpChar2Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpChar2Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform char width: 2! !

!GlorpChar2Test methodsFor: 'tests' stamp: ''!
testChar2

	stType := String.
	self helpTestValue: nil.
	self helpTestValue: '' compareWith: [:read :original |
		self platform usesNullForEmptyStrings ifTrue: [read = nil] ifFalse: [read = original]].
	self helpTestValue: 'a'.
	self helpTestValue: 'ab'.
	self helpTestInvalidValue: 'abc'.
	self helpTestInvalidValue: 'abcd'.
	self helpTestInvalidValue: 'abcde'.
	
	self assert: (type typeString asUppercase = (self platform char: 2) typeString asUppercase)! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpChar4Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpChar4Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform char width: 4.! !

!GlorpChar4Test methodsFor: 'tests' stamp: ''!
testChar4

	stType := String.
	self helpTestValue: nil.
	self helpTestValue: '' compareWith: [:read :original |
		self platform usesNullForEmptyStrings ifTrue: [read = nil] ifFalse: [read = original]].
	self helpTestValue: 'a'.
	self helpTestValue: 'ab'.
	self helpTestValue: 'abc'.
	self helpTestValue: 'abcd'.
	self helpTestInvalidValue: 'abcde'.
	stType := Symbol.
	self helpTestValue: #abcd.
	
	self assert: (type typeString asUppercase= (self platform char: 4) typeString asUppercase)! !

!GlorpDatabaseTypeIndividualDBTests class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpDatabaseLoginResource with: GlorpSessionResource.! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: ''!
compareFloatInModel: read with: original
	read isNil ifTrue: [^false].
	read test = original ifTrue: [^true].
	read test isNil ifTrue: [^false].
	^(read test - original) abs <= 0.00001.! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'setUp' stamp: ''!
createTypeTestTable
	| system |
	connection doDDLOperation: [
		connection
			dropTableNamed: 'TYPETESTS'
			ifAbsent: [:ex | ex return: nil].
		table := DatabaseTable named: 'TYPETESTS'.
		table
			createFieldNamed: 'test'
			type: type.
		(table createFieldNamed: 'id' type: session system platform inMemorySequence) bePrimaryKey.
		connection
			createTable: table
			ifError:
				[:ex | 
				Transcript show: 'CANNOT CREATE TABLE'.
             		self signalFailure: ex messageText.
				ex return: nil].
	].
	system := self systemFor: table.
	session system: system.
	^table.! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'setUp' stamp: ''!
defaultDatabaseType
	self subclassResponsibility.! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: 'GuillermoPolito 4/26/2011 04:57'!
helpTestExplicitReadBackFor: dbOutValue
	"Read back the row with the given value, using an explicit where clause."
	| queryStream result |
	queryStream := SQLStringSelectCommand sqlString: String new parameters: #() useBinding: false session: session.
	queryStream nextPutAll: 'SELECT TEST,ID FROM TYPETESTS WHERE TEST '.
	queryStream nextPutAll: (dbOutValue isNil ifTrue: [' IS '] ifFalse: [' = ']).
	type print: dbOutValue on: queryStream.
	result := connection executeSQLString: queryStream contents.
	result isEmpty ifTrue: [^nil].
	^(result at: 1) at: 1.! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: ''!
helpTestFloat: anObject
	"Don't try and read back an equal float, it'll likely fail on precision issues"

	self
		helpTestValue: anObject
		compareModelWith:
			[:read :original | self compareFloatInModel: read with: original]
		compareWith:
			[:read :original | original isNil or: [(read - original) abs <= 0.00001]].! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: ''!
helpTestInvalidValue: anObject
	self helpTestValue: anObject
		compareModelWith: [:read :original | read isNil or: [ read test ~= original ]]
		compareWith: [:read :original | read ~= original]! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: ''!
helpTestValue: anObject 
	"Don't try and read back an equal float, it'll likely fail on precision issues"

	self 
		helpTestValue: anObject
		compareModelWith: 
			[:read :original | 
			read notNil and: 
					[(original isKindOf: Float) or: 
							[original class == Dialect doublePrecisionFloatClass 
								or: [read test = original]]]]
		compareWith: [:read :original | read = original]! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: ''!
helpTestValue: anObject compareModelWith: modelBlock compareWith: aBlock 
	self 
		helpTestValueWithSQLWrite: anObject
		compareModelWith: modelBlock
		compareWith: aBlock.
	self 
		helpTestValueWithUnitOfWorkWrite: anObject
		compareModelWith: modelBlock! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: ''!
helpTestValue: anObject compareWith: aBlock 
	self 
		helpTestValue: anObject
		compareModelWith: [:read :original | true]
		compareWith: aBlock! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: 'GuillermoPolito 4/26/2011 04:55'!
helpTestValueWithSQLWrite: anObject compareModelWith: modelBlock compareWith: aBlock

	| dbInValue readObject row converter dbOutValue typeTestModel system dbInValue2 readObject2 command |

	system := self systemFor: table.
	session system: system.
	row := DatabaseRow newForTable: table.
	row owner: GlorpTypeTestsModelClass new.
	converter := type converterForStType: (stType isNil ifTrue: [anObject class] ifFalse: [stType]).
	dbOutValue := converter convert: anObject toDatabaseRepresentationAs: type.
	row atFieldNamed: 'test' put: dbOutValue.
	self inTransactionDo: [
	session writeRow: row.

	command := SQLStringCommand new setSQLString:  'SELECT TEST, ID FROM TYPETESTS'.
	command maximumLobSizeToRetrieveDirectly: ((dbOutValue class = ByteArray or: [dbOutValue isString]) ifTrue: [dbOutValue size] ifFalse: [42]).
	dbInValue := (session accessor executeCommand: command) first at: 1.
	readObject := converter convert: dbInValue fromDatabaseRepresentationAs: type.

	(session platform canUseInWhereClause: dbOutValue type: type) ifTrue: [
		dbInValue2 := self helpTestExplicitReadBackFor: dbOutValue.
		readObject2 := converter convert: dbInValue2 fromDatabaseRepresentationAs: type].

	(session platform canUseInWhereClause: anObject type: type) ifTrue: [
		typeTestModel := self readBackModelWithValue: anObject.
		self assert: (modelBlock value: typeTestModel value: anObject)]
	].
	self assert: (aBlock value: readObject value: anObject).
	readObject2 isNil ifFalse: [
		self assert: (aBlock value: readObject2 value: anObject)].! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: ''!
helpTestValueWithUnitOfWorkWrite: anObject compareModelWith: aBlock
	| typeTestModel system model |
	system := self systemFor: table.
	session system: system.
	self inTransactionDo: [
	session beginUnitOfWork.
	model := GlorpTypeTestsModelClass new test: anObject.
	session register: model.
	session commitUnitOfWork.
	session reset.
	(session platform canUseInWhereClause: anObject type: type) ifTrue: [
		typeTestModel := self readBackModelWithValue: anObject.
		self assert: (aBlock value: typeTestModel value: anObject)]].! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: ''!
inTransactionDo: aBlock
	"Like the one in session, but always rollback."
	| alreadyInTransaction result |
	[ 
	alreadyInTransaction := session isInTransaction.
	alreadyInTransaction ifFalse: [session beginTransaction].
	result := aBlock numArgs = 1 ifTrue: [aBlock value: self] ifFalse: [aBlock value].
	alreadyInTransaction ifFalse: [session rollbackTransaction]]
		ifCurtailed:
			[alreadyInTransaction ifFalse: [session rollbackTransaction]].
	^result.! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'initializing' stamp: ''!
initialize! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'accessing' stamp: ''!
platform
	^connection platform! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'helpers' stamp: ''!
readBackModelWithValue: anObject
	^session
		readOneOf: GlorpTypeTestsModelClass
		where: [:each | each test = anObject].! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'setUp' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	connection := session accessor.
	type := self defaultDatabaseType.
	table := self createTypeTestTable.! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'setUp' stamp: ''!
systemFor: aTable
	| system descriptor mapping model |
	system := DynamicDescriptorSystem new.
	model := system addClassModelFor: GlorpTypeTestsModelClass.
	model newAttributeNamed: #id.
	model newAttributeNamed: #test type: stType.
	system privateTableAt: aTable name put: aTable.
	descriptor := Descriptor new.
	descriptor system: system.
	descriptor classModel: model.
	descriptor table: aTable.
	descriptor
		addMapping: (DirectMapping from: #id to: (aTable fieldNamed: 'id')).
	stType isNil
		ifTrue:
			[mapping := DirectMapping from: #test to: (aTable fieldNamed: 'test')]
		ifFalse:
			[mapping := DirectMapping
				from: #test
				type: stType
				to: (aTable fieldNamed: 'test')].
	descriptor addMapping: mapping.
	system privateDescriptorAt: GlorpTypeTestsModelClass put: descriptor.
	^system.! !

!GlorpDatabaseTypeIndividualDBTests methodsFor: 'setUp' stamp: ''!
tearDown
	super tearDown.
	session doDDLOperation: [
		connection
			dropTableNamed: 'TYPETESTS'
			ifAbsent: [:ex | ex return: nil]].
	session reset.
	session := nil.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpDateTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpDateTest methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform date! !

!GlorpDateTest methodsFor: 'tests' stamp: ''!
testDate

	| date |
	Dialect isVisualAge ifTrue: [Transcript cr; show: 'Avoiding date test for VisualAge'. ^self].
	date := Date today.
	stType := Date.

	self helpTestValue: date.
	self helpTestValue: nil.! !

!GlorpDateTest methodsFor: 'tests' stamp: ''!
testDateCast
	"SQLite does not have time-related types:  they are all mapped to strings.  I have not debugged why this test fails yet but assume it is completely unrealistic to expect a string to be cast to a timestamp."

	| date model |
	Dialect isVisualAge ifTrue: [Transcript cr; show: 'Avoiding date test for VisualAge'. ^self].
	date := Date today.
	session transact: [session register: (GlorpTypeTestsModelClass new test: date)].
	model := session readOneOf: GlorpTypeTestsModelClass where: [:each | (each test cast: session platform date) = date].
	self deny: model isNil
		description: 'This test will fail on SQLite which has no DB Date type;  see test comment'.
	self assert: model test = date.

	model := session readOneOf: GlorpTypeTestsModelClass where: [:each | (each test cast: session platform timestamp) = date].
	self deny: model isNil.
	self assert: model test = date.

	model := session readOneOf: GlorpTypeTestsModelClass where: [:each | each test asDate = date].
	self deny: model isNil.
	self assert: model test = date.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpFloat4Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpFloat4Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform float4.! !

!GlorpFloat4Test methodsFor: 'tests' stamp: ''!
testFloat4

	self helpTestFloat: nil.
	self helpTestValue: 3.14 compareWith: [:read :original |
		read - original <= 0.00001].! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpFloat8Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpFloat8Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform float8! !

!GlorpFloat8Test methodsFor: 'types' stamp: ''!
testFloat8
	type := (self platform) double.

	self helpTestFloat: nil.
	self helpTestValue: (Dialect coerceToDoublePrecisionFloat: 3.14) compareWith: [:read :original |
		read - original <= 0.0000001].! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpInt2Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpInt2Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform int2.! !

!GlorpInt2Test methodsFor: 'tests' stamp: ''!
testBooleanToInteger
	stType := Boolean.
	self helpTestValue: nil.
	self helpTestValue: true.
	self helpTestValue: false.! !

!GlorpInt2Test methodsFor: 'tests' stamp: ''!
testInt2
	self helpTestValue: nil.
	self helpTestValue: 32123.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpInt4Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpInt4Test class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpInt4Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform int4! !

!GlorpInt4Test methodsFor: 'tests' stamp: ''!
testInt4
	self helpTestValue: nil.
	self helpTestValue: 3212321.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpInt8Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpInt8Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform int8! !

!GlorpInt8Test methodsFor: 'tests' stamp: ''!
testInt8
	type := (self platform) int8.
	self helpTestValue: nil.
	self helpTestValue: 3212321555.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpIntegerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpIntegerTest methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform integer.! !

!GlorpIntegerTest methodsFor: 'tests' stamp: ''!
testInteger
	type := (self platform) integer.
	self helpTestValue: nil.
	self helpTestValue: 3212321.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpNumeric52Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpNumeric52Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^(self platform numeric)
		precision: 5;
		scale: 2.! !

!GlorpNumeric52Test methodsFor: 'tests' stamp: ''!
testNumeric52
	self platform supportsVariableSizedNumerics ifFalse: [^self].
	self helpTestFloat: nil.
	self helpTestFloat: 12.
	self helpTestFloat: 321.12.
	self helpTestInvalidValue: 17.098.
	self helpTestFloat: 3.14.
	self helpTestFloat: (Dialect coerceToDoublePrecisionFloat: 12).
	self helpTestFloat: (Dialect coerceToDoublePrecisionFloat: 321.12).
	self helpTestInvalidValue: (Dialect coerceToDoublePrecisionFloat: 17.098).
	self helpTestFloat: (Dialect coerceToDoublePrecisionFloat: 3.14).
	self helpTestFloat: (Dialect readFixedPointFrom: '321.12').
	self helpTestInvalidValue: (Dialect readFixedPointFrom: '321.123').
	self helpTestFloat: (Dialect readFixedPointFrom: '3.14').
	self helpTestFloat: (Dialect readFixedPointFrom: '3.1' ).! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpNumeric5Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpNumeric5Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform numeric scale: 5.! !

!GlorpNumeric5Test methodsFor: 'tests' stamp: ''!
testNumeric5

	| fixed |

	self platform supportsVariableSizedNumerics ifFalse: [^self].

	self helpTestFloat: nil.
	self helpTestFloat: 12.
	self helpTestFloat: 10991.
	self helpTestFloat: (Dialect readFixedPointFrom: '3.0').
	fixed := Dialect readFixedPointFrom: '321.12'.
	self platform supportsDecimalsOnAllNumerics
		ifTrue: [
			self helpTestFloat: fixed.
			self helpTestFloat: 3.14]
		ifFalse: [
			self helpTestInvalidValue: fixed.
			self helpTestInvalidValue: 3.14].! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpNumeric5ZeroTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpNumeric5ZeroTest methodsFor: 'tests' stamp: ''!
defaultDatabaseType
	^self platform numeric precision: 5; scale: 0.! !

!GlorpNumeric5ZeroTest methodsFor: 'tests' stamp: ''!
testNumeric5Zero
	self platform supportsVariableSizedNumerics ifFalse: [^self].

	self helpTestFloat: nil.
	self helpTestFloat: 12.	
	self helpTestFloat: 10991.		
	self helpTestFloat: (Dialect coerceToDoublePrecisionFloat: 3.0).
	self helpTestInvalidValue: (Dialect coerceToDoublePrecisionFloat: 321.12).
	self helpTestFloat: (Dialect readFixedPointFrom: '3.0').
	self helpTestInvalidValue: (Dialect readFixedPointFrom: '321.12').
	self helpTestInvalidValue: 3.14.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpNumericTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpNumericTest methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform numeric! !

!GlorpNumericTest methodsFor: 'tests' stamp: ''!
testNumeric
	| floatSelector |
	self helpTestFloat: 12.	
	self helpTestFloat: nil.
	self helpTestFloat: (Dialect readFixedPointFrom: '12345678').
	self platform supportsDecimalsOnAllNumerics ifTrue: [
		self helpTestValue: 3.14  compareWith: [:read :original |
			read - original <= 0.00001]]
		ifFalse: [self helpTestInvalidValue: 3.14].
	floatSelector := self platform supportsDecimalsOnAllNumerics ifTrue: [#helpTestFloat:] ifFalse: [#helpTestInvalidValue:].
	self perform: floatSelector with: (Dialect coerceToDoublePrecisionFloat:  321.12).
	self perform: floatSelector with: (Dialect coerceToDoublePrecisionFloat: 321.123).
	self perform: floatSelector with: (Dialect coerceToDoublePrecisionFloat: (Dialect readFixedPointFrom: '321.123456789')).
	self perform: floatSelector with: (Dialect coerceToDoublePrecisionFloat: 3.14).
	self perform: floatSelector with: (Dialect coerceToDoublePrecisionFloat: 3.1).
	self perform: floatSelector with: (Dialect readFixedPointFrom:  '321.12').
	self perform: floatSelector with: (Dialect readFixedPointFrom: '321.123').
	self perform: floatSelector with: (Dialect readFixedPointFrom: '3.14').
	self perform: floatSelector with: (Dialect readFixedPointFrom: '3.1').! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpTimeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpTimeTest methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform time! !

!GlorpTimeTest methodsFor: 'tests' stamp: ''!
testTime
	self helpTestValue: nil.
	self
		helpTestValue: Time now
		compareWith:
			[:read :original | 
			(read hours = original hours and: [read minutes = original minutes])
				and: [read seconds truncated = original seconds truncated]].! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpTimestampTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpTimestampTest methodsFor: 'tests' stamp: ''!
compareTime: read to: original
	"GNU Smalltalk seems to be quite lone in following the ANSI standard..."
	| zeroDuration max min |
	Dialect isGNU
		ifTrue: [
			zeroDuration := (Smalltalk at: #Duration) zero.
			^(read offset: zeroDuration) = (original offset: zeroDuration)].
	"MS SQL Server fails randomly because it has a resolution of 3 ms only."
	session platform class = SQLServerPlatform ifTrue: [
		max := original addMilliseconds: 3.
		min := original addMilliseconds: -3.
		^read between: min and: max].
	"Otherwise"
	^session platform supportsMillisecondsInTimes 
		ifTrue: [read = original]
		ifFalse: [read = original or: [read = (original addMilliseconds: (-1 * original milliseconds))]].! !

!GlorpTimestampTest methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform timestamp! !

!GlorpTimestampTest methodsFor: 'tests' stamp: 'GuillermoPolito 4/26/2011 04:55'!
testTimeStamp
	"This has to be UTC because postgres has time zones and will try and compensate"

	| time |
	Dialect isVisualAge ifTrue: [Transcript cr; show: 'Avoiding timestamp test for VisualAge'. ^self].

	time := Dialect timestampNow.
	(session platform supportsMillisecondsInTimes not and: [ time respondsTo: #millisecond: ])
		ifTrue: [time millisecond: 0].
	self helpTestValue: nil.
	self
		helpTestValue: time
		compareWith:
			[:read :original | 
			"MS SQL Server fails randomly because it has a resolution of 3 ms only."
			"GNU Smalltalk seems to be quite lone in following the ANSI standard..."
			self compareTime: read to: original].! !

!GlorpTimestampTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/2/2011 09:32'!
testTimeStampBetweenAnd
	| record time newRecord |
	session system: (GlorpDemoDescriptorSystem forPlatform: session platform).
	self
		inTransactionDo:
			[record := (GlorpRecordWithUpdateTime new)
				id: 12;
				name: 'thing'.
			session modify: record in: [].
			session reset.
			record := session readOneOf: GlorpRecordWithUpdateTime.
			time := record updateTime.
			session reset.
			newRecord := session
				readOneOf: GlorpRecordWithUpdateTime
				where:
					[:each | 
					each updateTime
						between: (time minusSeconds: 1)
						and: (time plusSeconds: 1)].
			self assert: newRecord id = record id.].! !

!GlorpTimestampTest methodsFor: 'tests' stamp: ''!
testTimeStampCast
	"SQLite does not have time-related types:  they are all mapped to strings.  I have not debugged why this test fails yet but assume it is completely unrealistic to expect a string to be cast to a timestamp."

	| timestamp model |
	Dialect isVisualAge ifTrue: [Transcript cr; show: 'Avoiding testTimestampCast for VisualAge'. ^self].
	timestamp := Dialect timestampNow.
	timestamp millisecond: 0.
	session transact: [session register: (GlorpTypeTestsModelClass new test: timestamp)].
	model := session readOneOf: GlorpTypeTestsModelClass where: [:each | (each test cast: session platform timestamp) = timestamp].
	self deny: model isNil
		description: 'This test will fail on SQLite which has no DB Timestamp type;  see test comment'.
	self assert: (self compareTime: model test to: timestamp).
	model := session readOneOf: GlorpTypeTestsModelClass where: [:each | (each test cast: session platform date) = timestamp].
	self deny: model isNil.
	self assert: (self compareTime: model test to: timestamp).

	model := session readOneOf: GlorpTypeTestsModelClass where: [:each | each test asTimestamp = timestamp].
	self deny: model isNil.
	self assert: (self compareTime: model test to: timestamp).! !

!GlorpTimestampTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/2/2011 09:34'!
testTimeStampCompare
	| record time newRecord |
	session system: (GlorpDemoDescriptorSystem forPlatform: session platform).
	self
		inTransactionDo:
			[record := (GlorpRecordWithUpdateTime new)
				id: 12;
				name: 'thing'.
			session modify: record in: [].
			session reset.
			record := session readOneOf: GlorpRecordWithUpdateTime.
			time := record updateTime.
			session reset.
			newRecord := session
				readOneOf: GlorpRecordWithUpdateTime
				where:
					[:each | 
						(each updateTime > (time minusSeconds: 1))
						& (each updateTime < (time plusSeconds: 1))].
			self assert: newRecord id = record id].! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpVarchar10Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpVarchar10Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform varchar: 10.! !

!GlorpVarchar10Test methodsFor: 'tests' stamp: ''!
testVarCharWithEscapedCharacters
	stType := String.
	self helpTestValue: nil.
	#($\ $~ $` $!! $@ $# $$ $% $^ $& $* $( $) $_ $- $+ $= $| $} ${ $] $[ $" $' $: $; $? $/ $> $. $< $,)
		do: [:ea | self helpTestValue: 'abc' , (String with: ea) , 'def']! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpVarchar1Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpVarchar1Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform varchar: 1.! !

!GlorpVarchar1Test methodsFor: 'tests' stamp: ''!
testBooleanToTFString

	stType := Boolean.
	self helpTestValue: nil.
	self helpTestValue: true.
	self helpTestValue: false.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpVarchar2Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpVarchar2Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform varchar: 2.! !

!GlorpVarchar2Test methodsFor: 'tests' stamp: ''!
testVarChar2
	stType := String.
	self helpTestValue: nil.
	self helpTestValue: '' compareWith: [:read :original |
		self platform usesNullForEmptyStrings ifTrue: [read = nil] ifFalse: [read = original]].
	self helpTestValue: 'a'.
	self helpTestValue: 'ab'.
	self helpTestInvalidValue: 'abc'.
	self helpTestInvalidValue: 'abcd'.
	self helpTestInvalidValue: 'abcde'.! !

GlorpDatabaseTypeIndividualDBTests subclass: #GlorpVarchar4Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpVarchar4Test methodsFor: 'types' stamp: ''!
defaultDatabaseType
	^self platform varchar: 4.! !

!GlorpVarchar4Test methodsFor: 'tests' stamp: ''!
testVarChar4
	stType := String.
	self helpTestValue: nil.
	self helpTestValue: '' compareWith: [:read :original |
		self platform usesNullForEmptyStrings ifTrue: [read = nil] ifFalse: [read = original]].
	self helpTestValue: 'a'.
	self helpTestValue: 'ab'.
	self helpTestValue: 'abc'.
	self helpTestValue: 'abcd'.
	stType := Symbol.
	self helpTestValue: #abcd.
	stType := nil.
	self helpTestInvalidValue: 'abcde'.! !

GlorpTestCase subclass: #GlorpDeleteInUnitOfWorkTest
	instanceVariableNames: 'unitOfWork'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDeleteInUnitOfWorkTest methodsFor: 'support' stamp: ''!
setUp
	| session |
	session := GlorpMockSession new.
	session beginUnitOfWork.
	unitOfWork := session privateGetCurrentUnitOfWork.
	session system: (GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database)! !

!GlorpDeleteInUnitOfWorkTest methodsFor: 'As yet unclassified' stamp: ''!
system
	^unitOfWork session system.! !

!GlorpDeleteInUnitOfWorkTest methodsFor: 'As yet unclassified' stamp: ''!
tableNamed: aString 
	^self system tableNamed: aString.! !

!GlorpDeleteInUnitOfWorkTest methodsFor: 'support' stamp: ''!
tearDown

	unitOfWork := nil.! !

!GlorpDeleteInUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testCommit

	unitOfWork delete: GlorpAddress example1.
	unitOfWork commit.
	self assert: (unitOfWork numberOfRows) = 1.! !

!GlorpDeleteInUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testDeleteRegistration
	| obj |
	obj := Object new.
	unitOfWork delete: obj.
	self assert: (unitOfWork willDelete: obj).
	self deny: (unitOfWork willDelete: 3).! !

!GlorpDeleteInUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testDeletesComeAfterUpdates
	unitOfWork delete: GlorpAddress example1.
	unitOfWork register: GlorpCustomer example1.
	unitOfWork commit.
	self assert: unitOfWork session rows last table == (self tableNamed: 'GR_ADDRESS').! !

!GlorpDeleteInUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testDeletesInReverseOrder
	"Not that good a test, because it could be luck with only two tables. Should test this at a lower level"
	| cust trans |
	cust := GlorpCustomer example2.
	trans := cust transactions first.
	"We need to give it a primary key or else the delete logic fails"
	trans id: 42.
	unitOfWork delete: cust.
	unitOfWork delete: trans.
	unitOfWork commit.
	self assert: unitOfWork session rows last owner == cust.
	self assert: (unitOfWork session rows reverse at: 2) owner == trans.! !

!GlorpDeleteInUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testGeneratingDeleteRows
	unitOfWork delete: GlorpAddress example1.
	unitOfWork createRows.
	self assert: unitOfWork privateGetRowMap numberOfEntries = 1.
	unitOfWork rowsForTable: (self tableNamed: 'GR_ADDRESS')
		do: [:each | self assert: each forDeletion]! !

GlorpTestCase subclass: #GlorpDeleteTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDeleteTest commentStamp: '<historical>' prior: 0!
This tests deletions.!

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
checkCustomerDeletedInDatabase
	| result |
	result := session accessor 
				executeSQLString: 'SELECT * FROM GR_CUSTOMER WHERE ID=1'.
	self assert: result isEmpty! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
checkCustomerNotInCache
	self assert: (session cacheLookupForClass: GlorpCustomer key: 1) isNil! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
checkCustomerStillInCache
	self assert: (session cacheLookupForClass: GlorpCustomer key: 1) notNil! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
checkFoldersDeletedInDatabase
	| result generalResult |
	result := session accessor 
				executeSQLString: 'SELECT * FROM GR_FOLDER WHERE USER_ID=1'.
	self assert: result isEmpty.

	generalResult := session accessor 
				executeSQLString: 'SELECT * FROM GR_FOLDER'.
	self assert: generalResult size = 1.! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
checkFoldersExistInDatabase
	| result |
	result := session accessor 
				executeSQLString: 'SELECT * FROM GR_FOLDER WHERE USER_ID=1'.
	self assert: result size = 2.! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
checkFoldersNotInCache
	self assert: (session cacheLookupForClass: GlorpFolder key: #(1 'One')) isNil.
	self assert: (session cacheLookupForClass: GlorpFolder key: #(1 'One-A')) isNil.! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
checkMessagesDeletedInDatabase
	| result generalResult |
	result := session accessor 
				executeSQLString: 'SELECT * FROM GR_MESSAGE WHERE USER_ID = 1'.
	self assert: result isEmpty.

	generalResult := session accessor 
				executeSQLString: 'SELECT * FROM GR_MESSAGE'.
	self assert: generalResult size = 1.! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
checkPersonDeletedInDatabase
	| result |
	result := session accessor 
				executeSQLString: 'SELECT * FROM PERSON WHERE ID=1'.
	self assert: result isEmpty! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
doDeleteAndDoubleAddOfFoldersFor: user
	| new2 removed new |
	^session
		modify: user
		in:
			[user folders do: [:each | each printString].
			removed := user folders detect: [:each | each name = 'One'].
			user folders remove: removed.
			new := GlorpFolder new name: 'One'.
			new user: user.
			user folders add: new.
			new2 := GlorpFolder new name: 'One'.
			new2 user: user.
			user folders add: new2].! !

!GlorpDeleteTest methodsFor: 'support' stamp: 'GuillermoPolito 5/24/2011 04:45'!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
setUpCustomer
	session beginTransaction.
	session accessor 
		executeSQLStringNoResult: 'INSERT INTO GR_CUSTOMER VALUES (1,''Fred Flintstone'')'.
	^session readOneOf: GlorpCustomer where: [:each | each id = 1].! !

!GlorpDeleteTest methodsFor: 'support' stamp: 'GuillermoPolito 5/28/2011 07:06'!
setUpFolders
	[session system: (GlorpCompositeKeyDescriptorSystem forPlatform: session platform).
	session transact: [ session register: session system example1.].
	] on: Error 
		do: [:ex | 
				session rollbackTransaction.
				ex signal ].! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
setUpPersonWithAddress
	session beginTransaction.
	session accessor 
		executeSQLStringNoResult: 'INSERT INTO GR_ADDRESS VALUES (2,''Paseo Montril'', ''999'')'.
	session accessor 
		executeSQLStringNoResult: 'INSERT INTO PERSON VALUES (1,''Fred Flintstone'', 2)'.
	^session readOneOf: GlorpPerson where: [:each | each id = 1].! !

!GlorpDeleteTest methodsFor: 'support' stamp: 'GuillermoPolito 5/28/2011 06:33'!
tearDown
	super tearDown.
	session reset.
	session := nil.! !

!GlorpDeleteTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/28/2011 06:41'!
testCascadingDeleteAcrossNil
	| person addressRows personRows |
	session system: (GlorpDemoDescriptorSystem forPlatform: session platform).  
	session inTransactionDo: [session accessor dropConstraint: (session system tableNamed: 'PERSON') foreignKeyConstraints first].
	[
	((session system descriptorFor: GlorpPerson) mappingForAttributeNamed: #address) beExclusive.
	self setUpPersonWithAddress .

	session accessor executeSQLString: 'delete from GR_ADDRESS where ID = 2'.
	session reset.
	person := session readOneOf: GlorpPerson.
	"First, make sure this doesn't walkback"
	[session beginUnitOfWork.
	session delete: person] ensure: [session commitUnitOfWork].
	"Now make sure the address is left"
	addressRows := session accessor executeSQLString: 'SELECT * FROM GR_ADDRESS'.
	self assert: addressRows isEmpty.
	personRows :=  session accessor executeSQLString: 'SELECT * FROM PERSON'.
	self assert: personRows isEmpty.
	
	] ensure: [
		session rollbackTransaction.
		session inTransactionDo: [session accessor createTableFKConstraints: (session system tableNamed: 'PERSON') ifError: [:ex | ex signal]].
	]! !

!GlorpDeleteTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/28/2011 07:07'!
testCascadingDeleteInstantiating
	| user |
	
	self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session readOneOf: GlorpUser where: [:each | each name = 'User One'].
	session beginUnitOfWork.
	user folders do: [:each | each printString].
	session delete: user.
	session commitUnitOfWork.
	self checkFoldersNotInCache.
	self checkFoldersDeletedInDatabase.
	self checkMessagesDeletedInDatabase! !

!GlorpDeleteTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/28/2011 07:07'!
testCascadingDeleteWithoutInstantiating
	| user |
	
	self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session readOneOf: GlorpUser where: [:each | each name = 'User One'].
	session beginUnitOfWork.
	session delete: user.
	session commitUnitOfWork.
	self checkFoldersNotInCache.
	self checkFoldersDeletedInDatabase! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
testDeleteAndReinsertTwiceWhenRemovingFromAnExclusiveCollection
	"Make sure that if we add a new thing twice, we get the expected failure of a primary key conflict."

	| user |
	[self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session
		readOneOf: GlorpUser
		where: [:each | each name = 'User One'].
	self
		should: [self doDeleteAndDoubleAddOfFoldersFor: user]
		raise: GlorpDatabaseWriteError] ensure: [session rollbackTransaction].! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
testDeleteAndReinsertWhenRemovingFromAnExclusiveCollection
	"Test that if we delete something and then re-insert with the same primary key, that it gets converted to an update. This is not really advisable as something to do, since you're really violating the primary key = object identity correspondence, but it can reasonably happen if you're using domain primary keys, link tables reified as objects, or other circumstances"
	| user removed result generalResult new |
	
	[self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session readOneOf: GlorpUser where: [:each | each name = 'User One'].
	session modify: user in: [
		user folders do: [:each | each printString].
		removed := user folders detect: [:each | each name = 'One'].
		user folders remove: removed.
		new := GlorpFolder new name: 'One'.
		new user: user.
		user folders add: new].
	self assert: (session cacheLookupForClass: GlorpFolder key: #(1 'One')) == new.
	result := session accessor executeSQLString: 'SELECT * FROM GR_FOLDER WHERE USER_ID=1'.
	self assert: result size = 2.

	generalResult := session accessor executeSQLString: 'SELECT * FROM GR_FOLDER'.
	self assert: generalResult size = 3.

	result := session accessor executeSQLString: 'SELECT * FROM GR_MESSAGE WHERE USER_ID = 1'.
	self assert: result isEmpty.

	generalResult := session accessor executeSQLString: 'SELECT * FROM GR_MESSAGE'.
	self assert: generalResult size = 1.] 
			ensure: [session rollbackTransaction]! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
testDeleteFromAnExclusiveCollectionContainingNil
	| user removed |
	
	[self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session readOneOf: GlorpUser where: [:each | each name = 'User One'].
	session modify: user in: [
		user folders do: [:each | each printString].
		removed := user folders detect: [:each | each name = 'One'].
		user folders remove: removed.
		user folders add: nil]
	"The only assertion is that we shouldn't be crashing here."
	] 
			ensure: [session rollbackTransaction]! !

!GlorpDeleteTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/28/2011 06:41'!
testDeleteMultipleTable
	| passengerRow1 passengerRow2 passenger rows frequentFlyerRow1 frequentFlyerRow2 |
	
	[session system: (GlorpDemoDescriptorSystem forPlatform: session platform).  
	session beginTransaction.
	passengerRow1 := session system examplePassengerRow.
	session writeRow: passengerRow1.
	frequentFlyerRow1 := session system exampleFrequentFlyerRow.
	session writeRow: frequentFlyerRow1.
	passengerRow2 := session system examplePassengerRow2.
	session writeRow: passengerRow2.
	frequentFlyerRow2 := session system exampleFrequentFlyerRow2.
	session writeRow: frequentFlyerRow2.
	passenger := session readOneOf: GlorpPassenger.
	session delete: passenger.
	rows := session accessor executeSQLString: 'SELECT * FROM FREQUENT_FLYER'.
	self assert: rows size = 1.
	rows := session accessor executeSQLString: 'SELECT * FROM PASSENGER'.
	self assert: rows size = 1.
	] ensure: [
		session rollbackTransaction.
	]! !

!GlorpDeleteTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/28/2011 06:40'!
testDeleteMultipleTableWithOneMissing
	| passengerRow1 passengerRow2 passenger rows frequentFlyerRow2 |
	
	[session system: (GlorpDemoDescriptorSystem forPlatform: session platform).  
	session beginTransaction.
	passengerRow1 := session system examplePassengerRow.
	session writeRow: passengerRow1.
	passengerRow2 := session system examplePassengerRow2.
	session writeRow: passengerRow2.
	frequentFlyerRow2 := session system exampleFrequentFlyerRow2.
	session writeRow: frequentFlyerRow2.
	passenger := session readOneOf: GlorpPassenger where: [:each | each id = 1].
	session delete: passenger.
	rows := session accessor executeSQLString: 'SELECT * FROM FREQUENT_FLYER'.
	self assert: rows size = 1.
	rows := session accessor executeSQLString: 'SELECT * FROM PASSENGER'.
	self assert: rows size = 1.
	] ensure: [
		session rollbackTransaction.
	]! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
testDeleteWhenRemovingFromAnExclusiveCollection
	| user removed result generalResult |
	
	[self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session readOneOf: GlorpUser where: [:each | each name = 'User One'].
	session modify: user in: [
		user folders do: [:each | each printString].
		removed := user folders detect: [:each | each name = 'One'].
		user folders remove: removed].
	self assert: (session cacheLookupForClass: GlorpFolder key: #(1 'One')) isNil.
	result := session accessor executeSQLString: 'SELECT * FROM GR_FOLDER WHERE USER_ID=1'.
	self assert: result size = 1.

	generalResult := session accessor executeSQLString: 'SELECT * FROM GR_FOLDER'.
	self assert: generalResult size = 2.

	result := session accessor executeSQLString: 'SELECT * FROM GR_MESSAGE WHERE USER_ID = 1'.
	self assert: result isEmpty.

	generalResult := session accessor executeSQLString: 'SELECT * FROM GR_MESSAGE'.
	self assert: generalResult size = 1.] 
			ensure: [session rollbackTransaction]! !

!GlorpDeleteTest methodsFor: 'tests' stamp: ''!
testNormalDeleteOnFolders
	| user |
	
	[self setUpFolders.
	self checkFoldersExistInDatabase.
	user := session readOneOf: GlorpUser where: [:each | each name = 'User One'].
	session beginUnitOfWork.
	session delete: user.
	user folders do: [:each | 
		session delete: each.
		each messages do: [:eachMessage |
			session delete: eachMessage]].
	session commitUnitOfWork.
	self checkFoldersNotInCache.
	self checkFoldersDeletedInDatabase.
	self checkMessagesDeletedInDatabase] 
			ensure: [session rollbackTransaction]! !

!GlorpDeleteTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/28/2011 06:40'!
testUnitOfWorkDelete
	| customer result |
	
	[session system: (GlorpDemoDescriptorSystem forPlatform: session platform).  
	customer := self setUpCustomer.
	session beginUnitOfWork.
	session delete: customer.
	result := session accessor 
				executeSQLString: 'SELECT * FROM GR_CUSTOMER WHERE ID=1'.
	self assert: result size = 1.
	self 
		assert: (session readOneOf: GlorpCustomer where: [:each | each id = 1]) isNil.
	self checkCustomerStillInCache.
	session commitUnitOfWork.
	self checkCustomerNotInCache.
	self checkCustomerDeletedInDatabase
	] ensure: [
		session rollbackTransaction.
	]! !

!GlorpDeleteTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/28/2011 06:40'!
testUnitOfWorkDeleteOrder
	| person |
	
	[session system: (GlorpDemoDescriptorSystem forPlatform: session platform).  
	person := self setUpPersonWithAddress.
	session beginUnitOfWork.
	session delete: person.
	session delete: person address.
	session commitUnitOfWork.
	self checkPersonDeletedInDatabase.
	]ensure: [
		session rollbackTransaction.
	]! !

GlorpTestCase subclass: #GlorpDialectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDialectTest commentStamp: '<historical>' prior: 0!
Tests the portability methods in the Dialect class.!

!GlorpDialectTest methodsFor: 'tests' stamp: ''!
testBlockArgumentCount
	self assert: (Dialect argumentCountFor: [3]) = 0.
	self assert: (Dialect argumentCountFor: [:a | 3]) = 1.
	self assert: (Dialect argumentCountFor: [:a :b | a <= b]) = 2.! !

!GlorpDialectTest methodsFor: 'tests' stamp: ''!
testTimeSetMillisecond
	| time oldMs oldSec oldMin newMs |
	Dialect supportsMillisecondsInTimes ifFalse: [^self].
	time := Time now.
	oldMs := time milliseconds.
	oldSec := time seconds truncated.
	oldMin := time minutes.
	newMs := oldMs > 989 ifTrue: [3] ifFalse: [oldMs + 10].
	time millisecond: newMs.
	self assert: time milliseconds = newMs.
	self assert: time seconds truncated = oldSec.
	self assert: time minutes = oldMin.! !

!GlorpDialectTest methodsFor: 'tests' stamp: ''!
testTokensBasedOn
	self assert: (Dialect tokensBasedOn: '.' in: 'abc.def.ghi') asArray = #('abc' 'def' 'ghi').! !

GlorpTestCase subclass: #GlorpErrorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpErrorTest methodsFor: 'tests' stamp: ''!
testMessageTextVW
	| innerException text outerException error |
	Dialect isVisualWorks ifFalse: [^self].
	text := 'Something is horribly wrong'.
	error := (Dialect smalltalkAt: #ExternalDatabaseError) new dbmsErrorString: text.
	outerException := GlorpDatabaseReadError new databaseError: (Array with: error).
	[outerException signal] on: GlorpError do: [:ex |	
		self assert: ex messageText = text].
	
	innerException := (Dialect smalltalkAt: #ExternalDatabaseException) new.
	innerException parameter: (Array with: error).
	[DatabaseAccessor new handleError: innerException for: InsertCommand new ] on: GlorpError do: [:ex |	
		self assert: ex messageText = text.
		ex return: nil].! !

GlorpTestCase subclass: #GlorpExpressionBasicPropertiesTest
	instanceVariableNames: 'base'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpExpressionBasicPropertiesTest methodsFor: 'support' stamp: ''!
setUp

	base := BaseExpression new descriptor: (
		(GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database)
			descriptorFor: GlorpPerson)! !

!GlorpExpressionBasicPropertiesTest methodsFor: 'tests' stamp: ''!
testHasDescriptorForBase

	| exp |
	self assert: base hasDescriptor.
	exp := [:a | a ] asGlorpExpressionOn: base.
	self assert: exp hasDescriptor.! !

!GlorpExpressionBasicPropertiesTest methodsFor: 'tests' stamp: ''!
testHasDescriptorForDirect

	| exp |
	exp := [:a | a id ] asGlorpExpressionOn: base.
	self deny: exp hasDescriptor.! !

!GlorpExpressionBasicPropertiesTest methodsFor: 'tests' stamp: ''!
testHasDescriptorForOneToMany

	| exp |
	exp := [:a | a emailAddresses ] asGlorpExpressionOn: base.
	self assert: exp hasDescriptor.! !

!GlorpExpressionBasicPropertiesTest methodsFor: 'tests' stamp: ''!
testHasDescriptorForOneToOne

	| exp |
	exp := [:a | a address ] asGlorpExpressionOn: base.
	self assert: exp hasDescriptor.! !

!GlorpExpressionBasicPropertiesTest methodsFor: 'tests' stamp: ''!
testHasDescriptorForPrimaryKeyExpression

	| exp |
	exp := Join new.
	self deny: exp hasDescriptor.! !

!GlorpExpressionBasicPropertiesTest methodsFor: 'tests' stamp: ''!
testHasDescriptorForRelation

	| exp |
	exp := [:a | a = 3] asGlorpExpressionOn: base.
	self deny: exp hasDescriptor.! !

!GlorpExpressionBasicPropertiesTest methodsFor: 'tests' stamp: ''!
testHasDescriptorForTwoLevelDirect

	| exp |
	exp := [:a | a address street] asGlorpExpressionOn: base.
	self deny: exp hasDescriptor.
	self assert: exp base hasDescriptor.! !

!GlorpExpressionBasicPropertiesTest methodsFor: 'tests' stamp: ''!
testHasDescriptorForUninitializedBase

	self deny: BaseExpression new hasDescriptor.! !

GlorpTestCase subclass: #GlorpExpressionIterationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
helpTestSingleNodeDo: exp 
	self assert: (exp collect: [:each | each]) = (OrderedCollection with: exp)! !

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
testDoBase
	self helpTestSingleNodeDo:BaseExpression new! !

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
testDoCollection

	| exp l r |
	exp := CollectionExpression new.
	l := BaseExpression new.
	r := BaseExpression new.
	exp leftChild: l; rightChild: r.
	self assert: ((exp collect: [:each | each]) = (OrderedCollection with: r with: exp)).! !

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
testDoConstant
	self helpTestSingleNodeDo: ConstantExpression new! !

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
testDoField

	| exp |
	exp := FieldExpression new.
	exp field: nil base: BaseExpression new.
	self assert: ((exp collect: [:each | each]) = (OrderedCollection with: exp base with: exp)).! !

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
testDoMapping

	| exp |
	exp := MappingExpression new.
	exp named: 'foo' basedOn: BaseExpression new.
	self assert: ((exp collect: [:each | each]) = (OrderedCollection with: exp base with: exp)).! !

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
testDoParameter

	| exp |
	exp := ParameterExpression new.
	exp field: nil base: BaseExpression new.
	self assert: ((exp collect: [:each | each]) = (OrderedCollection with: exp base with: exp)).! !

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
testDoRelation

	| exp l r |
	exp := RelationExpression new.
	l := BaseExpression new.
	r := BaseExpression new.
	exp leftChild: l; rightChild: r.
	self assert: ((exp collect: [:each | each]) = (OrderedCollection with: l with: r with: exp)).! !

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
testDoTable

	| exp |
	exp := TableExpression new.
	exp table: nil base: BaseExpression new.
	self assert: ((exp collect: [:each | each]) = (OrderedCollection with: exp base with: exp)).! !

!GlorpExpressionIterationTest methodsFor: 'tests' stamp: ''!
testDoWithCommonBase

	| exp l r base |
	exp := RelationExpression new.
	base := BaseExpression new.
	l := MappingExpression new.
	l named: nil basedOn: base.
	r := MappingExpression new.
	r named: nil basedOn: base.
	exp leftChild: l; rightChild: r.	
	self assert: ((exp collect: [:each | each]) = (OrderedCollection with: base with: l with: r with: exp)).! !

GlorpTestCase subclass: #GlorpExpressionRebuildingTest
	instanceVariableNames: 'original newBase rebuilt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpExpressionRebuildingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	newBase := BaseExpression new.! !

!GlorpExpressionRebuildingTest methodsFor: 'tests' stamp: ''!
testLoneBase
	original := BaseExpression new.
	rebuilt := original rebuildOn: newBase.
	self assert: rebuilt == newBase.! !

!GlorpExpressionRebuildingTest methodsFor: 'tests' stamp: ''!
testMapping
	original := [:each | each foo] asGlorpExpression.
	rebuilt := original rebuildOn: newBase.
	self assert: rebuilt base == newBase.! !

!GlorpExpressionRebuildingTest methodsFor: 'tests-outer scope references' stamp: ''!
testRebuildDoubleInnerScopeNormallyWithExists
	original := [:each | 
	each foo
		anySatisfyExists:
			[:eachFoo | eachFoo things anySatisfyExists: [:eachThing | eachThing stuff = each fooThing]]]
		asGlorpExpression.
	rebuilt := original rebuildOn: newBase.
	rebuilt do: [:each | self assert: each ultimateBaseExpression == newBase].! !

!GlorpExpressionRebuildingTest methodsFor: 'tests-outer scope references' stamp: ''!
testRebuildDoubleInnerScopeNormallyWithJoins
	original := [:each | 
	each foo
		anySatisfyJoin:
			[:eachFoo | eachFoo things anySatisfyJoin: [:eachThing | eachThing stuff = each fooThing]]]
		asGlorpExpression.
	rebuilt := original rebuildOn: newBase.
	rebuilt do: [:each | self assert: each ultimateBaseExpression == newBase].! !

!GlorpExpressionRebuildingTest methodsFor: 'tests-outer scope references' stamp: ''!
testRebuildInnerScopeForSubquery

	| foo |
	original := [:each | each foo anySatisfyJoin: [:eachFoo | eachFoo thing = each fooThing]] asGlorpExpression.
	"Just so that it prints distinctively"
	original ultimateBaseExpression descriptor: (Descriptor new classModel: (GlorpClassModel for: Object)).
	foo := original leftChild.
	newBase := SubSelectBaseExpression on: foo.
	rebuilt := original rightChild rebuildOn: newBase startingFrom: foo.
	self assert: rebuilt leftChild base == newBase.
	self assert: rebuilt leftChild ultimateBaseExpression == original ultimateBaseExpression.
	self assert: rebuilt rightChild base == original ultimateBaseExpression.! !

!GlorpExpressionRebuildingTest methodsFor: 'tests-outer scope references' stamp: ''!
testRebuildInnerScopeNormally
	original := [:each | each foo anySatisfy: [:eachFoo | eachFoo thing = each fooThing]]
		asGlorpExpression.
	rebuilt := original rebuildOn: newBase.
	rebuilt do: [:each | self assert: each ultimateBaseExpression == newBase].! !

!GlorpExpressionRebuildingTest methodsFor: 'tests-outer scope references' stamp: ''!
testRebuildMappingWithStart

	original := [:each | each foo bar] asGlorpExpression.
	rebuilt := original rebuildOn: newBase startingFrom: original base.
	self assert: rebuilt base == newBase.! !

!GlorpExpressionRebuildingTest methodsFor: 'tests' stamp: ''!
testRelation
	original := [:each | each foo bar = each foo bletch] asGlorpExpression.
	rebuilt := original rebuildOn: newBase.
	self assert: rebuilt ultimateBaseExpression == newBase.
	self assert: rebuilt leftChild ultimateBaseExpression == newBase.
	self assert: rebuilt rightChild ultimateBaseExpression == newBase.
	self assert: rebuilt leftChild base base == newBase.! !

GlorpTestCase subclass: #GlorpFilteredInheritanceTest
	instanceVariableNames: 'session allEmployees'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpFilteredInheritanceTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource with: GlorpSessionResource! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
compareEmployees: employees

	employees do: [:each |
		| corresponding |
		corresponding := allEmployees detect: [:eachOriginal | each id = eachOriginal id].
		self assert: corresponding = each].! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
makeSupertaxIn: aCollection
	| previous newTax |
	previous := aCollection last.
	newTax := (GlorpBasicTax new)
		name: (String with: (Character value: previous name first asInteger + 1));
		yourself.
	previous addTax: newTax.
	aCollection add: newTax.! !

!GlorpFilteredInheritanceTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	session system: (GlorpInheritanceDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database).! !

!GlorpFilteredInheritanceTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session reset.
	session := nil.! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
testCacheLookup
	"Ask for an Employee which should be from the cache and which should return a Manager."

	| manager employee |
	session beginTransaction.
	
	[session beginUnitOfWork.
	self writeTestHarness.
	session commitUnitOfWork.
	session initializeCache.
	manager := session readOneOf: GlorpManager
				where: [:each | each id = 3 ].
	self assert: (session cacheLookupForClass: GlorpEmployee key: 3) == manager.
	employee := session readOneOf: GlorpEmployee
				where: [:each | each id = 3 ].
	self assert: employee == manager.

	manager := session readOneOf: GlorpRegionalManager
				where: [:each | each id = 12 ].
	employee := session readOneOf: GlorpEmployee
				where: [:each | each id = 12 ].
	self assert: employee == manager.

	employee := session readOneOf: GlorpManager
				where: [:each | each id = 11 ].
	manager := session readOneOf: GlorpEmployee
				where: [:each | each id = 11 ].
	self assert: employee == manager.

	"Test that the cache refuses to return an object which is not of the proper class or subclass."
	employee := session readOneOf: GlorpEmployee
				where: [:each | each id = 4 ].
	self assert: (session privateGetCache lookupClass: GlorpRegionalManager key: 4 ifAbsent: []) isNil.
	manager := session readOneOf: GlorpRegionalManager
				where: [:each | each id = 4 ].
	self assert: manager isNil.

	"Proxys seem to try a cache lookup before they execute their query...can we write a test which fails due to this?"] 
			ensure: [session rollbackTransaction]! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
testDirectQuery
	"Ask for all Employees, see if we get subclasses too"

	| employees offices |
	session beginTransaction.
	
	[session beginUnitOfWork.
	self writeTestHarness.
	session commitUnitOfWork.
	session initializeCache.
	employees := true 
				ifTrue: [session readManyOf: GlorpEmployee where: [:each | each name = 'Bob']]
				ifFalse: [session halt readManyOf: GlorpEmployee].
	self assert: employees size = 8.
	self 
		assert: (employees select: [:emp | emp isMemberOf: GlorpEmployee]) size = 1.
	self assert: (employees select: [:emp | emp isMemberOf: GlorpManager]) size = 2.
	self 
		assert: (employees select: [:emp | emp isMemberOf: GlorpLineWorker]) size = 4.
	self 
		assert: (employees select: [:emp | emp isMemberOf: GlorpRegionalManager]) size 
				= 1.
	self compareEmployees: employees.
	session initializeCache.
	offices := session readOneOf: GlorpOffice
				where: [:each | each employeeOfMonth name = 'Bob'].	"There is no regional manager with id = 4 but we can ensure that the type info is getting into the query's key by asking for one and seeing that it doesn't exist"
	session initializeCache.
	self 
		assert: (session readOneOf: GlorpRegionalManager where: [:each | each id = 4]) 
				== nil] 
			ensure: [session rollbackTransaction]! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
testDirectQuery2
	"Ask for all Employees, see if we get subclasses too"

	| employees |
	session beginTransaction.
	
	[session beginUnitOfWork.
	self writeTestHarness.
	session commitUnitOfWork.
	session initializeCache.
	employees := session readManyOf: GlorpEmployee.
	self assert: employees size = 12.
	self 
		assert: (employees select: [:emp | emp isMemberOf: GlorpEmployee]) size = 2.
	self assert: (employees select: [:emp | emp isMemberOf: GlorpManager]) size = 3.
	self 
		assert: (employees select: [:emp | emp isMemberOf: GlorpLineWorker]) size = 5.
	self 
		assert: (employees select: [:emp | emp isMemberOf: GlorpRegionalManager]) size 
				= 2.
] 
			ensure: [session rollbackTransaction]! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
testDirectQueryWithAbstractClass
	| fromdb |
	session beginTransaction.
	[session inUnitOfWorkDo: [	session register: (GlorpBasicTax new name: 'Tax'; yourself).
								session register: (GlorpSurcharge new name: 'Surcharge'; yourself)].
	session reset.
	fromdb := session readManyOf: GlorpTax.
	self assert: fromdb size = 2.
	self assert: (fromdb select: [:emp | emp isMemberOf: GlorpTax]) isEmpty.
	self assert: (fromdb select: [:emp | emp isMemberOf: GlorpBasicTax]) size = 1.
	self assert: (fromdb select: [:emp | emp isMemberOf: GlorpSurcharge]) size = 1] 
			ensure: [session rollbackTransaction]! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
testNotInSelect
	"Ask for an Employee which should be from the cache and which should return a Manager."

	| testEmployees |
	session beginTransaction.
	
	[session beginUnitOfWork.
	self writeTestHarness.
	session commitUnitOfWork.
	testEmployees := session readManyOf: GlorpManager
				where: [:each | each id notIn: #(1 2 3)].
	self assert: testEmployees notEmpty

	"Proxys seem to try a cache lookup before they execute their query...can we write a test which fails due to this?"] 
			ensure: [session rollbackTransaction]! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
testOrderBy
	| employees query |
	session beginTransaction.
	
	[session beginUnitOfWork.
	self writeTestHarness.
	session commitUnitOfWork.
	query := Query returningManyOf: GlorpEmployee
				where: [:each | each id <= 4].
	query orderBy: #name.
	query orderBy: #id.
	employees := session execute: query.
	self assert: (employees asSortedCollection: [:a :b | a name = b name ifTrue: [
		a id <= b id] ifFalse: [a name < b name]]) asArray = employees]
			ensure: [session rollbackTransaction]! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
testRelationshipQuery
	"Ask for all Employees in a given office and test that the return types are correct."

	| employees office |
	session beginTransaction.
	
	[session beginUnitOfWork.
	self writeTestHarness.
	session commitUnitOfWork.
	session initializeCache.
	office := session readOneOf: GlorpOffice
				where: [:each | each id = 1 ].
	employees := office employees.
	self assert: employees size = 6.
	self assert: (employees select: [:emp | emp isMemberOf: GlorpEmployee]) size = 2.
	self assert: (employees select: [:emp | emp isMemberOf: GlorpManager]) size = 1.
	self assert: (employees select: [:emp | emp isMemberOf: GlorpLineWorker]) size = 2.
	self assert: (employees select: [:emp | emp isMemberOf: GlorpRegionalManager]) size = 1] 
			ensure: [session rollbackTransaction]! !

!GlorpFilteredInheritanceTest methodsFor: 'tests' stamp: ''!
testTaxInsertOrdering
	| taxes first |
	taxes := OrderedCollection new.
	first := (GlorpBasicTax new)
		name: 'A';
		yourself.
	taxes add: first.
	10 timesRepeat: [self makeSupertaxIn: taxes].
	[session beginTransaction.
	session inUnitOfWorkDo: [session registerAll: taxes]]
		ensure: [session rollbackTransaction].! !

!GlorpFilteredInheritanceTest methodsFor: 'support' stamp: ''!
writeTestHarness
	| office1 office2  eom1 eom2 jim bob |
	session register: (office1 := GlorpOffice new id: 1; yourself).
	session register: (office2 := GlorpOffice new id: 2; yourself).
	office1 addEmployee: (eom1 := GlorpEmployee new id: 1; name: 'Bob'; yourself).
	office1 addEmployee: (GlorpEmployee new id: 2; name: 'Jim'; yourself).
	office1 addEmployee: (GlorpManager new id: 3; name: 'Bob'; branch: 'West'; yourself).
	office2 addEmployee: (eom2 := GlorpManager new id: 4; name: 'Steve'; branch: 'East'; yourself).
	office2 addEmployee: (GlorpManager new id: 5; name: 'Bob'; branch: 'South'; yourself).
	office1 addEmployee: (GlorpLineWorker new id: 6; name: 'Wally'; productionLine: 'Gold'; yourself).
	office1 addEmployee: (GlorpLineWorker new id: 7; name: 'Bob'; productionLine: 'Silver'; yourself).
	office2 addEmployee: (GlorpLineWorker new id: 8; name: 'Bob'; productionLine: 'Tin'; yourself).
	office2 addEmployee: (GlorpLineWorker new id: 9; name: 'Bob'; productionLine: 'Copper'; yourself).
	office2 addEmployee: (GlorpLineWorker new id: 10; name: 'Bob'; productionLine: 'Steel'; yourself).
	office1 addEmployee: (GlorpRegionalManager new id: 11; name: 'Bob'; branch: 'South'; region: 'MidWest'; yourself).
	office2 addEmployee: (GlorpRegionalManager new id: 12; name: 'Mike'; branch: 'North'; region: 'NorthEast'; yourself).
	office1 employeeOfMonth: eom1.
	office2 employeeOfMonth: eom2.

	session register: (jim := GlorpWorkingStiff new id: 13; name: 'Jim'; yourself).
	session register: (bob := GlorpWorkingStiff new id: 14; name: 'Bob'; yourself).

	allEmployees := (Array with: jim with: bob), office1 employees, office2 employees.! !

GlorpTestCase subclass: #GlorpHorizontalInheritanceTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpHorizontalInheritanceTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource with: GlorpSessionResource! !

!GlorpHorizontalInheritanceTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	session system: (GlorpInheritanceDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database).
	session beginTransaction.! !

!GlorpHorizontalInheritanceTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.
	session reset.! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests' stamp: ''!
testClassSelection

	| classes |
	classes := (session system descriptorFor: GlorpInventoryItem) classesRequiringIndependentQueries collect: [:each | each name ].
	self assert: classes size = 3.
	#(#GlorpNonperishableItem #GlorpPerishableItem #GlorpUnassembledItem) do: [:name |
		self assert: (classes includes: name) ].

	classes := (session system descriptorFor: GlorpPerishableItem) classesRequiringIndependentQueries collect: [:each | each name ].
	self assert: classes size = 1.
	#(#GlorpPerishableItem) do: [:name |
		self assert: (classes includes: name) ].

	classes := (session system descriptorFor: GlorpNonperishableItem) classesRequiringIndependentQueries collect: [:each | each name ].
	self assert: classes size = 2.
	#(#GlorpNonperishableItem #GlorpUnassembledItem) do: [:name |
		self assert: (classes includes: name)].! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests' stamp: ''!
testDirectQuery

	| items query |
	self writeTestHarness.
	query := Query readManyOf: GlorpInventoryItem
				where: [:each | each name = 'TV'].
	items := session execute: query.
	self assert: items size = 1.
	self assert: (items select: [:emp | emp isMemberOf: GlorpNonperishableItem]) size = 1.

	session reset.
	items := session readManyOf: GlorpInventoryItem
				where: [:each | each name = 'bicycle'].
	self assert: items size = 1.
	self assert: (items select: [:emp | emp isMemberOf: GlorpUnassembledItem]) size = 1.

	session reset.
	items := session readManyOf: GlorpPerishableItem
				where: [:each | each name = 'bicycle'].
	self assert: items size = 0.! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests' stamp: ''!
testDirectSingleQueryForSubclasses

	| items query |
	self writeTestHarness.
	(session system descriptorFor: GlorpInventoryItem) typeResolver useSingleQuery: true.
	query := Query readManyOf: GlorpInventoryItem
				where: [:each | each name = 'TV'].
	items := session execute: query.
	self assert: items size = 1.
	self assert: (items select: [:emp | emp isMemberOf: GlorpNonperishableItem]) size = 1.

	session reset.
	items := session readManyOf: GlorpInventoryItem
				where: [:each | each name = 'bicycle'].
	self assert: items size = 1.
	self assert: (items select: [:emp | emp isMemberOf: GlorpUnassembledItem]) size = 1.

	session reset.
	items := session readManyOf: GlorpPerishableItem
				where: [:each | each name = 'bicycle'].
	self assert: items size = 0.! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests' stamp: ''!
testDirectSingleQueryForSubclassesNoWhereClause

	| items query normalResult filtered |
	self writeTestHarness.
	(session system descriptorFor: GlorpInventoryItem) typeResolver useSingleQuery: true.
	query := Query readManyOf: GlorpInventoryItem.
	items := session execute: query.
	self assert: items size = 7.
	GlorpInventoryItem allSubclasses do: [:each |
		normalResult := session read: each.
		filtered := items select: [:other | other isKindOf: each].
		self assert: normalResult size = filtered size].! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests - tags' stamp: ''!
testForwardLookup
	| image |
	self writeTestHarnessForTags.
	image := session
				readOneOf: GlorpImage
				where: [:each | each location = 'testlocation'].
	self assert: image tags size = 2.
	self assert: (image tags contains: [:tag| tag key = 'first']).
	self assert: (image tags contains: [:tag| tag key = 'second']).! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests' stamp: ''!
testNilVariableJoin
	| warehouse |
	self writeTestHarness.
	warehouse := session readOneOf: GlorpWarehouse
				where: [:each | each address = '123 Some Ave.'].
	session beginUnitOfWork.
	session register: warehouse.
	warehouse currentItem: nil.
	session commitUnitOfWork.
	session reset.
	warehouse := session readOneOf: GlorpWarehouse
				where: [:each | each address = '123 Some Ave.'].
	self assert: warehouse currentItem isNil! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests' stamp: ''!
testOrderBy
	"We can't use database-level ordering in horizontal inheritance because it does multiple queries. We could, I suppose, sort after the fact, but we don't right now"

	| items query |
	query := Query returningManyOf: GlorpInventoryItem
				where: [:each | each id <= 4].
	query orderBy: #name.
	self should: [items := session execute: query] raise: Dialect error.! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests' stamp: ''!
testReadNonAbstractPolymorphic
	"This test excercises a problem that existed with MappingExpression caching its mapping
	and which was then copied to the newly created MappingExpressions for subclasses.
	This isn't a problem when reading from an abstract class since there are no mappings to cache."

	| wagon |
	
	self writeTestHarness.
	wagon := session readOneOf: GlorpNonperishableItem where: [:each | each name = 'wagon'].
	self should: [wagon serialNumber = 99958]! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests' stamp: ''!
testReadRelationship
	| warehouse |
	
	self writeTestHarness.
	warehouse := session readOneOf: GlorpWarehouse
				where: [:each | each address = '123 Some Ave.'].
	self assert: warehouse address = '123 Some Ave.'.
	self assert: warehouse currentItem name = 'squash'.
	self assert: (warehouse items collect: [:each | each name]) asSortedCollection asArray = #('apples' 'bicycle' 'fridge').! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests' stamp: ''!
testReadRelationshipSingleQuery
	| warehouse |

	(session system descriptorFor: GlorpInventoryItem) typeResolver useSingleQuery: true.	
	self writeTestHarness.
	warehouse := session readOneOf: GlorpWarehouse
				where: [:each | each address = '123 Some Ave.'].
	self assert: warehouse address = '123 Some Ave.'.
	self assert: warehouse currentItem name = 'squash'.
	self assert: (warehouse items collect: [:each | each name]) asSortedCollection asArray = #('apples' 'bicycle' 'fridge').! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests - tags' stamp: ''!
testReverseLookup
	| image |
	self writeTestHarnessForTags.
	image := session
				readOneOf: GlorpImage
				where: [:each | each tags anySatisfy: [:tag| tag key = 'third']].
	self assert: image tags size = 2.
	self assert: (image tags contains: [:tag| tag key = 'first'])not.
	self assert: (image tags contains: [:tag| tag key = 'second']).! !

!GlorpHorizontalInheritanceTest methodsFor: 'tests - tags' stamp: ''!
testTagObjectLookup
	| image |
	self writeTestHarnessForTags.
	image := session
				readOneOf: GlorpTaggableObject
				where: [:each | each tags
						anySatisfy: [:tag | tag key = 'third']].
	self assert: image tags size = 2.
	self assert: (image tags
			contains: [:tag | tag key = 'first']) not.
	self
		assert: (image tags
				contains: [:tag | tag key = 'second'])! !

!GlorpHorizontalInheritanceTest methodsFor: 'support' stamp: ''!
writeTestHarness

	| squash apples fridge bicycle warehouse |
	session inUnitOfWorkDo: [
	session register: (squash := GlorpPerishableItem new id: 1; name: 'squash'; age: 10; yourself).
	session register: (GlorpPerishableItem new id: 2; name: 'zucchini'; age: 14; yourself).
	session register: (apples := GlorpPerishableItem new id: 3; name: 'apples'; age: 4; yourself).
	session register: (GlorpNonperishableItem new id: 4; name: 'TV'; serialNumber: 56893; yourself).
	session register: (fridge := GlorpNonperishableItem new id: 5; name: 'fridge'; serialNumber: 12345; yourself).
	session register: (bicycle := GlorpUnassembledItem new id: 6; name: 'bicycle'; serialNumber: 83754; assemblyCost: 100; yourself).
	session register: (GlorpUnassembledItem new id: 7; name: 'wagon'; serialNumber: 99958; assemblyCost: 20; yourself).
	warehouse := GlorpWarehouse new.
	warehouse address: '123 Some Ave.'.
	warehouse currentItem: squash.
	warehouse items: (Array with: apples with: fridge with: bicycle).
	session register: warehouse].
	session reset.! !

!GlorpHorizontalInheritanceTest methodsFor: 'support' stamp: ''!
writeTestHarnessForTags

	| tag1 tag2 tag3 image1 image2 |
	session inUnitOfWorkDo: [
		tag1 := GlorpTag new label: 'First'; generateKeyFromLabel.
		tag2 := GlorpTag new label: 'Second'; generateKeyFromLabel.
		tag3 := GlorpTag new label: 'Third'; generateKeyFromLabel.

		image1 := GlorpImage new location: 'testlocation'.
		session register: image1.
	
		image1 tags add: tag1.
		image1 tags add: tag2.
	
		image2 := GlorpImage new location: 'testlocation2'.
		session register: image2.
	
		image2 tags add: tag2.
		image2 tags add: tag3].
	session reset.! !

GlorpTestCase subclass: #GlorpInsertUpdateTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpInsertUpdateTest class methodsFor: 'resources' stamp: ''!
resources

	^Array with: GlorpSessionResource.! !

!GlorpInsertUpdateTest methodsFor: 'support' stamp: ''!
rowFor: anObject

	| rowMap rows |
	rowMap := RowMap new.
	session createRowsFor: anObject in: rowMap.
	rows := rowMap rowsForKey: anObject.
	self assert: rows size = 1.
	^rows first.! !

!GlorpInsertUpdateTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.! !

!GlorpInsertUpdateTest methodsFor: 'tests' stamp: ''!
testFunctionalInsertUpdateForInsert

	| testObject |
	[session beginTransaction.
	session beginUnitOfWork.
	testObject := GlorpCustomer example1.
	testObject id: 876.
	session register: testObject.
	session commitUnitOfWork.
	self assert: (testObject seenPreWrite = true).
	self assert: (testObject seenPostWrite = true).

	session beginUnitOfWork.
	session register: testObject.
	testObject name: 'Change of name'.
	session commitUnitOfWork]
		ensure: [session rollbackTransaction]! !

!GlorpInsertUpdateTest methodsFor: 'tests' stamp: ''!
testRowOwnership

	| aCustomer rowMap |
	aCustomer := GlorpCustomer new.
	rowMap := RowMap new.
	(session descriptorFor: GlorpCustomer) createRowsFor: aCustomer in: rowMap.
	rowMap rowsDo: [:each | 
		self assert: each owner = aCustomer].! !

!GlorpInsertUpdateTest methodsFor: 'tests' stamp: ''!
testShouldInsertForInsert

	| testObject row |
	testObject := GlorpCustomer example1.
	testObject id: 876.
	row :=self rowFor: testObject.
	self assert: (session shouldInsert: row).! !

!GlorpInsertUpdateTest methodsFor: 'tests' stamp: ''!
testShouldInsertForUpdate

	| testObject row |
	session beginUnitOfWork.
	testObject := GlorpCustomer example1.
	testObject id: 876.
	session cacheAt: 876 put: testObject.
	row :=self rowFor: testObject.
	self deny: (session shouldInsert: row).! !

GlorpTestCase subclass: #GlorpMappingDBTest
	instanceVariableNames: 'system session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

GlorpMappingDBTest subclass: #GlorpConditionalMappingDBTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

GlorpMappingDBTest subclass: #GlorpDatabaseIndexTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDatabaseIndexTest methodsFor: 'tests' stamp: ''!
testNameGeneration
	| platform table field index |
	platform := GlorpDatabaseLoginResource defaultPlatform.
	table := DatabaseTable new
						name: 'A_VERY_VERY_VERY_LONG_TABLE_NAME';
						yourself.
	field := table createFieldNamed: 'A_VERY_LONG_FIELD_NAME' type: platform serial.
	index := table addIndexForField: field.
	self assert: index name size <= platform maximumLengthOfTableName.! !

!GlorpDatabaseIndexTest methodsFor: 'tests' stamp: ''!
testNameGenerationLong
	| platform table field index |
	"SQL Server has very long IDs"
	platform := SQLServerPlatform new.
	table := DatabaseTable new
						name: 'A_VERY_VERY_VERY_LONG_TABLE_NAME';
						yourself.
	field := table createFieldNamed: 'A_VERY_LONG_FIELD_NAME' type: platform serial.
	index := table addIndexForField: field.
	self assert: index name size <= platform maximumLengthOfTableName.
	self assert: index name = (table name, '_', field name, '_IDX').! !

GlorpMappingDBTest subclass: #GlorpDatabaseTableTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDatabaseTableTest methodsFor: 'tests' stamp: ''!
testFKUniqueNames

	| platform contactTable linkTable companyId personId |
	platform := GlorpDatabaseLoginResource defaultPlatform.
	contactTable := DatabaseTable new
						name: 'CONTACT';
						yourself.
	contactTable createFieldNamed: 'ID' type: platform serial.

	linkTable := DatabaseTable new
						name: 'COMPANY_PERSON_LINK';
						yourself.
	companyId := linkTable createFieldNamed: 'COMPANY_ID' type: platform int4.
	personId := linkTable createFieldNamed: 'PERSON_ID' type: platform int4.
	linkTable
		addForeignKeyFrom: companyId to: (contactTable fieldNamed: 'ID');
		addForeignKeyFrom: personId to: (contactTable fieldNamed: 'ID').
	self assert: (linkTable foreignKeyConstraints collect: [:ea | ea name asSymbol]) asSet size = linkTable foreignKeyConstraints size.! !

!GlorpDatabaseTableTest methodsFor: 'tests' stamp: ''!
testIndexes

	| platform contactTable idField name1Field name2Field |
	platform := GlorpDatabaseLoginResource defaultPlatform.
	contactTable := DatabaseTable new
						name: 'CONTACT';
						yourself.
	idField := (contactTable createFieldNamed: 'ID' type: platform serial) beIndexed.
	self assert: contactTable indexes size = 1.
	self assert: (contactTable hasIndexForField: idField).
	name1Field := contactTable createFieldNamed: 'NAME1' type: platform varchar.
	name2Field := contactTable createFieldNamed: 'NAME2' type: platform varchar.
	self deny: (contactTable hasIndexForField: name1Field).
	contactTable addIndexForFields: (Array with: name1Field with: name2Field).
	self assert: contactTable indexes size = 2.
	self assert: (contactTable hasIndexForField: idField).
	self assert: (contactTable hasIndexForField: name1Field).
	self assert: (contactTable hasIndexForField: name2Field).! !

GlorpMappingDBTest subclass: #GlorpDirectMappingDBTest
	instanceVariableNames: 'person personId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDirectMappingDBTest methodsFor: 'tests' stamp: ''!
readPerson
	| results query |
	query := Query
		returningManyOf: GlorpPerson
		where: [:pers | pers id = personId].
	results := query executeIn: session.
	self assert: results size = 1.
	person := results first.! !

!GlorpDirectMappingDBTest methodsFor: 'tests' stamp: ''!
testMappingToSubselect

	| personDescriptor people |
	[session beginTransaction.
	self writeAddress.
	self writeHomelessPerson.
	self writePersonWithEmailAddresses.
	personDescriptor := system descriptorFor: GlorpPerson.
	(personDescriptor newMapping: DirectMapping)
		beReadOnly;
		type: Boolean;
		from: #hasEmail 
		to: [:each | each emailAddresses notEmpty].
	people := session read: GlorpPerson orderBy: #id.
	self assert: people first hasEmail.
	self deny: people last hasEmail.
	self assert: people first emailAddresses notEmpty.
	self assert: people last emailAddresses isEmpty.
	 ]
		ensure: [session rollbackTransaction].! !

!GlorpDirectMappingDBTest methodsFor: 'tests' stamp: ''!
testReadWithDirectFieldReference

	| peopleWithAddresses peopleWithoutAddresses addressIdField |
	[session beginTransaction.
	self writeAddress.
	self writeHomelessPerson.
	self writeHomefulPerson.
	addressIdField := (session system tableNamed: 'PERSON') fieldNamed: 'ADDRESS_ID'.
	peopleWithAddresses := session readManyOf: GlorpPerson where: [:each | (each getField: addressIdField) notNIL].
	peopleWithoutAddresses := session readManyOf: GlorpPerson where: [:each | (each getField: addressIdField) isNIL].
	self assert: peopleWithAddresses size = 1.
	self assert: peopleWithoutAddresses size = 1.
	self assert: peopleWithAddresses first id = 3.
	self assert: peopleWithoutAddresses first id = 4.
	 ]
		ensure: [session rollbackTransaction].! !

!GlorpDirectMappingDBTest methodsFor: 'tests' stamp: ''!
testReadWithPseudoVariableMapping

	| peopleWithAddresses peopleWithoutAddresses |
	[session beginTransaction.
	self writeAddress.
	self writeHomelessPerson.
	self writeHomefulPerson.
	peopleWithAddresses := session readManyOf: GlorpPerson where: [:each | each addressId notNIL].
	peopleWithoutAddresses := session readManyOf: GlorpPerson where: [:each | each addressId isNIL].
	self assert: peopleWithAddresses size = 1.
	self assert: peopleWithoutAddresses size = 1.
	self assert: peopleWithAddresses first id = 3.
	self assert: peopleWithoutAddresses first id = 4.]
		ensure: [session rollbackTransaction].! !

!GlorpDirectMappingDBTest methodsFor: 'tests' stamp: ''!
testUpdate
	|newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson example1.
		personId := newPerson id.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		session inUnitOfWorkDo: [
			session register: person.
			person name: 'something else'].
		session reset.
		self readPerson.
		self assert: person id = newPerson id.
		self assert: person name = 'something else'].! !

!GlorpDirectMappingDBTest methodsFor: 'support' stamp: ''!
testUpdateWithDefaultValue
	"Check that default values don't revert if we update the object in some other way.  (This test does not in fact error in Glorp 7.7 - 69 and earlier, so needs revision.)"

	self inTransactionDo:
		[| thing |
		session transact: [thing := DefaultableThing new id: 1].
		session reset.
		thing := session readOneOf: DefaultableThing.
		self assert: thing integer = 4.
		session modify: thing in: [thing integer: 7].
		session reset.
		thing := session readOneOf: DefaultableThing.
		session modify: thing in: [thing boolean: true].
		self assert: thing integer = 7].! !

!GlorpDirectMappingDBTest methodsFor: 'support' stamp: ''!
testWriteWithDefaultValue

	| rawDBResult field converter dbValue raw |
	self inTransactionDo: [
		| thing |
		session transact: [thing := DefaultableThing new id: 1].
		session reset.
		thing := session readOneOf: DefaultableThing.
		self assert: thing boolean = false.
		field := (system tableNamed: 'DEFAULTABLE_THING') fieldNamed: 'MY_BOOL'.
		converter := field converterForStType: Boolean.
		dbValue := converter convert: false toDatabaseRepresentationAs: field type.
		rawDBResult := session accessor executeSQLString: 'select MY_BOOL from DEFAULTABLE_THING'.
		"The conversion back might get done by us, or might get done by the EXDI layer, Accomodate either possibility as long as it comes out to something equivalent to what we thought it ought to be."
		raw := rawDBResult first first.
		self assert: (raw = dbValue or: [raw = false]).
		self assert: thing integer = 4].! !

!GlorpDirectMappingDBTest methodsFor: 'support' stamp: ''!
writeAddress
	| addressRow |
	addressRow := session system exampleAddressRow.
	session writeRow: addressRow.! !

!GlorpDirectMappingDBTest methodsFor: 'support' stamp: ''!
writeHomefulPerson
	| personRow |
	personRow := session system examplePersonRow1.
	session writeRow: personRow.
	personId := personRow atFieldNamed: 'ID'.! !

!GlorpDirectMappingDBTest methodsFor: 'support' stamp: ''!
writeHomelessPerson
	| personRow |
	personRow := session system examplePersonRow2.
	session writeRow: personRow.
	personId := personRow atFieldNamed: 'ID'.! !

!GlorpDirectMappingDBTest methodsFor: 'support' stamp: ''!
writePersonWithEmailAddresses
	| personRow emailAddress1Row emailAddress2Row |
	personRow := session system examplePersonRow1.
	personId := personRow atFieldNamed: 'ID'.
	session writeRow: personRow.
	emailAddress1Row := session system exampleEmailAddressRow1.
	emailAddress2Row := session system exampleEmailAddressRow2.
	emailAddress1Row at: (emailAddress1Row table fieldNamed: 'ID').
	emailAddress2Row at: (emailAddress2Row table fieldNamed: 'ID').
	session writeRow: emailAddress1Row.
	session writeRow: emailAddress2Row.! !

GlorpMappingDBTest subclass: #GlorpEmbeddedMappingDBTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpEmbeddedMappingDBTest commentStamp: '<historical>' prior: 0!
This exercises the EmbeddedValueOneToOneMapping.!

!GlorpEmbeddedMappingDBTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session beginTransaction.
	self writeAgents.
	session reset.! !

!GlorpEmbeddedMappingDBTest methodsFor: 'support' stamp: ''!
tearDown
	session rollbackTransaction.
	super tearDown.! !

!GlorpEmbeddedMappingDBTest methodsFor: 'tests' stamp: ''!
testModifyAgents
	| agents |
	agents := session read: GlorpTravelAgent orderBy: #id.
	session modify: agents in: [
		agents first name: 'Something Else'.
		(agents at: 2) name: 'something else again'].
	session reset.
	agents := session read: GlorpTravelAgent orderBy: #id.
	self assert: agents size = 3.
	self assert: agents first agency yourSelf = (agents at: 2) agency yourSelf.
	self deny: agents first agency yourSelf == (agents at: 2) agency yourSelf.
	self deny: agents first agency yourSelf = agents last agency yourSelf.
	self assert: agents first name = 'Something Else'.
	self assert: (agents at: 2) name = 'something else again'.! !

!GlorpEmbeddedMappingDBTest methodsFor: 'tests' stamp: ''!
testModifyAgentsReadOnlyAgency
	"Test that if the agency mappings are all read-only, that we don't cause an error by trying to generate entries in the row map for its primary keys."

	(session system descriptorFor: GlorpTravelAgency) mappings do: [:each | each beReadOnly].
	[self testModifyAgents] ensure: [
		(session system descriptorFor: GlorpTravelAgency) mappings do: [:each | each readOnly: false]].! !

!GlorpEmbeddedMappingDBTest methodsFor: 'tests' stamp: ''!
testReadAgents
	| agents |
	agents := session read: GlorpTravelAgent orderBy: #id.
	self assert: agents size = 3.
	self assert: agents first agency yourSelf = (agents at: 2) agency yourSelf.
	self deny: agents first agency yourSelf == (agents at: 2) agency yourSelf.
	self deny: agents first agency yourSelf = agents last agency yourSelf.! !

!GlorpEmbeddedMappingDBTest methodsFor: 'support' stamp: ''!
writeAgents
	| agent1 agent2 agent3 |
	agent1 := GlorpTravelAgent new id: 1; name: 'Agent One'.
	agent1 agency: (GlorpTravelAgency new name: 'Agency One').
	agent2 := GlorpTravelAgent new id: 2; name: 'Agent Two'.
	agent2 agency: agent1 agency.
	agent3 := GlorpTravelAgent new id: 3; name: 'Agent Three'.
	agent3 agency: (GlorpTravelAgency new name: 'Agency Two').
	session modify: (Array with: agent1 with: agent2 with: agent3) in: [].! !

GlorpMappingDBTest subclass: #GlorpManyToManyDBTest
	instanceVariableNames: 'customer customerId accountId1 accountId2 accountId3 treeNodeClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpManyToManyDBTest methodsFor: 'support' stamp: ''!
checkAccounts

	| sorted numberOfAccounts |
	numberOfAccounts := (accountId1 isNil ifTrue: [0] ifFalse: [1]) + (accountId2 isNil ifTrue: [0] ifFalse: [1]) + (accountId3 isNil ifTrue: [0] ifFalse: [1]).
	self assert: customer accounts size = numberOfAccounts.
	sorted := customer accounts asSortedCollection: [:a :b | a id <= b id].
	accountId1 isNil ifFalse: [self assert: sorted first id = accountId1].
	accountId2 isNil ifFalse: [self assert: (sorted at: 2) id = accountId2].
	accountId3 isNil ifFalse: [self assert: sorted last id = accountId3].
	self assert: (customer accounts collect: [:each | each id]) asSet size = customer accounts size.! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
checkNumberOfAccounts: anInteger
	| accountRows |
	accountRows := session accessor
		executeSQLString: 'SELECT * FROM BANK_ACCT'.
	self assert: accountRows size = anInteger.! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
checkNumberOfLinkRows: anInteger
	| linkRows |
	linkRows := session accessor
		executeSQLString: 'SELECT * FROM CUSTOMER_ACCT_LINK'.
	self assert: linkRows size = anInteger.! !

!GlorpManyToManyDBTest methodsFor: 'support' stamp: ''!
inUnitOfWorkDo: aBlock initializeWith: initBlock
	"Set up a bunch of the normal data, read the objects, then run the block in a unit of work"
	initBlock value.
	session beginUnitOfWork.
	self readCustomer.
	aBlock value.
	session commitUnitOfWork.
	session reset.! !

!GlorpManyToManyDBTest methodsFor: 'support' stamp: ''!
readCustomer
	| results query |
	query := Query
		returningManyOf: GlorpCustomer
		where: [:cust | cust id = customerId].
	results := query executeIn: session.
	self assert: results size = 1.
	customer := results first.! !

!GlorpManyToManyDBTest methodsFor: 'As yet unclassified' stamp: ''!
session: aDatabaseSession
	session := aDatabaseSession.! !

!GlorpManyToManyDBTest methodsFor: 'tests-nodes' stamp: ''!
testReadAndDeleteNodes
	"Make sure we don't delete out of order and that links disappear."
	| nodes links |
	self inTransactionDo: [
		self writeNodeWithChildren.
		session reset.
		session transact: [
			nodes := session read: GlorpTreeNode.
			nodes do: [:each | session delete: each]].
		links := session accessor executeSQLString: 'SELECT * FROM TREE_NODE_LINK'.
		self assert: links size = 0].! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadCustomerAndAddAccount
	self
		inTransactionDo:
			[
			self 
				inUnitOfWorkDo: [customer addAccount: (GlorpBankAccount new id: 77473)]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId3 := 77473.
			self checkAccounts.
			self checkNumberOfLinkRows: 4].! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadCustomerAndDeleteAccount
	self
		inTransactionDo:[
			[self 
				inUnitOfWorkDo: [ | account| 
					account := customer accounts detect: [:each | each id = 9874].
					session delete: account]
				initializeWith: [self writeCustomerWithAccounts]]
				on: Dialect error
				do: [:ex | Transcript show: 'integrity violation'; cr.^self].
			self readCustomer.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2.
			self checkNumberOfAccounts: 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadCustomerAndDeleteAccountProperly
	"Do both the delete and the patching up of relationships"
	self
		inTransactionDo:
			[
			self 
				inUnitOfWorkDo: [ | account| 
					account := customer accounts detect: [:each | each id = 9874].
					session delete: account.
					customer removeAccount: account]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2.
			self checkNumberOfAccounts: 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadCustomerAndRemoveAccount
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[| account |
					account := customer accounts detect: [:each | each id = 9874].
					customer accounts remove: account]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadCustomerAndReplaceAccounts
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[| account |
					account := GlorpBankAccount new id: 99999.
					customer accounts do: [:each | each accountHolders remove: customer].
					customer accounts: (Array with: account)]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId1 := 99999.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2.
			self checkNumberOfAccounts: 4].! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadCustomerAndReplaceAccountsWithRemoval
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[| account |
					account := customer accounts detect: [:each | each id = 6].
					customer accounts: (Array with: account)]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadCustomerAndReplaceAccountsWithoutInstantiatingHolders
	"This works, but only fortuitously. If the accounts haven't been read into memory, we don't have to remove their object-level references to the account holder, because changing one side of the relationship in memory is enough to cause the link rows to be deleted"
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[| account |
					account := GlorpBankAccount new id: 99999.
					customer accounts: (Array with: account)]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId1 := 99999.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 2.
			self checkNumberOfAccounts: 4].! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadCustomerAndReplaceInstantiatedAccountsWithEmpty
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[customer accounts yourSelf. 
					customer accounts: #()]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId1 := nil.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 1.
			self checkNumberOfAccounts: 3].! !

!GlorpManyToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadCustomerAndReplaceUninstantiatedAccountsWithEmpty
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[customer accounts: #()]
				initializeWith: [self writeCustomerWithAccounts].
			self readCustomer.
			accountId1 := nil.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfLinkRows: 1.
			self checkNumberOfAccounts: 3].! !

!GlorpManyToManyDBTest methodsFor: 'tests-join' stamp: ''!
testReadCustomerOrderByLinkTableField

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session readManyOf: GlorpCustomer.
			customers do: [:each | | sortedAccounts |
				sortedAccounts := each accounts asSortedCollection: [:a :b | a id <= b id].
				self assert: each accountsSortedById asArray = sortedAccounts asArray]].! !

!GlorpManyToManyDBTest methodsFor: 'tests-join' stamp: ''!
testReadCustomerOrderByLinkTableFieldDescending

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session readManyOf: GlorpCustomer.
			customers do: [:each | | sortedAccounts |
				sortedAccounts := each accounts asSortedCollection: [:a :b | a id <= b id].
				self assert: each accountsSortedByIdDescending asArray = sortedAccounts asArray reverse]].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerOrderingByCountSubselect
	| customers |
	self
		inTransactionDo:
			[| sorted |
			self writeCustomerWithAccounts.
			customers := session
				read: GlorpCustomer
				orderBy: [:each | each accounts sqlCount].
			sorted := (customers asSortedCollection: [:a :b | a accounts size <= b accounts size]) asArray.
			self assert: customers = sorted.
			customers := session
				read: GlorpCustomer
				orderBy: [:each | each accounts sqlCount descending].
			self assert: customers reverse = sorted].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithAverageSubselect

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				read: GlorpCustomer
				where:
					[:each | 
					(each average: [:eachCust | eachCust accounts id]) = ((9874 + 6) / 2)].
		self assert: customers size = 1.
		self assert: customers first id = 27.
		customers := session
				read: GlorpCustomer
				where:
					[:each | 
					(each accounts id sqlAverage) = 22].
		self assert: customers size = 1.
		self assert: customers first id = 28.].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithCountSubselect

	| customersWithOneAccount customersWithTwoAccounts customersWithLessThanFiveAccounts |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customersWithOneAccount := session
				read: GlorpCustomer
				where:
					[:each | 
					(each getConstant: 1) = (each count: [:eachCust | eachCust accounts])].
			customersWithTwoAccounts := session
				read: GlorpCustomer
				where:
					[:each | 
					(each count: [:eachCust | eachCust accounts]) = 2].
			"fully general syntax. doesn't require any helper methods, just the existence of the function"
			customersWithLessThanFiveAccounts := session
				read: GlorpCustomer
				where: [:each | (each aggregate: each accounts as: #countStar) < 5].
		self assert: customersWithOneAccount size = 1.
		self assert: customersWithOneAccount first id = 28.
		self assert: customersWithTwoAccounts size = 1.
		self assert: customersWithTwoAccounts first id = 27.
		self assert: customersWithLessThanFiveAccounts size = 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithCountSubselectAlternateSyntax

	| customersWithOneAccount customersWithTwoAccounts |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customersWithOneAccount := session
				read: GlorpCustomer
				where:
					[:each | 
					(each getConstant: 1) = (each accounts sqlCount)].
			customersWithTwoAccounts := session
				read: GlorpCustomer
				where:
					[:each | 
					each accounts sqlCount = 2].
		self assert: customersWithOneAccount size = 1.
		self assert: customersWithOneAccount first id = 28.
		self assert: customersWithTwoAccounts size = 1.
		self assert: customersWithTwoAccounts first id = 27].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithEqualityComparisonToAccountsUsingInClause
	"Test doing a primary key rewrite of the where clause in a subselect, to the result of another subselect"

	| customers accounts |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			accounts := session read: GlorpBankAccount where: [:each | each id = 22].
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount in: accounts]].
			self assert: customers size = 1.
			customers do: [:each | each accounts do: [:eachAcct | self assert: eachAcct id = 22]]].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithEqualitySubSelectToAccountsUsingInClause
	"Test doing a primary key rewrite of the where clause in a subselect, to the result of another subselect"

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount in: (Query read: GlorpBankAccount where: [:accts2 | accts2 id <> 12])]].
			self assert: customers size = 2.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | 
							eachAccount in: (Query read: GlorpBankAccount where: [:accts2 | accts2 id = 22])]].
			self assert: customers size = 1.
			customers do: [:each | each accounts do: [:eachAcct | self assert: eachAcct id = 22]]].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithFilteredSubSelectToAccounts1

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					(eachCustomer accounts select: [:eachAccount | eachAccount accountNumber bankCode = '2']) sqlCount > 0].
			self assert: customers size = 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithFilteredSubSelectToAccounts2

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					(eachCustomer accounts select: [:eachAccount | eachAccount accountNumber bankCode = '2']) sqlCount > 1].
			self assert: customers size = 0].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithFilteredSubSelectToAccounts3

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					(eachCustomer accounts select: [:eachAccount | eachAccount id > 5 ]) sqlCount <= 1].
			self assert: customers size = 1].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithFilteredSubSelectToAccountsAndAggregate
	"Test with a filtered read to accounts, where we've used an aggregate function to find the customer with the maximum primary key for the given id"

	| query |
	((session system descriptorFor: GlorpCustomer) mappingForAttributeNamed: #accounts) useFilteredRead.
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			query := Query readOneOf: GlorpCustomer where: [:each |
				| q |
				q := Query read: GlorpCustomer where: [:eachPundle |
					eachPundle name = 'aCustomer'].
				q retrieve: [:x | x id max].
				each id = q].
			customer := session execute: query.
			self assert: customer name = 'aCustomer'.
			self assert: (customer accounts collect: [:each | each id]) asSortedCollection asArray = #(6 9874)].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithIsEmptyAndFilteredSubselectsToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | (eachCustomer accounts select: [:each | each id > 30]) isEmpty].
			self assert: customers size = 1.
			self assert: customers first id = 28].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithIsEmptySubselectToAccounts1

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | eachCustomer accounts isEmpty].
			self assert: customers size = 0].! !

!GlorpManyToManyDBTest methodsFor: 'tests-join' stamp: ''!
testReadCustomerWithJoinToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyJoin: [:eachAccount | eachAccount accountNumber bankCode = '2']].
			self assert: customers size = 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-join' stamp: ''!
testReadCustomerWithJoinToAccounts2

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyJoin: [:eachAccount | eachAccount accountNumber branchNumber = 3]].
			self assert: customers size = 1].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithMaxSubselect

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				read: GlorpCustomer
				where:
					[:each | 
					(each max: [:eachCust | eachCust accounts id]) = 9874].
		self assert: customers size = 1.
		self assert: customers first id = 27.
		customers := session
				read: GlorpCustomer
				where:
					[:each | 
					(each accounts id sqlMax) = 22].
		self assert: customers size = 1.
		self assert: customers first id = 28.].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithMinSubselect

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				read: GlorpCustomer
				where:
					[:each | 
					(each min: [:eachCust | eachCust accounts id]) = 6].
		self assert: customers size = 1.
		self assert: customers first id = 27.
		customers := session
				read: GlorpCustomer
				where:
					[:each | 
					(each accounts id sqlMin) = 22].
		self assert: customers size = 1.
		self assert: customers first id = 28.].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithNotEmptyAndFilteredSubselectsToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | (eachCustomer accounts select: [:each | each id > 30]) notEmpty].
			self assert: customers size = 1.
			self assert: customers first id = 27].! !

!GlorpManyToManyDBTest methodsFor: 'tests-join' stamp: ''!
testReadCustomerWithSimpleJoinToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyJoin: [:eachAccount | eachAccount id <> 12]].
			self assert: customers size = 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithSimpleSubSelectToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount id <> 12]].
			self assert: customers size = 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithSubSelectToAccounts

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount accountNumber bankCode = '2']].
			self assert: customers size = 2].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithSubSelectToAccounts2

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | eachAccount accountNumber branchNumber = 3]].
			self assert: customers size = 1].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithSubSelectToAccountsUsingInClause
	"Test doing an IN clause with the result of another subselect"

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | 
							| q |
							q := Query read: GlorpBankAccount where: [:accts2 | accts2 id <> 12].
							q retrieve: #id.
							eachAccount id in: q]].
			self assert: customers size = 2.
			customers := session
				readManyOf: GlorpCustomer
				where:
					[:eachCustomer | 
					eachCustomer accounts
						anySatisfyExists: [:eachAccount | 
							| q |
							q := Query read: GlorpBankAccount where: [:accts2 | accts2 id = 22].
							q retrieve: #id.
							eachAccount id in: q]].
			self assert: customers size = 1.
			customers do: [:each | each accounts do: [:eachAcct | self assert: eachAcct id = 22]].
].! !

!GlorpManyToManyDBTest methodsFor: 'tests-subselect' stamp: ''!
testReadCustomerWithSumSubselect

	| customers |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			customers := session
				read: GlorpCustomer
				where:
					[:each | 
					(each sum: [:eachCust | eachCust accounts accountNumber branchNumber]) < 6].
		self assert: customers size = 1.
		self assert: customers first id = 27.
		customers := session
				read: GlorpCustomer
				where:
					[:each | 
					(each accounts accountNumber branchNumber sqlSum) > 6].
		self assert: customers size = 1.
		self assert: customers first id = 28.].! !

!GlorpManyToManyDBTest methodsFor: 'tests-nodes' stamp: ''!
testReadNode
	| allNodes |
	self inTransactionDo: [
		self writeNodeWithChildren.
		session reset.
		allNodes := session readManyOf: GlorpTreeNode orderBy: #id.
		self assert: (allNodes collect: [:each | each id]) asArray = #( 1 2 3).
		self assert: allNodes first children size = 2.
		self assert: (allNodes first children collect: [:each | each id]) asSortedCollection asArray = #( 2 3)].! !

!GlorpManyToManyDBTest methodsFor: 'tests-nodes' stamp: ''!
testReadNodeAndDeleteChild
	| node1 node3 links nodes |
	self inTransactionDo: [
		self writeNodeWithChildren.
		session reset.
		node1 := session readOneOf: GlorpTreeNode where: [:each | each id = 1].
		node3 := (node1 children asSortedCollection: [:a :b | a id <= b id]) last.
		session transact: [
			session register: node1.
			session delete: node3.
			node1 children remove: node3].
		links := session accessor executeSQLString: 'SELECT * FROM TREE_NODE_LINK'.
		self assert: links size = 1.
		self assert: links first first = 1. 
		self assert: links first last = 2.
		nodes := session accessor executeSQLString: 'SELECT * FROM TREE_NODE'.
		self deny: (nodes anySatisfy: [:each | each first = node3 id]).
		self assert: nodes size = 2
		].! !

!GlorpManyToManyDBTest methodsFor: 'tests-nodes' stamp: ''!
testReadNodeAndDeleteChildWithSaveAndContinue
	"This exercises two things. First, there's a bug in save and continue with deleted objects resurrecting. Second, make sure that if we delete both parent and child, the link goes away"
	| node1 node3 links nodes node2 |
	self inTransactionDo: [
		self writeNodeWithChildren.
		session reset.
		node1 := session readOneOf: GlorpTreeNode where: [:each | each id = 1].
		node2 := (node1 children asSortedCollection: [:a :b | a id <= b id]) first.
		node3 := (node1 children asSortedCollection: [:a :b | a id <= b id]) last.
		session beginUnitOfWork.
			[session register: node1.
			session delete: node3.
			node1 children remove: node3] ensure: [session saveAndContinue].
		links := session accessor executeSQLString: 'SELECT * FROM TREE_NODE_LINK'.
		self assert: links size = 1.
		self assert: links first first = 1. 
		self assert: links first last = 2.
		nodes := session accessor executeSQLString: 'SELECT * FROM TREE_NODE'.
		self deny: (nodes anySatisfy: [:each | each first = node3 id]).
		self assert: nodes size = 2.

			[session register: node1.
			session delete: node2.
			session delete: node1] ensure: [session saveAndContinue].
		nodes := session accessor executeSQLString: 'SELECT * FROM TREE_NODE'.
		self assert: nodes isEmpty.
		links := session accessor executeSQLString: 'SELECT * FROM TREE_NODE_LINK'.
		self assert: links isEmpty.
		].! !

!GlorpManyToManyDBTest methodsFor: 'tests-nodes' stamp: ''!
testReadNodeAndDeleteParent
	| node1 links |
	self inTransactionDo: [
		self writeNodeWithChildren.
		session reset.
		session transact: [
		node1 := session readOneOf: GlorpTreeNode where: [:each | each id = 1].
		session delete: node1.
		node1 children: #()].
		links := session accessor executeSQLString: 'SELECT * FROM TREE_NODE_LINK'.
		self assert: links size = 0].! !

!GlorpManyToManyDBTest methodsFor: 'tests-write' stamp: ''!
testWriteCustomerWithAccounts
	| newCustomer |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			session beginUnitOfWork.
			newCustomer := GlorpCustomer example1.
			newCustomer id: 12.
			customerId := 12.
			newCustomer accounts: (OrderedCollection with: (GlorpBankAccount new id: 223)).
			session register: newCustomer.
			session commitUnitOfWork.
			session reset.
			self readCustomer.
			accountId1 := 223.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfAccounts: 4.
			self checkNumberOfLinkRows: 4].! !

!GlorpManyToManyDBTest methodsFor: 'tests-write' stamp: ''!
testWriteCustomerWithNoAccounts
	| newCustomer |
	self
		inTransactionDo:
			[session beginUnitOfWork.
			newCustomer := GlorpCustomer example1.
			newCustomer id: 12.
			customerId := 12.
			session register: newCustomer.
			session commitUnitOfWork.
			session reset.
			self readCustomer.
			accountId1 := nil.
			accountId2 := nil.
			self checkAccounts.
			self checkNumberOfAccounts: 0.
			self checkNumberOfLinkRows: 0].! !

!GlorpManyToManyDBTest methodsFor: 'tests-write' stamp: ''!
testWriteCustomerWithTwoAccounts
	| newCustomer |
	self
		inTransactionDo:
			[self writeCustomerWithAccounts.
			session beginUnitOfWork.
			newCustomer := GlorpCustomer example1.
			newCustomer id: 12.
			customerId := 12.
			newCustomer accounts: (OrderedCollection with: (GlorpBankAccount new id: 223)).
			newCustomer accounts add: (GlorpBankAccount new id: 224).
			session register: newCustomer.
			session commitUnitOfWork.
			session reset.
			self readCustomer.
			accountId1 := 223.
			accountId2 := 224.
			self checkAccounts.
			self checkNumberOfAccounts: 5.
			self checkNumberOfLinkRows: 5].! !

!GlorpManyToManyDBTest methodsFor: 'tests-nodes' stamp: ''!
treeNodeClass
	treeNodeClass isNil ifTrue: [treeNodeClass := GlorpTreeNode].
	^treeNodeClass.! !

!GlorpManyToManyDBTest methodsFor: 'tests-nodes' stamp: ''!
treeNodeClass: aClass
	treeNodeClass := aClass.! !

!GlorpManyToManyDBTest methodsFor: 'support' stamp: ''!
writeCustomerWithAccounts
	| customerRow accountRow1 accountRow2 linkRow1 linkRow2 customerRow2 accountRow3 linkRow3 |

	customerRow := session system exampleCustomerRow1.
	customerId := customerRow atFieldNamed: 'ID'.
	customerRow2 := session system exampleCustomerRow2.
	accountRow1 := session system exampleAccountRow1. 
	accountId2 := accountRow1 atFieldNamed: 'ID'.
	accountRow2 := session system exampleAccountRow2.
	accountId1 := accountRow2 atFieldNamed: 'ID'.
	accountRow3 := session system exampleAccountRow3.
	linkRow1 := session system exampleCALinkRow1.
	linkRow2 := session system exampleCALinkRow2.
	linkRow3 := session system exampleCALinkRow3.
	session writeRow: customerRow.
	session writeRow: customerRow2.
	session writeRow: accountRow1.
	session writeRow: accountRow2.
	session writeRow: accountRow3.
	session writeRow: linkRow1.
	session writeRow: linkRow2.
	session writeRow: linkRow3.! !

!GlorpManyToManyDBTest methodsFor: 'tests-nodes' stamp: ''!
writeNodeWithChildren
	| node1 node2 node2a |
	node1 := self treeNodeClass new id: 1.
	node2 := self treeNodeClass new id: 2.
	node2a := self treeNodeClass new id: 3.
	node1 children add: node2.
	node1 children add: node2a.
	session inUnitOfWorkDo: [session register: node1].! !

!GlorpMappingDBTest class methodsFor: 'testing' stamp: 'GuillermoPolito 4/9/2011 15:25'!
isAbstract
	^self class name = #GlorpMappingDBTest! !

!GlorpMappingDBTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpSessionResource.! !

!GlorpMappingDBTest methodsFor: 'support' stamp: ''!
inTransactionDo: aBlock
	[session beginTransaction.
	aBlock value] ensure: [session rollbackTransaction].! !

!GlorpMappingDBTest methodsFor: 'Running' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	system := session system.! !

!GlorpMappingDBTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session reset.
	session resetPermissions.
	session := nil.
	system := nil.! !

GlorpMappingDBTest subclass: #GlorpOneToManyDBTest
	instanceVariableNames: 'person personId emailId1 emailId2 emailId3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpOneToManyDBTest methodsFor: 'support' stamp: ''!
checkEmailAddresses: emailAddresses
	| sorted numberOfAddresses |
	numberOfAddresses := (emailId1 isNil ifTrue: [0] ifFalse: [1]) + (emailId2 isNil ifTrue: [0] ifFalse: [1]) + (emailId3 isNil ifTrue: [0] ifFalse: [1]).
	self assert: emailAddresses size = numberOfAddresses.
	sorted := emailAddresses asSortedCollection: [:a :b | a id <= b id].
	emailId1 isNil ifFalse: [self assert: sorted first id = emailId1].
	emailId2 isNil ifFalse: [self assert: (sorted at: 2) id = emailId2].
	emailId3 isNil ifFalse: [self assert: sorted last id = emailId3].
	self assert: (emailAddresses collect: [:each | each id]) asSet size = emailAddresses size.! !

!GlorpOneToManyDBTest methodsFor: 'support' stamp: ''!
checkNumberOfEmailAddressesInDB: numberOfAddresses
	| databaseAddresses |
	databaseAddresses := session accessor executeSQLString: 'SELECT * FROM EMAIL_ADDRESS'.
	self assert: databaseAddresses size = numberOfAddresses.! !

!GlorpOneToManyDBTest methodsFor: 'support' stamp: ''!
inUnitOfWorkDo: aBlock initializeWith: initBlock
	"Set up a bunch of the normal data, read the objects, then run the block in a unit of work"
	initBlock value.
	session beginUnitOfWork.
	self readPerson.
	aBlock value.
	session commitUnitOfWork.
	session reset.! !

!GlorpOneToManyDBTest methodsFor: 'support' stamp: ''!
readPerson
	| results query |
	query := Query
		returningManyOf: GlorpPerson
		where: [:pers | pers id = personId].
	results := query executeIn: session.
	self assert: results size = 1.
	person := results first.! !

!GlorpOneToManyDBTest methodsFor: 'tests-join' stamp: ''!
setUpSomeExtraPeople
	self
		inUnitOfWorkDo:
			[| otherPerson |
			session register: (GlorpPerson new id: 9924365).
			otherPerson := GlorpPerson new id: 12121.
			otherPerson
				emailAddresses:
					(OrderedCollection
						with:
							((GlorpEmailAddress new)
								id: 7;
								host: 'asdfasdf')).
			session register: otherPerson]
		initializeWith: [self writePersonWithEmailAddresses].! !

!GlorpOneToManyDBTest methodsFor: 'support' stamp: ''!
tearDown
	((session system descriptorFor: GlorpPerson) mappingForAttributeNamed: #emailAddresses) isExclusive: false.
	super tearDown.! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadActionWithCollection
	| collection |
	self
		inTransactionDo:
			[collection := OrderedCollection new.
			self writePersonWithEmailAddresses.
			((session system descriptorFor: GlorpPerson)
				mappingForAttributeNamed: #emailAddresses)
				proxyAction:
					[:owner :object | object do: [:each | collection add: owner -> each]].
			person := session readOneOf: GlorpPerson.
			person emailAddresses yourSelf.
			self assert: collection size = 2.
			self assert: collection first = (person -> person emailAddresses first).
			self assert: collection last = (person -> person emailAddresses last)].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: 'GuillermoPolito 4/26/2011 04:57'!
testReadJustTheEmailAddressNotThePersonAndWriteBackWithChanges
	| addresses addressRows |
	"We won't have the person object to set a value for the PERSON_ID field. Ensure that we don't write a null for that field, or otherwise modify things."
	self inTransactionDo: [	
		self writePersonWithEmailAddresses.
		session beginUnitOfWork.
		addresses := session readManyOf: GlorpEmailAddress.
		addresses do: [:each | each host: 'bar.org'].
		self assert: ((session privateGetCache cacheForClass: GlorpPerson) at: 3 ifAbsent: [nil]) isNil.
		session commitUnitOfWork.
		addressRows := session accessor executeSQLString: 'SELECT PERSON_ID, HOST_NAME from EMAIL_ADDRESS'.
		self assert: addressRows size = 2.
		addressRows do: [:each | 
			self assert: (each at: 1) = 3.
			self assert: (each at: 2) = 'bar.org'].
].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: 'GuillermoPolito 4/26/2011 04:58'!
testReadJustTheEmailAddressNotThePersonAndWriteBackWithNoChanges
	| addresses addressRows |
	"We won't have the person object to set a value for the PERSON_ID field. Ensure that we don't write a null for that field, or otherwise modify things."
	self inTransactionDo: [	
		self writePersonWithEmailAddresses.
		session beginUnitOfWork.
		addresses := session readManyOf: GlorpEmailAddress.
		self assert: ((session privateGetCache cacheForClass: GlorpPerson) at: 3 ifAbsent: [nil]) isNil.
		session commitUnitOfWork.
		addressRows := session accessor executeSQLString: 'SELECT PERSON_ID from EMAIL_ADDRESS'.
		self assert: addressRows size = 2.
		addressRows do: [:each | self assert: (each at: 1) = 3].
].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndAddEmailAddress
	| |
	self
		inTransactionDo:
			[
			self 
				inUnitOfWorkDo: [person emailAddresses add: (GlorpEmailAddress new id: 99876; user: 'postmaster'; host: 'foo.com')]
				initializeWith: [self writePersonWithEmailAddresses].
			emailId3 := 99876.
			self readPerson.
			self checkEmailAddresses: person emailAddresses].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndAddEmailAddressProxy
	| address |
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[| query |
					query := Query
						readOneOf: GlorpEmailAddress
						where: [:each | each id = 99876].
					query proxyType: Proxy.
					address := session execute: query.
					address yourself.
					person emailAddresses add: address.
					self assert: person emailAddresses last class == Proxy]
				initializeWith:
					[self writePersonWithEmailAddresses.
					session
						transact:
							[(GlorpEmailAddress new)
								id: 99876;
								user: 'postmaster';
								host: 'foo.com'].
						session reset].
			emailId3 := 99876.
			self readPerson.
			self checkEmailAddresses: person emailAddresses].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndAddEmailAddressTwice
	| personRow addressRow |
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[person emailAddresses
						add:
							((GlorpEmailAddress new)
								id: 99876;
								user: 'postmaster';
								host: 'foo.com')]
				initializeWith:
					[addressRow := session system exampleAddressRow.
					session writeRow: addressRow.
					personRow := session system examplePersonRow1.
					personId := personRow atFieldNamed: 'ID'.
					session writeRow: personRow].
			self
				inUnitOfWorkDo:
					[person emailAddresses
						add:
							((GlorpEmailAddress new)
								id: 99991;
								user: 'someone';
								host: 'foo.com')]
				initializeWith: [].
			emailId1 := 99876.
			emailId2 := 99991.
			self readPerson.
			self checkEmailAddresses: person emailAddresses].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndDeleteEmailAddress
	self
		inTransactionDo:
			[
			self 
				inUnitOfWorkDo: [
					session delete: person emailAddresses last. 
					person emailAddresses removeLast. ]
				initializeWith: [self writePersonWithEmailAddresses].
			emailId2 := nil.
			self readPerson.
			self checkEmailAddresses: person emailAddresses.
			self checkNumberOfEmailAddressesInDB: 1].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndRemoveEmailAddress
	self
		inTransactionDo:
			[
			self 
				inUnitOfWorkDo: [person emailAddresses removeLast]
				initializeWith: [self writePersonWithEmailAddresses].
			emailId2 := nil.
			self readPerson.
			self checkEmailAddresses: person emailAddresses.
			self checkNumberOfEmailAddressesInDB: 2].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndReplaceEmailAddressesWithDifferent
	| |
	self
		inTransactionDo:
			[self 
				inUnitOfWorkDo: [
					person emailAddresses: (Array with: (GlorpEmailAddress new id: 99876; user: 'postmaster'; host: 'foo.com')) ]
				initializeWith: [self writePersonWithEmailAddresses].
			emailId1 := 99876.
			emailId2 := nil.
			self readPerson.
			self checkEmailAddresses: person emailAddresses.
			self checkNumberOfEmailAddressesInDB: 3].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndReplaceEmailAddressesWithRemoval
	| |
	self
		inTransactionDo:
			[self 
				inUnitOfWorkDo: [
					person emailAddresses: (Array with: person emailAddresses first) ]
				initializeWith: [self writePersonWithEmailAddresses].
			emailId2 := nil.
			self readPerson.
			self checkEmailAddresses: person emailAddresses.
			self checkNumberOfEmailAddressesInDB: 2].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndReplaceInstantiatedEmailAddressesWithEmpty
	| |
	self
		inTransactionDo:
			[self 
				inUnitOfWorkDo: [
					person emailAddresses yourSelf. 
					person emailAddresses: #(). ]
				initializeWith: [self writePersonWithEmailAddresses].
			emailId1 := nil.
			emailId2 := nil.
			self readPerson.
			self checkEmailAddresses: person emailAddresses.
			self checkNumberOfEmailAddressesInDB: 2].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndReplaceUninstantiatedEmailAddressesWithEmpty
	| |
	self
		inTransactionDo:
			[
			self 
				inUnitOfWorkDo: [person emailAddresses: #().]
				initializeWith: [self writePersonWithEmailAddresses].
			emailId1 := nil.
			emailId2 := nil.
			self readPerson.
			self checkEmailAddresses: person emailAddresses.
			self checkNumberOfEmailAddressesInDB: 2].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonWithEmailAddresses
	| query result emailAddresses |
	self
		inTransactionDo:
			[self writePersonWithEmailAddresses.
			query := Query
				returningOneOf: GlorpPerson
				where: [:eachPerson | eachPerson id = personId].
			result := query executeIn: session.
			emailAddresses := result emailAddresses getValue.
			self checkEmailAddresses: emailAddresses].! !

!GlorpOneToManyDBTest methodsFor: 'tests' stamp: ''!
testReadPersonWithExistsWithAlsoFetch
	| people query |
	self inTransactionDo: 
			[self setUpSomeExtraPeople.
			query := ReadQuery readManyOf: GlorpPerson
						where: 
							[:eachPerson | 
							eachPerson emailAddresses 
								anySatisfyExists: [:eachEmail | eachEmail host = 'objectpeople.com']].
			query alsoFetch: [:e | e address].
			people := query executeIn: session.
			self assert: people size = 1]! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonWithIsEmptySubselectToEmailAddresses
	| people |
	self
		inTransactionDo:
			[self setUpSomeExtraPeople.
			people := session
				readManyOf: GlorpPerson
				where:
					[:eachPerson | eachPerson emailAddresses isEmpty].
			self assert: people size = 1].! !

!GlorpOneToManyDBTest methodsFor: 'tests-join' stamp: ''!
testReadPersonWithJoinToEmailAddresses
	| people |
	self
		inTransactionDo:
			[self setUpSomeExtraPeople.
			people := session
				readManyOf: GlorpPerson
				where:
					[:eachPerson | 
					eachPerson emailAddresses
						anySatisfyJoin: [:eachEmail | eachEmail host = 'objectpeople.com']].
			self assert: people size = 1].! !

!GlorpOneToManyDBTest methodsFor: 'tests' stamp: ''!
testReadPersonWithJoinWithAlsoFetch
	| people query |
	self inTransactionDo: 
			[self setUpSomeExtraPeople.
			query := ReadQuery readManyOf: GlorpPerson
						where: 
							[:eachPerson | 
							eachPerson emailAddresses anySatisfyJoin: 
									[:eachEmail | 
									eachEmail host = 'objectpeople.com' | (eachEmail host = 'cs.uiuc.edu')]].
			query alsoFetch: [:e | e address].
			people := query executeIn: session.
			self assert: people size = 1]! !

!GlorpOneToManyDBTest methodsFor: 'tests-join' stamp: ''!
testReadPersonWithNegativeJoinToEmailAddresses
	"Read with a negative condition. Note that this excludes the person with no e-mail addresses, as we're not doing an outer join"
	| people |
	self
		inTransactionDo:
			[self setUpSomeExtraPeople.
			people := session
				readManyOf: GlorpPerson
				where:
					[:eachPerson | 
					eachPerson emailAddresses
						anySatisfyJoin: [:eachEmail | eachEmail host ~= 'objectpeople.com']].
			self assert: people size = 2].! !

!GlorpOneToManyDBTest methodsFor: 'tests-join' stamp: ''!
testReadPersonWithNegativeSubselectToEmailAddresses
	"Read with a negative condition. Note that this excludes the person with no e-mail addresses, as we're not doing an outer join"
	| people |
	self
		inTransactionDo:
			[self setUpSomeExtraPeople.
			people := session
				readManyOf: GlorpPerson
				where:
					[:eachPerson | 
					eachPerson emailAddresses
						anySatisfyExists: [:eachEmail | eachEmail host ~= 'objectpeople.com']].
			self assert: people size = 2].! !

!GlorpOneToManyDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonWithNotEmptySubselectToEmailAddresses
	| people |
	self
		inTransactionDo:
			[self setUpSomeExtraPeople.
			people := session
				readManyOf: GlorpPerson
				where:
					[:eachPerson | eachPerson emailAddresses notEmpty].
			self assert: people size = 2].! !

!GlorpOneToManyDBTest methodsFor: 'tests-join' stamp: ''!
testReadPersonWithSubselectToEmailAddresses
	| people |
	self
		inTransactionDo:
			[self setUpSomeExtraPeople.
			people := session
				readManyOf: GlorpPerson
				where:
					[:eachPerson | 
					eachPerson emailAddresses
						anySatisfyExists: [:eachEmail | eachEmail host = 'objectpeople.com']].
			self assert: people size = 1].! !

!GlorpOneToManyDBTest methodsFor: 'tests-write' stamp: ''!
testUpdatePersonWithNilEmailAddressesAndExclusiveRelationship

	| newPerson |
	((session system descriptorFor: GlorpPerson) mappingForAttributeNamed: #emailAddresses) beExclusive.
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson new.
		newPerson id: 231.
		personId := 231.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		session modify: person in: [person emailAddresses: nil].
		session reset.
		self readPerson.
		self assert: person emailAddresses isEmpty.
		self checkNumberOfEmailAddressesInDB: 0].! !

!GlorpOneToManyDBTest methodsFor: 'tests-write' stamp: ''!
testWritePersonWithEmailAddresses

	| newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson example1.
		newPerson id: 231.
		personId := 231.	
		newPerson emailAddresses: OrderedCollection new.
		newPerson emailAddresses add: (GlorpEmailAddress new id: 2; user: 'one'; host: 'blorch.ca').
		newPerson emailAddresses add: (GlorpEmailAddress new id: 3; user: 'two'; host: 'blorch.ca').
		emailId1 := 2.
		emailId2 := 3.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		self checkEmailAddresses: person emailAddresses.
		self checkNumberOfEmailAddressesInDB: 2].! !

!GlorpOneToManyDBTest methodsFor: 'tests-write' stamp: ''!
testWritePersonWithNilEmailAddresses

	| newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson new.
		newPerson id: 231.
		personId := 231.
		newPerson emailAddresses: nil.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		self assert: person emailAddresses isEmpty.
		self checkNumberOfEmailAddressesInDB: 0].! !

!GlorpOneToManyDBTest methodsFor: 'tests-write' stamp: ''!
testWritePersonWithNilEmailAddressesAndExclusiveRelationship

	| newPerson |
	((session system descriptorFor: GlorpPerson) mappingForAttributeNamed: #emailAddresses) beExclusive.
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson new.
		newPerson id: 231.
		personId := 231.
		newPerson emailAddresses: nil.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		self assert: person emailAddresses isEmpty.
		self checkNumberOfEmailAddressesInDB: 0].! !

!GlorpOneToManyDBTest methodsFor: 'tests-write' stamp: ''!
testWritePersonWithNoEmailAddresses

	| newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson new.
		newPerson id: 231.
		personId := 231.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		self assert: person emailAddresses isEmpty.
		self checkNumberOfEmailAddressesInDB: 0].! !

!GlorpOneToManyDBTest methodsFor: 'tests-write' stamp: ''!
testWritePersonWithNoEmailAddresses2

	| newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson new.
		newPerson id: 231.
		personId := 231.
		newPerson emailAddresses: OrderedCollection new.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		self assert: person emailAddresses isEmpty.
		self checkNumberOfEmailAddressesInDB: 0].! !

!GlorpOneToManyDBTest methodsFor: 'support' stamp: ''!
writePersonWithEmailAddresses
	| addressRow personRow emailAddress1Row emailAddress2Row |
	addressRow := session system exampleAddressRow.
	session writeRow: addressRow.
	personRow := session system examplePersonRow1.
	personId := personRow atFieldNamed: 'ID'.
	session writeRow: personRow.
	emailAddress1Row := session system exampleEmailAddressRow1.
	emailAddress2Row := session system exampleEmailAddressRow2.
	emailId1 := emailAddress1Row at: (emailAddress1Row table fieldNamed: 'ID').
	emailId2 := emailAddress2Row at: (emailAddress2Row table fieldNamed: 'ID').
	session writeRow: emailAddress1Row.
	session writeRow: emailAddress2Row.! !

GlorpMappingDBTest subclass: #GlorpOneToOneDBTest
	instanceVariableNames: 'person personId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
additionalTests

	"It would be good to have tests here for a foreign key 'pointing' the other direction. Also composite keys (once those work)"! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
checkAddress
	self assert: person address isGlorpProxy.
	self assert: person address getValue id = 123.
	^self assert: person address getValue class == GlorpAddress.! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
checkNoAddress
	self deny: person address isGlorpProxy.
	self assert: person address == nil.! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: 'GuillermoPolito 4/26/2011 04:58'!
checkNoAddressesInDB
	| addresses addressKeys |
	addresses := session accessor executeSQLString: 'SELECT * FROM GR_ADDRESS'.
	self assert: addresses isEmpty.
	addressKeys := session accessor executeSQLString: 'SELECT ADDRESS_ID FROM PERSON'.
	self assert: (addressKeys size = 1).
	self assert: (addressKeys first at: 1) = nil.! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
checkPerson
	self assert: person class = GlorpPerson.
	self assert: person id = personId.
	self assert: person name = 'aPerson'.! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
inUnitOfWorkDo: aBlock
	"Set up a bunch of the normal data, read the objects, then run the block in a unit of work"
	self inUnitOfWorkDo: aBlock initializeWith: [self writePersonWithAddress].! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
inUnitOfWorkDo: aBlock initializeWith: initBlock
	"Set up a bunch of the normal data, read the objects, then run the block in a unit of work"
	initBlock value.
	session beginUnitOfWork.
	self readPerson.
	aBlock value.
	session commitUnitOfWork.
	session reset.! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
readPerson
	| results query |
	query := Query
		returningManyOf: GlorpPerson
		where: [:pers | pers id = personId].
	results := query executeIn: session.
	self assert: results size = 1.
	person := results first.! !

!GlorpOneToOneDBTest methodsFor: 'tests-link table' stamp: ''!
testLinkTableRead
	| customers |
	system := GlorpLinkTableAnywhereDescriptorSystem forPlatform: system platform.
	session system: system.
	self inTransactionDo: [
		self writeCustomersWithOneAccountEach.
		session reset.
		customers := session readManyOf: GlorpCustomer.
		self assert: ((customers collect: [:each | each accounts id]) asSortedCollection asArray = #(22 9874))].! !

!GlorpOneToOneDBTest methodsFor: 'tests-link table' stamp: ''!
testLinkTableWrite
	system := GlorpLinkTableAnywhereDescriptorSystem
		forPlatform: system platform.
	session system: system.
	self
		inTransactionDo:
			[| rows row |
			self
				inUnitOfWorkDo:
					[| cust1 acct1 |
					cust1 := GlorpCustomer new.
					cust1 id: 2.
					cust1 name: 'Violet Baudelaire'.
					acct1 := GlorpBankAccount new.
					acct1 id: 3.
					cust1 accounts: acct1.
					session register: cust1].
			session reset.
			rows := session accessor
				executeSQLString:
					'select * from GR_CUSTOMER t1, BANK_ACCT t2, CUSTOMER_ACCT_LINK t3 WHERE t3.ACCT_ID = t2.ID AND t3.CUSTOMER_ID = t1.ID'.
			self assert: rows size = 1.
			row := rows first.
			self assert: (row asArray = (OrderedCollection new add: 2; add: 'Violet Baudelaire'; add: 3; add: nil; add: nil; add: nil; add: 2; add: 3; yourself) asArray)].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReRegisteringAfterUpdate
	"Test that even if we re-register the object after it's been modified, we still write the changes."

	self
		inTransactionDo:
			[self writePersonWithAddress.
			session reset.
			session beginUnitOfWork.
			person := (session read: GlorpPerson) first.
			person name: 'new name'.
			session register: person.
			session commitUnitOfWork.
			session reset.
			self readPerson.
			self assert: person name = 'new name'].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadAction
	| collection |
	self
		inTransactionDo:
			[collection := OrderedCollection new.
			self writePersonWithAddress.
			((session system descriptorFor: GlorpPerson)
				mappingForAttributeNamed: #address)
				proxyAction: [:owner :object | collection add: owner -> object].
			person := session readOneOf: GlorpPerson.
			person address yourSelf.
			self assert: collection size = 1.
			self assert: collection first = (person -> person address yourSelf)].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndAddAddress
	self inTransactionDo: [
		self 
			inUnitOfWorkDo: [person address: (GlorpAddress new id: 5555; street: 'hello'; number: 'world')]
			initializeWith: [self writeHomelessPerson].
		self readPerson.
		self checkPerson].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndAddBogusCollectionInsteadOfAddress
	self inTransactionDo: [
		self should: [self 
				inUnitOfWorkDo: [person address: #()]
				initializeWith: [self writeHomelessPerson]]
			raise: GlorpInvalidTypeError].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndAddBogusObjectInsteadOfAddress
	self inTransactionDo: [
		self should: [self 
				inUnitOfWorkDo: [person address: GlorpPerson new]
				initializeWith: [self writeHomelessPerson]]
			raise: GlorpInvalidTypeError].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndAddBogusObjectInsteadOfEmailAddresses
	self inTransactionDo: [
		self should: [self 
				inUnitOfWorkDo: [person emailAddresses: 42]
				initializeWith: [self writeHomelessPerson]]
			raise: GlorpInvalidTypeError].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndAddBogusObjectInsteadOfEmailAddresses2
	self inTransactionDo: [
		self should: [self 
				inUnitOfWorkDo: [person emailAddresses: GlorpEmailAddress new]
				initializeWith: [self writeHomelessPerson]]
			raise: GlorpInvalidTypeError].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndRemoveAddress
	self inTransactionDo: [
		self inUnitOfWorkDo: [
			session delete: person address.
			person address: nil.].
		self readPerson.
		self checkPerson.
		self checkNoAddress.
		self checkNoAddressesInDB].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndReplaceAddress
	self inTransactionDo:
		[self inUnitOfWorkDo: 
			[person	address: ((GlorpAddress new) id: 12; street: 'foo'; number: '1234')].
		self readPerson.
		self checkPerson.
		self assert: person address isGlorpProxy.
		self assert: person address getValue id = 12.
		self assert: person address getValue street = 'foo'.].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndReplaceAdressWithDifferentProxy
	| proxy |
	self inTransactionDo: 
			[
			session writeRow: session system exampleAddressRowForOrdering1.
			self inUnitOfWorkDo: 
					[ | field |
					proxy := Proxy new.
					proxy query: person address query copy.
					field := (session system tableNamed: 'PERSON') fieldNamed: 'ADDRESS_ID'.
					proxy parameters: (IdentityDictionary new at: field put: 1; yourself).
					proxy session: person address session.
					person address: proxy].
			self readPerson.
			self checkPerson.
			self assert: person address isGlorpProxy.
			self assert: person address getValue notNil.
			self assert: person address getValue id = 1]! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndReplaceAdressWithProxy
	| proxy |
	self inTransactionDo: 
			[self inUnitOfWorkDo: 
					[proxy := Proxy new.
					proxy query: person address query copy.
					proxy parameters: person address parameters copy.
					proxy session: person address session.
					person address: proxy].
			self readPerson.
			self checkPerson.
			self assert: person address isGlorpProxy.
			self assert: person address getValue notNil.
			self assert: person address getValue id = 123]! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonAndUpdateName
	"Test to make sure that if we never instantiate the address, we don't have problems with that relationship"
	self inTransactionDo:
		[self inUnitOfWorkDo: 
			[person	name: 'new name'].
		self deny: person address isInstantiated.
		session reset.
		self readPerson.
		self assert: person name = 'new name'.
		self assert: person address isGlorpProxy.
		self assert: person address getValue id = 123.
		self assert: person address getValue street = 'Paseo Montril'.].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonWithAddress
	self inTransactionDo: [
		self writePersonWithAddress.
		self readPerson.
		self checkPerson.
		self checkAddress].! !

!GlorpOneToOneDBTest methodsFor: 'tests-join' stamp: ''!
testReadPersonWithJoinToAddress
	| people |
	self inTransactionDo: [
		self writePersonWithAddress.
		people := session readManyOf: GlorpPerson where: [:eachPerson | eachPerson address street = 'Paseo Montril'].
		self assert: people size = 1.
		person := people first.
		self assert: person address street = 'Paseo Montril'.
		self assert: person address id = 123].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadPersonWithoutAddress
	self inTransactionDo: [
		self writeHomelessPerson.
		self writeAddress.
		self readPerson.
		self checkPerson.
		self checkNoAddress].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadWithCacheHitFromExpression
	self
		inTransactionDo:
			[self
				inUnitOfWorkDo:
					[self assert: person address yourSelf id = 123.
					session accessor permitNothing.
					session readOneOf: GlorpAddress where: [:each | each id = 123]]].! !

!GlorpOneToOneDBTest methodsFor: 'tests-read' stamp: ''!
testReadWithCacheHitFromJoin
	self inTransactionDo: [
		self 
			inUnitOfWorkDo: [
				session readOneOf: GlorpAddress.
				session accessor permitNothing.
				self assert: (person address yourSelf id = 123)]].! !

!GlorpOneToOneDBTest methodsFor: 'tests-write' stamp: ''!
testWritePersonWithAddress
	|newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson example1.
		personId := newPerson id.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		self assert: person id = newPerson id.
		self assert: person name = newPerson name.
		self assert: person address id = newPerson address id.
		self assert: person address street = newPerson address street].! !

!GlorpOneToOneDBTest methodsFor: 'tests-write' stamp: ''!
testWritePersonWithoutAddress
	|newPerson |
	self inTransactionDo: [
		session beginUnitOfWork.
		newPerson := GlorpPerson example1.
		newPerson address: nil.
		personId := newPerson id.
		session register: newPerson.
		session commitUnitOfWork.
		session reset.
		self readPerson.
		self assert: person id = newPerson id.
		self assert: person name = newPerson name.
		self assert: person address yourSelf == nil.
		self checkNoAddressesInDB].! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
writeAddress
	| addressRow |
	addressRow := session system exampleAddressRow.
	session writeRow: addressRow.! !

!GlorpOneToOneDBTest methodsFor: 'tests-link table' stamp: ''!
writeCustomersWithOneAccountEach
	| customerRow accountRow1 accountRow2 linkRow1 linkRow2 customerRow2 |

	customerRow := session system exampleCustomerRow1.
	customerRow2 := session system exampleCustomerRow2.
	accountRow1 := session system exampleAccountRow1. 
	accountRow2 := session system exampleAccountRow3.
	linkRow1 := session system exampleCALinkRow1.
	linkRow2 := session system exampleCALinkRow3.
	session writeRow: customerRow.
	session writeRow: customerRow2.
	session writeRow: accountRow1.
	session writeRow: accountRow2.
	session writeRow: linkRow1.
	session writeRow: linkRow2.! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
writeHomefulPerson
	| personRow |
	personRow := session system examplePersonRow1.
	session writeRow: personRow.
	personId := personRow atFieldNamed: 'ID'.! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
writeHomelessPerson
	| personRow |
	personRow := session system examplePersonRow2.
	session writeRow: personRow.
	personId := personRow atFieldNamed: 'ID'.! !

!GlorpOneToOneDBTest methodsFor: 'support' stamp: ''!
writePersonWithAddress
	self writeAddress.
	self writeHomefulPerson.! !

GlorpMappingDBTest subclass: #GlorpOwnerSlaveTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpOwnerSlaveTest methodsFor: 'tests' stamp: ''!
test1
	| owner slaves table field |
	self
		inTransactionDo:
			[owner := GlorpOwner new.
			owner id: 1.
			slaves := OrderedCollection new.
			1 to: 5 do: [:id | slaves add: (GlorpSlave new id: id)].
			owner slaves: slaves.
			owner slave: slaves last.
			session beginUnitOfWork.
			session register: owner.
			session commitUnitOfWork.
			session reset.
			owner := session readOneOf: GlorpOwner where: [:each | each id = 1].
			self assert: owner slaves isEmpty not.
			table := session system tableNamed: 'GLORP_SLAVE'.
			field := table fieldNamed: 'OWNER_ID'.
			slaves := session
				readManyOf: GlorpSlave
				where: [:each | ((each getTable: table) getField: field) ~= nil].
			self assert: slaves size = 5].! !

!GlorpOwnerSlaveTest methodsFor: 'tests' stamp: ''!
testSortOrder
	"Validate that it doesn't matter what order we put in the input, the tables come out in the correct sort order."
	| justTheseTables sorted sorted2 |
	justTheseTables := Array with: (system tableNamed: 'GLORP_OWNER') with: (system tableNamed: 'GLORP_SLAVE').
	sorted := (TableSorter for: justTheseTables) sort.
	sorted2 := (TableSorter for: justTheseTables reverse) sort.
	self assert: sorted = sorted2.
	self assert: sorted first name = 'GLORP_SLAVE'.! !

GlorpMappingDBTest subclass: #GlorpVariableTypeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpVariableTypeTest commentStamp: '<historical>' prior: 0!
This exercises trying to map to objects that might be of multiple types, not related by inheritance. !

!GlorpVariableTypeTest methodsFor: 'Running' stamp: ''!
setUp
	super setUp.
	system := GlorpVariableTypeTestSystem forPlatform: session system platform.
	session system: system.! !

!GlorpVariableTypeTest methodsFor: 'testing' stamp: ''!
testRead
	| gallery image |
	self inTransactionDo: [
		gallery := GlorpGallery new name: 'Some Gallery'.
		gallery images add: (GlorpImageFile example1).
		session modify: gallery in: [].
		session reset.
		gallery := session readOneOf: GlorpGallery.
		self assert: gallery images size = 1.
		image := gallery images first.
		self assert: image class = GlorpImageFile.
		self assert: image bits = GlorpImageFile example1 bits.
		].! !

!GlorpVariableTypeTest methodsFor: 'testing' stamp: ''!
testReadBoth
	| gallery image link |
	self inTransactionDo: [
		gallery := GlorpGallery new name: 'Some Gallery'.
		gallery images add: (GlorpImageFile example2); add: (GlorpImageFile example1).
		session modify: gallery in: [].
		session reset.
		gallery := session readOneOf: GlorpGallery.
		self assert: gallery images size = 2.
		self assert: (gallery images select: [:each | each class = GlorpImageFile]) size = 1.
		image := gallery images detect: [:each | each class = GlorpImageFile].
		link := gallery images detect: [:each | each class = GlorpImageLink].
		self assert: (gallery images collect: [:each | each id]) asSet size = 2.
		self assert: image bits = (GlorpImageFile example1 bits).
		self assert: image thumbnailBits = (GlorpImageFile example1 bits).
		self assert: image thumbnailBits = (GlorpImageFile example2 thumbnailBits).


		].! !

!GlorpVariableTypeTest methodsFor: 'testing' stamp: ''!
testReadLarge
	| gallery image |
	self inTransactionDo: [
		gallery := GlorpGallery new name: 'Some Gallery'.
		gallery images add: (GlorpImageFile example2).
		session modify: gallery in: [].
		session reset.
		gallery := session readOneOf: GlorpGallery.
		self assert: gallery images size = 1.
		image := gallery images first.
		self assert: image class = GlorpImageLink.
		self assert: image thumbnailBits = (GlorpImageFile example2 thumbnailBits) .
		].! !

!GlorpVariableTypeTest methodsFor: 'testing' stamp: ''!
testWrite
	| gallery images links |
	self inTransactionDo: [
		gallery := GlorpGallery new name: 'Some Gallery'.
		gallery images add: (GlorpImageFile example1).
		session modify: gallery in: [].
		session reset.
		images := session accessor executeSQLString: 'SELECT * FROM GLORP_IMAGE_FILE'.
		self assert: images size = 1.
		links := session accessor executeSQLString: 'SELECT GALLERY_ID, IMAGE_ID, LARGE FROM GALLERY_LINK'.
		self assert: links size = 1.
		self assert: links first last ='F'].! !

!GlorpVariableTypeTest methodsFor: 'testing' stamp: ''!
testWriteLarge
	| gallery images links |
	self inTransactionDo: [
		gallery := GlorpGallery new name: 'Some Gallery'.
		gallery images add: (GlorpImageFile example2).
		session modify: gallery in: [].
		session reset.
		images := session accessor executeSQLString: 'SELECT * FROM GLORP_IMAGE_FILE'.
		self assert: images size = 1.
		links := session accessor executeSQLString: 'SELECT GALLERY_ID, IMAGE_ID, LARGE FROM GALLERY_LINK'.
		self assert: links size = 1.
		self assert: links first last = 'T'].! !

GlorpTestCase subclass: #GlorpMappingFromClassModelTests
	instanceVariableNames: 'system platform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpMappingFromClassModelTests methodsFor: 'running' stamp: ''!
setUp
	platform := PostgreSQLPlatform new.
	system := DynamicDescriptorSystem forPlatform: platform! !

!GlorpMappingFromClassModelTests methodsFor: 'Tests' stamp: ''!
someCode
	| model descriptor mapping table |
	table := DatabaseTable named: 'TYPETESTS'.
	table createFieldNamed: 'test' type: system platform boolean.
	model := system addClassModelFor: GlorpTypeTestsModelClass.
	model newAttributeNamed: #test type: Boolean.
	system privateTableAt: table name put: table.
	descriptor := Descriptor new.
	descriptor system: system.
	descriptor classModel: model.
	descriptor table: table.
	descriptor 
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'id')).
	Boolean isNil 
		ifTrue: [mapping := DirectMapping from: #test to: (table fieldNamed: 'test')]
		ifFalse: 
			[mapping := DirectMapping 
						from: #test
						type: Boolean
						to: (table fieldNamed: 'test')].
	descriptor addMapping: mapping.
	system privateDescriptorAt: GlorpTypeTestsModelClass put: descriptor.
	^system! !

!GlorpMappingFromClassModelTests methodsFor: 'Tests' stamp: ''!
testAllAttributes
	| bankTransTable customerTable ownerId btOwnerId customerModel customerDescriptor attributes |
	bankTransTable := system addEmptyTableNamed: 'BANK_TRANS'.
	customerTable := system addEmptyTableNamed: 'GR_CUSTOMER'.
	ownerId := customerTable createFieldNamed: 'ID' type: platform int4.
	btOwnerId := bankTransTable createFieldNamed: 'OWNER_ID'
				type: platform int4.
	bankTransTable addForeignKeyFrom: btOwnerId to: ownerId.
	customerModel := system addClassModelFor: GlorpCustomer.
	customerModel newAttributeNamed: #id type: Integer.
	customerModel newAttributeNamed: #transactions
		collectionOf: GlorpBankTransaction.
	system addClassModelFor: GlorpBankTransaction.
	system addEmptyDescriptorFor: GlorpBankTransaction.
	(system descriptorFor: GlorpBankTransaction) table: (system tableNamed: 'BANK_TRANS').
	customerDescriptor := system addEmptyDescriptorFor: GlorpCustomer.
	customerDescriptor table: customerTable.
	system createMappingsFromClassModelIn: customerDescriptor.
	self assert: (customerDescriptor mappingForAttributeNamed: #id) notNil.
	self 
		assert: (customerDescriptor mappingForAttributeNamed: #transactions) notNil.
	system validate.
	attributes := OrderedCollection new.
	customerDescriptor classModel attributesDo: [:each | attributes add: each].
	self assert: attributes size = 2.! !

!GlorpMappingFromClassModelTests methodsFor: 'Tests' stamp: ''!
testAllAttributesSkipDefined
	| bankTransTable customerTable ownerId btOwnerId customerModel customerDescriptor ownerId2 |
	bankTransTable := system addEmptyTableNamed: 'BANK_TRANS'.
	customerTable := system addEmptyTableNamed: 'GR_CUSTOMER'.
	ownerId := customerTable createFieldNamed: 'ID' type: platform int4.
	ownerId2 := customerTable createFieldNamed: 'ID2' type: platform int4.
	btOwnerId := bankTransTable createFieldNamed: 'OWNER_ID'
				type: platform int4.
	bankTransTable addForeignKeyFrom: btOwnerId to: ownerId.
	customerModel := system addClassModelFor: GlorpCustomer.
	customerModel newAttributeNamed: #id type: Integer.
	customerModel newAttributeNamed: #transactions
		collectionOf: GlorpBankTransaction.
	system addClassModelFor: GlorpBankTransaction.
	system addEmptyDescriptorFor: GlorpBankTransaction.
	customerDescriptor := system addEmptyDescriptorFor: GlorpCustomer.
	customerDescriptor table: customerTable.
	(customerDescriptor newMapping: DirectMapping) from: #id to: ownerId2.
	system createMappingsFromClassModelIn: customerDescriptor.
	self assert: customerDescriptor mappings size = 2.
	self 
		assert: (customerDescriptor mappingForAttributeNamed: #id) field = ownerId2! !

!GlorpMappingFromClassModelTests methodsFor: 'Tests' stamp: ''!
testDirectMapping
	| customerTable ownerId descriptor mapping customerModel |
	customerTable := system addEmptyTableNamed: 'GR_CUSTOMER'.
	ownerId := customerTable createFieldNamed: 'ID' type: platform int4.
	customerModel := system addClassModelFor: GlorpCustomer.
	customerModel newAttributeNamed: #id type: Integer.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	descriptor table: customerTable.
	system createMappingIn: descriptor fromClassModelForAttributeNamed: #id.
	mapping := descriptor mappingForAttributeNamed: #id.
	self assert: mapping class = DirectMapping.
	self assert: mapping field = ownerId! !

!GlorpMappingFromClassModelTests methodsFor: 'Tests' stamp: ''!
testDirectMappingNameFieldNameComputation
	| customerTable ownerId descriptor mapping customerModel |
	customerTable := system addEmptyTableNamed: 'GR_CUSTOMER'.
	ownerId := customerTable createFieldNamed: 'OWNER_ID' type: platform int4.
	customerModel := system addClassModelFor: GlorpCustomer.
	customerModel newAttributeNamed: #id type: Integer.
	descriptor := system addEmptyDescriptorFor: GlorpCustomer.
	descriptor table: customerTable.
	system attributeNameToFieldNameBlock: [:symbol | 'OWNER_ID'].
	system createMappingIn: descriptor fromClassModelForAttributeNamed: #id.
	mapping := descriptor mappingForAttributeNamed: #id.
	self assert: mapping class = DirectMapping.
	self assert: mapping field = ownerId! !

!GlorpMappingFromClassModelTests methodsFor: 'Tests' stamp: ''!
testOneToOneRelation
	| bankTransTable customerTable ownerId btOwnerId bankTransModel descriptor mapping |
	bankTransTable := system addEmptyTableNamed: 'BANK_TRANS'.
	customerTable := system addEmptyTableNamed: 'GR_CUSTOMER'.
	ownerId := customerTable createFieldNamed: 'ID' type: platform int4.
	btOwnerId := bankTransTable createFieldNamed: 'OWNER_ID'
				type: platform int4.
	bankTransTable addForeignKeyFrom: btOwnerId to: ownerId.
	system
		addClassModelFor: GlorpCustomer;
		addEmptyDescriptorFor: GlorpCustomer.
	bankTransModel := system addClassModelFor: GlorpBankTransaction.
	bankTransModel newAttributeNamed: #owner type: GlorpCustomer.
	descriptor := system addEmptyDescriptorFor: GlorpBankTransaction.
	descriptor table: bankTransTable.
	system createMappingIn: descriptor fromClassModelForAttributeNamed: #owner.
	mapping := descriptor mappingForAttributeNamed: #owner.
	self assert: mapping class = OneToOneMapping! !

!GlorpMappingFromClassModelTests methodsFor: 'Tests' stamp: ''!
testToManyRelation
	| bankTransTable customerTable ownerId btOwnerId mapping customerModel customerDescriptor |
	bankTransTable := system addEmptyTableNamed: 'BANK_TRANS'.
	customerTable := system addEmptyTableNamed: 'GR_CUSTOMER'.
	ownerId := customerTable createFieldNamed: 'ID' type: platform int4.
	btOwnerId := bankTransTable createFieldNamed: 'OWNER_ID'
				type: platform int4.
	bankTransTable addForeignKeyFrom: btOwnerId to: ownerId.
	customerModel := system addClassModelFor: GlorpCustomer.
	customerModel newAttributeNamed: #transactions
		collectionOf: GlorpBankTransaction.
	system addClassModelFor: GlorpBankTransaction.
	system addEmptyDescriptorFor: GlorpBankTransaction.
	customerDescriptor := system addEmptyDescriptorFor: GlorpCustomer.
	customerDescriptor table: customerTable.
	system createMappingIn: customerDescriptor
		fromClassModelForAttributeNamed: #transactions.
	mapping := customerDescriptor mappingForAttributeNamed: #transactions.
	self assert: mapping class = ToManyMapping.
	self assert: mapping referenceClass = GlorpBankTransaction! !

GlorpTestCase subclass: #GlorpMessageCollectorTest
	instanceVariableNames: 'collector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpMessageCollectorTest methodsFor: 'tests' stamp: ''!
messageCollectorDNUObjectStudio
	| caught |
	caught := false.
	[collector basicDoesNotUnderstand: #foo args: #()] on: MessageNotUnderstood
		do: [:signal | caught := true. signal sunitExitWith: nil].
	self assert: caught! !

!GlorpMessageCollectorTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	collector := MessageArchiver new! !

!GlorpMessageCollectorTest methodsFor: 'tests' stamp: ''!
testExpressionCreation
	| exp |
	exp := collector foo asGlorpExpression.
	self assert: exp name == #foo.
	self assert: exp base class == BaseExpression! !

!GlorpMessageCollectorTest methodsFor: 'tests' stamp: ''!
testMessageCollectorDNU
	| message caught |
	Dialect isObjectStudio ifTrue: [
		^self messageCollectorDNUObjectStudio].
	message := Message selector: #foo arguments: #().
	caught := false.
	[collector basicDoesNotUnderstand: message] on: MessageNotUnderstood
		do: [:signal | caught := true. signal sunitExitWith: nil].
	self assert: caught! !

!GlorpMessageCollectorTest methodsFor: 'tests' stamp: ''!
testMessageIntercept

	| foo |
	foo := collector foo.
	self assert: foo privateGlorpSelector == #foo.
	self assert: foo privateGlorpReceiver == collector.! !

GlorpTestCase subclass: #GlorpObjectTransactionTest
	instanceVariableNames: 'transaction objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpObjectTransactionTest methodsFor: 'support' stamp: ''!
setUp

	transaction := ObjectTransaction new! !

!GlorpObjectTransactionTest methodsFor: 'tests' stamp: ''!
testArray
	| object |
	object := #(1 2 3 4 5) copy.
	transaction begin.
	transaction register: object.
	object
		at: 1 put: #one;
		at: 2 put: object.
	transaction abort.
	self
		assert: (object at: 1) == 1;
		assert: (object at: 2) == 2! !

!GlorpObjectTransactionTest methodsFor: 'tests' stamp: ''!
testBecome
	| object |
	object := 'hello' copy.
	transaction begin.
	transaction register: object.
	object become: Set new.
	transaction abort.
	self 
		assert: object class == '' class;
		assert: object = 'hello'.! !

!GlorpObjectTransactionTest methodsFor: 'tests' stamp: ''!
testCommit
	| array |
	array := #(1 2 3 4 5) copy.
	transaction begin.
	transaction register: array.
	array
		at: 1 put: #one;
		at: 2 put: array.
	transaction commit.
	self
		assert: (array at: 1) == #one;
		assert: (array at: 2) == array! !

!GlorpObjectTransactionTest methodsFor: 'tests' stamp: ''!
testHashedCollection
	| object originalMembers |
	object := Set new.
	originalMembers := #(#one #two #three 'four' 5 'vi' (1 2 3 4 5 6 7)) collect: [:each | each copy].
	object addAll: originalMembers.
	transaction begin.
	transaction register: object.
	object remove: #one; remove: (originalMembers at: 4).
	object add: 1.
	originalMembers last at: 7 put: 'seven'.
	transaction abort.
	self
		assert: object size = originalMembers size;
		assert: (object includes: originalMembers first);
		assert: (object includes: (originalMembers at: 4));
		assert: object size = (object rehash; size).
	originalMembers do: [:each | self assert: (object includes: each)]! !

!GlorpObjectTransactionTest methodsFor: 'tests' stamp: ''!
testRegisteredObjectsDoRecursively

	| link |
	link := 1 -> (2 -> (3 -> (4 ->5))).
	transaction register: link.
	self assert: transaction undoMap size = 1.
	transaction registeredObjectsDo: [:each |
		transaction register: each value].
	self assert: transaction undoMap size = 4.! !

!GlorpObjectTransactionTest methodsFor: 'tests' stamp: ''!
testString
	| object |
	object := 'Hello, World!!' copy.
	transaction begin.
	transaction register: object.
	object
		at: 1 put: $h;
		at: 2 put: $E.
	transaction abort.
	self
		assert: object first  == $H;
		assert: (object at: 2) == $e! !

GlorpTestCase subclass: #GlorpPartialWritesTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpPartialWritesTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.! !

!GlorpPartialWritesTest methodsFor: 'tests' stamp: ''!
testNoDifferenceNoWrite
self todo! !

!GlorpPartialWritesTest methodsFor: 'tests' stamp: ''!
testWritingNewObject
self todo! !

!GlorpPartialWritesTest methodsFor: 'tests' stamp: ''!
testWritingObjectWithAddedComponent
self todo! !

!GlorpPartialWritesTest methodsFor: 'tests' stamp: ''!
testWritingObjectWithChangedAttribute
self todo! !

!GlorpPartialWritesTest methodsFor: 'tests' stamp: ''!
testWritingObjectWithChangedComponent
self todo! !

!GlorpPartialWritesTest methodsFor: 'tests' stamp: ''!
testWritingObjectWithChangedComponentAttribute
self todo! !

!GlorpPartialWritesTest methodsFor: 'tests' stamp: ''!
testWritingObjectWithDeletedComponent
self todo! !

!GlorpPartialWritesTest methodsFor: 'tests' stamp: ''!
testWritingObjectWithDeletedRelationship
self todo! !

GlorpTestCase subclass: #GlorpProxyEqualityTest
	instanceVariableNames: 'session proxy proxy2 stub2 stub'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpProxyEqualityTest class methodsFor: 'Accessing' stamp: ''!
resources
	^Array with: GlorpSessionResource! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
setUp 

	super setUp. 
	session := GlorpSessionResource current newSession.
	proxy := Proxy new.
	proxy session: session.
	stub := GlorpQueryStub returningOneOf: GlorpAddress where: [:address | address id = 1].
	stub result: 'abc'.
	proxy query: stub.
	proxy parameters: #().
	proxy2 := Proxy new.
	proxy2 session: session.
	stub2 := GlorpQueryStub returningOneOf: GlorpAddress where: [:address | address id = 1].
	stub2 result: 'abc'.
	proxy2 query: stub2.
	proxy2 parameters: #().! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
setUp2
	proxy query result: 'def'! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
testEquality
	self assert: proxy = proxy2.! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
testEquality2
	self assert: proxy2 = proxy.! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
testEqualityFailing
	self setUp2.
	self deny: proxy = proxy2.! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
testEqualityFailing2
	self setUp2.
	self deny: proxy2 = proxy.! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
testInequality
	self setUp2.
	self deny: proxy = proxy2.! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
testInequality2
	self setUp2.
	self deny: proxy2 = proxy.! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
testInequalityFailing
	self deny: proxy ~= proxy2.! !

!GlorpProxyEqualityTest methodsFor: 'initialize-release' stamp: ''!
testInequalityFailing2
	self deny: proxy2 ~= proxy.! !

GlorpTestCase subclass: #GlorpProxyTest
	instanceVariableNames: 'session proxy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpProxyTest class methodsFor: 'resources' stamp: ''!
resources

	^Array with: GlorpSessionResource.! !

!GlorpProxyTest methodsFor: 'support' stamp: ''!
setUp

	| stub |
	super setUp.
	session := GlorpSessionResource current newSession.
	proxy := Proxy new.
	proxy session: session.
	stub := GlorpQueryStub returningOneOf: GlorpAddress where: [:address | address id = 1].
	stub result: 42.
	proxy query: stub.
	proxy parameters: #().! !

!GlorpProxyTest methodsFor: 'tests' stamp: ''!
testAdHocProxy
	"Some dialects (e.g. VA) inline yourself so we must send yourSelf."

	| p |
	p := AdHocProxy readBlock: ['instantiated!!'].
	self assert: p printString = '{uninstantiated AdHocProxy}'.
	self assert: p isInstantiated not.
	p yourSelf.
	self assert: p isInstantiated.
	self assert: p printString = '{''instantiated!!''}'.
	p uninstantiate.
	self deny: p isInstantiated.! !

!GlorpProxyTest methodsFor: 'tests' stamp: ''!
testCreation
	| otherProxy |
	self deny: proxy isInstantiated.
	otherProxy := Proxy new.
	self deny: otherProxy isInstantiated.! !

!GlorpProxyTest methodsFor: 'tests' stamp: ''!
testInstantiationFromStub

	self assert: (proxy getValue notNil).
	self assert: proxy = 42.
	self assert: proxy isInstantiated.! !

!GlorpProxyTest methodsFor: 'tests' stamp: ''!
testPrintingInstantiated

	proxy getValue.
	self assert: proxy printString = ('{', proxy getValue printString, '}').! !

!GlorpProxyTest methodsFor: 'tests' stamp: ''!
testPrintingUninstantiated

	self assert: proxy printString = '{uninstantiated GlorpAddress}'.! !

!GlorpProxyTest methodsFor: 'tests' stamp: ''!
testPrintingUninstantiatedCollection
	proxy query readsOneObject: false.
	self assert: proxy printString = '{uninstantiated collection of GlorpAddress}'.! !

GlorpTestCase subclass: #GlorpReadQueryTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpReadQueryTest class methodsFor: 'resources' stamp: ''!
resources

	^Array with: GlorpSessionResource.! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
helpTestSingleOrderingBy: aBlock setup: setupBlock

	| query result realBlock |
	query := Query returningManyOf: GlorpAddress.
	query orderBy: aBlock.
	realBlock := query expressionBlockFor: aBlock.
	result := session execute: query.
	self verifyOrderFor: result byAttribute: realBlock.! !

!GlorpReadQueryTest methodsFor: 'support' stamp: ''!
setUp

	super setUp.
	session := GlorpSessionResource current newSession.
	session system: (GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database).
	session beginTransaction.
	self writeAddressDoubleOrderingRows.
	self writePersonOrderingRows.! !

!GlorpReadQueryTest methodsFor: 'support' stamp: ''!
tearDown

	super tearDown.
	session rollbackTransaction.
	session reset.
	session := nil.! !

!GlorpReadQueryTest methodsFor: 'tests' stamp: ''!
testCriteriaSetup
	| query |
	query := Query returningOneOf: GlorpAddress where: [:each | each id = 12].
	query session: session.
	query setUpExpressions.
	self assert: query whereClause class == RelationExpression.
	self assert: query whereClause ultimateBaseExpression descriptor 
				== (session descriptorFor: GlorpAddress)! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
testDescendingSingleOrdering
	| query result sortedResult |
	
	query := Query returningManyOf: GlorpAddress where: [:each | each street ~= 'Beta'].
	query orderBy: [:each | each street descending].
	result := session execute: query.
	sortedResult := result 
				asSortedCollection: [:a :b | a street > b street].
	self assert: sortedResult asArray = result asArray.! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
testDoubleOrderingAddress
	| query |
	query := Query returningManyOf: GlorpAddress.
	query orderBy: [:each | each street].
	query orderBy: [:each | each number].
	self validateDoubleOrderFor: query.! !

!GlorpReadQueryTest methodsFor: 'tests' stamp: ''!
testIn
	| query result |
	
	query := Query returningManyOf: GlorpAddress where: [:each | each street in: #('Beta' 'Alpha')].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | #('Beta' 'Alpha') includes: each street]).
	self assert: result size = 4.! !

!GlorpReadQueryTest methodsFor: 'tests' stamp: ''!
testInEmpty
	| query result |
	
	query := Query returningManyOf: GlorpAddress where: [:each | each street in: #()].
	result := session execute: query.
	self assert: result isEmpty.! !

!GlorpReadQueryTest methodsFor: 'tests' stamp: ''!
testInInteger
	| query result |
	
	query := Query returningManyOf: GlorpAddress where: [:each | each id in: #(1 2)].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | #(1 2) includes: each id]).
	self assert: result size = 2.! !

!GlorpReadQueryTest methodsFor: 'tests' stamp: ''!
testInSymbol
	| query result transRow transRow2 |
	transRow := session system exampleBankTransactionRow.
	session writeRow: transRow.	
	transRow2 := session system exampleBankTransactionRow2.
	session writeRow: transRow2.	
	query := Query returningManyOf: GlorpBankTransaction where: [:each | each amount currency in: #(#USD #CDN)].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | each amount currency = #CDN]).
	self assert: result size = 1.
	query := Query returningManyOf: GlorpBankTransaction where: [:each | each amount currency in: #(#USD #DM)].
	result := session execute: query.
	self assert: result isEmpty.! !

!GlorpReadQueryTest methodsFor: 'tests' stamp: ''!
testLike
	| query result |
	
	query := Query returningManyOf: GlorpAddress where: [:each | each street like: 'Be%'].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | each street = 'Beta']).
	self assert: result size = 3.! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
testMixedDoubleOrderingAddress
	| query |
	query := Query returningManyOf: GlorpAddress.
	query orderBy: [:each | each street descending].
	query orderBy: [:each | each number].
	self validateDoubleOrderMixedFor: query.! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
testOrderingByRelatedObjectAttribute
	| query result |
	query := Query returningManyOf: GlorpPerson.
	query orderBy: [:each | each address street].
	result := session execute: query.
	self verifyOrderFor: result byAttribute: [:each | each address street].! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
testOrderingWithNonEmptyWhereClause

	| query result |
	query := Query returningManyOf: GlorpPerson where: [:each | each id ~= 12].
	query orderBy: #(#address #street).
	result := session execute: query.
	self verifyOrderFor: result byAttribute: [:each | each address street].! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: ''!
testReadDataItemsFromEmbeddedObject

	| query result transRow id |
	transRow := session system exampleBankTransactionRow.
	session writeRow: transRow.
	id := transRow atFieldNamed: 'ID'.
	query := Query returningManyOf: GlorpBankTransaction.
	query retrieve: [:each | each id].
	query retrieve: [:each | each serviceCharge description].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = (Array with: id with: 'additional overcharge').! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: ''!
testReadDistinctIds

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each id distinct].
	result := query executeIn: session.
	self assert: (result asSortedCollection asArray = #(86 87 88)).! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: ''!
testReadDistinctIdsWithWhereClause

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each id distinct].
	query whereClause: [:each | each id ~= 423421].
	result := query executeIn: session.
	self assert: (result asSortedCollection asArray = #(86 87 88)).! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: ''!
testReadDistinctRelatedAttribute

	| query result allStreets |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each address street distinct].
	result := query executeIn: session.
	self assert: (result asSortedCollection asArray = #('Alpha' 'Beta' 'Gamma')).
	allStreets := (session readManyOf: GlorpAddress) collect: [:each | each street].
	self assert: allStreets size = 5.! !

!GlorpReadQueryTest methodsFor: 'tests' stamp: ''!
testReadMultipleObjects

	| query result tracing addressReadSeparately personReadSeparately allResults |
	query := Query returningManyOf: GlorpPerson.
	tracing := Tracing new.
	tracing retrieve: [:each | each ].
	tracing retrieve: [:each | each address].
	query tracing: tracing.
	query orderBy: #id.
	allResults := query executeIn: session.
	self assert: allResults size = 3.
	result := allResults first.
	self assert: (result at: 1) id = 86.
	self assert: ((result at: 1) class == GlorpPerson).
	self assert: (result at: 1) address == (result at: 2).
	self assert: ((result at: 2) class == GlorpAddress).
	self assert: ((result at: 2) id = 2).
	addressReadSeparately := session readOneOf: GlorpAddress where: [:each | each id = 2].
	self assert: result last == addressReadSeparately.
	personReadSeparately := session readOneOf: GlorpPerson where: [:each | each id = 86].
	self assert: personReadSeparately == result first.
	self assert: personReadSeparately address yourSelf == result last.! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: ''!
testReadObjectsAndData

	| query result tracing addressReadSeparately personReadSeparately |
	query := Query returningManyOf: GlorpPerson.
	tracing := Tracing new.
	tracing retrieve: [:each | each id].
	tracing retrieve: [:each | each address].
	query tracing: tracing.
	query orderBy: #id.
	result := query executeIn: session.
	self assert: result size = 3.
	self assert: (result first at: 1) = 86.
	self assert: ((result first at: 2) class == GlorpAddress).
	self assert: ((result first at: 2) id = 2).
	addressReadSeparately := session readOneOf: GlorpAddress where: [:each | each id = 2].
	self assert: result first last == addressReadSeparately.
	personReadSeparately := session readOneOf: GlorpPerson where: [:each | each id = 86].
	self assert: personReadSeparately address yourSelf == result first last.! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: ''!
testReadOneWithObjects

	| query result |
	query := Query returningOneOf: GlorpPerson where: [:each | each id = 86].
	query retrieve: [:each | each].
	query retrieve: [:each | each address].
	query orderBy: [:each | each id].
	result := query executeIn: session.
	self assert: result first id = 86.
	self assert: (result first address yourSelf == result last).! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: ''!
testReadOnlyPrimaryKeys

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each id].
	result := query executeIn: session.
	self assert: (result asSortedCollection asArray = #(86 87 88)).! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: ''!
testReadTwoDataItems

	| query result tracing|
	query := Query returningManyOf: GlorpPerson.
	tracing := Tracing new.
	tracing retrieve: [:each | each id].
	tracing retrieve: [:each | each name].
	query tracing: tracing.
	result := query executeIn: session.
	self assert: result size = 3.
	self assert: result first = #(86 'person1').
	self assert: (result at: 2) = #(87 'person2').
	self assert: result last = #(88 'person3').! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: ''!
testReadTwoDataItemsFromDifferentObjects

	| query result tracing|
	query := Query returningManyOf: GlorpPerson.
	tracing := Tracing new.
	tracing retrieve: [:each | each id].
	tracing retrieve: [:each | each address street].
	query tracing: tracing.
	query orderBy: #id.
	result := query executeIn: session.
	self assert: result size = 3.
	self assert: result first = #(86 'Beta').
	self assert: (result at: 2) = #(87 'Gamma').
	self assert: result last = #(88 'Alpha').! !

!GlorpReadQueryTest methodsFor: 'tests-data reading' stamp: 'GuillermoPolito 4/26/2011 04:58'!
testReadTwoObjects

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query retrieve: [:each | each].
	query retrieve: [:each | each address].
	query orderBy: [:each | each id].
	result := query executeIn: session.
	self assert: result size = 3.
	self assert: (result first at: 1) id = 86.
	self assert: ((result first at: 1) address yourSelf == result first last).! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
testSingleOrderingAddress1

	self 
		helpTestSingleOrderingBy: [:each | each street]
		setup: [self writeAddressOrderingRows].! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
testSingleOrderingAddress2

	self 
		helpTestSingleOrderingBy: [:each | each number]
		setup: [self writeAddressOrderingRows].! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
testSingleOrderingBySymbol

	self 
		helpTestSingleOrderingBy: #street
		setup: [self writeAddressOrderingRows].! !

!GlorpReadQueryTest methodsFor: 'tests-ordering' stamp: ''!
testSymbolsOrderingByRelatedObjectAttribute

	| query result |
	query := Query returningManyOf: GlorpPerson.
	query orderBy: #(#address #street).
	result := session execute: query.
	self verifyOrderFor: result byAttribute: [:each | each address street].! !

!GlorpReadQueryTest methodsFor: 'support' stamp: ''!
validateDoubleOrderFor: query 
	| result sortedResult |
	result := session execute: query.
	sortedResult := result asSortedCollection: 
					[:a :b | 
					a street = b street 
						ifTrue: [a number <= b number]
						ifFalse: [a street < b street]].
	^self assert: sortedResult asArray = result asArray! !

!GlorpReadQueryTest methodsFor: 'support' stamp: ''!
validateDoubleOrderMixedFor: query 
	| result sortedResult |
	result := session execute: query.
	sortedResult := result asSortedCollection: 
					[:a :b | 
					a street = b street 
						ifTrue: [a number <= b number]
						ifFalse: [a street > b street]].
	^self assert: sortedResult asArray = result asArray! !

!GlorpReadQueryTest methodsFor: 'support' stamp: ''!
verifyOrderFor: result byAttribute: aBlock
	1 to: result size - 1
		do: 
			[:i | 
			| a b |
			a := result at: i.
			b := result at: i + 1.
			self assert: (aBlock value: a) <= (aBlock value: b)]! !

!GlorpReadQueryTest methodsFor: 'support' stamp: ''!
writeAddressDoubleOrderingRows

	self writeAddressOrderingRows.
	session writeRow: session system exampleAddressRowForOrdering4.
	session writeRow: session system exampleAddressRowForOrdering5.! !

!GlorpReadQueryTest methodsFor: 'support' stamp: ''!
writeAddressOrderingRows

	session writeRow: session system exampleAddressRowForOrdering1.
	session writeRow: session system exampleAddressRowForOrdering2.
	session writeRow: session system exampleAddressRowForOrdering3.! !

!GlorpReadQueryTest methodsFor: 'support' stamp: ''!
writePersonOrderingRows

	session writeRow: session system examplePersonRowForOrdering1.
	session writeRow: session system examplePersonRowForOrdering2.
	session writeRow: session system examplePersonRowForOrdering3.! !

GlorpTestCase subclass: #GlorpReadingDifferentCollectionsTest
	instanceVariableNames: 'system session singleQuery allQuery singleResult allResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpReadingDifferentCollectionsTest methodsFor: 'tests' stamp: ''!
check: aClass
	| collectionClass |
	collectionClass := aClass isBehavior ifTrue: [aClass] ifFalse: [aClass class].
	singleQuery collectionType: aClass.
	allQuery collectionType: aClass.
	singleResult := session execute: singleQuery.
	allResult := session execute: allQuery.
	self assert: singleResult class == collectionClass.
	self assert: allResult class == collectionClass.
	self assert: singleResult size = 1.
	self assert: allResult size = 2.! !

!GlorpReadingDifferentCollectionsTest methodsFor: 'setup' stamp: ''!
setUp
	
	| |
	super setUp.
	session := GlorpSessionResource current newSession.
	system := session system.
	session beginTransaction.
	self writeAccountRows.
	singleQuery := Query returningManyOf: GlorpBankAccount
				where: [:passenger | passenger id = 6].
	allQuery := Query returningManyOf: GlorpBankAccount.! !

!GlorpReadingDifferentCollectionsTest methodsFor: 'setup' stamp: ''!
tearDown

	session rollbackTransaction.! !

!GlorpReadingDifferentCollectionsTest methodsFor: 'tests' stamp: ''!
testArray
	self check: Array.! !

!GlorpReadingDifferentCollectionsTest methodsFor: 'tests' stamp: ''!
testBlank
	singleResult := session execute: singleQuery.
	allResult := session execute: allQuery.
	self assert: singleResult class == Array.
	self assert: allResult class == Array.
	self assert: singleResult size = 1.
	self assert: allResult size = 2.! !

!GlorpReadingDifferentCollectionsTest methodsFor: 'tests' stamp: ''!
testOrderedCollection
	self check: OrderedCollection.! !

!GlorpReadingDifferentCollectionsTest methodsFor: 'tests' stamp: ''!
testSet
	self check: Set.! !

!GlorpReadingDifferentCollectionsTest methodsFor: 'tests' stamp: ''!
testSortedCollection
	self check: SortedCollection.
	self assert: allResult first <= allResult last.! !

!GlorpReadingDifferentCollectionsTest methodsFor: 'tests' stamp: ''!
testSortedCollectionReversed
	self check: (SortedCollection sortBlock: [:a :b | b <= a]).
	self assert: allResult last <= allResult first.! !

!GlorpReadingDifferentCollectionsTest methodsFor: 'setup' stamp: ''!
writeAccountRows
	| accountRow1 accountRow2 |

	accountRow1 := session system exampleAccountRow1. 
	accountRow2 := session system exampleAccountRow2.
	session writeRow: accountRow1.
	session writeRow: accountRow2.! !

GlorpTestCase subclass: #GlorpReadingDifferentCollectionsThroughMappingsTest
	instanceVariableNames: 'system session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpReadingDifferentCollectionsThroughMappingsTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource with: GlorpSessionResource.! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'setup' stamp: ''!
descriptorClass
	^GlorpCollectionTypesDescriptorSystem.! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'setup' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	system := self descriptorClass forPlatform: session platform.
	session system: system.
	session beginTransaction.! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'setup' stamp: ''!
tearDown
	super tearDown.
	session rollbackTransaction.! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'tests' stamp: ''!
testReadBack
	"Some dialects (e.g. VA) inline yourself so we must send yourSelf."

	| thing list |
	self writeRows.
	session reset.
	list := session readManyOf: GlorpThingWithLotsOfDifferentCollections.
	self assert: list size = 1.
	thing := list first.
	self assert: thing array size = 3.
	self
		assert:
			(self validateFor: thing array against: #('array1' 'array2' 'array3')).
	self assert: thing array yourSelf class = Array.
	self assert: thing set size = 2.
	self assert: thing set yourSelf class = Set.
	self assert: (self validateFor: thing set against: #('set1' 'set2')).
	self assert: thing orderedCollection size = 2.
	self assert: thing orderedCollection yourSelf class = OrderedCollection.
	self
		assert:
			(self
				validateFor: thing orderedCollection
				against: #('orderedCollection1' 'orderedCollection2')).
	self assert: thing orderedCollection first name = 'orderedCollection1'.
	self assert: thing bag size = 2.
	self assert: thing bag yourSelf class = Bag.
	self assert: (self validateFor: thing bag against: #('bag1' 'bag2')).
	self assert: thing sortedCollection size = 4.
	self assert: thing sortedCollection yourSelf class = SortedCollection.
	self assert:
			(thing sortedCollection collect: [:each | each name]) asArray
				= #('sorted1' 'sorted2' 'sorted3' 'sorted4').! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'tests' stamp: ''!
testReadBackOneOfSeveral
	"Some dialects (e.g. VA) inline yourself so we must send yourSelf."

	| thing list |
	self writeMore.
	session reset.
	list := session readManyOf: GlorpThingWithLotsOfDifferentCollections where: [:each | each name = 'fred'].
	self assert: list size = 1.
	thing := list first.
	self assert: thing array size = 3.
	self
		assert:
			(self validateFor: thing array against: #('array1' 'array2' 'array3')).
	self assert: thing array yourSelf class = Array.
	self assert: thing set size = 2.
	self assert: thing set yourSelf class = Set.
	self assert: (self validateFor: thing set against: #('set1' 'set2')).
	self assert: thing orderedCollection size = 2.
	self assert: thing orderedCollection yourSelf class = OrderedCollection.
	self
		assert:
			(self
				validateFor: thing orderedCollection
				against: #('orderedCollection1' 'orderedCollection2')).
	self assert: thing bag size = 2.
	self assert: thing bag yourSelf class = Bag.
	self assert: (self validateFor: thing bag against: #('bag1' 'bag2')).
	self assert: thing sortedCollection size = 4.
	self assert: thing sortedCollection yourSelf class = SortedCollection.
	self assert:
			(thing sortedCollection collect: [:each | each name]) asArray
				= #('sorted1' 'sorted2' 'sorted3' 'sorted4').! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'tests' stamp: ''!
testReadCollectionWithOrder
	| thing list |
	session transact: [session register: GlorpThingWithLotsOfDifferentCollections exampleForOrdering].
	session reset.
	list := session readManyOf: GlorpThingWithLotsOfDifferentCollections.
	self assert: list size = 1.
	thing := list first.
	self assert: thing orderedCollection size = 6.
	self
		assert:
			(thing orderedCollection collect: [:each | each name]) asArray = #('oc6' 'oc5' 'oc4' 'oc3' 'oc7' 'oc8')! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'tests' stamp: ''!
testReadManyToManyWithOrder
	| thing list |
	session transact: [session register: GlorpThingWithLotsOfDifferentCollections exampleForOrdering].
	session reset.
	list := session readManyOf: GlorpThingWithLotsOfDifferentCollections.
	self assert: list size = 1.
	thing := list first.
	self assert: thing orderedCollection size = 6.
	self
		assert:
			(thing orderedCollection collect: [:each | each name]) asArray = #('oc6' 'oc5' 'oc4' 'oc3' 'oc7' 'oc8')! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'tests' stamp: ''!
testReadOneToManyWithOrder
	| thing list |
	session transact: [session register: GlorpThingWithLotsOfDifferentCollections exampleForOrdering].
	session reset.
	list := session readManyOf: GlorpThingWithLotsOfDifferentCollections.
	self assert: list size = 1.
	thing := list first.
	self assert: thing array size = 6.
	self
		assert:
			(thing array collect: [:each | each name]) asArray = #('a1' 'a2' 'a3' 'a9' 'a8' 'a7')! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'tests' stamp: ''!
validateFor: aCollection against: expectedArrayContents
	^(aCollection collect: [:each | each name]) asSortedCollection asArray
		= expectedArrayContents.! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'setup' stamp: ''!
writeMore

	| other |
	session transact: [
		session register: GlorpThingWithLotsOfDifferentCollections example1.
		other := GlorpThingWithLotsOfDifferentCollections example1.
		other name: 'barney'.
		session register: other].! !

!GlorpReadingDifferentCollectionsThroughMappingsTest methodsFor: 'setup' stamp: ''!
writeRows

	session transact: [
		session register: GlorpThingWithLotsOfDifferentCollections example1].! !

GlorpReadingDifferentCollectionsThroughMappingsTest subclass: #GlorpReadingDifferentCollectionsThroughMappingsWithClassModelsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpReadingDifferentCollectionsThroughMappingsWithClassModelsTest methodsFor: 'setup' stamp: ''!
descriptorClass
	^GlorpCollectionTypesWithClassModelsDescriptorSystem.! !

GlorpTestCase subclass: #GlorpReadingPersonWithEmailAddressesTest
	instanceVariableNames: 'session personRow addressRow emailAddress1Row emailAddress2Row id1 id2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpReadingPersonWithEmailAddressesTest class methodsFor: 'resources' stamp: ''!
resources
	^Array with: GlorpSessionResource.! !

!GlorpReadingPersonWithEmailAddressesTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	session beginTransaction.
	addressRow := session system exampleAddressRow.
	session writeRow: addressRow.
	personRow := session system examplePersonRow1.
	session writeRow: personRow.
	emailAddress1Row := session system exampleEmailAddressRow1.
	emailAddress2Row := session system exampleEmailAddressRow2.
	id1 := emailAddress1Row at: (emailAddress1Row table fieldNamed: 'ID').
	id2 := emailAddress2Row at: (emailAddress2Row table fieldNamed: 'ID').
	session writeRow: emailAddress1Row.
	session writeRow: emailAddress2Row.! !

!GlorpReadingPersonWithEmailAddressesTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session rollbackTransaction.
	session reset.
	session := nil.! !

!GlorpReadingPersonWithEmailAddressesTest methodsFor: 'tests' stamp: ''!
testObjectsNotAddedTwiceWhenReadingMultipleObjectsOneToMany
	"Read in the objects first, so they're in cache. Make sure they don't get the collection built up twice."
	| people |

	people := session readManyOf: GlorpPerson.
	self testReadMultipleObjectsOneToMany.
	^people.! !

!GlorpReadingPersonWithEmailAddressesTest methodsFor: 'tests' stamp: ''!
testPreparedStatementsAreFaster
	"Not really a good test, since there are lots of other factors. And since we don't support this on all dialects/databases, they might easily be the same. Maybe should remove this test, but on the other hand it's the most useful feedback that the prepared statements are actually good for something"
	| timePrepared timeUnPrepared| 
	session reusePreparedStatements: true.
	session reset.
	timeUnPrepared := Time millisecondsToRun: [
		session readManyOf: GlorpPerson where: [:eachPerson | eachPerson id = 3].
		session readManyOf: GlorpPerson where: [:eachPerson | eachPerson id ~= 3].
		session readManyOf: GlorpPerson where: [:eachPerson | eachPerson id >= 3]].
	session reset.
	timePrepared := Time millisecondsToRun: [
		3 timesRepeat: [ | query |
			query := Query returningManyOf: GlorpPerson where: [:eachPerson | eachPerson id = (eachPerson parameter: 1)].
			query executeWithParameters: #(3) in: session]].
	(session accessor numberOfPreparedStatements < 1) ifTrue: [^self "Unsupported, don't bother testing"].
	Transcript cr; show: 'Time reusing prepared statements = ', timePrepared printString.
	Transcript cr; show: 'Time not reusing prepared statements = ', timeUnPrepared printString.
	"Give a little bit of room, so if they take roughly the same amount of time it'll still pass"
	self assert: (timePrepared * 0.8 )< timeUnPrepared.! !

!GlorpReadingPersonWithEmailAddressesTest methodsFor: 'tests' stamp: ''!
testPreparedStatementsAreReused

	| | 
	"This test only makes sense if binding is on" 
	session useBinding ifFalse: [^self].
	session reusePreparedStatements: true.
	session reset.
	session readManyOf: GlorpPerson where: [:eachPerson | eachPerson id = 3].
	session readManyOf: GlorpPerson where: [:eachPerson | eachPerson id ~= 3].
	session readManyOf: GlorpPerson where: [:eachPerson | eachPerson id >= 3].
	self assert: session accessor numberOfPreparedStatements = 3.
	session reset.
	1 to: 3 do: [:i | | query |
			query := Query returningManyOf: GlorpPerson where: [:eachPerson | eachPerson id = (eachPerson parameter: 1)].
			query executeWithParameters: (Array with: i) in: session].
	self assert: session accessor numberOfPreparedStatements = 1.! !

!GlorpReadingPersonWithEmailAddressesTest methodsFor: 'tests' stamp: ''!
testReadMultipleObjectsOneToMany
	| query result person addresses |

	query := Query returningManyOf: GlorpPerson where: [:eachPerson | eachPerson id = 3].
	query alsoFetch: [:each | each emailAddresses].
	result := query executeIn: session.
	self assert: result size = 1.
	person := result first.
	addresses := person emailAddresses.
	self deny: addresses isGlorpProxy.
	self assert: addresses size = 2.
	self 
		assert: (addresses first id = id1 or: [addresses last id = id1]).
	self 
		assert: (addresses first id = id2 or: [addresses last id = id2]).
	self assert: addresses first id ~= addresses last id.! !

!GlorpReadingPersonWithEmailAddressesTest methodsFor: 'tests' stamp: ''!
testReadPersonWithEmailAddresses
	| query result emailAddresses |
	query := Query
		returningOneOf:GlorpPerson
		where: [:person | person id = 3].
	result := query executeIn: session.
	emailAddresses := result emailAddresses getValue.
	self assert: emailAddresses size = 2.
	self
		assert:
			(emailAddresses first id = id1 or: [emailAddresses last id = id1]).
	self
		assert:
			(emailAddresses first id = id2 or: [emailAddresses last id = id2]).
	self assert: emailAddresses first id ~= emailAddresses last id.! !

GlorpTestCase subclass: #GlorpReadingTest
	instanceVariableNames: 'system session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpReadingTest commentStamp: '<historical>' prior: 0!
This tests the full reading mechanism, writing out some rows manually and then doing various read operations.

Instance Variables:
	session	<Session>	
	system	<GlorpDemoDescriptorSystem>	

!

!GlorpReadingTest class methodsFor: 'resources' stamp: ''!
resources

	^Array with: GlorpSessionResource with: GlorpDemoTablePopulatorResource.! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
checkExpiryTime
	""
	| rowToWrite address modifiedRow expiryTime newExpiryTime |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	address := session readOneOf: GlorpAddress
				where: [:each | each id = 123].
	expiryTime := ((session cacheFor: address) basicAt: 123 ifAbsent: [#(nil)]) at: 1.
	modifiedRow := session system exampleModifiedAddressRow.
	modifiedRow owner: address. "Otherwise it thinks it's an insert"
	session writeRow: modifiedRow.
	(Delay forSeconds: 1) wait.
	address := session readOneOf: GlorpAddress
				where: [:each | each id = 123].
	newExpiryTime := ((session cacheFor: address) basicAt: 123 ifAbsent: [#(nil)]) at: 1.
	self assert: newExpiryTime > expiryTime.
	self assert: address street = 'Paseo Montril'] 
			ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
checkExpiryTime: aBlock
	""
	| rowToWrite address modifiedRow expiryTime newExpiryTime |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	address := session readOneOf: GlorpAddress
				where: [:each | each id = 123].
	expiryTime := ((session cacheFor: address) basicAt: 123 ifAbsent: [#(nil)]) at: 1.
	modifiedRow := session system exampleModifiedAddressRow.
	modifiedRow owner: address. "Otherwise it thinks it's an insert"
	session writeRow: modifiedRow.
	(Delay forSeconds: 1) wait.
	address := session readOneOf: GlorpAddress
				where: [:each | each id = 123].
	newExpiryTime := ((session cacheFor: address) basicAt: 123 ifAbsent: [#(nil)]) at: 1.
	aBlock value: expiryTime value: newExpiryTime value: address] 
			ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
checkRefreshDoing: aBlock
	"Check that we refresh correctly doing the action specified by aBlock"
	| rowToWrite address modifiedRow |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	address := session readOneOf: GlorpAddress
				where: [:each | each id = 123].
	modifiedRow := session system exampleModifiedAddressRow.
	modifiedRow owner: address. "Otherwise it thinks it's an insert"
	session writeRow: modifiedRow.
	aBlock value: address.
	self assert: address street = 'Something Else'.] 
			ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
helperForTestReadEmbeddedOneToOne
	| transRow query result |
	
	[session beginTransaction.
	transRow := session system exampleBankTransactionRow.
	session writeRow: transRow.
	query := Query returningOneOf: GlorpBankTransaction
				where: [:each | each id = each id].
	result := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: result serviceCharge notNil.
	self assert: result serviceCharge description = 'additional overcharge'.
	self assert: result amount currency = #CDN.
	self assert: result amount amount = 7.
	self assert: result serviceCharge amount currency = #USD.
	self assert: result serviceCharge amount amount = 2.
	^result.! !

!GlorpReadingTest methodsFor: 'accessing' stamp: ''!
session
	^session.! !

!GlorpReadingTest methodsFor: 'accessing' stamp: ''!
session: anObject
	session := anObject! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.
	system := session system.! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session reset.
	session := nil.
	system := nil.! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testCompoundAND
	| object query results rowToWrite |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	query := Query returningManyOf: GlorpAddress
				where: [:address | address id < 124 AND: (address id  > 122) AND: (address id ~= nil)].
	results := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpAddress.
	self assert: object id = 123.
	self assert: object street = 'Paseo Montril'.
	self assert: object number = '10185'! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testConversionOnConstantQueryParams
	| query result transRow | 
	
	[
	session beginTransaction.
	transRow := session system exampleBankTransactionRow.
	session writeRow: transRow.
	query := Query returningManyOf: GlorpBankTransaction where: [:trans | trans amount currency = #CDN].
	result := session execute: query.
	self assert: result size = 1.
	self assert: result first amount amount = 7] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testConversionOnConstantQueryParams2
	| query result transRow |
	
	[
	session beginTransaction.
	transRow := session system exampleBankTransactionRow.
	session writeRow: transRow.
	query := Query returningManyOf: GlorpBankTransaction where: [:trans | trans amount amount = 7].
	result := session execute: query.
	self assert: result size = 1.
	self assert: result first amount amount = 7] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-functions' stamp: ''!
testCount
	| query result account extraAccount |
	
	[session beginTransaction.
	self writeCustomer1Rows.
	extraAccount := session system exampleAccountRow3.
	session writeRow: extraAccount.


	query := Query returningManyOf: GlorpBankAccount.
	query retrieve: [:each | each id count].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = 3.
	account := result first] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-functions' stamp: ''!
testCountOnRelatedAttribute
	| query result account extraAccount |
	
	[session beginTransaction.
	self writeCustomer1Rows.
	extraAccount := session system exampleAccountRow3.
	session writeRow: extraAccount.


	query := Query returningManyOf: GlorpBankAccount.
	query retrieve: [:each | each accountHolders id count].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = 2.
	account := result first] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-functions' stamp: ''!
testCountStar
	| query result account extraAccount |
	
	[session beginTransaction.
	self writeCustomer1Rows.
	extraAccount := session system exampleAccountRow3.
	session writeRow: extraAccount.


	query := Query returningManyOf: GlorpBankAccount.
	query retrieve: [:each | each countStar].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = 3.
	account := result first] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-functions' stamp: ''!
testCountStarOnRelatedObject
	| query result account extraAccount |
	
	[session beginTransaction.
	self writeCustomer1Rows.
	extraAccount := session system exampleAccountRow3.
	session writeRow: extraAccount.

	query := Query returningManyOf: GlorpBankAccount.
	query retrieve: [:each | each accountHolders countStar].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = 2.
	account := result first] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-functions' stamp: ''!
testCountStarOnRelatedObjectAttribute
	| query result account extraAccount |
	
	[session beginTransaction.
	self writeCustomer1Rows.
	extraAccount := session system exampleAccountRow3.
	session writeRow: extraAccount.


	query := Query returningManyOf: GlorpBankAccount.
	query retrieve: [:each | each accountHolders id countStar].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = 2.
	account := result first] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-functions' stamp: ''!
testCountStarOnRelatedObjectWithGroupBy
	| query result extraAccount |
	
	[session beginTransaction.
	self writeCustomer1Rows.
	extraAccount := session system exampleAccountRow3.
	session writeRow: extraAccount.

	query := Query returningManyOf: GlorpBankAccount.
	query retrieve: [:each | each accountHolders countStar].
	query groupBy: [:each | each id].
	result := query executeIn: session.
	self assert: result = #(1 1 )] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-sql' stamp: ''!
testCustomSQLInAccountsMapping
	"Test using custom sql for the mapping's query. This will only work if using binding."

	| query id1 id2 result accounts backRef1 backRef2 accountIds |
	session useBinding ifFalse: [^self].
	((session system descriptorFor: GlorpCustomer) 
		mappingForAttributeNamed: #accounts)
			query sql: 'SELECT t1.ID, t1.BANK_CODE, t1.BRANCH_NO, t1.ACCT_NO
 FROM BANK_ACCT t1, CUSTOMER_ACCT_LINK t2
 WHERE ((t2.ACCT_ID = t1.ID) AND (t2.CUSTOMER_ID = ?))'.
	[session beginTransaction.
	accountIds := self writeCustomer1Rows.
	id1 := accountIds at: 1.
	id2 := accountIds at: 2.
	query := Query returningOneOf: GlorpCustomer
				where: [:person | person id = 27].
	result := session execute: query.
	accounts := result accounts getValue.
	self assert: accounts size = 2.
	self assert: (accounts first id = id1 or: [accounts last id = id1]).
	self assert: (accounts first id = id2 or: [accounts last id = id2]).
	self assert: accounts first id ~= accounts last id.
	backRef1 := accounts first accountHolders getValue.
	self assert: backRef1 size = 1.
	self assert: backRef1 first = result.
	backRef2 := accounts first accountHolders getValue.
	self assert: backRef2 size = 1.
	self assert: backRef2 first = result] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-sql' stamp: ''!
testCustomSQLInAddressMapping
	| object query results rowToWrite |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	query := Query returningManyOf: GlorpAddress.
	query sql: 'select id, street, house_num from gr_address where id = 123'.
	results := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpAddress.
	self assert: object id = 123.
	self assert: object street = 'Paseo Montril'.
	self assert: object number = '10185'! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testDeleteWithEmbedded
	| transaction uow |
	
	[session beginTransaction.
	session beginUnitOfWork.
	self writeCustomer1RowsWithTransactions.
	transaction := (session readManyOf: GlorpBankTransaction) first.
	session delete: transaction.
	uow := session privateGetCurrentUnitOfWork.
	self assert: (uow willDelete: transaction).
	self assert: (uow willDelete: transaction amount).
	self assert: (uow willDelete: transaction serviceCharge).
	self assert: (uow willDelete: transaction serviceCharge amount).
	session commitUnitOfWork] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testDeletedObjectsNotReadInManyQuery
	| query anAccount accounts someAccountId |
	[session beginTransaction.
	someAccountId := self writeCustomer1Rows first.
	session beginUnitOfWork.
	query := Query returningOneOf: GlorpBankAccount
				where: [:each | each id = someAccountId].
	anAccount := session execute: query.
	self assert: anAccount class == GlorpBankAccount.
	self assert: anAccount id = someAccountId.
	session delete: anAccount.
	accounts := session readManyOf: GlorpBankAccount.
	self assert: accounts size = 1.
	self deny: (accounts includes: anAccount).
	self deny: (accounts anySatisfy: [:each | each id = someAccountId]).
] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testDeletedObjectsNotReadInOneQuery
	| query anAccount someAccountId account |
	[session beginTransaction.
	someAccountId := self writeCustomer1Rows first.
	session beginUnitOfWork.
	query := Query returningOneOf: GlorpBankAccount
				where: [:each | each id = someAccountId].
	anAccount := session execute: query.
	self assert: anAccount class == GlorpBankAccount.
	self assert: anAccount id = someAccountId.
	session delete: anAccount.
	account := session readOneOf: GlorpBankAccount where: [:each | each id = someAccountId].
	self assert: account isNil] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testExpiryTimeNotReset
	| cachePolicy |
	cachePolicy := TimedExpiryCachePolicy new.
	cachePolicy timeoutSeconds: 3.
	cachePolicy expiryAction: #refresh.
	(session descriptorFor: GlorpAddress) cachePolicy: cachePolicy.
	self checkExpiryTime: [:expiryTime :newExpiryTime :address | 
		self assert: newExpiryTime = expiryTime.
		self assert: address street = 'Paseo Montril'].! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testExpiryTimeReset
	| cachePolicy |
	cachePolicy := TimedExpiryCachePolicy new.
	cachePolicy timeoutSeconds: 4.
	cachePolicy expiryAction: #remove.
	(session descriptorFor: GlorpAddress) cachePolicy: cachePolicy.
	self checkExpiryTime: [:expiryTime :newExpiryTime :address | 
		self assert: newExpiryTime > expiryTime.
		self assert: address street = 'Paseo Montril'].! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testExpiryTimeReset2
	| cachePolicy |
	cachePolicy := TimedExpiryCachePolicy new.
	cachePolicy timeoutSeconds: 4.
	cachePolicy expiryAction: #notifyAndRemove.
	(session descriptorFor: GlorpAddress) cachePolicy: cachePolicy.
	self checkExpiryTime: [:expiryTime :newExpiryTime :address | 
		self assert: newExpiryTime > expiryTime.
		self assert: address street = 'Paseo Montril'].! !

!GlorpReadingTest methodsFor: 'tests-in' stamp: ''!
testInFromJoin
	| query result |
	[session beginTransaction.
	self writeReservationData.
	query := Query returningManyOf: GlorpItinerary where: [:each | each reservation passengers 
		anySatisfyJoin: [:eachPassenger | eachPassenger airline id in: #(73 74)]].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | each reservation passengers anySatisfy: [:eachPassenger | #(73 74) includes: eachPassenger airline id]]).
	self assert: result size = 2] ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'tests-in' stamp: ''!
testInFromSubselect
	| query result |
	[session beginTransaction.
	self writeReservationData.
	query := Query returningManyOf: GlorpItinerary where: [:each | each reservation passengers 
		anySatisfyExists: [:eachPassenger | eachPassenger airline id in: #(73 74)]].
	result := session execute: query.
	self assert: (result allSatisfy: [:each | each reservation passengers anySatisfy: [:eachPassenger | #(73 74) includes: eachPassenger airline id]]).
	self assert: result size = 2] ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'tests-functions' stamp: ''!
testMax
	| query result account |
	
	[session beginTransaction.
	self writeCustomer1Rows.

	query := Query returningManyOf: GlorpBankAccount.
	query retrieve: [:each | each accountNumber accountNumber max].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = '4'.
	account := result first] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-functions' stamp: ''!
testMin
	| query result account |
	
	[session beginTransaction.
	self writeCustomer1Rows.

	query := Query returningManyOf: GlorpBankAccount.
	query retrieve: [:each | each accountNumber accountNumber min].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = '3'.
	account := result first] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testNonIntrusiveAlsoFetch
	| alsoFetchQuery query results | 

	"If a platform has no outer joins (!!) we cannot execute
	 this test."
	(session system platform supportsANSIJoins or: [
		session system platform useMicrosoftOuterJoins or: [
		session system platform useOracleOuterJoins ]])
			ifFalse: [ ^self ].

	[session beginTransaction.
	session beginUnitOfWork.
	session register: GlorpPerson example1.
	session register: (GlorpPerson example1
							id: 2;
							address: nil;
							yourself).
	session commitUnitOfWork.
	alsoFetchQuery := Query
						returningManyOf: GlorpPerson
						where: nil.
	alsoFetchQuery alsoFetch: [:ea | ea address asOuterJoin].
	query := Query
					returningManyOf: GlorpPerson
					where: nil.
	results := alsoFetchQuery executeIn: session.
	self assert: results size = 2.
	results := query executeIn: session.
	self assert: results size = 2] 
			ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testNonRefreshAddress
	"Test that if we don't set the refresh flag on the query we don't re-read the data"
	| query rowToWrite address modifiedRow |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	address := session readOneOf: GlorpAddress
				where: [:each | each id = 123].
	modifiedRow := session system exampleModifiedAddressRow.
	modifiedRow owner: address. "Otherwise it thinks it's an insert"
	session writeRow: modifiedRow.
	query := Query returningOneOf: GlorpAddress where: [:each | each id = 123].
	query executeIn: session.
	self assert: address street = 'Paseo Montril'.] 
			ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testQueryHasLimit
	self deny: (SimpleQuery returningManyOf: GlorpAddress) hasLimit.
	self assert: (SimpleQuery returningManyOf: GlorpAddress limit: 10) hasLimit.
	self assert: (SimpleQuery returningManyOf: GlorpAddress limit: -10) hasLimit.

	self assert: (SimpleQuery returningManyOf: GlorpAddress) limitMakesSense.
	self assert: (SimpleQuery returningManyOf: GlorpAddress limit: 10) limitMakesSense.
	self deny: (SimpleQuery returningManyOf: GlorpAddress limit: 0) limitMakesSense.
	self deny: (SimpleQuery returningManyOf: GlorpAddress limit: -10) limitMakesSense.! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testQueryHasOffset
	self deny: (SimpleQuery returningManyOf: GlorpAddress) hasOffset.
	self assert: ((SimpleQuery returningManyOf: GlorpAddress) offset: 10; yourself) hasOffset.
	self deny: ((SimpleQuery returningManyOf: GlorpAddress) offset: 0; yourself) hasOffset.
	self deny: ((SimpleQuery returningManyOf: GlorpAddress) offset: -10; yourself) hasOffset.! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithCompoundAnySatisfyExists
	| query result |
	
	[| block |
	session beginTransaction.
	self writeCustomer1Rows.
	block := 
			[:account | 
			account accountHolders anySatisfyExists: [:each | (each id = 27) & (each name = 'aCustomer')]].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithCompoundAnySatisfyJoin
	| query result |
	
	[| block |
	session beginTransaction.
	self writeCustomer1Rows.
	block := 
			[:account | 
			account accountHolders anySatisfyJoin: [:each | (each id = 27) & (each name = 'aCustomer')]].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithMultipleAnySatisfyExists
	| query result |
	 
	[| block |
	session beginTransaction.
	self writeCustomer1Rows.
	block := 
			[:account | 
			(account accountHolders anySatisfyExists: [:each | each id = 24]) 
				| (account accountHolders anySatisfyExists: [:each | each id = 27])].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithMultipleAnySatisfyJoin
	| query result |
	 
	[| block |
	session beginTransaction.
	self writeCustomer1Rows.
	block := 
			[:account | 
			(account accountHolders anySatisfyJoin: [:each | each id = 24]) 
				| (account accountHolders anySatisfyJoin: [:each | each id = 27])].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithMultipleAnySatisfyMixed1
	| query result |
	 
	[| block |
	session beginTransaction.
	self writeCustomer1Rows.
	block := 
			[:account | 
			(account accountHolders anySatisfyJoin: [:each | each id = 24]) 
				| (account accountHolders anySatisfyExists: [:each | each id = 27])].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithMultipleAnySatisfyMixed2
	| query result |
	 
	[| block |
	session beginTransaction.
	self writeCustomer1Rows.
	block := 
			[:account | 
			(account accountHolders anySatisfyExists: [:each | each id = 24]) 
				| (account accountHolders anySatisfyJoin: [:each | each id = 27])].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithNestedAnySatisfyExists
	| query result |
	[| block |
	session beginTransaction.
	self writeCustomer1RowsWithTransactions.
	block := 
			[:account | 
			account accountHolders anySatisfyExists: [:each | each transactions anySatisfyExists: [:eachTrans |
			eachTrans id ~= nil]]].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithNestedAnySatisfyJoin
	| query result |
	[| block |
	session beginTransaction.
	self writeCustomer1RowsWithTransactions.
	block := 
			[:account | 
			account accountHolders anySatisfyJoin: [:each | each transactions anySatisfyJoin: [:eachTrans |
			eachTrans id ~= nil]]].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithNestedAnySatisfyMixed1
	| query result |
	[| block |
	session beginTransaction.
	self writeCustomer1RowsWithTransactions.
	block := 
			[:account | 
			account accountHolders anySatisfyExists: [:each | each transactions anySatisfyJoin: [:eachTrans |
			eachTrans id ~= nil]]].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAccountsWithNestedAnySatisfyMixed2
	| query result |
	[| block |
	session beginTransaction.
	self writeCustomer1RowsWithTransactions.
	block := 
			[:account | 
			account accountHolders anySatisfyJoin: [:each | each transactions anySatisfyExists: [:eachTrans |
			eachTrans id ~= nil]]].
	query := Query returningManyOf: GlorpBankAccount where: block.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAdHoc 
	| queryTime table row time idField times |
	
	[
	session beginTransaction.
	
	table := session system tableNamed: 'TRANSFORMED_TIME'.
	row := DatabaseRow newForTable: table.
	idField := (table fieldNamed: 'ID').
	row at: idField put: 3.
	time := Time now.
	row at: (table fieldNamed: 'TIMEFIELD') put: time asSeconds.

	session writeRow: row.
	queryTime := GlorpTransformedTime  new id: 3; time: time.
	times := session readManyOf: GlorpTransformedTime where: [:each | each time = time].
	self assert: times size = 1.
	self assert: times first time asSeconds = time asSeconds.
	] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAdHocProxied
	| table row time idField times time2 |
	((session system descriptorFor: GlorpTransformedTime) mappingForAttributeNamed: #time) shouldProxy: true.
	[
	session beginTransaction.
	
	table := session system tableNamed: 'TRANSFORMED_TIME'.
	row := DatabaseRow newForTable: table.
	idField := (table fieldNamed: 'ID').
	row at: idField put: 3.
	time := Time now.
	row at: (table fieldNamed: 'TIMEFIELD') put: time asSeconds.
	session writeRow: row.

	row := DatabaseRow newForTable: table.
	row at: idField put: 4.
	time2 := Dialect addSeconds: 10 to: Time now.
	row at: (table fieldNamed: 'TIMEFIELD') put: time2 asSeconds.
	session writeRow: row.

	times := session read: GlorpTransformedTime orderBy: [:each | each time].
	self assert: times size = 2.
	self assert: times first time class == AdHocProxy.
	self assert: times first time asSeconds = time asSeconds.
	self assert: times last time asSeconds = time2 asSeconds.
	] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAddress
	| object query results rowToWrite |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	query := Query returningManyOf: GlorpAddress
				where: [:address | address id = 123].
	results := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpAddress.
	self assert: object id = 123.
	self assert: object street = 'Paseo Montril'.
	self assert: object number = '10185'! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAddressProxy
	| object query results rowToWrite proxy | 

	[session beginTransaction.
		rowToWrite := session system exampleAddressRow.
		session writeRow: rowToWrite.
		query := (Query
			returningManyOf: GlorpAddress
			where: ([:address | address id = 123]) ) returnProxies: true.
		results := query executeIn: session.
		self assert: (results size = 1).
		proxy := results first.
		object := proxy getValue]
	ensure: [session rollbackTransaction].

	self assert: (proxy isGlorpProxy).
	self assert: (object class = GlorpAddress).
	self assert: (object id = 123).
	self assert: (object street = 'Paseo Montril').
	self assert: (object number = '10185').! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAddressProxyAlreadyInMemory
	"Check that if the object is already in memory we don't create a proxy for it, just return the instance Unless explicitly asked, in which case we return a proxy."
	| object object2 query results rowToWrite |

	[session beginTransaction.
		rowToWrite := session system exampleAddressRow.
		session writeRow: rowToWrite.
		object := session readOneOf: GlorpAddress where: [:address | address id = 123].
		self deny: (object class inheritsFrom: Proxy).
		object2 := session readOneOf: GlorpAddress where: [:address | address id = 123].
		self deny: (object class inheritsFrom: Proxy).
		self assert: object2 == object.
		object := nil.
		object2 := nil.
		Dialect garbageCollect.
		(Delay forSeconds: 2) wait.
		query := (Query
			returningManyOf: GlorpAddress
			where: ([:address | address id = 123]) ) returnProxies: true.
		results := query executeIn: session.
		session accessor denyReads.
		self assert: (results size = 1).
		object := results first]
	ensure: [session rollbackTransaction].

	self assert: (object class = Proxy).
	self deny: object isInstantiated.
	self assert: (object id = 123).
	self assert: (object street = 'Paseo Montril').
	self assert: (object number = '10185').
	session accessor permitEverything.! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAddressSubstring
	| object query results rowToWrite results2 |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	query := Query returningManyOf: GlorpAddress
				where: [:address | (address street copyFrom: 1 to: 5) = 'Paseo'].
	results := query executeIn: session.
	query := Query returningManyOf: GlorpAddress
				where: [:address | (address street copyFrom: 7 to: 13) = 'Montril'].
	results2 := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpAddress.
	self assert: object id = 123.
	self assert: object street = 'Paseo Montril'.
	self assert: object number = '10185'.
	self assert: results2 size = 1.
	self assert: results2 first = results first.! !

!GlorpReadingTest methodsFor: 'tests-sql' stamp: ''!
testReadAddressWithCustomSQL
	| object query results rowToWrite |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	query := Query returningManyOf: GlorpAddress.
	query sql: 'select id, street, house_num from gr_address where id = 123'.
	results := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpAddress.
	self assert: object id = 123.
	self assert: object street = 'Paseo Montril'.
	self assert: object number = '10185'! !

!GlorpReadingTest methodsFor: 'tests-sql' stamp: ''!
testReadAddressWithCustomSQLBinding
	"Test that we can bind to custom SQL. If not binding, don't run this test."
	| object query results rowToWrite |
	
	session useBinding ifFalse: [^self].
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	query := Query returningManyOf: GlorpAddress.
	query sql: 'select id, street, house_num from gr_address where id = ?'.
	results := query executeWithParameters: #(123) in: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpAddress.
	self assert: object id = 123.
	self assert: object street = 'Paseo Montril'.
	self assert: object number = '10185'! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadAllAddress
	| object results rowToWrite |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	results := session readManyOf: GlorpAddress] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpAddress.
	self assert: object id = 123.
	self assert: object street = 'Paseo Montril'.
	self assert: object number = '10185'! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadCompressedMoney
	| object query results rowToWrite |
	
	[session beginTransaction.
	rowToWrite := session system exampleCompressedMoneyRow.
	session writeRow: rowToWrite.
	query := Query returningManyOf: GlorpCompressedMoney
				where: [:money | money id ~= 0].
	results := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpCompressedMoney.
	self assert: object amount = 12.
	self assert: object currency = 'CDN'.! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadCompressedMoney2
	| object query results rowToWrite |
	
	[session beginTransaction.
	rowToWrite := session system exampleCompressedMoneyRow.
	session writeRow: rowToWrite.
	query := Query returningManyOf: GlorpCompressedMoney
				where: [:money | money = (GlorpCompressedMoney new id: 123; currency: 'CDN'; amount: 12)].
	results := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpCompressedMoney.
	self assert: object amount = 12.
	self assert: object currency = 'CDN'.! !

!GlorpReadingTest methodsFor: 'tests' stamp: 'GuillermoPolito 4/26/2011 04:58'!
testReadCustomerAndAddTransaction
	| query  customer accountIds newCustomer rawRows |
	
	[session beginTransaction.
	accountIds := self writeCustomer1Rows.
	session beginUnitOfWork.
	query := Query returningOneOf: GlorpCustomer
				where: [:person | person id = 27].
	customer := session execute: query.
	customer addTransaction: GlorpBankTransaction example1.
	session commitUnitOfWork.
	newCustomer := session execute: query.
	self assert: customer == newCustomer.
	self assert: customer transactions first owner yourSelf == customer.
	rawRows := session accessor executeSQLString: 'SELECT ID, NAME FROM GR_CUSTOMER'.
	self assert: rawRows size = 1.
	self assert: (rawRows first at: 1) = 27.
	] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadCustomerWithAccounts
	| query id1 id2 result accounts backRef1 backRef2 accountIds |
	
	[session beginTransaction.
	accountIds := self writeCustomer1Rows.
	id1 := accountIds at: 1.
	id2 := accountIds at: 2.
	query := Query returningOneOf: GlorpCustomer
				where: [:person | person id = 27].
	result := session execute: query.
	self assert: result seenPostFetch = true.
	accounts := result accounts getValue.
	self assert: accounts size = 2.
	self assert: (accounts first id = id1 or: [accounts last id = id1]).
	self assert: (accounts first id = id2 or: [accounts last id = id2]).
	self assert: accounts first id ~= accounts last id.
	backRef1 := accounts first accountHolders getValue.
	self assert: backRef1 size = 1.
	self assert: backRef1 first = result.
	backRef2 := accounts first accountHolders getValue.
	self assert: backRef2 size = 1.
	self assert: backRef2 first = result] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadCustomerWithAnySatisfyExists
	| query result accounts |
	
	[session beginTransaction.
	self writeCustomer1Rows.
	query := Query returningManyOf: GlorpCustomer
				where: 
					[:person | 
					person accounts anySatisfyExists: [:each | each accountNumber branchNumber > 0]].
	result := session execute: query.
	self assert: result size = 1.
	accounts := result first accounts getValue.
	self assert: accounts size = 2.
	query := Query returningManyOf: GlorpCustomer
				where: 
					[:person | 
					person accounts anySatisfy: [:each | each accountNumber branchNumber = 2]].
	result := session execute: query.
	self assert: result size = 1.
	accounts := result first accounts getValue.
	self assert: accounts size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadCustomerWithAnySatisfyJoin
	| query result accounts |
	
	[session beginTransaction.
	self writeCustomer1Rows.
	query := Query returningManyOf: GlorpCustomer
				where: 
					[:person | 
					person accounts anySatisfyJoin: [:each | each accountNumber branchNumber > 0]].
	result := session execute: query.
	self assert: result size = 1.
	accounts := result first accounts getValue.
	self assert: accounts size = 2.
	query := Query returningManyOf: GlorpCustomer
				where: 
					[:person | 
					person accounts anySatisfy: [:each | each accountNumber branchNumber = 2]].
	result := session execute: query.
	self assert: result size = 1.
	accounts := result first accounts getValue.
	self assert: accounts size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadCustomerWithIncludes
	| query result accounts account |
	
	[session beginTransaction.
	self writeCustomer1Rows.
	account := session readOneOf: GlorpBankAccount where: [:each | each accountNumber branchNumber = 2].
	query := Query returningManyOf: GlorpCustomer
				where: 
					[:person | 
					person accounts includes: account].
	result := session execute: query.
	self assert: result size = 1.
	accounts := result first accounts getValue.
	self assert: (accounts includes: account).
	self assert: accounts size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadCustomerWithMultipleAnySatisfyExists
	| query result |
	
	[| block |
	session beginTransaction.
	self writeCustomer1Rows.
	block := 
			[:person | 
			(person accounts anySatisfyExists: [:each | each accountNumber branchNumber = 2]) 
				& (person accounts 
						anySatisfyExists: [:each | each accountNumber branchNumber = 3])].
	query := Query returningManyOf: GlorpCustomer where: block.
	result := session execute: query.
	self assert: result size = 1] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadCustomerWithMultipleAnySatisfyJoin
	| query result |
	
	[| block |
	session beginTransaction.
	self writeCustomer1Rows.
	block := 
			[:person | 
			(person accounts anySatisfyJoin: [:each | each accountNumber branchNumber = 2]) 
				& (person accounts 
						anySatisfyJoin: [:each | each accountNumber branchNumber = 3])].
	query := Query returningManyOf: GlorpCustomer where: block.
	result := session execute: query.
	self assert: result size = 1] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadEmbeddedObjectDirectly
	| serviceCharges transRow |
	
	[
	session beginTransaction.
	transRow := session system exampleBankTransactionRow.
	session writeRow: transRow.
	transRow := session system exampleBankTransactionRow2.
	session writeRow: transRow.
	serviceCharges := session readManyOf: GlorpServiceCharge.
	self assert: serviceCharges size = 2.
	self deny: serviceCharges first == serviceCharges last.
	] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadEmbeddedOneToOne

	self helperForTestReadEmbeddedOneToOne.! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadMultiFieldAdHoc
	| object query results rowToWrite row2 |
	
	[session beginTransaction.
	rowToWrite := session system exampleCompressedMoneyRow.
	row2 := session system exampleCompressedMoneyRow2.
	session writeRow: rowToWrite.
	session writeRow: row2.

	query := Query returningManyOf: GlorpCompressedMoney
				where: [:money | money array = #('CDN' 12)].
	results := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 1.
	object := results first.
	self assert: object class = GlorpCompressedMoney.
	self assert: object amount = 12.
	self assert: object currency = 'CDN'.! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadMultipleObjectsManyToMany1
	| query result account |
	
	[session beginTransaction.
	self writeCustomer1Rows.

	query := Query returningManyOf: GlorpBankAccount.
	query alsoFetch: [:each | each accountHolders].
	result := query executeIn: session.
	self assert: result size = 2.
	account := result first.
	self deny: account accountHolders isGlorpProxy.
	self assert: account accountHolders size = 1.
	self assert: (account accountHolders first == (result last accountHolders first))] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadMultipleObjectsManyToMany2
	| query result customer |
	
	[session beginTransaction.
	self writeCustomer1Rows.

	query := Query returningManyOf: GlorpCustomer.
	query retrieve: [:each | each ].
	query alsoFetch: [:each | each accounts].
	result := query executeIn: session.
	self assert: result size = 1.
	customer := result first.
	self deny: customer accounts class isGlorpProxy.
	self assert: customer accounts size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadMultipleObjectsManyToMany3
	| query result customer array |
	
	[session beginTransaction.
	self writeCustomer1Rows.

	query := Query returningManyOf: GlorpCustomer.
	query retrieve: [:each | each ].
	query retrieve: [:each | each accounts].
	result := query executeIn: session.
	self assert: result size = 1.
	array := result first.
	customer := array first.
	self deny: customer accounts isGlorpProxy.
	self assert: customer accounts size = 2.
	self assert: customer accounts == array last] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadMultipleObjectsManyToMany4
	"If we read just a to-many relation, then it doesn't get knitted and we just get a collection of the target"
	| query result |
	
	[session beginTransaction.
	self writeCustomer1Rows.

	query := Query returningManyOf: GlorpCustomer where: [:each | each id ~= 7].
	query retrieve: [:each | each accounts].
	result := query executeIn: session.
	self assert: result size = 2.
	self assert: session privateGetCache numberOfElements = 2.
	self assert: (result allSatisfy: [:each | each class == GlorpBankAccount]).
	self assert: result first ~= result last] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadMultipleObjectsToManyAlsoFetchTwoLevels
	| query result account transactions |
	
	[session beginTransaction.
	self write2CustomerRowsWith3Transactions.
	session beginUnitOfWork.

	query := Query returningManyOf: GlorpBankAccount.
	query alsoFetch: [:each | each accountHolders].
	query alsoFetch: [:each | each accountHolders transactions].
	result := query executeIn: session.
	self assert: result size = 2.
	account := result detect: [:each | each id = 9874].
	self deny: result first == result last.
	self deny: account accountHolders isGlorpProxy.
	self assert: account accountHolders size = 1.
	transactions := account accountHolders first transactions.
	self deny: transactions isGlorpProxy.
	self assert: transactions size = 2.
	self assert: account accountHolders first == (result at: 2) accountHolders first] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadPassenger
	| passengerRow1 passengerRow2 query result |
	
	[session beginTransaction.
	passengerRow1 := session system examplePassengerRow.
	session writeRow: passengerRow1.
	passengerRow2 := session system exampleFrequentFlyerRow.
	session writeRow: passengerRow2.
	query := Query returningOneOf: GlorpPassenger
				where: [:passenger | passenger id = 1].
	result := query executeIn: session.
	self assert: result id = 1.
	self assert: result name = 'Some Passenger'.
	self assert: result frequentFlyerPoints = 10000] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadRelationshipField
	| query results |
	
	[session beginTransaction.
	query := Query returningManyOf: GlorpItinerary where: [:each | ((each getTable: 'ITINERARY') getField: 'RES_ID') = nil].
	results := query executeIn: session] 
			ensure: [session rollbackTransaction].
	self assert: results size = 0.! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadReservationWithJoinToPassenger
	| reservations |
	
	[session beginTransaction.
	self writeReservationData.
	session beginUnitOfWork.
	reservations := session readManyOf: GlorpReservation
				where: [:each | each passenger id = 3].
	self assert: reservations size = 1.
	self assert: reservations first passengers size = 1.] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadReservationWithPassenger
	
	| reservation passenger reservations |
	[session beginTransaction.
	self writeReservationData.
	session beginUnitOfWork.
	"This doesn't validate so well. We want to make sure that the passenger table read uses a join and gets back only the one row, but it's hard to test that. Putting in an error check in the query for readOne... that returns multiple would work, but is kind of intrusive"
	reservations := session readManyOf: GlorpReservation where: [:each | each id = 2].
	self assert: reservations size = 1.
	reservation := reservations first.
	passenger := reservation passenger.
	passenger id.
	] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadTransactionsWithIndependentSubselect
	| query result query2 matchingResult |
	
	[session beginTransaction.
	self write2CustomerRowsWith3Transactions.
	query := Query returningManyOf: GlorpBankTransaction where: [:each |
		each exists: (Query readManyOf: GlorpCustomer where: [:eachCustomer |
			(eachCustomer id = each owner id) & (eachCustomer name = 'anotherCustomer')])].
	query orderBy: [:each | each owner name].
	"Another way of saying the same thing"
	query2 := Query returningManyOf: GlorpBankTransaction where: [:each | each owner name = 'anotherCustomer'].
	result := session execute: query.
	matchingResult := session execute: query2.
	self assert: result size = matchingResult size.
	result with: matchingResult do: [:one :other |
		self assert: one = other]] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadTransactionsWithIndependentSubselectAndNot
	| query result query2 matchingResult |
	
	[session beginTransaction.
	self write2CustomerRowsWith3Transactions.
	query := Query returningManyOf: GlorpBankTransaction where: [:each |
		each notExists: (Query readManyOf: GlorpCustomer where: [:eachCustomer |
			(eachCustomer id = each owner id) & (eachCustomer name = 'anotherCustomer')])].
	query orderBy: [:each | each owner name].
	"Another way of saying the same thing"
	query2 := Query returningManyOf: GlorpBankTransaction where: [:each | each owner name ~= 'anotherCustomer'].
	query2 orderBy: [:each | each owner name].
	result := session execute: query.
	matchingResult := session execute: query2.
	self assert: result size = matchingResult size.
	result with: matchingResult do: [:one :other |
		self assert: one = other]] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadTransactionsWithMaxSubselect
	| query result |
	
	[session beginTransaction.
	self write2CustomerRowsWith3Transactions.
	query := Query returningManyOf: GlorpBankTransaction where: [:each |
		each amount amount = ((Query readManyOf: GlorpBankTransaction
			where: [:eachTrans | eachTrans owner = each owner]) 
				retrieve: [:eachTrans | eachTrans amount amount max])].
	query orderBy: [:each | each owner name].
	result := session execute: query.
	self assert: result size = 2.
	self assert: result first amount amount = 10.
	self assert: result last amount amount = 11.
	self assert: result first owner id ~= result last owner id] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadTransactionsWithMaxSubselectAndRetrieveMax
	| query result |
	
	[session beginTransaction.
	self write2CustomerRowsWith3Transactions.
	query := Query returningManyOf: GlorpBankTransaction where: [:each |
		each amount amount = ((Query readManyOf: GlorpBankTransaction
			where: [:eachTrans | eachTrans owner = each owner]) 
				retrieveMax: [:eachTrans | eachTrans amount amount])].
	query orderBy: [:each | each owner name].
	result := session execute: query.
	self assert: result size = 2.
	self assert: result first amount amount = 10.
	self assert: result last amount amount = 11.
	self assert: result first owner id ~= result last owner id] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadWithCacheHits
	| query addressRow result1 result2 |
	
	[session beginTransaction.
	addressRow := session system exampleAddressRow.
	session writeRow: addressRow.
	query := Query returningOneOf: GlorpAddress
				where: [:address | address id = 123].
	result1 := query executeIn: session.
	result2 := query executeIn: session.
	self assert: result1 == result2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadWithFalseWhereClause
	| query id1 id2 result accountIds |
	
	[session beginTransaction.
	accountIds := self writeCustomer1Rows.
	id1 := accountIds at: 1.
	id2 := accountIds at: 2.
	query := Query returningManyOf: GlorpBankAccount
				where: false.
	result := session execute: query.
	self assert: result size = 0] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadWithLimit
	| results |
	session platform supportsLimit ifFalse: [^self].
	results := Set new.
	[session beginTransaction.
	session
		inUnitOfWorkDo:
			[(1 to: 5) do: [:i | session register: ((GlorpAddress new)
													id: i;
													yourself)]].
	results
		add: (session execute: (SimpleQuery returningManyOf: GlorpAddress limit: 3)) size;
		add: (session execute: (SimpleQuery returningManyOf: GlorpAddress where: true limit: 3)) size;
		add: (session execute: (SimpleQuery readManyOf: GlorpAddress limit: 3)) size;
		add: (session execute: (SimpleQuery readManyOf: GlorpAddress where: true limit: 3)) size;
		add: (session readManyOf: GlorpAddress limit: 3) size;
		add: (session readManyOf: GlorpAddress where: true limit: 3) size.
	self assert: (results allSatisfy: [:ea | ea = 3])] ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadWithLimitAndOffset

	session platform supportsOffset ifFalse: [^self].
	[session beginTransaction.
	session
		inUnitOfWorkDo:
			[(1 to: 5) do: [:i | session register: ((GlorpAddress new)
													id: i;
													yourself)]].
	self assert: (session execute: ((SimpleQuery returningManyOf: GlorpAddress limit: 2)
									offset: 2;
									yourself)) size = 2.
	self assert: (session execute: ((SimpleQuery returningManyOf: GlorpAddress limit: 0)
									offset: 2;
									yourself)) size = 0.
	self assert: (session execute: ((SimpleQuery returningManyOf: GlorpAddress)
									offset: 6;
									yourself)) size = 0] ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadWithNilWhereClause
	| query id1 id2 result accountIds |
	
	[session beginTransaction.
	accountIds := self writeCustomer1Rows.
	id1 := accountIds at: 1.
	id2 := accountIds at: 2.
	query := Query returningManyOf: GlorpBankAccount
				where: nil.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadWithObjectEqualityToEmbeddedObject
	| query result |
	 
	[session beginTransaction.
	self writeCustomer1RowsWith2Transactions.
	query := Query
	returningManyOf: GlorpBankTransaction
	where: [:trans | trans amount = (GlorpMoney currency: #CDN amount: 7)].
	result := session execute: query.
	self assert: result size = 1.
	self assert: result first amount amount = 7.
	self assert: result first amount currency = #CDN] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadWithObjectEqualityToEmbeddedObjectNested
	| query result |
	 
	[session beginTransaction.
	self writeCustomer1RowsWith2Transactions.
	query := Query
		 returningManyOf: GlorpBankTransaction
	where: [:trans | trans serviceCharge amount = (GlorpMoney currency: #CHF amount: 2)].
	result := session execute: query.
	self assert: result size = 1.
	self assert: result first serviceCharge amount amount = 2.
	self assert: result first serviceCharge amount currency = #CHF] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadWithOffset
	session platform supportsOffset ifFalse: [^self].
	[session beginTransaction.
	session
		inUnitOfWorkDo:
			[(1 to: 5) do: [:i | session register: ((GlorpAddress new)
													id: i;
													yourself)]].
	self assert: (session execute: ((SimpleQuery returningManyOf: GlorpAddress)
									offset: 2;
									yourself)) size = 3] ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testReadWithTrueWhereClause
	| query id1 id2 result accountIds |
	
	[session beginTransaction.
	accountIds := self writeCustomer1Rows.
	id1 := accountIds at: 1.
	id2 := accountIds at: 2.
	query := Query returningManyOf: GlorpBankAccount
				where: true.
	result := session execute: query.
	self assert: result size = 2] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testRefreshAddress
	"Check that we refresh correctly when the refresh flag is set"
	| query rowToWrite address modifiedRow |
	
	[session beginTransaction.
	rowToWrite := session system exampleAddressRow.
	session writeRow: rowToWrite.
	address := session readOneOf: GlorpAddress
				where: [:each | each id = 123].
	modifiedRow := session system exampleModifiedAddressRow.
	modifiedRow owner: address. "Otherwise it thinks it's an insert"
	session writeRow: modifiedRow.
	query := Query returningOneOf: GlorpAddress where: [:each | each id = 123].
	query shouldRefresh: true.
	query executeIn: session.
	self assert: address street = 'Something Else'.] 
			ensure: [session rollbackTransaction].! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testRegisteringWithEmbeddedMapping

	| bankTrans |
	session beginUnitOfWork.
	bankTrans := self helperForTestReadEmbeddedOneToOne.
	self assert: (session isRegistered: bankTrans).
	self assert: (session isRegistered: bankTrans serviceCharge).
	self assert: (session isRegistered: bankTrans serviceCharge amount).! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testSequencePolicyForInsert
	| testObject |

	GlorpInMemorySequenceDatabaseType reset.
	[session beginTransaction.
	session beginUnitOfWork.
	testObject := GlorpAirline new.
	session register: testObject.
	session commitUnitOfWork.
	self assert: testObject id = 1]
		ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testSessionRefresh
	"Check that we refresh correctly when the refresh flag is set"
	self checkRefreshDoing: [:anAddress | session refresh: anAddress].! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testSessionRefreshOnExpiry
	"Check that we refresh correctly when an object has expired"
	| cachePolicy |
	cachePolicy := TimedExpiryCachePolicy new.
	cachePolicy timeout: 0.
	cachePolicy expiryAction: #refresh.
	(session descriptorFor: GlorpAddress) cachePolicy: cachePolicy.
	self checkRefreshDoing: [:anAddress |
		session readOneOf: GlorpAddress where: [:each | each id = 123]].! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testSessionRefreshOnExpiryWithCacheLookupOnly
	"Check that we refresh correctly when an object has expired, doing only a cache lookup, not an explicit read"
	| cachePolicy |
	cachePolicy := TimedExpiryCachePolicy new.
	cachePolicy timeout: 0.
	cachePolicy expiryAction: #refresh.
	(session descriptorFor: GlorpAddress) cachePolicy: cachePolicy.
	self checkRefreshDoing: [:anAddress |
		session privateGetCache lookupClass: GlorpAddress key: 123 ifAbsent: [nil]].! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testSessionRefreshOnProxyExpiry
	"Check that we refresh correctly when an object has expired"
	| cachePolicy |
	cachePolicy := TimedStrongCachePolicy new.
	cachePolicy timeout: 0.
	cachePolicy expiryAction: #refresh.
	(session descriptorFor: GlorpAddress) cachePolicy: cachePolicy.
	self checkRefreshDoing: [:anAddress |
		session readOneOf: GlorpAddress where: [:each | each id = 123]].! !

!GlorpReadingTest methodsFor: 'tests-refreshing' stamp: ''!
testSessionRefreshOnProxyExpiryWithCacheLookupOnly
	"Check that we refresh correctly when an object has expired, doing only a cache lookup, not an explicit read"
	| cachePolicy |
	cachePolicy := TimedStrongCachePolicy new.
	cachePolicy timeout: 0.
	cachePolicy expiryAction: #refresh.
	(session descriptorFor: GlorpAddress) cachePolicy: cachePolicy.
	self checkRefreshDoing: [:anAddress |
		session privateGetCache lookupClass: GlorpAddress key: 123 ifAbsent: [nil]].! !

!GlorpReadingTest methodsFor: 'tests' stamp: ''!
testShortCircuitEmptyReturnWhenLimitMeansNoResults
	self deny: (SimpleQuery returningManyOf: GlorpAddress limit: 3) shortCircuitEmptyReturn.
	self assert: (SimpleQuery returningManyOf: GlorpAddress limit: 0) shortCircuitEmptyReturn.
	self assert: (SimpleQuery returningManyOf: GlorpAddress limit: -3) shortCircuitEmptyReturn! !

!GlorpReadingTest methodsFor: 'tests-functions' stamp: ''!
testSum
	| query result |
	
	[session beginTransaction.
	self writeCustomer1Rows.

	query := Query returningManyOf: GlorpBankAccount.
	query retrieve: [:each | each accountNumber branchNumber sum].
	result := query executeIn: session.
	self assert: result size = 1.
	self assert: result first = 5] 
			ensure: [session rollbackTransaction]! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
write2CustomerRowsWith3Transactions
	| table row aGlorpDemoDescriptorSystem |
	session system writeCustomerWithDualAccountOwners.
	aGlorpDemoDescriptorSystem := session system.
	table := aGlorpDemoDescriptorSystem tableNamed: 'BANK_TRANS'.
	row := DatabaseRow newForTable: table.
	row atFieldNamed: 'ID' put: nil.
	row atFieldNamed: 'AMT_CURR' put: 'CDN'.
	row atFieldNamed: 'AMT_AMT' put: 9.
	row atFieldNamed: 'SRVC_DESC' put: 'additional overcharge'.
	row atFieldNamed: 'SRVC_AMT_CURR' put: 'CHF'.
	row atFieldNamed: 'SRVC_AMT_AMT' put: 2.
	row atFieldNamed: 'OWNER_ID' put: 27.
	session writeRow: row.
	row := DatabaseRow newForTable: table.
	row atFieldNamed: 'ID' put: nil.
	row atFieldNamed: 'AMT_CURR' put: 'USD'.
	row atFieldNamed: 'AMT_AMT' put: 10.
	row atFieldNamed: 'SRVC_DESC' put: 'additional overcharge'.
	row atFieldNamed: 'SRVC_AMT_CURR' put: 'CHF'.
	row atFieldNamed: 'SRVC_AMT_AMT' put: 2.
	row atFieldNamed: 'OWNER_ID' put: 27.
	session writeRow: row.
	row := DatabaseRow newForTable: table.
	row atFieldNamed: 'ID' put: nil.
	row atFieldNamed: 'AMT_CURR' put: 'EUR'.
	row atFieldNamed: 'AMT_AMT' put: 11.
	row atFieldNamed: 'SRVC_DESC' put: 'additional overcharge'.
	row atFieldNamed: 'SRVC_AMT_CURR' put: 'EUR'.
	row atFieldNamed: 'SRVC_AMT_AMT' put: 2.
	row atFieldNamed: 'OWNER_ID' put: 28.
	session writeRow: row.! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
writeCustomer1Rows
	^system writeCustomer1Rows.! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
writeCustomer1RowsWith2Transactions
	| table row aGlorpDemoDescriptorSystem |
	self writeCustomer1RowsWithTransactions.
	aGlorpDemoDescriptorSystem := session system.
	table := aGlorpDemoDescriptorSystem tableNamed: 'BANK_TRANS'.
	row := DatabaseRow newForTable: table.
	row atFieldNamed: 'ID' put: nil.
	row atFieldNamed: 'AMT_CURR' put: 'USD'.
	row atFieldNamed: 'AMT_AMT' put: 9.
	row atFieldNamed: 'SRVC_DESC' put: 'additional overcharge'.
	row atFieldNamed: 'SRVC_AMT_CURR' put: 'CHF'.
	row atFieldNamed: 'SRVC_AMT_AMT' put: 2.
	row atFieldNamed: 'OWNER_ID' put: 27.
	session writeRow: row.! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
writeCustomer1RowsWithTransactions
	| table row aGlorpDemoDescriptorSystem |
	self writeCustomer1Rows.
	aGlorpDemoDescriptorSystem := session system.
	table := aGlorpDemoDescriptorSystem tableNamed: 'BANK_TRANS'.
	row := DatabaseRow newForTable: table.
	row atFieldNamed: 'ID' put: nil.
	row atFieldNamed: 'AMT_CURR' put: 'CDN'.
	row atFieldNamed: 'AMT_AMT' put: 7.
	row atFieldNamed: 'SRVC_DESC' put: 'additional overcharge'.
	row atFieldNamed: 'SRVC_AMT_CURR' put: 'USD'.
	row atFieldNamed: 'SRVC_AMT_AMT' put: 2.
	row atFieldNamed: 'OWNER_ID' put: 27.
	session writeRow: row.! !

!GlorpReadingTest methodsFor: 'support' stamp: ''!
writeReservationData
	session beginUnitOfWork.
	session register: GlorpItinerary example1.
	session register: GlorpItinerary example2.
	session commitUnitOfWork.
	session writeRow: session system examplePassengerRow.
	session initializeCache! !

GlorpTestCase subclass: #GlorpRowMapForMementosCollectionTest
	instanceVariableNames: 'transaction rowMap collection newCollection copy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpRowMapForMementosCollectionTest methodsFor: 'tests' stamp: ''!
createRowMap
	rowMap := RowMapForMementos withCorrespondenceMap: transaction undoMap.
	rowMap collectionMementos: transaction collectionMementos.
	newCollection := rowMap collectionMementoFor: collection.! !

!GlorpRowMapForMementosCollectionTest methodsFor: 'support' stamp: ''!
setUp
	transaction := ObjectTransaction new.
	rowMap := nil.! !

!GlorpRowMapForMementosCollectionTest methodsFor: 'tests' stamp: ''!
testArray
	collection := Array with: 1 with: 2 with: 3 with: 4.
	copy := collection copy.
	transaction register: collection.
	collection at: 3 put: 'hello'.
	self createRowMap.
	self assert: newCollection = copy.
	self deny: newCollection = collection
		description: 'Copying is too shallow both here and in registerTransientInternalsOfCollection:'.! !

!GlorpRowMapForMementosCollectionTest methodsFor: 'tests' stamp: ''!
testDictionary
	collection := Dictionary new.
	1 to: 50 do: [:i | collection at: i put: i printString].
	copy := collection copy.
	transaction register: collection.
	#( 1 2 7 43 15 9 10 17 18 19 33 23) do: [:i | collection removeKey: i].
	collection at: 3 put: 'hello'.
	self createRowMap.
	self assert: newCollection size = copy size.
	self assert: newCollection keys asSortedCollection = copy keys asSortedCollection.
	self assert: newCollection values asSortedCollection = copy values asSortedCollection.
	self deny: newCollection values asSortedCollection = collection values asSortedCollection
		description: 'Copying is too shallow both here and in registerTransientInternalsOfCollection:'.! !

!GlorpRowMapForMementosCollectionTest methodsFor: 'tests' stamp: ''!
testIdentityDictionary
	collection := IdentityDictionary new.
	1 to: 50 do: [:i | collection at: i put: i printString].
	copy := collection copy.
	transaction register: collection.
	#( 1 2 7 43 15 9 10 17 18 19 33 23) do: [:i | collection removeKey: i].
	collection at: 3 put: 'hello'.
	self createRowMap.
	self assert: newCollection size = copy size.
	self assert: newCollection keys asSortedCollection = copy keys asSortedCollection.
	self assert: newCollection values asSortedCollection = copy values asSortedCollection.
	self deny: newCollection values asSortedCollection = collection values asSortedCollection
		description: 'Copying is too shallow both here and in registerTransientInternalsOfCollection:'.! !

!GlorpRowMapForMementosCollectionTest methodsFor: 'tests' stamp: ''!
testOrderedCollection
	collection := (1 to: 50) asOrderedCollection.
	copy := collection copy.
	transaction register: collection.
	12 timesRepeat: [collection removeLast].
	collection at: 3 put: 'hello'.
	self createRowMap.
	self assert: newCollection = copy.
	self deny: newCollection = collection
		description: 'Copying is too shallow both here and in registerTransientInternalsOfCollection:'.! !

!GlorpRowMapForMementosCollectionTest methodsFor: 'tests' stamp: ''!
testOrderedSet
	collection := (Dialect smalltalkAt: #OrderedSet ifAbsent: [^self]) new.
	1 to: 50 do: [:i | collection add: i].
	copy := collection copy.
	transaction register: collection.
	#( 1 2 7 43 15 9 10 17 18 19 33 23) do: [:i | collection remove: i].
	collection add: 999.
	self createRowMap.
	self assert: newCollection asSortedCollection = copy asSortedCollection.
	self deny: newCollection asSortedCollection = collection asSortedCollection
		description: 'Copying is too shallow both here and in registerTransientInternalsOfCollection:'.! !

!GlorpRowMapForMementosCollectionTest methodsFor: 'tests' stamp: ''!
testRBSmallDictionary
	collection := (Dialect smalltalkAt: #RBSmallDictionary ifAbsent: [^self]) new.
	1 to: 50 do: [:i | collection at: i put: i printString].
	copy := collection copy.
	transaction register: collection.
	#( 1 2 7 43 15 9 10 17 18 19 33 23) do: [:i | collection removeKey: i].
	collection at: 3 put: 'hello'.
	self createRowMap.
	self assert: newCollection size = copy size.
	self assert: newCollection keys asSortedCollection = copy keys asSortedCollection.
	self assert: newCollection values asSortedCollection = copy values asSortedCollection.
	self deny: newCollection values asSortedCollection = collection values asSortedCollection
		description: 'Copying is too shallow both here and in registerTransientInternalsOfCollection:'.! !

!GlorpRowMapForMementosCollectionTest methodsFor: 'tests' stamp: ''!
testSet
	collection := (1 to: 50) asSet.
	copy := collection copy.
	transaction register: collection.
	1 to: 12 do: [:i | collection remove: i].
	collection add: 'hello'.
	self createRowMap.
	self assert: newCollection asSortedCollection = copy asSortedCollection.
	self deny: newCollection size = collection size
		description: 'Copying is too shallow both here and in registerTransientInternalsOfCollection:'.! !

GlorpTestCase subclass: #GlorpRowMapForMementosTest
	instanceVariableNames: 'rowMap a1 a2 b1 b2 table1 row1 row2 transaction c1 c2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpRowMapForMementosTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	a1 := IdentityHashPrintingObject new.
	b1 := IdentityHashPrintingObject new.
	c1 := IdentityHashPrintingObject new.
	transaction := ObjectTransaction new.
	transaction register: a1.
	transaction register: b1.
	transaction register: c1.
	a2 := transaction undoMap at: a1.
	b2 := transaction undoMap at: b1.
	c2 := transaction undoMap at: c1.
	rowMap := RowMapForMementos withCorrespondenceMap: transaction undoMap.
	table1 := DatabaseTable new.
	row1 := 'row1'.
	row2 := 'row2'.! !

!GlorpRowMapForMementosTest methodsFor: 'tests' stamp: ''!
testAdjustMultipleRowMapKey
	| rowMapKey adjustedKey  |
	rowMapKey := MultipleRowMapKey with: a1 with: b1 with: c1.
	adjustedKey := rowMapKey adjustIn: rowMap correspondenceMap.
	self assert: (adjustedKey keys includes: a2).
	self assert: (adjustedKey keys includes: b2).
	self assert: (adjustedKey keys includes: c2).! !

!GlorpRowMapForMementosTest methodsFor: 'tests' stamp: ''!
testAdjustRowMapKey
	| rowMapKey adjustedKey  |
	rowMapKey := RowMapKey new.
	rowMapKey key1: a1; key2: b1.
	adjustedKey := rowMapKey adjustIn: rowMap correspondenceMap.
	self assert: (adjustedKey key1 == a2).
	self assert: (adjustedKey key2 == b2).! !

!GlorpRowMapForMementosTest methodsFor: 'tests' stamp: ''!
testReverseAdjustMultipleRowMapKey
	| rowMapKey adjustedKey  |
	rowMapKey := MultipleRowMapKey with: a2 with: b2 with: c2.
	adjustedKey := rowMapKey reverseAdjustIn: rowMap.
	self assert: (adjustedKey keys includes: a1).
	self assert: (adjustedKey keys includes: b1).
	self assert: (adjustedKey keys includes: c1).! !

!GlorpRowMapForMementosTest methodsFor: 'tests' stamp: ''!
testReverseAdjustRowMapKey
	| rowMapKey adjustedKey  |
	rowMapKey := RowMapKey new.
	rowMapKey key1: a2; key2: b2.
	adjustedKey := rowMapKey reverseAdjustIn: rowMap.
	self assert: (adjustedKey key1 == a1).
	self assert: (adjustedKey key2 == b1).! !

!GlorpRowMapForMementosTest methodsFor: 'tests' stamp: ''!
testRowForTableMultiple
	| rowMapKey row |
	rowMapKey := MultipleRowMapKey with: a1 with: b1 with: c1.
	row := rowMap findOrAddRowForTable: table1 withKey: rowMapKey.
	row1 := rowMap findOrAddRowForTable: table1 withKey: rowMapKey.
	row2 := rowMap rowForTable: table1 withKey: rowMapKey.
	self assert: row == row1.
	self assert: row == row2.! !

!GlorpRowMapForMementosTest methodsFor: 'tests' stamp: ''!
testRowForTableSingle
	| rowMapKey row |
	rowMapKey := RowMapKey new.
	rowMapKey key1: a1; key2: b1.
	row := rowMap findOrAddRowForTable: table1 withKey: rowMapKey.
	row1 := rowMap findOrAddRowForTable: table1 withKey: rowMapKey.
	row2 := rowMap rowForTable: table1 withKey: rowMapKey.
	self assert: row == row1.
	self assert: row == row2.! !

GlorpTestCase subclass: #GlorpRowMapTest
	instanceVariableNames: 'rowMap a1 b1 table1 row2 platform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpRowMapTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	a1 := IdentityHashPrintingObject new.
	b1 := IdentityHashPrintingObject new.
	rowMap := RowMap new.
	table1 := DatabaseTable new name: 'SomeTable'.
	platform := OraclePlatform new.
	table1 addField: (DatabaseField named: 'ONE_ID' type: platform integer) bePrimaryKey.
	table1 addField: (DatabaseField named: 'OTHER_ID' type: platform integer).
	table1 addField: (DatabaseField named: 'KEY' type: platform integer) bePrimaryKey.
	row2 := 'row2'.! !

!GlorpRowMapTest methodsFor: 'tests' stamp: ''!
testCollapseMatchingInsertsAndDeletes
	| c1 key1 key2 row1 remainingRow updates |
	c1 := Object new.
	key1 := RowMapKey new key1: a1; key2: b1.
	key2 := RowMapKey new key1: a1; key2: c1.
	row1 := rowMap findOrAddRowForTable: table1 withKey: key1.
	row1 atFieldNamed: 'ONE_ID' put: 99.
	row1 atFieldNamed: 'OTHER_ID' put: 98.
	row1 atFieldNamed: 'KEY' put: 5.
	row2 := rowMap findOrAddRowForTable: table1 withKey: key2.
	row2 atFieldNamed: 'ONE_ID' put: 99.
	row2 atFieldNamed: 'OTHER_ID' put: 14.
	row2 atFieldNamed: 'KEY' put: 5.
	row1 forDeletion: true.
	updates := rowMap collapseMatchingInsertsAndDeletes.
	self assert: rowMap numberOfEntries = 1.
	self assert: updates size = 1.
	self assert: updates asArray first = row2.
	remainingRow := rowMap rowForTable: table1 withKey: key2 ifAbsent: [self assert: false].
	rowMap rowForTable: table1 withKey: key1 ifPresent: [:shouldntBeThereAnyMore | self assert: false].
	self assert: (remainingRow atFieldNamed: 'ONE_ID') = 99.
	self assert: (remainingRow atFieldNamed: 'OTHER_ID') = 14.
	self assert: (remainingRow atFieldNamed: 'KEY') = 5.
	self assert: remainingRow owner = key2.! !

!GlorpRowMapTest methodsFor: 'tests' stamp: ''!
testCollapseMatchingInsertsAndDeletesWithUnknownFields
	"Check that we throw an error for unknown values in the update row"
	| c1 key1 key2 row1 |
	c1 := Object new.
	key1 := RowMapKey new key1: a1; key2: b1.
	key2 := RowMapKey new key1: a1; key2: c1.
	row1 := rowMap findOrAddRowForTable: table1 withKey: key1.
	row1 atFieldNamed: 'ONE_ID' put: 99.
	row1 atFieldNamed: 'OTHER_ID' put: 98.
	row1 atFieldNamed: 'KEY' put: 5.
	row2 := rowMap findOrAddRowForTable: table1 withKey: key2.
	row2 atFieldNamed: 'ONE_ID' put: 99.
	row2 atFieldNamed: 'KEY' put: 5.
	row1 forDeletion: true.
	self should: [rowMap collapseMatchingInsertsAndDeletes] raise: Dialect error.! !

!GlorpRowMapTest methodsFor: 'tests' stamp: ''!
testRowForTableSingle
	| rowMapKey row |
	rowMapKey := RowMapKey new.
	rowMapKey key1: a1; key2: b1.
	row := rowMap findOrAddRowForTable: table1 withKey: rowMapKey.
	row2 := rowMap rowForTable: table1 withKey: rowMapKey.
	self assert: row == row2.! !

GlorpTestCase subclass: #GlorpRowMapUnificationTest
	instanceVariableNames: 't1 t2 t3 f1 f2 f3 o1 o2 o3 rowMap platform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpRowMapUnificationTest methodsFor: 'support' stamp: ''!
setUp

	super setUp.
	platform := GlorpDatabaseLoginResource defaultPlatform.
	t1 := DatabaseTable named: 'T1'.
	t2 := DatabaseTable named: 'T2'.
	t3 := DatabaseTable named: 'T3'.
	f1 := t1 createFieldNamed: 'f1' type: (platform varChar: 10).
	f2 := t2 createFieldNamed: 'f2' type: (platform varChar: 10).
	f3 := t3 createFieldNamed: 'f3'  type: (platform varChar: 10).
	rowMap := RowMap new.
	o1 := 'one'.
	o2 := 'two'.
	o3 := 'three'.! !

!GlorpRowMapUnificationTest methodsFor: 'tests' stamp: ''!
testDoubleRowUnificationDifferentRows
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2.
	rowMap
		unifyField: f3
		key: o3
		withField: f2
		key: o2.
	(rowMap rowForTable: t1 withKey: o1) at: f1 put: 42.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 42.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 42.
	self assert: ((rowMap rowForTable: t3 withKey: o3) at: f3) = 42.! !

!GlorpRowMapUnificationTest methodsFor: 'tests' stamp: ''!
testDoubleRowUnificationDifferentRows2
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2.
	rowMap
		unifyField: f2
		key: o2
		withField: f3
		key: o3.
	(rowMap rowForTable: t1 withKey: o1) at: f1 put: 42.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 42.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 42.
	self assert: ((rowMap rowForTable: t3 withKey: o3) at: f3) = 42.! !

!GlorpRowMapUnificationTest methodsFor: 'tests' stamp: ''!
testDoubleRowUnificationDifferentRows3
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2.
	rowMap
		unifyField: f2
		key: o2
		withField: f3
		key: o3.
	(rowMap rowForTable: t3 withKey: o3) at: f3 put: 42.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 42.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 42.
	self assert: ((rowMap rowForTable: t3 withKey: o3) at: f3) = 42.! !

!GlorpRowMapUnificationTest methodsFor: 'tests' stamp: ''!
testDoubleRowUnificationDifferentRows4
	| t4 f4 o4 |
	t4 := DatabaseTable named: 'T4'.
	f4 := t4 createFieldNamed: 'f4' type: (platform varChar: 10).
	o4 := 'four'.
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2.
	rowMap
		unifyField: f3
		key: o3
		withField: f4
		key: o4.
	rowMap
		unifyField: f2
		key: o2
		withField: f3
		key: o3.
	(rowMap rowForTable: t1 withKey: o1) at: f1 put: 42.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 42.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 42.
	self assert: ((rowMap rowForTable: t3 withKey: o3) at: f3) = 42.
	self assert: ((rowMap rowForTable: t4 withKey: o4) at: f4) = 42.! !

!GlorpRowMapUnificationTest methodsFor: 'tests' stamp: ''!
testDoubleRowUnificationSameRow
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2.
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2.
	(rowMap rowForTable: t1 withKey: o1) at: f1 put: 42.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 42.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 42.! !

!GlorpRowMapUnificationTest methodsFor: 'tests' stamp: ''!
testDoubleRowUnificationSameRow2
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2.
	rowMap
		unifyField: f2
		key: o2
		withField: f1
		key: o1.
	(rowMap rowForTable: t1 withKey: o1) at: f1 put: 42.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 42.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 42.! !

!GlorpRowMapUnificationTest methodsFor: 'tests' stamp: ''!
testIteration

	| rows r1 r2 r3 count |
	r1 := rowMap findOrAddRowForTable: t1 withKey: o1.
	r2 := rowMap findOrAddRowForTable: t1 withKey: o2.
	r3 := rowMap findOrAddRowForTable: t2 withKey: o2.
	rows := IdentitySet new.
	count := 0.
	rowMap rowsDo: [:each | 
		count := count + 1.
		rows add: each].
	self assert: count = 3.
	self assert: (rows includes: r1).
	self assert: (rows includes: r3).
	self assert: (rows includes: r2).! !

!GlorpRowMapUnificationTest methodsFor: 'tests-relatedrows' stamp: ''!
testRelatedRowInternal
	| cf1 r1 r2 r3 |
	cf1 := t1 createFieldNamed: 'cf1' type: (platform varChar: 10).
	t1 addForeignKeyFrom: cf1 to: f1.
	rowMap
		unifyField: f1
		key: o1
		withField: cf1
		key: o2.
	rowMap
		unifyField: f1
		key: o1
		withField: cf1
		key: o3.
	r1 := rowMap rowForTable: t1 withKey: o1.
	r1 at: f1 put: o1.
	r2 := rowMap rowForTable: t1 withKey: o2.
	r2 at: f1 put: o2.
	r3 := rowMap rowForTable: t1 withKey: o3.
	r3 at: f1 put: o3.
	self assert: (r1 relatedRowsIn: rowMap) isEmpty.
	self assert: (r2 relatedRowsIn: rowMap) asArray = (Array with: r1).
	self assert: (r3 relatedRowsIn: rowMap) asArray = (Array with: r1).! !

!GlorpRowMapUnificationTest methodsFor: 'tests' stamp: ''!
testStoreThenUnify
	rowMap at: f1 key: o1 put: 12.
	rowMap
		unifyField: f2
		key: o2
		withField: f3
		key: o3.
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 12.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 12.
	self assert: ((rowMap rowForTable: t3 withKey: o3) at: f3) = 12.! !

!GlorpRowMapUnificationTest methodsFor: 'tests' stamp: ''!
testStoreWithRowMapKey

	| a b key1 key2 key3 table r1 r2 r3 |
	a := Object new.
	b := Object new.
	key1 := RowMapKey new key1: a; key2: b.
	key2 := RowMapKey new key1: a; key2: b.
	key3 := RowMapKey new key1: b; key2: a.
	table := DatabaseTable new.

	r1 := rowMap findOrAddRowForTable: table withKey: key1.
	r2 := rowMap findOrAddRowForTable: table withKey: key2.
	r3 := rowMap findOrAddRowForTable: table withKey: key3.

	self assert: r1 == r2.
	self assert: r2 == r3.
	self assert: r1 owner == key1.! !

!GlorpRowMapUnificationTest methodsFor: 'tests-transformation' stamp: ''!
testUnifyExtraOnSourceSideWithTransformationBeforeStoreForward
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2
		transformationForward: [:value | value * 2]
		inverse: [:value | value / 2].
	rowMap
		unifyField: f1
		key: o1
		withField: f3
		key: o3.
	(rowMap findOrAddRowForTable: t1 withKey: o1) at: f1 put: 12.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 12.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 24.
	self assert: ((rowMap rowForTable: t3 withKey: o3) at: f3) = 12.! !

!GlorpRowMapUnificationTest methodsFor: 'tests-transformation' stamp: ''!
testUnifyExtraOnSourceSideWithTransformationBeforeStoreInverse
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2
		transformationForward: [:value | value * 2]
		inverse: [:value | value / 2].
	rowMap
		unifyField: f1
		key: o1
		withField: f3
		key: o3.
	(rowMap findOrAddRowForTable: t2 withKey: o2) at: f2 put: 24.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 12.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 24.
	self assert: ((rowMap rowForTable: t3 withKey: o3) at: f3) = 12.! !

!GlorpRowMapUnificationTest methodsFor: 'tests-transformation' stamp: ''!
testUnifyExtraOnTargetSideWithTransformationBeforeStoreForward
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2
		transformationForward: [:value | value * 2]
		inverse: [:value | value / 2].
	rowMap
		unifyField: f2
		key: o2
		withField: f3
		key: o3.
	(rowMap findOrAddRowForTable: t1 withKey: o1) at: f1 put: 12.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 12.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 24.
	self assert: ((rowMap rowForTable: t3 withKey: o3) at: f3) = 24.! !

!GlorpRowMapUnificationTest methodsFor: 'tests-transformation' stamp: ''!
testUnifyExtraOnTargetSideWithTransformationBeforeStoreInverse
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2
		transformationForward: [:value | value * 2]
		inverse: [:value | value / 2].
	rowMap
		unifyField: f2
		key: o2
		withField: f3
		key: o3.
	(rowMap findOrAddRowForTable: t2 withKey: o2) at: f2 put: 24.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 12.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 24.
	self assert: ((rowMap rowForTable: t3 withKey: o3) at: f3) = 24.! !

!GlorpRowMapUnificationTest methodsFor: 'tests-transformation' stamp: ''!
testUnifyWithTransformationAfterStoreForward
	(rowMap findOrAddRowForTable: t1 withKey: o1) at: f1 put: 12.
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2
		transformationForward: [:value | value * 2]
		inverse: [:value | value / 2].
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 12.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 24.! !

!GlorpRowMapUnificationTest methodsFor: 'tests-transformation' stamp: ''!
testUnifyWithTransformationAfterStoreInverse
	"This is a little tricky. If we set the value first, then impose the constraint, then we're effectively doubling what was set. I think this is actually right, but also a very weird case on top of weird scenarios to begin with"

	(rowMap findOrAddRowForTable: t2 withKey: o2) at: f2 put: 12.
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2
		transformationForward: [:value | value * 2]
		inverse: [:value | value / 2].	"Note: vs. the 6 and 12 you might otherwise expect"
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 12.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 24.! !

!GlorpRowMapUnificationTest methodsFor: 'tests-transformation' stamp: ''!
testUnifyWithTransformationBeforeStoreForward
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2
		transformationForward: [:value | value * 2]
		inverse: [:value | value / 2].
	(rowMap findOrAddRowForTable: t1 withKey: o1) at: f1 put: 12.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 12.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 24.! !

!GlorpRowMapUnificationTest methodsFor: 'tests-transformation' stamp: ''!
testUnifyWithTransformationBeforeStoreInverse
	rowMap
		unifyField: f1
		key: o1
		withField: f2
		key: o2
		transformationForward: [:value | value * 2]
		inverse: [:value | value / 2].
	rowMap at: f2 key: o2 put: 12.
	self assert: ((rowMap rowForTable: t1 withKey: o1) at: f1) = 6.
	self assert: ((rowMap rowForTable: t2 withKey: o2) at: f2) = 12.! !

GlorpTestCase subclass: #GlorpSQLPrintingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpSQLPrintingTest methodsFor: 'tests' stamp: ''!
testDatePrinting
	| date stream |
	date := Dialect newDateWithYears: 1997 months: 11 days: 14.
	stream := WriteStream on: String new.
	date glorpPrintSQLOn: stream.
	self assert: stream contents = '''1997-11-14'''.

	date := Dialect newDateWithYears: 2002 months: 5 days: 2.
	stream := WriteStream on: String new.
	date glorpPrintSQLOn: stream.
	self assert: stream contents = '''2002-05-02'''.! !

!GlorpSQLPrintingTest methodsFor: 'As yet unclassified' stamp: ''!
testGroupWriting
	| system platform row1 row2 command string session |
	platform := PostgreSQLPlatform new.
	session := GlorpDemoDescriptorSystem sessionForLogin: (Login new database: platform).
	system := session system.
	platform useBinding: false.
	row1 := DatabaseRow newForTable: (system tableNamed: 'GR_ADDRESS').
	row1 atFieldNamed: 'ID' put: 1.
	row2 := DatabaseRow newForTable: (system tableNamed: 'GR_ADDRESS').
	row2 atFieldNamed: 'ID' put: 2.
	command := InsertCommand forRows: (Array with: row1 with: row2) useBinding: false session: session.
	string := command sqlString.
	self assert: (string includes: $;).
	self assert: string = 'INSERT INTO GR_ADDRESS (ID,STREET,HOUSE_NUM)  VALUES (1,NULL,NULL); INSERT INTO GR_ADDRESS (ID,STREET,HOUSE_NUM)  VALUES (2,NULL,NULL)'.! !

GlorpTestCase subclass: #GlorpSimpleQueryTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpSimpleQueryTest class methodsFor: 'resources' stamp: ''!
resources

	^Array with: GlorpSessionResource.! !

!GlorpSimpleQueryTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
setUpQueryBasic: query 
	| |
	query session: session.
	query setUpExpressions.
	query setupTracing.! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
setUpQueryFields: query 
	self setUpQueryBasic: query.
	query computeFields.! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
setUpQueryFully: query 
	self setUpQueryBasic: query.
	query prepare! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
skipToString
	^session platform supportsANSIJoins ifTrue: ['join '] ifFalse: ['where '].! !

!GlorpSimpleQueryTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session reset.
	session := nil.! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
testCaseInsensitiveQuery
	| result |
	session platform supportsCaseInsensitiveLike ifFalse: [^self].
	[session beginUnitOfWork.
	session beginTransaction.
	session register: GlorpAddress example1.
	session commitUnitOfWork.
	result := session
				readOneOf: GlorpAddress
				where: [:address | address street ilike: 'WeSt%'].
	self assert: result street = 'West 47th Ave'] 
			ensure: [session rollbackTransaction]! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
testComputingFieldsForDirectMappings

	| query table |
	query := SimpleQuery returningOneOf: GlorpAddress where: [:each | each id = 1].
	self setUpQueryFields: query.
	
	table := session system tableNamed: 'GR_ADDRESS'.
	self assert: (query fields = table fields).! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
testComputingFieldsForDirectMappingsWithFullPrepare

	| query table |
	query := Query readOneOf: GlorpAddress where: [:each | each id = 1].
	query session: session.
	query prepare.
	
	table := session system tableNamed: 'GR_ADDRESS'.
	self assert: ((query fields collect: [:each | each table parent fieldNamed: each name])= table fields).! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
testComputingFieldsForReferenceMappings

	| query table |
	query := SimpleQuery returningOneOf: GlorpPerson where: [:each | each id = 1].
	self setUpQueryFields: query.
	
	table := session system tableNamed: 'PERSON'.
	self assert: (query fields = table fields).! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
testComputingFieldsForReferenceMappingsWithFullPrepare

	| query table |
	query := SimpleQuery returningOneOf: GlorpPerson where: [:each | each id = 1].
	query session: session.
	query prepare.
	
	table := session system tableNamed: 'PERSON'.
	self assert: ((query fields collect: [:each | each table parent fieldNamed: each name])= table fields).! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
testDescriptorAssignmentToCriteria

	| query |
	query := SimpleQuery returningOneOf: GlorpAddress where: [:each | each id = 1].
	query session: session.
	query setUpExpressions.
	self assert: query whereClause ultimateBaseExpression descriptor == (session descriptorFor: GlorpAddress).! !

!GlorpSimpleQueryTest methodsFor: 'tests-ordering' stamp: ''!
testDoubleOrderSQL

	| query sql |
	query := SimpleQuery returningManyOf: GlorpAddress.
	query orderBy: [:each | each id].
	query orderBy: [:each | each number].
	self setUpQueryFully: query.
	sql := (query sqlWith: Dictionary new) sqlString.
	self assert: ('* from gr_address t1 order by t1.id, t1.house_num' match: sql asLowercase).! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: ''!
testFieldAliasingForEmbeddedMappings

	| query table |
	query := SimpleQuery returningOneOf: GlorpBankTransaction where: [:each | each id = 1].
	self setUpQueryFields: query.
	table := session system tableNamed: 'BANK_TRANS'.
	self assert: (query fields = table fields).
	self assert: (query builders first translateFieldPosition: (table fieldNamed: 'ID')) = 1.
	self assert: (query builders first translateFieldPosition: (table fieldNamed: 'OWNER_ID')) = 2.! !

!GlorpSimpleQueryTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/2/2011 08:21'!
testJoinExpressionWithMultipleTables
	| query sql sqlStream result command |
	query := SimpleQuery returningOneOf: GlorpPassenger where: [:each | each id = 1].
	query session: session.
	query prepare.
	command := query sqlWith: Dictionary new.
	command useBinding: false.
	sql := command sqlString.
	sqlStream := ReadStream on: sql asLowercase.
	sqlStream upToAll: self skipToString.
	Dialect isVisualWorks ifTrue: [sqlStream skip: self skipToString size].	"<Grumble grumble> stupid incompatibilities"
	result := sqlStream upToEnd.
	session platform supportsANSIJoins ifTrue: [
		self assert: ('frequent_flyer t2 on *t1.id = t2.id*<n> where *t1.id = 1*' expandMacros match: result)]
		ifFalse: [self assert:  ('*(t1.id = 1) and ((t1.id = t2.id*' match: result)].! !

!GlorpSimpleQueryTest methodsFor: 'tests-ordering' stamp: ''!
testOrderSQL

	| query sql |
	query := SimpleQuery returningManyOf: GlorpAddress.
	query orderBy: [:each | each id].
	self setUpQueryFully: query.
	sql := (query sqlWith: Dictionary new) sqlString.
	self assert: ('* from gr_address t1 order by t1.id' match: sql asLowercase).! !

GlorpTestCase subclass: #GlorpSortedCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

GlorpTestCase subclass: #GlorpStringTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpStringTest commentStamp: '<historical>' prior: 0!
Test string extensions in Glorp.!

!GlorpStringTest methodsFor: 'tests' stamp: ''!
testGlorpIsCollection
	self deny: 'abc' glorpIsCollection.
	self deny: Object new glorpIsCollection.
	self deny: $a glorpIsCollection.
	self assert: #() glorpIsCollection.! !

!GlorpStringTest methodsFor: 'tests' stamp: ''!
testLikePercent
	self deny: ('abc' like: 'def').
	self assert: ('' like: '%').
	self assert: ('abc' like: '%').
	self assert: ('abc' like: '%bc').
	self deny: ('abc' like: 'a%ac').
	self assert: ('aaababc' like: '%bc').
	self assert: ('abcd' like: 'ab%cd').
	self assert: ('abcdef' like: 'ab%ef').
	self assert: ('abcdef' like: 'ab%ef%').
	self assert: ('abcdefg' like: 'ab%ef%').! !

!GlorpStringTest methodsFor: 'tests' stamp: ''!
testLikeUnderscore
	self deny: ('' like: '_').
	self assert: ('a' like: '_').
	self assert: ('abc' like: '_bc').
	self deny: ('aaababc' like: '_bc').
	self deny: ('abcd' like: 'ab_cd').
	self assert: ('abcd' like: 'a_cd').
	self assert: ('abcdef' like: 'ab__ef').
	self assert: ('abcdef' like: 'ab__ef%').
	self assert: ('abcdefg' like: 'ab%ef_').
	self deny: ('abcdefgh' like: 'ab%ef_').! !

!GlorpTestCase class methodsFor: 'testing' stamp: 'GuillermoPolito 4/9/2011 15:25'!
isAbstract
	^self class name = #GlorpTestCase! !

!GlorpTestCase methodsFor: 'hacks' stamp: ''!
knownFailure
	"This does nothing, but makes it easier to search for known failures."
	^self.! !

!GlorpTestCase methodsFor: 'hacks' stamp: ''!
unfinished! !

GlorpTestCase subclass: #GlorpTimedProxyReaperTest
	instanceVariableNames: 'reaper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpTimedProxyReaperTest methodsFor: 'tests' stamp: ''!
README

	"Unfortunately, these tests are timing sensitive. They may intermittently fail, when in fact they are working as advertised.
	Rerun failed tests individually. If a test consistently fails, then it likely indicates a bug."! !

!GlorpTimedProxyReaperTest methodsFor: 'support' stamp: ''!
setUp
	Dialect isVisualAge ifTrue: [Transcript cr; show: 'Avoiding ', self class name, ' for VisualAge'. ^self].
	Dialect isObjectStudio ifTrue: [Transcript cr; show: 'Avoiding ', self class name, ' for ObjectStudio'. ^self].

	reaper := TimedProxyReaper new.
	reaper delay: 0.5.! !

!GlorpTimedProxyReaperTest methodsFor: 'support' stamp: ''!
tearDown

	reaper release.
	reaper := nil.! !

!GlorpTimedProxyReaperTest methodsFor: 'tests' stamp: ''!
testAddObjects

	| proxies |
	Dialect isVisualAge ifTrue: [Transcript cr; show: 'Avoiding ', self class name, '>>testAddObjects for VisualAge'. ^self].
	Dialect isObjectStudio ifTrue: [Transcript cr; show: 'Avoiding ', self class name, '>>testAddObjects for ObjectStudio'. ^self].
	proxies := OrderedCollection new.
	1 to: 10 do: [:index | 
		proxies add: (TimedProxy returningOneOf: Object where: nil in: nil).
		proxies last glorpSecondsToLive: 1000.
		reaper add: proxies last].

	(Delay forSeconds: 1) wait.
	proxies do: [:proxy | self should: [reaper includes: proxy]].! !

!GlorpTimedProxyReaperTest methodsFor: 'tests' stamp: ''!
testExpireObjects

	| proxies |
	Dialect isVisualAge ifTrue: [Transcript cr; show: 'Avoiding ', self class name, '>>testExpireObjects for VisualAge'. ^self].
	Dialect isObjectStudio ifTrue: [Transcript cr; show: 'Avoiding t', self class name, '>>estExpireObjects for ObjectStudio'. ^self].

	proxies := OrderedCollection new.
	1 to: 10 do: [:index | 
		proxies add: (TimedProxy returningOneOf: Object where: nil in: nil).
		proxies last glorpSecondsToLive: 1.
		reaper add: proxies last].

	(Delay forSeconds: 5) wait.
	proxies do: [:proxy | self shouldnt: [reaper includes: proxy]].! !

!GlorpTimedProxyReaperTest methodsFor: 'tests' stamp: ''!
testTouchAndExpireObjectsWhileOtherObjectsAreBeingReaped

	| expiringProxy proxy |
	Dialect isVisualAge ifTrue: [Transcript cr; show: 'Avoiding ', self class name, '>>testTouchAndExpireObjectsWhileOtherObjectsAreBeingReaped for VisualAge'. ^self].
	Dialect isObjectStudio ifTrue: [Transcript cr; show: 'Avoiding ', self class name, '>>testTouchAndExpireObjectsWhileOtherObjectsAreBeingReaped for ObjectStudio'. ^self].


	reaper delay: 0.25.
	expiringProxy := TimedProxy returningOneOf: Object where: nil in: nil.
	expiringProxy glorpSecondsToLive: 1.
	proxy := TimedProxy returningOneOf: Object where: nil in: nil.
	proxy glorpSecondsToLive: 2.

	reaper add: proxy; add: expiringProxy.
	self assert: (reaper includes: proxy).
	self assert: (reaper includes: expiringProxy).
	(Delay forSeconds: 1) wait.
	proxy glorpTouch.
	(Delay forSeconds: 1.5) wait.
	self assert: (reaper includes: proxy).
	self deny: (reaper includes: expiringProxy).
	(Delay forSeconds: 2) wait.
	self deny: (reaper includes: proxy).! !

!GlorpTimedProxyReaperTest methodsFor: 'tests' stamp: ''!
testTouchObjects

	| proxy |
	Dialect isVisualAge ifTrue: [Transcript cr; show: 'Avoiding ', self class name, '>>testTouchObjects for VisualAge'. ^self].
	Dialect isObjectStudio ifTrue: [Transcript cr; show: 'Avoiding ', self class name, '>>testTouchObjects for ObjectStudio'. ^self].

	reaper delay: 0.25.
	proxy := TimedProxy returningOneOf: Object where: nil in: nil.
	proxy glorpSecondsToLive: 2.
	reaper add: proxy.
	(Delay forSeconds: 1) wait.
	proxy glorpTouch.
	(Delay forSeconds: 1.1) wait.
	self should: [reaper includes: proxy].
	(Delay forSeconds: 2) wait.
	self shouldnt: [reaper includes: proxy]! !

GlorpTestCase subclass: #GlorpTracingTest
	instanceVariableNames: 'tracing'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpTracingTest methodsFor: 'support' stamp: ''!
setUp

	tracing := Tracing new.
	tracing setup.! !

!GlorpTracingTest methodsFor: 'tests' stamp: ''!
testAddDuplicateTracings

	| all |
	tracing addExpression: (tracing base get: #foo).
	tracing addExpression: (tracing base get: #foo).
	all := tracing allTracings.
	self assert: all size = 2.
	self assert: all first == tracing base.! !

!GlorpTracingTest methodsFor: 'tests' stamp: ''!
testAddRecursiveTracings

	| all |
	tracing addExpression: (tracing base get: #foo).
	tracing addExpression: ((tracing base get: #foo) get: #bar).
	all := tracing allTracings.
	self assert: all size = 3.
	self assert: all first == tracing base.
	self assert: all last base == (all at: 2).! !

!GlorpTracingTest methodsFor: 'tests' stamp: ''!
testAddTracing

	| all |
	tracing addExpression: (tracing base get: #foo).
	all := tracing allTracings.
	self assert: all size = 2.
	self assert: all first == tracing base.
	self assert: all last == (tracing base get: #foo).! !

!GlorpTracingTest methodsFor: 'tests' stamp: ''!
testAddTwoTracings

	tracing addExpression: (tracing base get: #foo).
	tracing addExpression: (tracing base get: #bar).
	self assert: tracing allTracings size = 3.
	self assert: tracing allTracings first == tracing base.! !

GlorpTestCase subclass: #GlorpTypeComparisonTest
	instanceVariableNames: 'platform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-DatabaseTypes'!

!GlorpTypeComparisonTest methodsFor: 'running' stamp: ''!
setUp
	super setUp.
	platform := DatabasePlatform new! !

!GlorpTypeComparisonTest methodsFor: 'testing' stamp: ''!
testIntegerToInteger
	self assert: (GlorpIntegerType new platform: platform) = (GlorpIntegerType new platform: platform).
	self assert: (GlorpIntegerType new platform: platform) hash = (GlorpIntegerType new platform: platform) hash.! !

!GlorpTypeComparisonTest methodsFor: 'testing' stamp: ''!
testIntegerToIntegerishNumber
	self assert: (GlorpIntegerType new platform: platform) = (GlorpNumericType new platform: platform).
	self assert: (GlorpIntegerType new platform: platform) hash = (GlorpNumericType new platform: platform) hash.
	self assert: (GlorpNumericType new platform: platform) = (GlorpIntegerType new platform: platform).
	self assert: (GlorpNumericType new platform: platform) hash = (GlorpIntegerType new platform: platform) hash.! !

!GlorpTypeComparisonTest methodsFor: 'testing' stamp: ''!
testIntegerToNumber
	self deny: (GlorpIntegerType new platform: platform) = (GlorpNumericType new platform: platform; precision: 2).
	self deny: (GlorpIntegerType new platform: platform) hash = (GlorpNumericType new platform: platform; precision: 2) hash.
	self deny: (GlorpNumericType new platform: platform; precision: 2) = (GlorpIntegerType new platform: platform).
	self deny: (GlorpNumericType new platform: platform; precision: 2) hash = (GlorpIntegerType new platform: platform) hash.! !

!GlorpTypeComparisonTest methodsFor: 'testing' stamp: ''!
testIntegerishNumberToNumber
	self deny: (GlorpNumericType new platform: platform) = (GlorpNumericType new platform: platform; precision: 2).
	self deny: (GlorpNumericType new platform: platform) hash = (GlorpNumericType new platform: platform; precision: 2) hash.
	self deny: (GlorpNumericType new platform: platform; precision: 2) = (GlorpNumericType new platform: platform).
	self deny: (GlorpNumericType new platform: platform; precision: 2) hash = (GlorpNumericType new platform: platform) hash.! !

!GlorpTypeComparisonTest methodsFor: 'testing' stamp: ''!
testNumberToNumber
	self assert: (GlorpNumericType new platform: platform; precision: 2) = (GlorpNumericType new platform: platform; precision: 2).
	self assert: (GlorpNumericType new platform: platform; precision: 2) hash = (GlorpNumericType new platform: platform; precision: 2) hash.
	self assert: (GlorpNumericType new platform: platform; precision: 2) = (GlorpNumericType new platform: platform; precision: 2).
	self assert: (GlorpNumericType new platform: platform; precision: 2) hash = (GlorpNumericType new platform: platform; precision: 2) hash.! !

GlorpTestCase subclass: #GlorpUnitOfWorkTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpUnitOfWorkTest class methodsFor: 'resources' stamp: ''!
resources

	^Array with: GlorpSessionResource.! !

!GlorpUnitOfWorkTest methodsFor: 'support' stamp: ''!
exampleCustomerProxy
	| p |
	p := Proxy new.
	p session: session.
	p query: (GlorpQueryStub returningOneOf: GlorpCustomer where: [:a | a id = 3]).
	p query result: (GlorpCustomer new id: 3).
	^p! !

!GlorpUnitOfWorkTest methodsFor: 'support' stamp: ''!
exampleCustomerWithTransactionsProxy
	| customer |
	customer := GlorpCustomer new.
	customer transactions: self exampleTransactionsProxy.
	^customer! !

!GlorpUnitOfWorkTest methodsFor: 'support' stamp: ''!
exampleTransactionWithCustomerProxy
	| transaction |
	transaction := GlorpBankTransaction example1.
	transaction owner: self exampleCustomerProxy.
	^transaction! !

!GlorpUnitOfWorkTest methodsFor: 'support' stamp: ''!
exampleTransactionsProxy
	| p |
	p := Proxy new.
	p session: session.
	p query: (GlorpQueryStub returningOneOf: GlorpBankTransaction where: [:a | a id ~= 0]).
	p query result: (Array with: GlorpBankTransaction example1 with: GlorpBankTransaction example1).
	^p! !

!GlorpUnitOfWorkTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.! !

!GlorpUnitOfWorkTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session reset.
	session resetPermissions.
	session := nil.! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testAutomaticRegistrationOnRead
	| p c |
	p := self exampleCustomerProxy.
	c := p getValue.
	session beginUnitOfWork.
	session register: p.
	self assert: (session isRegistered: p).
	self assert: (session isRegistered: c).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction isRegistered: p).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction isRegistered: c).! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testCheckIfInstantiationRequiredForDirectMapping
	| c1 mapping proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	c1 name: proxy.
	session beginUnitOfWork.
	session register: c1.
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #name.
	self deny: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testCheckIfInstantiationRequiredForRelationshipInstantiatedProxy
	| c1 mapping proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	proxy query: (GlorpQueryStub new session: session; result: 'foo').
	proxy session: session.
	proxy yourSelf.
	c1 accounts: proxy.
	session beginUnitOfWork.
	session register: c1.
	c1 accounts: #().
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #accounts.
	self deny: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testCheckIfInstantiationRequiredForRelationshipNoChange
	"Note that we explicitly register the objects as being old (even though they aren't) so that we don't force instantation of the related objects, which is what we're trying to test."
	| c1 mapping proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	c1 accounts: proxy.
	session beginUnitOfWork.
	session privateRegisterAsOld: c1.
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #accounts.
	self deny: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testCheckIfInstantiationRequiredForRelationshipNoProxy
	| c1 mapping |
	c1 := GlorpCustomer new.
	c1 accounts: #().
	session beginUnitOfWork.
	session register: c1.
	c1 accounts: nil.
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #accounts.
	self deny: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testCheckIfInstantiationRequiredForRelationshipWithChange
	"Note that we explicitly register the objects as being old (even though they aren't) so that we don't force instantation of the related objects, which is what we're trying to test."
	| c1 mapping proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	proxy session: session.
	proxy query: (GlorpQueryStub new result: 'foo').
	c1 accounts: proxy.
	session beginUnitOfWork.
	session privateRegisterAsOld: c1.
	c1 accounts: #().
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #accounts.
	self assert: (session privateGetCurrentUnitOfWork instantiateIfRequiredFor: c1 mapping: mapping).! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testCheckIfInstantiationRequiredForToManyRelationshipFromNew
	"Note that we force the object to be registered as new, just to make absolutely sure it's being treated as a new object."
	| c1 proxy |
	c1 := GlorpCustomer new.
	proxy := Proxy new.
	proxy session: session.
	c1 accounts: proxy.
	session beginUnitOfWork.
	session privateRegisterAsNew: c1.
	self assert: (proxy isInstantiated).! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testCommitOrderAtSessionLevel

	| tables | 
	tables := session tablesInCommitOrder.
	tables first name = 'CUSTOMER'.
	self unfinished.! !

!GlorpUnitOfWorkTest methodsFor: 'tests-transaction wrappers' stamp: ''!
testInTransactionDoSuccessful
	"This has to test that a transaction completed successfully, so unlike most other tests, we have to clean up the evidence afterwards"

	| result endResult |
	[session inTransactionDo: [
		session writeRow: session system exampleAddressRow].
	result := session readManyOf: GlorpAddress.
	self assert: result size = 1.
	self assert: result first id = 123]
		ensure: [session inTransactionDo: [session accessor executeSQLStringNoResult: 'DELETE FROM GR_ADDRESS']].
	endResult := session readManyOf: GlorpAddress.
	self assert: endResult size = 0.! !

!GlorpUnitOfWorkTest methodsFor: 'tests-transaction wrappers' stamp: ''!
testInTransactionDoUnsuccessful

	| result initial|
	initial := session readManyOf: GlorpAddress.
	self assert: initial isEmpty.
	[session inTransactionDo: [
		session writeRow: session system exampleAddressRow.
		self error: 'no you don''t']] on: Dialect error do: [:ex | ex return: nil].
	result := session readManyOf: GlorpAddress.
	self assert: result size = 0.! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testOriginalValueFor
	| c1 mapping |
	c1 := GlorpCustomer new.
	c1 name: 'fred'.
	session beginUnitOfWork.
	session register: c1.
	c1 name: 'barney'.
	mapping := (session descriptorFor: GlorpCustomer) mappingForAttributeNamed: #name.
	self assert: (session privateGetCurrentUnitOfWork originalValueFor: c1 mapping: mapping) = 'fred'.! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testPostRegister
	| c1 t1 t2 |
	c1 := GlorpCustomer example2.
	[session beginTransaction.
	session beginUnitOfWork.
	t1 := GlorpBankTransaction new.
	t2 := GlorpBankTransaction new.
	c1 addTransaction: t1.
	c1 addTransaction: t2.
	session register: c1.
	self assert: (session isRegistered: c1).
	self assert: (session isRegistered: t1).
	self assert: (session isRegistered: t2).
	session commitUnitOfWork]
		ensure: [session rollbackTransaction].
	"Need some assertions on what was written"
	self unfinished.! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testPreRegister
	
	| c1 t1 t2 trans |
	c1 := GlorpCustomer example2.
	[session beginTransaction.
	session beginUnitOfWork.
	session register: c1.
	t1 := GlorpBankTransaction new.
	t2 := GlorpBankTransaction new.
	c1 addTransaction: t1.
	c1 addTransaction: t2.
	trans := session privateGetCurrentUnitOfWork privateGetTransaction.
	session commitUnitOfWork.
	self assert: (trans isRegistered: c1).
	self assert: (trans isRegistered: t1).
	self assert: (trans isRegistered: t2).]
		ensure: [session rollbackTransaction].
	"Need some assertions on what got written"
	self unfinished.! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterAll
	| c1 c2 collection |
	c1 := GlorpCustomer new.
	c2 := GlorpCustomer new.
	session beginUnitOfWork.
	collection := Array with: c1 with: c2.
	session registerAll: collection.
	self assert: (session isRegistered: c1).
	self assert: (session isRegistered: c2).
	self deny: (session isRegistered: collection).! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterCollection
	| c1 c2 collection |
	c1 := GlorpCustomer new.
	c2 := GlorpCustomer new.
	session beginUnitOfWork.
	collection := Array with: c1 with: c2.
	session register: collection.
	self assert: (session isRegistered: c1).
	self assert: (session isRegistered: collection).! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterEmbeddedInTargetDontRefetch
	" This tests for a problem with embedded objects suddenly becoming nil. This can happen when an object is part of a to-many relation. If the relation is followed during the registering phase the query will wrongly replace the embedded object even though requiresPopulating is false. When the object itself is already registered the replaced embedded object won't be registered."

	| transactionId customer |
	
	[session beginTransaction.
	session inUnitOfWorkDo: 
			[customer := GlorpCustomer example1.
			customer id: 42.
			session register: customer].
	transactionId := customer transactions first id.
	customer := nil.
	session initializeCache.
	session inUnitOfWorkDo: 
			[| transaction amount |
			transaction := session readOneOf: GlorpBankTransaction
						where: [:each | each id = transactionId].
			amount := transaction amount.
			transaction owner transactions getValue.
			self assert: amount == transaction amount]] 
			ensure: [session rollbackTransaction]! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterExistingCollection
	| c1 |
	c1 := GlorpCustomer new.
	session beginUnitOfWork.
	session register: c1.
	session register: c1 transactions.
	self assert: (session isRegistered: c1).
	self assert: (session isRegistered: c1 transactions).
	self deny: (session isNew: c1 transactions).! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterInstantiatedProxy
	| p c |
	p := self exampleCustomerProxy.
	c := p getValue.
	session beginUnitOfWork.
	session register: p.
	self assert: (session isRegistered: p).
	self assert: (session isRegistered: c).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction isRegistered: p).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction isRegistered: c).! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterObjectWithCollectionProxyThenInstantiate
	"Note that we explicitly register the objects as being old (even though they aren't) so that we don't force instantation of the related objects, which is what we're trying to test."
	| customer transactions |
	customer := self exampleCustomerWithTransactionsProxy.
	session beginUnitOfWork.
	session privateRegisterAsOld: customer.
	self deny: customer transactions isInstantiated.
	transactions := customer transactions getValue.
	self assert: customer transactions isInstantiated.
	session register: transactions.
	self assert: (session isRegistered: transactions first).
	self assert: (session isRegistered: customer).
	self assert: (session isRegistered: transactions).
	self assert: (session isRegistered: customer transactions).! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterObjectWithInstantiatedProxy
	| transaction customer |
	transaction := self exampleTransactionWithCustomerProxy.
	customer := transaction owner getValue.
	session beginUnitOfWork.
	session register: transaction.
	self assert: (session isRegistered: transaction).
	self assert: (session isRegistered: customer).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction 
				isRegistered: transaction).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction 
				isRegistered: customer).
	self assert: transaction owner isInstantiated.! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterObjectWithNilCollection
	| c1 |
	c1 := GlorpCustomer new.
	c1 transactions: nil.
	session beginUnitOfWork.
	session register: c1.
	self assert: (session isRegistered: c1).! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterObjectWithProxy
	"Note that we explicitly register the objects as being old (even though they aren't) so that we don't force instantation of the related objects, which is what we're trying to test."
	| transaction |
	transaction := self exampleTransactionWithCustomerProxy.
	session beginUnitOfWork.
	session privateRegisterAsOld: transaction.
	self assert: (session isRegistered: transaction).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction 
				isRegistered: transaction).
	self deny: transaction owner isInstantiated.! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterObjectWithProxyThenInstantiate
	"Note that we explicitly register the objects as being old (even though they aren't) so that we don't force instantation of the related objects, which is what we're trying to test."
	| transaction customer |
	transaction := self exampleTransactionWithCustomerProxy.
	session beginUnitOfWork.
	session privateRegisterAsOld: transaction.
	self deny: transaction owner isInstantiated.
	customer := transaction owner getValue.
	session register: transaction.
	self assert: (session isRegistered: transaction).
	self assert: (session isRegistered: customer).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction 
				isRegistered: transaction).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction 
				isRegistered: customer).
	self assert: transaction owner isInstantiated! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterObjectWithProxyThenInstantiateAndReregister
	| transaction customer |
	transaction := self exampleTransactionWithCustomerProxy.
	session beginUnitOfWork.
	session register: transaction.
	customer := transaction owner getValue.
	session register: transaction.
	self assert: (session isRegistered: transaction).
	self assert: (session isRegistered: customer).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction 
				isRegistered: transaction).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction 
				isRegistered: customer).
	self assert: transaction owner isInstantiated! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterProxy
	| p |
	p := self exampleCustomerProxy.
	session beginUnitOfWork.
	session register: p.
	self deny: (session isRegistered: p).
	self deny: (session isRegistered: p query result).
	p getValue.
	self assert: (session isRegistered: p).
	self assert: (session isRegistered: p query result).! !

!GlorpUnitOfWorkTest methodsFor: 'tests-registration' stamp: ''!
testRegisterProxyThenInstantiateAndReregister
	| p c |
	p := self exampleCustomerProxy.
	session beginUnitOfWork.
	session register: p.
	c := p getValue.
	session register: p.
	self assert: (session isRegistered: p).
	self assert: (session isRegistered: c).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction 
				isRegistered: p).
	self assert: (session privateGetCurrentUnitOfWork privateGetTransaction 
				isRegistered: c)! !

!GlorpUnitOfWorkTest methodsFor: 'tests-forcing update' stamp: ''!
testRegisteringAndForcingUpdate
	"Test that we can force the first customer to be rewritten, even if it hasn't been modified by using the forceSaveOf: call"
	| customer triedToWrite row table map otherCustomer |
	session beginTransaction.
	customer := GlorpCustomer example1 id: 7.
	otherCustomer := GlorpCustomer example2 id: 8.
	session modify: (Array with: customer with: otherCustomer) in: [].
	session accessor denyWrites.
	session modify: customer in: [].
	session beginUnitOfWork.
	session register: otherCustomer.
	session save: customer.
	[session commitUnitOfWork] on: GlorpIllegalCommand do: [:ex |
		triedToWrite := true.
		table := session system tableNamed: 'GR_CUSTOMER'.
		map := session privateGetCurrentUnitOfWork privateGetRowMap subMapForTable: table.
		row := map at: map keys asArray first.
		self assert: (row at: row fields first) = 7.
		self assert: (row at: row fields last) = 'Fred Flintstone'.
		self assert: map size = 1.
	].
	self assert: triedToWrite.
	session rollbackTransaction.! !

!GlorpUnitOfWorkTest methodsFor: 'tests-forcing update' stamp: ''!
testRegisteringAndForcingUpdateToDatabase
	"Test going round trip to the database with forceSaveOf:. This odesn't assert much, but is mostly to make sure we aren't doing an insert instead of an update."
	| customer otherCustomer |
	session beginTransaction.
	customer := GlorpCustomer example1 id: 7.
	otherCustomer := GlorpCustomer example2 id: 8.
	session modify: (Array with: customer with: otherCustomer) in: [].
	session inUnitOfWorkDo: [
		session register: otherCustomer.
		session save: customer].
	session rollbackTransaction.! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testRollbackOneToManyAfterAdd
	| customer t1 t2 transList amount |
	customer := GlorpCustomer example1.
	t1 := customer transactions first.
	t2 := customer transactions last.
	transList := customer transactions.
	amount := t1 amount.
	session beginUnitOfWork.
	session register: customer.
	20 timesRepeat: [customer addTransaction: (GlorpBankTransaction new)].
	customer transactions first amount: 65543.
	session rollbackUnitOfWork.
	self assert: customer transactions == transList.
	self assert: customer transactions size = 2.
	self assert: customer transactions first == t1.
	self assert: customer transactions last == t2.
	self assert: t1 amount == amount.! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testRollbackOneToManyAfterReplace
	| customer t1 t2 transList |
	customer := GlorpCustomer example1.
	t1 := customer transactions first.
	t2 := customer transactions last.
	transList := customer transactions.
	session beginUnitOfWork.
	session register: customer.
	customer transactions: OrderedCollection new.
	session rollbackUnitOfWork.
	self assert: customer transactions == transList.
	self assert: customer transactions size = 2.
	self assert: customer transactions first == t1.
	self assert: customer transactions last == t2.! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testRollbackOneToManyProxy
	| customer t1 t2 transList |
	customer := GlorpCustomer example1.
	t1 := customer transactions first.
	t2 := customer transactions last.
	transList := customer transactions.
	session beginUnitOfWork.
	session register: customer.
	customer transactions: OrderedCollection new.
	session rollbackUnitOfWork.
	self assert: customer transactions == transList.
	self assert: customer transactions size = 2.
	self assert: customer transactions first == t1.
	self assert: customer transactions last == t2.! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testRollbackOneToManyWithList
	"Check that dependents aren't being registered for the collection"
	| customer marker |
	"Lists only exist in VW"
	Dialect isVisualWorks ifFalse: [^self].
	marker := Object new.
	customer := GlorpCustomer example1.
	customer transactions: (customer transactions asList).
	customer transactions addDependent: marker.
	session beginUnitOfWork.
	session register: customer.
	20 timesRepeat: [customer addTransaction: (GlorpBankTransaction new)].
	session rollbackUnitOfWork.
	self assert: customer transactions class == (Dialect smalltalkAt: #List).
	self assert: customer transactions size = 2.
	self should: [customer transactions privateAt: 3] raise: Object subscriptOutOfBoundsSignal.
	self assert: (customer transactions dependents includes: marker).
	self deny: (session isRegistered: marker).! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testRollbackOneToOne
	| transaction customer |
	transaction := GlorpBankTransaction new.
	customer := GlorpCustomer new.
	transaction owner: customer.
	session beginUnitOfWork.
	session register: transaction.
	transaction owner: GlorpCustomer new.
	session rollbackUnitOfWork.
	self assert: transaction owner == customer.! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testRollbackOneToOneWithProxy
	| transaction customerProxy |
	transaction := self exampleTransactionWithCustomerProxy.
	customerProxy := transaction owner.
	session beginUnitOfWork.
	session register: transaction.
	transaction owner: GlorpCustomer new.
	session rollbackUnitOfWork.
	self assert: transaction owner == customerProxy! !

!GlorpUnitOfWorkTest methodsFor: 'tests-transaction wrappers' stamp: ''!
testTransactSuccessful
	"This has to test that a transaction completed successfully, so unlike most other tests, we have to clean up the evidence afterwards"

	| result |
	[session transact: [
		session register: (GlorpReservation new id: 345)].
	result := session readManyOf: GlorpReservation.
	self assert: result size = 1.
	self assert: result first id = 345]
		ensure: [session transact: [session accessor executeSQLStringNoResult: 'DELETE FROM RESERVATION']].! !

!GlorpUnitOfWorkTest methodsFor: 'tests-transaction wrappers' stamp: ''!
testTransactUnsuccessful

	| result |
	[session transact: [
		session register: (GlorpReservation new id: 345).
		self error: 'didn''t work']] on: Dialect error do: [:ex | ex return: nil].
	result := session readManyOf: GlorpReservation.
	self assert: result size = 0.! !

!GlorpUnitOfWorkTest methodsFor: 'tests' stamp: ''!
testWriteObjectWithNilCollection
	| c1 query customer |
	c1 := GlorpCustomer new.
	c1 transactions: nil.
	c1 id: 9999.
	[session beginTransaction.
	session beginUnitOfWork.
	session register: c1.
	session commitUnitOfWork.
	query := Query returningOneOf: GlorpCustomer where: [:each | each id = 9999].
	query shouldRefresh: true.
	customer := session execute: query.
	self assert: customer transactions notNil.
	self assert: customer transactions isEmpty.
	]
		ensure: [session rollbackTransaction].! !

!GlorpUnitOfWorkTest methodsFor: 'tests-transaction wrappers' stamp: ''!
testinUnitOfWorkSuccessful

	| result |
	[session beginTransaction.
	session inUnitOfWorkDo: [
		session register: (GlorpReservation new id: 345)].
	result := session readManyOf: GlorpReservation.
	self assert: result size = 1.
	self assert: result first id = 345]
		ensure: [session rollbackTransaction].! !

!GlorpUnitOfWorkTest methodsFor: 'tests-transaction wrappers' stamp: ''!
testinUnitOfWorkUnsuccessful

	| result |
	[session beginTransaction.
	[session inUnitOfWorkDo: [
		session register: (GlorpReservation new id: 345).
		self error: 'aaaagh']] on: Dialect error do: [:ex | ex return: nil].
	result := session readManyOf: GlorpReservation.
	self assert: result size = 0]
		ensure: [session rollbackTransaction].! !

GlorpTestCase subclass: #GlorpWritingTest
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpWritingTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	session := GlorpSessionResource current newSession.! !

!GlorpWritingTest methodsFor: 'support' stamp: ''!
tearDown
	super tearDown.
	session reset.
	session := nil.! !

!GlorpWritingTest methodsFor: 'tests' stamp: ''!
testLargeWrite
	"This is really just a performance test. No assertions. It should take on the order of 10 seconds. If it starts taking 10 minutes, it indicates a potential problem...."
	| customer |
	session beginTransaction.
	[customer := GlorpCustomer new.
	customer name: 'foo'.
     customer id: 123.
	1 to: 5000 do: [:i | | transaction |
		transaction := GlorpBankTransaction new.
		transaction amount amount: i.
		customer addTransaction: transaction].
	session beginUnitOfWork.
	session register: customer.
	session commitUnitOfWork]
		ensure: [session rollbackTransaction].! !

!GlorpWritingTest methodsFor: 'tests' stamp: ''!
testMultipleWritesAndReads
	"Trying to narrow down on a problem with SQLiteEXDI"

	| names customers |
	session beginTransaction.
	[session beginUnitOfWork.
	names := OrderedCollection new.
	1 to: 10
		do: 
			[:i | 
			| nm |
			nm := 'John Doe ' , i printString.
			names add: nm.
			session register: ((GlorpCustomer new)
									id: i;
									name: nm;
									yourself)].
	session commitUnitOfWork.
	customers := session readManyOf: GlorpCustomer.
	self assert: (names asSortedCollection isSameSequenceAs: (customers collect: [:ea | ea name]) asSortedCollection).
	] 
			ensure: [session rollbackTransaction]! !

!GlorpWritingTest methodsFor: 'tests' stamp: 'GuillermoPolito 4/26/2011 04:58'!
testRegistrationFromWrittenObject
	| customer trans transactions |
	session beginTransaction.
	[customer := GlorpCustomer new.
	customer name: 'foo'.
     customer id: 123.
	session beginUnitOfWork.
	session register: customer.
	session commitUnitOfWork.
	trans := GlorpBankTransaction new.
	session beginUnitOfWork.
	session readOneOf: GlorpCustomer where: [:each | each id = customer id].
	customer addTransaction: trans.
	session commitUnitOfWork.
	transactions := session accessor executeSQLString: 'SELECT ID FROM BANK_TRANS WHERE OWNER_ID = ', customer id printString.
	self assert: transactions size = 1.
	self assert: (trans id = (transactions first at: 1))]
		ensure: [session rollbackTransaction].! !

TestCase subclass: #GlorpTransformedFieldValueWrapperTest
	instanceVariableNames: 'inner outer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpTransformedFieldValueWrapperTest methodsFor: 'accessing' stamp: ''!
inner
	^inner! !

!GlorpTransformedFieldValueWrapperTest methodsFor: 'accessing' stamp: ''!
inner: aFieldValueWrapper
	inner := aFieldValueWrapper! !

!GlorpTransformedFieldValueWrapperTest methodsFor: 'accessing' stamp: ''!
outer
	^outer! !

!GlorpTransformedFieldValueWrapperTest methodsFor: 'accessing' stamp: ''!
outer: aTransformingFieldValueWrapper
	outer := aTransformingFieldValueWrapper! !

!GlorpTransformedFieldValueWrapperTest methodsFor: 'support' stamp: ''!
setUp
	super setUp.
	inner := FieldValueWrapper new.
	outer := TransformingFieldValueWrapper new innerWrapper: inner; forwardTransformation: [:a | a * 2]; inverseTransformation: [:a | a / 2].! !

!GlorpTransformedFieldValueWrapperTest methodsFor: 'tests' stamp: ''!
testPrivateSetOuter
	self deny: inner hasValue.
	self deny: outer hasValue.
	outer privateSetContents: 8.
	self assert: outer contents = 8.
	self assert: inner contents = 4.
	self assert: inner hasValue.
	self assert: outer hasValue.! !

!GlorpTransformedFieldValueWrapperTest methodsFor: 'tests' stamp: ''!
testSetInner
	self deny: inner hasValue.
	self deny: outer hasValue.
	inner contents: 3.
	self assert: outer contents = 6.
	self assert: inner contents = 3.
	self assert: inner hasValue.
	self assert: outer hasValue.! !

!GlorpTransformedFieldValueWrapperTest methodsFor: 'tests' stamp: ''!
testSetOuter
	self deny: inner hasValue.
	self deny: outer hasValue.
	outer contents: 8.
	self assert: outer contents = 8.
	self assert: inner contents = 4.
	self assert: inner hasValue.
	self assert: outer hasValue.! !

!GlorpTransformedFieldValueWrapperTest methodsFor: 'tests' stamp: ''!
testSetOuterFails
	self deny: inner hasValue.
	self deny: outer hasValue.
	outer contents: 8.
	self should: [outer contents: 9] raise: Dialect error.
	self assert: outer contents = 8.
	self assert: inner contents = 4.
	self assert: inner hasValue.
	self assert: outer hasValue.! !

!DescriptorSystem class methodsFor: '*GlorpTests' stamp: ''!
createTables
	"Create the tables for this descriptor system. This will fail if the tables already exist. Use recreateTables for that."
	^self createTablesFor: GlorpDatabaseLoginResource current login.! !

!DescriptorSystem class methodsFor: '*GlorpTests' stamp: ''!
dropTables
	"Drop and re-create the tables for this descriptor system."
	^self dropTablesFor: GlorpDatabaseLoginResource current login.! !

!DescriptorSystem class methodsFor: '*GlorpTests' stamp: ''!
recreateTables
	"Drop and re-create the tables for this descriptor system."
	^self recreateTablesFor: GlorpDatabaseLoginResource current login.! !

DescriptorSystem subclass: #GlorpTestDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpTestDescriptorSystem commentStamp: '<historical>' prior: 0!
This is an abstract superclass for all descriptor systems whose tables should be set up as part of the standard GLORP testing process. See GlorpDemoTablePopulatorResource.!

GlorpTestDescriptorSystem subclass: #GlorpCollectionTypesDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-CollectionTypeModels'!

!GlorpCollectionTypesDescriptorSystem methodsFor: 'accessing' stamp: ''!
allTableNames

	^#('GR_THINGWITHCOLLECTIONS' 'GR_THINGONE' 'GR_THING_LINK').! !

!GlorpCollectionTypesDescriptorSystem methodsFor: 'accessing' stamp: ''!
constructAllClasses
	^super constructAllClasses add: GlorpThingWithLotsOfDifferentCollections; add: GlorpThingOne; yourself.! !

!GlorpCollectionTypesDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpThingOne: aDescriptor 
	aDescriptor table: self thingOneTable.
	aDescriptor addMapping: (DirectMapping from: #id to: (self thingOneTable fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (self thingOneTable fieldNamed: 'NAME')).! !

!GlorpCollectionTypesDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpThingWithLotsOfDifferentCollections: aDescriptor
	| ocMapping |
	aDescriptor table: self ownerTable.
	aDescriptor
		addMapping:
			(DirectMapping from: #id to: (self ownerTable fieldNamed: 'ID')).
	aDescriptor
		addMapping:
			(DirectMapping from: #name to: (self ownerTable fieldNamed: 'NAME')).
	aDescriptor
		addMapping:
			((OneToManyMapping new)
				attributeName: #array;
				referenceClass: GlorpThingOne;
				collectionType: Array;
				orderBy: [:each | (each getTable: self thingOneTable) getField: 'ARRAY_POSITION'];
				writeTheOrderField;
				join:
						(Join
								from: (self ownerTable fieldNamed: 'ID')
								to: (self thingOneTable fieldNamed: 'ARRAY_OWNER'))).
	aDescriptor
		addMapping:
			((OneToManyMapping new)
				attributeName: #set;
				referenceClass: GlorpThingOne;
				collectionType: Set;
				join:
						(Join
								from: (self ownerTable fieldNamed: 'ID')
								to: (self thingOneTable fieldNamed: 'SET_OWNER'))).
	ocMapping := ((ManyToManyMapping new)
				attributeName: #orderedCollection;
				referenceClass: GlorpThingOne;
				collectionType: OrderedCollection;
				join:
						(Join
								from: (self ownerTable fieldNamed: 'ID')
								to: (self linkTable fieldNamed: 'OWNER_ID')
								from: 'O'
								to: (self linkTable fieldNamed: 'TYPE'))).
	ocMapping orderBy: [:each | (each getTable: self linkTable) getField: 'POSITION'].
	ocMapping writeTheOrderField.
	aDescriptor addMapping: ocMapping.

	aDescriptor
		addMapping:
			((ManyToManyMapping new)
				attributeName: #bag;
				referenceClass: GlorpThingOne;
				collectionType: Bag;
				join:
						(Join
								from: (self ownerTable fieldNamed: 'ID')
								to: (self linkTable fieldNamed: 'OWNER_ID')
								from: 'B'
								to: (self linkTable fieldNamed: 'TYPE'))).
	aDescriptor
		addMapping:
			((ManyToManyMapping new)
				attributeName: #sortedCollection;
				referenceClass: GlorpThingOne;
				collectionType: SortedCollection;
				join:
						(Join
								from: (self ownerTable fieldNamed: 'ID')
								to: (self linkTable fieldNamed: 'OWNER_ID')
								from: 'S'
								to: (self linkTable fieldNamed: 'TYPE'))).! !

!GlorpCollectionTypesDescriptorSystem methodsFor: 'tables' stamp: ''!
linkTable
	^self tableNamed: 'GR_THING_LINK'.! !

!GlorpCollectionTypesDescriptorSystem methodsFor: 'tables' stamp: ''!
ownerTable
	^self tableNamed: 'GR_THINGWITHCOLLECTIONS'.! !

!GlorpCollectionTypesDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGR_THINGONE: aTable
	| setOwnerId arrayOwnerId |
	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).
	setOwnerId := aTable createFieldNamed: 'SET_OWNER' type: platform int4.
	aTable addForeignKeyFrom: setOwnerId
		to: (self ownerTable fieldNamed: 'ID').

	arrayOwnerId := aTable createFieldNamed: 'ARRAY_OWNER' type: platform int4.
	aTable addForeignKeyFrom: arrayOwnerId
		to: (self ownerTable fieldNamed: 'ID').
	aTable createFieldNamed: 'ARRAY_POSITION' type: platform int4.! !

!GlorpCollectionTypesDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGR_THINGWITHCOLLECTIONS: aTable

	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).! !

!GlorpCollectionTypesDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGR_THING_LINK: aTable
	| ownerId thingId |
	ownerId := aTable createFieldNamed: 'OWNER_ID' type: platform int4.
	aTable addForeignKeyFrom: ownerId to: (self ownerTable fieldNamed: 'ID').
	thingId := aTable createFieldNamed: 'THING_ID' type: platform int4.
	aTable
		addForeignKeyFrom: thingId
		to: (self thingOneTable fieldNamed: 'ID').
	aTable createFieldNamed: 'TYPE' type: (platform char: 1).
	aTable createFieldNamed: 'POSITION' type: platform int4.! !

!GlorpCollectionTypesDescriptorSystem methodsFor: 'tables' stamp: ''!
thingOneTable
	^self tableNamed: 'GR_THINGONE'.! !

GlorpCollectionTypesDescriptorSystem subclass: #GlorpCollectionTypesWithClassModelsDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-CollectionTypeModels'!

!GlorpCollectionTypesWithClassModelsDescriptorSystem methodsFor: 'class models' stamp: ''!
classModelForGlorpThingWithLotsOfDifferentCollections: aClassModel

	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #array collection: Array of: GlorpThingOne.
	aClassModel newAttributeNamed: #set collection: Set of: GlorpThingOne.
	aClassModel newAttributeNamed: #orderedCollection collection: OrderedCollection of: GlorpThingOne.
	aClassModel newAttributeNamed: #bag collection: Bag of: GlorpThingOne.
	aClassModel newAttributeNamed: #sortedCollection collection: SortedCollection of: GlorpThingOne.! !

!GlorpCollectionTypesWithClassModelsDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpThingWithLotsOfDifferentCollections: aDescriptor
	| ocMapping |
	aDescriptor table: self ownerTable.
	(aDescriptor newMapping: DirectMapping)
		from: #id to: (self ownerTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping)
		from: #name to: (self ownerTable fieldNamed: 'NAME').
	(aDescriptor newMapping: OneToManyMapping)
		attributeName: #array;
		orderBy: [:each | (each getTable: self thingOneTable) getField: 'ARRAY_POSITION'];
		writeTheOrderField;
			join:
				(Join
					from: (self ownerTable fieldNamed: 'ID')
					to: (self thingOneTable fieldNamed: 'ARRAY_OWNER')).
	(aDescriptor newMapping: OneToManyMapping)
		attributeName: #set;
		join:
			(Join
				from: (self ownerTable fieldNamed: 'ID')
				to: (self thingOneTable fieldNamed: 'SET_OWNER')).
	ocMapping := (aDescriptor newMapping: ManyToManyMapping)
		attributeName: #orderedCollection;
		join:
			(Join
				from: (self ownerTable fieldNamed: 'ID')
				to: (self linkTable fieldNamed: 'OWNER_ID')
				from: 'O'
				to: (self linkTable fieldNamed: 'TYPE')).
	ocMapping orderBy: [:each | (each getTable: self linkTable) getField: 'POSITION'].
	ocMapping writeTheOrderField.

	(aDescriptor newMapping: ManyToManyMapping)
		attributeName: #bag;
		join:
			(Join
				from: (self ownerTable fieldNamed: 'ID')
				to: (self linkTable fieldNamed: 'OWNER_ID')
				from: 'B'
				to: (self linkTable fieldNamed: 'TYPE')).
	(aDescriptor newMapping: ManyToManyMapping)
		attributeName: #sortedCollection;
		join:
			(Join
				from: (self ownerTable fieldNamed: 'ID')
				to: (self linkTable fieldNamed: 'OWNER_ID')
				from: 'S'
				to: (self linkTable fieldNamed: 'TYPE')).! !

GlorpTestDescriptorSystem subclass: #GlorpCompositeKeyDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpCompositeKeyDescriptorSystem commentStamp: '<historical>' prior: 0!
This is a system designed to test the use of composite keys. Its model is an email message system, where we have users, folders, keyed by name and owning user, and messages, keyed by a message id, folder name, and owning user.!

!GlorpCompositeKeyDescriptorSystem methodsFor: 'accessing' stamp: ''!
allTableNames
	^#('GR_USER' 'GR_FOLDER' 'GR_MESSAGE').! !

!GlorpCompositeKeyDescriptorSystem methodsFor: 'accessing' stamp: ''!
constructAllClasses
	^(super constructAllClasses)
		add: GlorpUser;
		add: GlorpFolder;
		add: GlorpMessage;
		yourself! !

!GlorpCompositeKeyDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpFolder: aDescriptor

	| table userMapping messageMapping |
	table := self tableNamed: 'GR_FOLDER'.
	aDescriptor table: table.

	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).

	userMapping := OneToOneMapping new
		attributeName: #user;
		referenceClass: GlorpUser.
"		join: (Join from: (table fieldNamed: 'USER_ID') to: (userTable fieldNamed: 'ID'))."
	aDescriptor addMapping: userMapping.
	messageMapping := OneToManyMapping new
		attributeName: #messages;
		referenceClass: GlorpMessage.
"		join: (Join 
			from: (table fieldNamed: 'USER_ID')
			to: (messageTable fieldNamed: 'USER_ID') 	
			from: (table fieldNamed: 'NAME')
			to: (messageTable fieldNamed: 'FOLDER_NAME'))."
	messageMapping beExclusive.
	aDescriptor addMapping: messageMapping.! !

!GlorpCompositeKeyDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpMessage: aDescriptor

	| table userMapping folderMapping |
	table := self tableNamed: 'GR_MESSAGE'.
	aDescriptor table: table.
	self tableNamed: 'GR_USER'.
	self tableNamed: 'GR_FOLDER'.

	aDescriptor addMapping: (DirectMapping from: #subject to: (table fieldNamed: 'SUBJECT')).
	aDescriptor addMapping: (DirectMapping from: #contents to: (table fieldNamed: 'CONTENTS')).

	userMapping := OneToOneMapping new
		attributeName: #user;
		referenceClass: GlorpUser.
"		mappingCriteria: (Join from: (table fieldNamed: 'USER_ID') to: (userTable fieldNamed: 'ID'))."
	aDescriptor addMapping: userMapping.
	folderMapping := OneToOneMapping new
		attributeName: #folder;
		referenceClass: GlorpFolder.
	"Let it figure this out for itself"
"		mappingCriteria: (Join 
			from: (table fieldNamed: 'USER_ID')
			to: (userTable fieldNamed: 'ID') 	
			from: (table fieldNamed: 'NAME')
			to: (folderTable fieldNamed: 'FOLDER_NAME'))."
	aDescriptor addMapping: folderMapping.! !

!GlorpCompositeKeyDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpUser: aDescriptor

	| table folderMapping |
	table := self tableNamed: 'GR_USER'.
	aDescriptor table: table.

	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).

	folderMapping := OneToManyMapping new
		attributeName: #folders;
		referenceClass: GlorpFolder.
"		mappingCriteria: 
						(Join
								from: (table fieldNamed: 'ID')
								to: (folderTable fieldNamed: 'USER_ID'))."
	folderMapping beExclusive.
	aDescriptor addMapping: folderMapping.! !

!GlorpCompositeKeyDescriptorSystem methodsFor: 'examples' stamp: ''!
example1
	| user1 user2 folder1 folder1a folder2 message1 message2 message3 |
	user1 := GlorpUser new id: 1; name: 'User One'.
	user2 := GlorpUser new id: 2; name: 'User Two'.
	folder1 := GlorpFolder new name: 'One'; user: user1.
	user1 folders add: folder1.
	folder1a := GlorpFolder new name: 'One-A'; user: user1.
	user1 folders add: folder1a.
	folder2 := GlorpFolder new name: 'Two'; user: user2.
	user2 folders add: folder2.
	
	message1 := GlorpMessage new subject: 'goes in folder 1'; user: user1; folder: folder1.
	folder1 messages add: message1.
	message2 := GlorpMessage new subject: 'also goes in folder 1'; user: user1; folder: folder1.
	folder1 messages add: message2.
	message3 := GlorpMessage new subject: 'goes in folder 2'; user: user2; folder: folder2.
	folder2 messages add: message3.
	^Array with: user1 with: user2.! !

!GlorpCompositeKeyDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGR_FOLDER: aTable 
	| userId |
	userId := aTable createFieldNamed: 'USER_ID' type: platform integer.
	userId bePrimaryKey.
	aTable addForeignKeyFrom: userId
		to: ((self tableNamed: 'GR_USER') fieldNamed: 'ID').
	(aTable createFieldNamed: 'NAME' type: (platform varChar: 50)) bePrimaryKey.! !

!GlorpCompositeKeyDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGR_MESSAGE: aTable 
	| userId folderName |
	userId := aTable createFieldNamed: 'USER_ID' type: platform integer.
	userId bePrimaryKey.
	folderName := aTable createFieldNamed: 'FOLDER_NAME' type: (platform varChar: 50).
	folderName bePrimaryKey.
	aTable addForeignKeyFrom: userId
		to: ((self tableNamed: 'GR_USER') fieldNamed: 'ID').

	aTable addForeignKeyFrom: userId
		to: ((self tableNamed: 'GR_FOLDER') fieldNamed: 'USER_ID')
		from: folderName
		to: ((self tableNamed: 'GR_FOLDER') fieldNamed: 'NAME').


	(aTable createFieldNamed: 'SUBJECT' type: (platform varChar: 50)) bePrimaryKey.
	aTable createFieldNamed: 'CONTENTS' type: (platform varChar: 100).! !

!GlorpCompositeKeyDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGR_USER: aTable 
	| |
	(aTable createFieldNamed: 'ID' type: platform integer)
		bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 50).! !

GlorpTestDescriptorSystem subclass: #GlorpDemoDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDemoDescriptorSystem class methodsFor: 'accessing' stamp: ''!
default
	Default isNil ifTrue: [Default := self new].
	^Default! !

!GlorpDemoDescriptorSystem methodsFor: 'other' stamp: ''!
allTableNames
	^#('GR_ADDRESS' 'PERSON' 'GR_CUSTOMER' 'BANK_TRANS' 'BANK_ACCT' 'CUSTOMER_ACCT_LINK' 'EMAIL_ADDRESS' 'STUFF' 'PASSENGER' 'AIRLINE' 'FREQUENT_FLYER' 'COMPRESSED_MONEY_TABLE' 'RESERVATION' 'ITINERARY' 'TRANSFORMED_TIME' 'FKCONTACT' 'FKADDRESS' 'GLORP_OWNER' 'GLORP_SLAVE' 'TREE_NODE' 'TREE_NODE_LINK' 'RECORD_WITH_UPDATE' 'GR_TRAVEL_AGENT').! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/other' stamp: ''!
classModelForDefaultableThing: aClassModel

	aClassModel newAttributeNamed: #id type: Integer.
	aClassModel newAttributeNamed: #boolean type: Boolean.
	aClassModel newAttributeNamed: #integer type: Integer.! !

!GlorpDemoDescriptorSystem methodsFor: 'class models/airline' stamp: ''!
classModelForGlorpAirline: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpBankAccount: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #accountNumber type: GlorpBankAccountNumber.
	aClassModel newAttributeNamed: #accountHolders collectionOf: GlorpCustomer.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpBankAccountNumber: aClassModel

	aClassModel newAttributeNamed: #bankCode.
	aClassModel newAttributeNamed: #branchNumber.
	aClassModel newAttributeNamed: #accountNumber.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpBankBranch: aClassModel
	aClassModel newAttributeNamed: #branchNumber.
	aClassModel newAttributeNamed: #accounts collectionOf: GlorpBankAccount.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpBankTransaction: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #owner type: GlorpCustomer.
	aClassModel newAttributeNamed: #amount type: GlorpMoney.
	aClassModel newAttributeNamed: #serviceCharge type: GlorpServiceCharge.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpCustomer: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #transactions collectionOf: GlorpBankTransaction.
	aClassModel newAttributeNamed: #accounts collectionOf: GlorpBankAccount.
	aClassModel newAttributeNamed: #accountsSortedById collectionOf: GlorpBankAccount.
	aClassModel newAttributeNamed: #accountsSortedByIdDescending collectionOf: GlorpBankAccount.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpEmailAddress: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #user.
	aClassModel newAttributeNamed: #host.! !

!GlorpDemoDescriptorSystem methodsFor: 'class models/airline' stamp: ''!
classModelForGlorpItinerary: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #reservation type: GlorpReservation.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpMoney: aClassModel

	aClassModel newAttributeNamed: #currency.
	aClassModel newAttributeNamed: #amount.! !

!GlorpDemoDescriptorSystem methodsFor: 'class models/airline' stamp: ''!
classModelForGlorpPassenger: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #frequentFlyerMiles.
	aClassModel newAttributeNamed: #airline type: GlorpAirline.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpPerson: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #address type: GlorpAddress.
	aClassModel newAttributeNamed: #emailAddresses collectionOf: GlorpEmailAddress.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/other' stamp: ''!
classModelForGlorpRecordWithUpdateTime: aClassModel

	aClassModel newAttributeNamed: #id type: Integer.
	aClassModel newAttributeNamed: #name type: String.
	aClassModel newAttributeNamed: #updateTime type: Timestamp.
	aClassModel newAttributeNamed: #updateSomeOtherThing type: String.! !

!GlorpDemoDescriptorSystem methodsFor: 'class models/airline' stamp: ''!
classModelForGlorpReservation: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #passengers collectionOf: GlorpPassenger.
	aClassModel newAttributeNamed: #passenger type: GlorpPassenger.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpServiceCharge: aClassModel

	aClassModel newAttributeNamed: #description.
	aClassModel newAttributeNamed: #amount type: GlorpMoney.! !

!GlorpDemoDescriptorSystem methodsFor: 'class models/airline' stamp: ''!
classModelForGlorpTravelAgency: aClassModel
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #agents collectionOf: GlorpTravelAgent.! !

!GlorpDemoDescriptorSystem methodsFor: 'class models/airline' stamp: ''!
classModelForGlorpTravelAgent: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #agency type: GlorpTravelAgency.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/tree nodes' stamp: ''!
classModelForGlorpTreeNode: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #children collectionOf: GlorpTreeNode.! !

!GlorpDemoDescriptorSystem methodsFor: 'other' stamp: ''!
constructAllClasses
	"This includes a list of classes as an example, but we don't actually use most of them, because the superclass method will figure it out if the descriptor methods are written in this class."
	^(super constructAllClasses) 
		add: GlorpAddress;
		yourself.
"	^(super constructAllClasses)
		add: GlorpPerson;
		add: GlorpAddress;
		add: GlorpCustomer;
		add: GlorpBankTransaction;
		add: GlorpBankAccount;
		add: GlorpMoney;
		add: GlorpCompressedMoney;
		add: GlorpServiceCharge;
		add: GlorpBankAccountNumber;
		add: GlorpEmailAddress;
		add: GlorpPassenger;
		add: GlorpAirline;
		add: GlorpReservation;
		add: GlorpItinerary;
		add: GlorpTransformedTime;
		add: GlorpBankBranch;
		add: GlorpOwner;
		add: GlorpSlave;
		add: GlorpTreeNode;
		add: GlorpRecordWithUpdateTime;
		add: GlorpTravelAgent;
		add: GlorpTravelAgency;
		yourself."! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/other' stamp: ''!
descriptorForDefaultableThing: aDescriptor
	| table |
	table := self tableNamed: 'DEFAULTABLE_THING'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping)
		from: #id
		to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping)
		from: #boolean
		to: (table fieldNamed: 'MY_BOOL').
	(aDescriptor newMapping: DirectMapping)
		from: #integer
		to: (table fieldNamed: 'MY_INT').! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/airline' stamp: ''!
descriptorForGlorpAirline: aDescriptor 
	aDescriptor table: (self tableNamed: 'AIRLINE').
	aDescriptor directMappingFor: #id.
	aDescriptor directMappingFor: #name.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpBankAccount: aDescriptor 

	| table |
	table := self tableNamed: 'BANK_ACCT'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id
				to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: ToManyMapping)
				attributeName: #accountHolders;
				useLinkTable.
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
				attributeName: #accountNumber.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpBankAccountNumber: aDescriptor

	| table |
	table := self tableNamed: 'BANK_ACCT'.
	aDescriptor table: table.
"	aDescriptor addMapping: ((DirectMapping from: #bankCode to: (table fieldNamed: 'BANK_CODE')) type: Integer)."
	(aDescriptor newMapping: DirectMapping) from: #bankCode to: (table fieldNamed: 'BANK_CODE').
	(aDescriptor newMapping: DirectMapping) from: #branchNumber to: (table fieldNamed: 'BRANCH_NO').
	(aDescriptor newMapping: DirectMapping) from: #accountNumber to: (table fieldNamed: 'ACCT_NO').! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpBankBranch: aDescriptor

	| table distinctBranchNumber mapping |
	table := self tableNamed: 'BANK_ACCT'.
	aDescriptor table: table.
	distinctBranchNumber := [:each | ((each getTable: table) getField: (table fieldNamed: 'BRANCH_NO')) distinct] asGlorpExpression.
	mapping := aDescriptor newMapping: DirectMapping.
	mapping from: #branchNumber to: distinctBranchNumber.
	mapping readOnly: true.

	aDescriptor keyFields: (Array with: distinctBranchNumber).

	(aDescriptor newMapping: ToManyMapping)
		attributeName: #accounts;
		readOnly: true;
		join: 
						(Join
								from: distinctBranchNumber
								to: (table fieldNamed: 'BRANCH_NO')).! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpBankTransaction: aDescriptor
	| table | 
	table := self tableNamed: 'BANK_TRANS'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id
				to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: OneToOneMapping)
				attributeName: #owner.
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
				attributeName: #amount;
				fieldTranslation: ((Join new)
							addSource: (table fieldNamed: 'AMT_AMT')
								target: ((self tableNamed: 'MONEY_IMAGINARY_TABLE') fieldNamed: 'AMOUNT');
							addSource: (table fieldNamed: 'AMT_CURR')
								target: ((self tableNamed: 'MONEY_IMAGINARY_TABLE') fieldNamed: 'CURRENCY');
							yourself).
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
				attributeName: #serviceCharge.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/other' stamp: ''!
descriptorForGlorpCompressedMoney: aDescriptor 
	| table currencyField amountField |
	table := self tableNamed: 'COMPRESSED_MONEY_TABLE'.
	aDescriptor 
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	currencyField := table fieldNamed: 'CURRENCY_NAME'.
	amountField := table fieldNamed: 'AMOUNT'.
	aDescriptor table: table.
	aDescriptor addMapping: (AdHocMapping 
				forAttribute: #array
				fromDb: 
					[:row :elementBuilder :context| 
					Array with: (elementBuilder valueOfField: (context translateField: currencyField) in: row)
						with: (elementBuilder valueOfField: (context translateField: amountField) in: row)]
				toDb: 
					[:rows :attribute :attributeRows | 
					(rows at: table) at: currencyField put: (attribute at: 1).
					(rows at: table) at: amountField put: (attribute at: 2)]
				mappingFields: (Array with: currencyField with: amountField)).
	"Note that position won't work if we have a join. We need to take the elementbuilder into account"
	^aDescriptor! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpCustomer: aDescriptor

	| table |
	table := self tableNamed: 'GR_CUSTOMER'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (table fieldNamed: 'NAME'); shouldProxy: true.
	(aDescriptor newMapping: ToManyMapping)
			attributeName: #transactions.
	(aDescriptor newMapping: ToManyMapping)
			attributeName: #accounts;
			useLinkTable.

	"Two additional relationships, there to test ordering within a mapping, where the order is determined by a field in the link table"
	(aDescriptor newMapping: ToManyMapping)
			attributeName: #accountsSortedById;
			readOnly: true;
			useLinkTable;
			orderBy: [:each | (each getTable: 'CUSTOMER_ACCT_LINK') getField: 'ACCT_ID'].
	(aDescriptor newMapping: ManyToManyMapping)
			attributeName: #accountsSortedByIdDescending;
			readOnly: true;
			useLinkTable;
			orderBy: [:each | ((each getTable: 'CUSTOMER_ACCT_LINK') getField: 'ACCT_ID') descending].! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpEmailAddress: aDescriptor

	| table |
	table := self tableNamed: 'EMAIL_ADDRESS'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #user to: (table fieldNamed: 'USER_NAME').
	(aDescriptor newMapping: DirectMapping) from: #host to: (table fieldNamed: 'HOST_NAME').! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/airline' stamp: ''!
descriptorForGlorpItinerary: aDescriptor 
	aDescriptor table: (self tableNamed: 'ITINERARY').
	aDescriptor directMappingFor: #id.
	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #reservation.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpMoney: aDescriptor

	| table |
	table := self tableNamed: 'MONEY_IMAGINARY_TABLE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #currency type: Symbol to: (table fieldNamed: 'CURRENCY').
	(aDescriptor newMapping: DirectMapping) from: #amount to: (table fieldNamed: 'AMOUNT').
	^aDescriptor.! !

!GlorpDemoDescriptorSystem methodsFor: 'owner' stamp: ''!
descriptorForGlorpOwner: aDescriptor
	| table slaveTable |
	table := self tableNamed: 'GLORP_OWNER'.
	slaveTable := self tableNamed: 'GLORP_SLAVE'.
	aDescriptor table: table.
	aDescriptor
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID'));
		addMapping:
				((OneToManyMapping new)
						attributeName: #slaves;
						referenceClass: GlorpSlave;
						collectionType: OrderedCollection;
						orderBy:
								[:each | (each getTable: 'GLORP_SLAVE') getField: 'POSITION_IN_OWNER'];
						writeTheOrderField;
						mappingCriteria:
								(Join
										from: (table fieldNamed: 'ID')
										to: (slaveTable fieldNamed: 'OWNER_ID')));
		addMapping:
				((OneToOneMapping new)
						attributeName: #slave;
						referenceClass: GlorpSlave;
						mappingCriteria:
								(Join
										from: (table fieldNamed: 'SLAVE_ID')
										to: (slaveTable fieldNamed: 'ID'))).! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/airline' stamp: ''!
descriptorForGlorpPassenger: aDescriptor 
	| passTable ffTable ffMiles airlineMapping |
	passTable := self tableNamed: 'PASSENGER'.
	ffTable := self tableNamed: 'FREQUENT_FLYER'.
	aDescriptor table: passTable.
	aDescriptor addTable: ffTable.
	aDescriptor addMultipleTableJoin: (	
		Join 
			from: (passTable fieldNamed: 'ID')
			to: (ffTable fieldNamed: 'ID')) beOuterJoin.
	(aDescriptor newMapping: DirectMapping) from: #id to: (passTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (passTable fieldNamed: 'NAME').

	ffMiles := aDescriptor newMapping: ConditionalMapping.
	ffMiles 
		forField: (ffTable fieldNamed: 'POINTS')
		attribute: [:object | object frequentFlyerPoints]
		if: [:x | x notNil]
		useMapping: ((ffMiles newMapping: DirectMapping) from: #frequentFlyerMiles to: (ffTable fieldNamed: 'POINTS')).
	ffMiles otherwise: (ffMiles newMapping: ConstantMapping).
	
	airlineMapping := aDescriptor newMapping: ConditionalMapping.
	airlineMapping
		forField: (ffTable fieldNamed: 'AIRLINE_ID')
		attribute: [:object | object airline]
		if: [:x | x notNil]
		useMapping: ((airlineMapping newMapping: OneToOneMapping) attributeName: #airline).
	airlineMapping otherwise: (airlineMapping newMapping: ConstantMapping).! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpPerson: aDescriptor

	| table |
	table := self tableNamed: 'PERSON'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (table fieldNamed: 'NAME').
	(aDescriptor newMapping: DirectMapping) fromPseudoVariable: #addressId to: (table fieldNamed: 'ADDRESS_ID').
	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #address.
	(aDescriptor newMapping: ToManyMapping)
			attributeName: #emailAddresses;
			orderBy: #id.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/other' stamp: ''!
descriptorForGlorpRecordWithUpdateTime: aDescriptor
	| table |
	table := self tableNamed: 'RECORD_WITH_UPDATE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping)
		from: #id
		to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping)
		from: #name
		to: (table fieldNamed: 'NAME').
	(aDescriptor newMapping: DirectMapping)
		from: #updateTime
		to: (table fieldNamed: 'UPDATE_TIME').
	(aDescriptor newMapping: DirectMapping)
		from: #updateSomeOtherThing
		to: (table fieldNamed: 'UPDATE_OTHER_THING').! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/airline' stamp: ''!
descriptorForGlorpReservation: aDescriptor 
	| table |
	table := self tableNamed: 'RESERVATION'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	"The res->passenger relationship is actually 1-1, but map it as both 1-1 and 1-many so that we can more easily verify that only one object comes back, i.e. that joins are being done correctly"
	(aDescriptor newMapping: OneToManyMapping)
			attributeName: #passengers.
	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #passenger.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpServiceCharge: aDescriptor 
	
	| table |
	table := self tableNamed: 'BANK_TRANS'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #description
				to: (table fieldNamed: 'SRVC_DESC').
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
				attributeName: #amount;
				fieldTranslation: ((Join new)
							addSource: (table fieldNamed: 'SRVC_AMT_AMT')
								target: ((self tableNamed: 'MONEY_IMAGINARY_TABLE') fieldNamed: 'AMOUNT');
							addSource: (table fieldNamed: 'SRVC_AMT_CURR')
								target: ((self tableNamed: 'MONEY_IMAGINARY_TABLE') fieldNamed: 'CURRENCY');
							yourself).! !

!GlorpDemoDescriptorSystem methodsFor: 'owner' stamp: ''!
descriptorForGlorpSlave: aDescriptor
	| table |
	table := self tableNamed: 'GLORP_SLAVE'.
	aDescriptor table: table.
	aDescriptor
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/other' stamp: ''!
descriptorForGlorpTransformedTime: aDescriptor 
	| table timeField |
	table := self tableNamed: 'TRANSFORMED_TIME'.
	aDescriptor 
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	timeField := table fieldNamed: 'TIMEFIELD'.
	aDescriptor table: table.
	aDescriptor addMapping: (AdHocMapping 
				forAttribute: #time
				fromDb: 
					[:row :elementBuilder :context| 
					Time fromSeconds: (elementBuilder valueOfField: (context translateField: timeField) in: row) ]
				toDb: 
					[:rows :attribute :attributeRows | 
					(rows at: table) at: timeField put: attribute asSeconds]
				mappingFields: (Array with: timeField)).
	"Note that position won't work if we have a join. We need to take the elementbuilder into account"
	^aDescriptor! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/airline' stamp: ''!
descriptorForGlorpTravelAgency: aDescriptor
	| table |
	table := self tableNamed: 'GR_TRAVEL_AGENT'.
	aDescriptor table: table.
	aDescriptor directMapping from: #name to: (table fieldNamed: 'AGENCY_NAME').
	aDescriptor toManyMapping
			attributeName: #agents;
			join: (Join from: (table fieldNamed: 'AGENCY_NAME') to: (table fieldNamed: 'AGENCY_NAME')).! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/airline' stamp: ''!
descriptorForGlorpTravelAgent: aDescriptor
	| table |
	table := self tableNamed: 'GR_TRAVEL_AGENT'.
	aDescriptor table: table.
	aDescriptor directMapping from: #id to: (table fieldNamed: 'ID').
	aDescriptor directMapping from: #name to: (table fieldNamed: 'AGENT_NAME').
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
			attributeName: #agency.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/tree nodes' stamp: ''!
descriptorForGlorpTreeNode: aDescriptor
	| table |
	table := self tableNamed: 'TREE_NODE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping)
		from: #id
		to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: ToManyMapping)
		attributeName: #children;
		useLinkTable;
		join:
				(Join
						from: (table fieldNamed: 'ID')
						to: ((self tableNamed: 'TREE_NODE_LINK') fieldNamed: 'PARENT')).! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAccountRow1

	| accountTable row |
	accountTable := self tableNamed: 'BANK_ACCT'.
	row := DatabaseRow newForTable: accountTable.
	row at: (accountTable fieldNamed: 'ID') put: 9874.
	row at: (accountTable fieldNamed: 'BANK_CODE') put: '1'.
	row at: (accountTable fieldNamed: 'BRANCH_NO') put: 2.
	row at: (accountTable fieldNamed: 'ACCT_NO') put: '3'.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAccountRow2

	| accountTable row |
	accountTable := self tableNamed: 'BANK_ACCT'.
	row := DatabaseRow newForTable: accountTable.
	row at: (accountTable fieldNamed: 'ID') put: 6.
	row at: (accountTable fieldNamed: 'BANK_CODE') put: '2'.
	row at: (accountTable fieldNamed: 'BRANCH_NO') put: 3.
	row at: (accountTable fieldNamed: 'ACCT_NO') put: '4'.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAccountRow3

	| accountTable row |
	accountTable := self tableNamed: 'BANK_ACCT'.
	row := DatabaseRow newForTable: accountTable.
	row at: (accountTable fieldNamed: 'ID') put: 22.
	row at: (accountTable fieldNamed: 'BANK_CODE') put: '2'.
	row at: (accountTable fieldNamed: 'BRANCH_NO') put: 712.
	row at: (accountTable fieldNamed: 'ACCT_NO') put: '5551212'.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAddressRow
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 123.
	row at: (addressTable fieldNamed: 'STREET') put: 'Paseo Montril'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '10185'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAddressRowForOrdering1
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 1.
	row at: (addressTable fieldNamed: 'STREET') put: 'Alpha'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '300'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAddressRowForOrdering2
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 2.
	row at: (addressTable fieldNamed: 'STREET') put: 'Beta'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '200'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAddressRowForOrdering3
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 3.
	row at: (addressTable fieldNamed: 'STREET') put: 'Gamma'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '100'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAddressRowForOrdering4
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 6.
	row at: (addressTable fieldNamed: 'STREET') put: 'Beta'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '120'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAddressRowForOrdering5
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 4.
	row at: (addressTable fieldNamed: 'STREET') put: 'Beta'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '130'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleAddressRowWithDifferentStreet
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 123.
	row at: (addressTable fieldNamed: 'STREET') put: 'Garden of the Gods'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '10185'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleBankTransactionRow
	| table row |
	table := self tableNamed: 'BANK_TRANS'.
	row := DatabaseRow newForTable: table.

	row atFieldNamed: 'ID' put: nil.
	row atFieldNamed: 'OWNER_ID' put: nil.
	row atFieldNamed: 'AMT_CURR' put: 'CDN'.
	row atFieldNamed: 'AMT_AMT' put: 7.
	row atFieldNamed: 'SRVC_DESC' put: 'additional overcharge'.
	row atFieldNamed: 'SRVC_AMT_CURR' put: 'USD'.
	row atFieldNamed: 'SRVC_AMT_AMT' put: 2.

	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleBankTransactionRow2
	| table row |
	table := self tableNamed: 'BANK_TRANS'.
	row := DatabaseRow newForTable: table.

	row atFieldNamed: 'ID' put: nil.
	row atFieldNamed: 'OWNER_ID' put: nil.
	row atFieldNamed: 'AMT_CURR' put: 'EUR'.
	row atFieldNamed: 'AMT_AMT' put: 45.
	row atFieldNamed: 'SRVC_DESC' put: 'deposit'.
	row atFieldNamed: 'SRVC_AMT_CURR' put: 'EUR'.
	row atFieldNamed: 'SRVC_AMT_AMT' put: 1.

	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCALinkRow1

	| linkTable row |
	linkTable := self tableNamed: 'CUSTOMER_ACCT_LINK'.
	row := DatabaseRow newForTable: linkTable.
	row at: (linkTable fieldNamed: 'ACCT_ID') put: 9874.
	row at: (linkTable fieldNamed: 'CUSTOMER_ID') put: 27.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCALinkRow2

	| linkTable row |
	linkTable := self tableNamed: 'CUSTOMER_ACCT_LINK'.
	row := DatabaseRow newForTable: linkTable.
	row at: (linkTable fieldNamed: 'ACCT_ID') put: 6.
	row at: (linkTable fieldNamed: 'CUSTOMER_ID') put: 27.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCALinkRow3

	| linkTable row |
	linkTable := self tableNamed: 'CUSTOMER_ACCT_LINK'.
	row := DatabaseRow newForTable: linkTable.
	row at: (linkTable fieldNamed: 'ACCT_ID') put: 22.
	row at: (linkTable fieldNamed: 'CUSTOMER_ID') put: 28.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCALinkRow4

	| linkTable row |
	linkTable := self tableNamed: 'CUSTOMER_ACCT_LINK'.
	row := DatabaseRow newForTable: linkTable.
	row at: (linkTable fieldNamed: 'ACCT_ID') put: 6.
	row at: (linkTable fieldNamed: 'CUSTOMER_ID') put: 28.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCompressedMoneyRow
	| table row |
	table := self  tableNamed: 'COMPRESSED_MONEY_TABLE'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'ID') put: 123.
	row at: (table fieldNamed: 'AMOUNT') put: 12.
	row at: (table fieldNamed: 'CURRENCY_NAME') put: 'CDN'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCompressedMoneyRow2
	| table row |
	table := self  tableNamed: 'COMPRESSED_MONEY_TABLE'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'ID') put: 124.
	row at: (table fieldNamed: 'AMOUNT') put: 15.
	row at: (table fieldNamed: 'CURRENCY_NAME') put: 'CDN'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomerRow1

	| customerTable row |
	customerTable := self tableNamed: 'GR_CUSTOMER'.
	row := DatabaseRow newForTable: customerTable.
	row at: (customerTable fieldNamed: 'ID') put: 27.
	row at: (customerTable fieldNamed: 'NAME') put: 'aCustomer'.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomerRow2

	| customerTable row |
	customerTable := self tableNamed: 'GR_CUSTOMER'.
	row := DatabaseRow newForTable: customerTable.
	row at: (customerTable fieldNamed: 'ID') put: 28.
	row at: (customerTable fieldNamed: 'NAME') put: 'anotherCustomer'.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleEmailAddressRow1

	| personTable row |
	personTable := self tableNamed: 'EMAIL_ADDRESS'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 42.
	row at: (personTable fieldNamed: 'USER_NAME') put: 'alan'.
	row at: (personTable fieldNamed: 'HOST_NAME') put: 'objectpeople.com'.
	row at: (personTable fieldNamed: 'PERSON_ID') put: 3.

	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleEmailAddressRow2

	| personTable row |
	personTable := self tableNamed: 'EMAIL_ADDRESS'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 54321.
	row at: (personTable fieldNamed: 'USER_NAME') put: 'johnson'.
	row at: (personTable fieldNamed: 'HOST_NAME') put: 'cs.uiuc.edu'.
	row at: (personTable fieldNamed: 'PERSON_ID') put: 3.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleFrequentFlyerRow
	| ffTable row |
	ffTable := self tableNamed: 'FREQUENT_FLYER'.
	row := DatabaseRow newForTable: ffTable.
	row at: (ffTable fieldNamed: 'ID') put: 1.
	row at: (ffTable fieldNamed: 'POINTS') put: 10000.
	row at: (ffTable fieldNamed: 'AIRLINE_ID') put: nil.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleFrequentFlyerRow2
	| ffTable row |
	ffTable := self tableNamed: 'FREQUENT_FLYER'.
	row := DatabaseRow newForTable: ffTable.
	row at: (ffTable fieldNamed: 'ID') put: 2.
	row at: (ffTable fieldNamed: 'POINTS') put: 7.
	row at: (ffTable fieldNamed: 'AIRLINE_ID') put: nil.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleModifiedAddressRow
	| addressTable row |
	addressTable := self  tableNamed: 'GR_ADDRESS'.
	row := DatabaseRow newForTable: addressTable.
	row at: (addressTable fieldNamed: 'ID') put: 123.
	row at: (addressTable fieldNamed: 'STREET') put: 'Something Else'.
	row at: (addressTable fieldNamed: 'HOUSE_NUM') put: '10185'.
	^row! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePassengerRow
	| passengerTable row |
	passengerTable := self tableNamed: 'PASSENGER'.
	row := DatabaseRow newForTable: passengerTable.
	row at: (passengerTable fieldNamed: 'ID') put: 1.
	row at: (passengerTable fieldNamed: 'NAME') put: 'Some Passenger'.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePassengerRow2
	| passengerTable row |
	passengerTable := self tableNamed: 'PASSENGER'.
	row := DatabaseRow newForTable: passengerTable.
	row at: (passengerTable fieldNamed: 'ID') put: 2.
	row at: (passengerTable fieldNamed: 'NAME') put: 'Another Passenger'.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePersonRow1

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 3.
	row at: (personTable fieldNamed: 'NAME') put: 'aPerson'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: 123.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePersonRow2

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 4.
	row at: (personTable fieldNamed: 'NAME') put: 'aPerson'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: nil.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePersonRowForOrdering1

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 86.
	row at: (personTable fieldNamed: 'NAME') put: 'person1'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: 2.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePersonRowForOrdering2

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 87.
	row at: (personTable fieldNamed: 'NAME') put: 'person2'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: 3.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePersonRowForOrdering3

	| personTable row |
	personTable := self tableNamed: 'PERSON'.
	row := DatabaseRow newForTable: personTable.
	row at: (personTable fieldNamed: 'ID') put: 88.
	row at: (personTable fieldNamed: 'NAME') put: 'person3'.
	row at: (personTable fieldNamed: 'ADDRESS_ID') put: 1.
	^row.! !

!GlorpDemoDescriptorSystem methodsFor: 'tables/airline' stamp: ''!
tableForAIRLINE: aTable

	(aTable createFieldNamed: 'ID' type: platform inMemorySequence) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20)! !

!GlorpDemoDescriptorSystem methodsFor: 'tables/airline' stamp: ''!
tableForAIRLINE_MEAL: aTable

	aTable createFieldNamed: 'ID' type: platform int4.
	aTable createFieldNamed: 'DESCR' type: (platform varChar: 20).
	aTable createFieldNamed: 'FLIGHT_ID' type: platform int4! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForBANK_ACCT: aTable

	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'BANK_CODE' type: (platform varChar: 10).
	aTable createFieldNamed: 'BRANCH_NO' type: (platform int4).
	aTable createFieldNamed: 'ACCT_NO' type: (platform varChar: 10).! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForBANK_TRANS: aTable 
	| ownerId |
	(aTable createFieldNamed: 'ID' type: platform serial)
		bePrimaryKey.
	ownerId := aTable createFieldNamed: 'OWNER_ID' type: platform int4.
	aTable addForeignKeyFrom: ownerId
		to: ((self tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
	aTable createFieldNamed: 'AMT_CURR' type: (platform varChar: 5).
	aTable createFieldNamed: 'AMT_AMT' type: platform int4.
	aTable createFieldNamed: 'SRVC_DESC' type: (platform varChar: 30).
	aTable createFieldNamed: 'SRVC_AMT_CURR' type: (platform varChar: 5).
	aTable createFieldNamed: 'SRVC_AMT_AMT' type: platform int4.! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForCOMPRESSED_MONEY_TABLE: aTable
	(aTable createFieldNamed: 'ID' type: platform int4)
		bePrimaryKey.
	aTable createFieldNamed: 'CURRENCY_NAME' type: (platform varChar: 20).
	aTable createFieldNamed: 'AMOUNT' type: platform int4.! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForCUSTOMER_ACCT_LINK: aTable

	| customerId accountId |
	customerId := (aTable createFieldNamed: 'CUSTOMER_ID' type: platform int4).
	aTable addForeignKeyFrom: customerId to: ((self tableNamed: 'GR_CUSTOMER') fieldNamed: 'ID').
	accountId := aTable createFieldNamed: 'ACCT_ID' type: platform int4.
	aTable addForeignKeyFrom: accountId to: ((self tableNamed: 'BANK_ACCT') fieldNamed: 'ID').
	aTable addIndexForFields: (Array with: customerId with: accountId).! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/other' stamp: ''!
tableForDEFAULTABLE_THING: aTable 
	| boolean converter |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	boolean := aTable createFieldNamed: 'MY_BOOL' type: (platform boolean).
	"We want a default value for this field, but we can't just use a boolean, because in some database we can represent that directly, but in others it will need to be converted to an integer. And likewise we can't just use an integer, so we need to do an explicit conversion."
	converter := boolean converterForStType: Boolean.
	boolean defaultValue: (converter convert: false toDatabaseRepresentationAs: boolean type).
	(aTable createFieldNamed: 'MY_INT' type: (platform int4)) defaultValue: 4.! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForEMAIL_ADDRESS: aTable

	| personId |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	(aTable createFieldNamed: 'USER_NAME' type: (platform varChar: 20)) beIndexed.
	aTable createFieldNamed: 'HOST_NAME' type: (platform varChar: 20).
	personId := aTable createFieldNamed: 'PERSON_ID' type: platform int4.
	personId beIndexed.
	(aTable addForeignKeyFrom: personId to: ((self tableNamed: 'PERSON') fieldNamed: 'ID')).! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForFKADDRESS: aTable 
	| contact |
	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	contact := aTable createFieldNamed: 'CONTACT_ID' type: platform int4.
	aTable addForeignKeyFrom: contact
		to: ((self tableNamed: 'FKCONTACT') fieldNamed: 'ID')! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForFKCONTACT: aTable 
	| address |
	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	address := aTable createFieldNamed: 'ADDRESS_ID' type: platform int4.
	aTable addForeignKeyFrom: address
		to: ((self tableNamed: 'FKADDRESS') fieldNamed: 'ID')! !

!GlorpDemoDescriptorSystem methodsFor: 'tables/airline' stamp: ''!
tableForFLIGHT: aTable

	aTable name: 'FLIGHT'.
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'FLIGHT_NUM' type: platform int4! !

!GlorpDemoDescriptorSystem methodsFor: 'tables/airline' stamp: ''!
tableForFLIGHT_PASS: aTable

	| flight pass airline |
	aTable name: 'FLIGHT_PASS'.
	flight := aTable createFieldNamed: 'FLIGHT_ID' type: platform int4.
	aTable addForeignKeyFrom: flight
		to: ((self tableNamed: 'FLIGHT') fieldNamed: 'ID').
	pass := aTable createFieldNamed: 'PASS_ID' type: platform int4.
	aTable addForeignKeyFrom: pass
		to: ((self tableNamed: 'PASSENGER') fieldNamed: 'ID').
	airline := aTable createFieldNamed: 'AIRLINE_ID' type: platform int4.
	aTable addForeignKeyFrom: airline
		to: ((self tableNamed: 'AIRLINE') fieldNamed: 'ID').! !

!GlorpDemoDescriptorSystem methodsFor: 'tables/airline' stamp: ''!
tableForFREQUENT_FLYER: aTable 
	| airlineId |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'POINTS' type: platform int4.
	airlineId := aTable createFieldNamed: 'AIRLINE_ID' type: platform int4.
	aTable addForeignKeyFrom: airlineId to: ((self tableNamed: 'AIRLINE') fieldNamed: 'ID').! !

!GlorpDemoDescriptorSystem methodsFor: 'owner' stamp: ''!
tableForGLORP_OWNER: aTable
	| field |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	field := aTable createFieldNamed: 'SLAVE_ID' type: platform int4.
	aTable
		addForeignKeyFrom: field
		to: ((self tableNamed: 'GLORP_SLAVE') fieldNamed: 'ID').! !

!GlorpDemoDescriptorSystem methodsFor: 'owner' stamp: ''!
tableForGLORP_SLAVE: aTable
	| field constraint |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'POSITION_IN_OWNER' type: platform int4.
	field := aTable createFieldNamed: 'OWNER_ID' type: platform int4.
	"Don't actually create this constraint because it causes a cycle that can confuse our sorting mechanism."
	"constraint := aTable
		addForeignKeyFrom: field
		to: ((self tableNamed: 'GLORP_OWNER') fieldNamed: 'ID').
	constraint shouldCreateInDatabase: false."! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGR_ADDRESS: aTable 

	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'STREET' type: (platform varChar: 20).
	aTable createFieldNamed: 'HOUSE_NUM' type: (platform varChar: 20)! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGR_CUSTOMER: aTable 
	(aTable createFieldNamed: 'ID' type: platform int4)
		bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20)! !

!GlorpDemoDescriptorSystem methodsFor: 'tables/airline' stamp: ''!
tableForGR_TRAVEL_AGENT: aTable 

	(aTable createFieldNamed: 'ID' type: platform int4)
		bePrimaryKey.
	aTable createFieldNamed: 'AGENT_NAME' type: (platform varchar: 50).
	aTable createFieldNamed: 'AGENCY_NAME' type: (platform varchar: 50).! !

!GlorpDemoDescriptorSystem methodsFor: 'tables/airline' stamp: ''!
tableForITINERARY: aTable 

	| res |
	(aTable createFieldNamed: 'ID' type: platform serial)
		bePrimaryKey.
	res := aTable createFieldNamed: 'RES_ID' type: (platform int4).
	aTable addForeignKeyFrom: res
		to: ((self tableNamed: 'RESERVATION') fieldNamed: 'ID').! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForMONEY_IMAGINARY_TABLE: aTable

	aTable isImaginary: true.
	aTable createFieldNamed: 'CURRENCY' type: (platform varChar: 5).
	aTable createFieldNamed: 'AMOUNT' type: platform int4! !

!GlorpDemoDescriptorSystem methodsFor: 'tables/airline' stamp: ''!
tableForPASSENGER: aTable
	| |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20)! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForPERSON: aTable
	| addrId|
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).
	addrId := aTable createFieldNamed: 'ADDRESS_ID' type: platform int4.
	aTable addForeignKeyFrom: addrId to: ((self tableNamed: 'GR_ADDRESS') fieldNamed: 'ID').! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/other' stamp: ''!
tableForRECORD_WITH_UPDATE: aTable 
	| |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	(aTable createFieldNamed: 'NAME' type: (platform varchar: 20)).
	aTable createFieldNamed: 'UPDATE_TIME' type: (platform versionFieldFor: platform timestamp) generateOverExistingValues.
	"OK, it's not much of a comment, we just print the current time. Note that this field is completely unmapped"
	aTable createFieldNamed: 'UPDATE_COMMENT' type: (platform generatorFor: (platform varchar: 20) withBlock: [Time now printString]) generateOverExistingValues.
	"But this one *is* mapped"
	aTable createFieldNamed: 'UPDATE_OTHER_THING' type: (platform generatorFor: (platform varchar: 20) withBlock: [Time now printString]) generateOverExistingValues.! !

!GlorpDemoDescriptorSystem methodsFor: 'tables/airline' stamp: ''!
tableForRESERVATION: aTable 

	| pass |
	(aTable createFieldNamed: 'ID' type: platform int4)
		bePrimaryKey.
	pass := aTable createFieldNamed: 'PASS_ID' type: (platform int4).
	aTable addForeignKeyFrom: pass
		to: ((self tableNamed: 'PASSENGER') fieldNamed: 'ID').! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForSTUFF: aTable 

	(aTable createFieldNamed: 'ID' type: platform inMemorySequence)
		bePrimaryKey.
	aTable createFieldNamed: 'THING' type: (platform varChar: 20)! !

!GlorpDemoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForTRANSFORMED_TIME: aTable
	(aTable createFieldNamed: 'ID' type: platform int4)
		bePrimaryKey.
	aTable createFieldNamed: 'TIMEFIELD' type: platform int4.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/tree nodes' stamp: ''!
tableForTREE_NODE: aTable

	| |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.! !

!GlorpDemoDescriptorSystem methodsFor: 'descriptors/tree nodes' stamp: ''!
tableForTREE_NODE_LINK: aTable

	| parent child |
	parent := (aTable createFieldNamed: 'PARENT' type: platform int4) bePrimaryKey.
	(aTable addForeignKeyFrom: parent
		to: ((self tableNamed: 'TREE_NODE') fieldNamed: 'ID')) "shouldCreateInDatabase: false"  .
	child := (aTable createFieldNamed: 'CHILD' type: platform int4) bePrimaryKey.
	(aTable addForeignKeyFrom: child
		to: ((self tableNamed: 'TREE_NODE') fieldNamed: 'ID')) "shouldCreateInDatabase: false".! !

!GlorpDemoDescriptorSystem methodsFor: 'test support' stamp: ''!
testIDLessThan5: aValue
	"A test method, used for conditional mapping tests"
	^aValue < 5.! !

!GlorpDemoDescriptorSystem methodsFor: 'test support' stamp: ''!
testIDMoreThan5: aValue
	"A test method, used for conditional mapping tests"
	^aValue > 5.! !

!GlorpDemoDescriptorSystem methodsFor: 'support' stamp: ''!
writeCustomer1Rows
	| linkRow2 id2 id1 accountRow1 customerRow accountRow2 linkRow1 |
	customerRow := self exampleCustomerRow1.
	accountRow1 := self exampleAccountRow1.
	accountRow2 := self exampleAccountRow2.
	linkRow1 := self exampleCALinkRow1.
	linkRow2 := self exampleCALinkRow2.
	session writeRow: customerRow.
	session writeRow: accountRow1.
	session writeRow: accountRow2.
	session writeRow: linkRow1.
	session writeRow: linkRow2.
	id1 := accountRow1 at: (accountRow1 table fieldNamed: 'ID').
	id2 := accountRow2 at: (accountRow2 table fieldNamed: 'ID').
	^Array with: id1 with: id2.! !

!GlorpDemoDescriptorSystem methodsFor: 'support' stamp: ''!
writeCustomerWithDualAccountOwners
	| linkRow2 id2 id1 accountRow1 accountRow2 linkRow1 customer1Row customer2Row linkRow4 |
	customer1Row := self exampleCustomerRow1.
	customer2Row := self exampleCustomerRow2.
	accountRow1 := self exampleAccountRow1.
	accountRow2 := self exampleAccountRow2.
	linkRow1 := self exampleCALinkRow1.
	linkRow2 := self exampleCALinkRow2.
	linkRow4 := self exampleCALinkRow4.
	session writeRow: customer1Row.
	session writeRow: customer2Row.
	session writeRow: accountRow1.
	session writeRow: accountRow2.
	session writeRow: linkRow1.
	session writeRow: linkRow2.
	session writeRow: linkRow4.
	id1 := accountRow1 at: (accountRow1 table fieldNamed: 'ID').
	id2 := accountRow2 at: (accountRow2 table fieldNamed: 'ID').
	^Array with: id1 with: id2.! !

GlorpDemoDescriptorSystem subclass: #GlorpLinkTableAnywhereDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpLinkTableAnywhereDescriptorSystem commentStamp: '<historical>' prior: 0!
This is the same as the demo descriptor system, but with small modifications to use link tables in different places.

!

!GlorpLinkTableAnywhereDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
classModelForGlorpCustomer: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #transactions collectionOf: GlorpBankTransaction.
	"Note: accounts is a 1-1 in this model"
	aClassModel newAttributeNamed: #accounts type: GlorpBankAccount.
	aClassModel newAttributeNamed: #accountsSortedById collectionOf: GlorpBankAccount.
	aClassModel newAttributeNamed: #accountsSortedByIdDescending collectionOf: GlorpBankAccount.! !

!GlorpLinkTableAnywhereDescriptorSystem methodsFor: 'descriptors/bank' stamp: ''!
descriptorForGlorpCustomer: aDescriptor

	| table |
	table := self tableNamed: 'GR_CUSTOMER'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (table fieldNamed: 'NAME').
	(aDescriptor newMapping: OneToManyMapping)
			attributeName: #transactions;
			referenceClass: GlorpBankTransaction.
	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #accounts;
			referenceClass: GlorpBankAccount;
			usesLinkTable: true.! !

GlorpTestDescriptorSystem subclass: #GlorpDescriptorSystemWithNamespaces
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDescriptorSystemWithNamespaces methodsFor: 'accessing' stamp: ''!
allTableNames
	^#()! !

!GlorpDescriptorSystemWithNamespaces methodsFor: 'accessing' stamp: ''!
constructAllClasses
	^(super constructAllClasses)
		add: GlorpTestClassInNamespace;
		yourself! !

!GlorpDescriptorSystemWithNamespaces methodsFor: 'accessing' stamp: ''!
descriptorForGlorpTestClassInNamespace: aDescriptor 
	^aDescriptor! !

GlorpTestDescriptorSystem subclass: #GlorpDocumentDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpDocumentDescriptorSystem commentStamp: '<historical>' prior: 0!
This is a test descriptor system for documents with attachments that have their attachment bytes in a separate table, so reading them can be deferred. Neither attachments nor attachmentBytes have references to their containing document, but do have primary keys as foreign keys to it. When writing, this exhibited a problem that if you didn't read the bytes, then the attachment was written properly, because it could get its documentId primary key component from the proxy to the attachmentBytes. But if you had read the attachmentBytes, then it tried to get the value from the attachmentBytes object, which didn't know about that value. So it couldn't find the primary key of the attachment, and therefore assumed it was new.!

!GlorpDocumentDescriptorSystem class methodsFor: 'none' stamp: ''!
doTest
	"   AADescriptor doTest    "

	| glorpSession doc |
	glorpSession := self connectToDatabase.

	"Try #1 -- this works as advertized"
	doc := self readDocumentAndAttachmentsFrom: glorpSession.
	"Something changed? Should not."
	glorpSession modify: doc in: [].
	doc := nil.
	glorpSession accessor logout.
	glorpSession := self connectToDatabase.

	"Try #2 -- this makes Glorp believe the attachment was changed"
	doc := self readDocumentAndAttachmentsFrom: glorpSession.
	doc attachments do: 
			[:ea | 
			"Force attachmentBytes to be instanciated"
			Transcript nextPutAll: '----> ', ea attachmentBytes content; cr].
	"Something changed? Should not, but does."
	"Glorp now tries to update the attachment AND save the attachmentBytes again. Boom"
	glorpSession modify: doc in: [].
	glorpSession accessor logout! !

!GlorpDocumentDescriptorSystem class methodsFor: 'none' stamp: ''!
readDocumentAndAttachmentsFrom: glorpSession 
	| doc |
	doc := glorpSession readOneOf: GlorpDocument.
	Transcript
		cr;
		nextPutAll: '---> Read document ' , doc id printString;
		cr.
	doc attachments do: 
			[:att | 
			Transcript
				cr;
				nextPutAll: '------> Attachment ' , att index printString , ' ' 
							, att mimeType;
				cr].
	^doc! !

!GlorpDocumentDescriptorSystem methodsFor: 'glorp' stamp: ''!
allTableNames
	^#('document' 'attachment' 'attachmentbytes')! !

!GlorpDocumentDescriptorSystem methodsFor: 'glorp' stamp: ''!
constructAllClasses
	^(super constructAllClasses)
		add: GlorpDocument;
		add: GlorpAttachment;
		add: GlorpAttachmentBytes;
		yourself! !

!GlorpDocumentDescriptorSystem methodsFor: 'glorp' stamp: ''!
tableForATTACHMENT: aTable 
	| d |
	d := (aTable createFieldNamed: 'document_id' type: platform int4) 
				bePrimaryKey.
	(aTable createFieldNamed: 'indexInDocument' type: platform int) bePrimaryKey.
	aTable createFieldNamed: 'mimetype' type: (platform varchar: 32).

	"FK Constraints"
	aTable addForeignKeyFrom: d
		to: ((self tableNamed: 'document') fieldNamed: 'id')! !

!GlorpDocumentDescriptorSystem methodsFor: 'glorp' stamp: ''!
tableForATTACHMENTBYTES: aTable 
	| d i |
	d := (aTable createFieldNamed: 'attachment_document_id'
				type: platform int4) bePrimaryKey.
	i := (aTable createFieldNamed: 'attachment_index' type: platform int) 
				bePrimaryKey.
	aTable createFieldNamed: 'content' type: platform text.

	"FK"
	aTable 
		addForeignKeyFrom: d
		to: ((self tableNamed: 'attachment') fieldNamed: 'document_id')
		from: i
		to: ((self tableNamed: 'attachment') fieldNamed: 'indexInDocument')! !

!GlorpDocumentDescriptorSystem methodsFor: 'glorp' stamp: ''!
tableForDOCUMENT: aTable 
	(aTable createFieldNamed: 'id' type: platform sequence) bePrimaryKey.
	aTable createFieldNamed: 'whatever' type: (platform integer).! !

GlorpTestDescriptorSystem subclass: #GlorpEncyclopediaDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpEncyclopediaDescriptorSystem commentStamp: '<historical>' prior: 0!
This is a set of test descriptors for dealing with different variations of dictionary mappings.!

!GlorpEncyclopediaDescriptorSystem methodsFor: 'other' stamp: ''!
allTableNames
	^#('ENCYC' 'ENCYC_ENTRY' 'ENCYC_ENTRY_LINK' 'IMAGINARY_MEDIA_TYPE' 'GR_PUBLISHER' 'PUBLISHER_TITLE' 'PUB_TITLES_STOCK' 'PUB_TITLE_LINK' 'PUBLISHER_TITLE2' 'PUB_EMP_LINK' 'PUBLISHER_EMP').! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
classModelForGlorpEncyclopedia: aClassModel 
 
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name. 
	aClassModel newAttributeNamed: #entries dictionaryFrom: String to: GlorpEncyclopediaEntry.
	"A slightly contrived way of having a mapping to a type of entries that don't use a link table."
	aClassModel newAttributeNamed: #biographicalEntries dictionaryFrom: String to: GlorpEncyclopediaBioEntry.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
classModelForGlorpEncyclopediaBioEntry: aClassModel 
	aClassModel newAttributeNamed: #owner type: GlorpEncyclopedia.
	aClassModel newAttributeNamed: #title.
	aClassModel newAttributeNamed: #text.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
classModelForGlorpEncyclopediaEntry: aClassModel 
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #title.
	aClassModel newAttributeNamed: #text.
	aClassModel newAttributeNamed: #mediaType type: GlorpMediaType! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
classModelForGlorpMediaType: aClassModel 
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #mimeType! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
classModelForGlorpPublisher: aClassModel 
 
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name. 
	aClassModel newAttributeNamed: #titles collection: Array of: String.
	aClassModel newAttributeNamed: #titlesWithNumberInStock dictionaryFrom: String to: Integer.
	aClassModel newAttributeNamed: #employees dictionaryFrom: String to: GlorpPublisherEmployee.
	aClassModel newAttributeNamed: #differentBunchOfEmployeesById dictionaryFrom: Integer to: GlorpPublisherEmployee.

"	aClassModel newAttributeNamed: #titlesWithNumberInStockLink collection: Dictionary of: Integer."! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
classModelForGlorpPublisherEmployee: aClassModel 
 
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name. 
	aClassModel newAttributeNamed: #position.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'other' stamp: ''!
constructAllClasses
	^(super constructAllClasses)
		add: GlorpEncyclopedia;
		add: GlorpMediaType;
		add: GlorpEncyclopediaEntry;
		add: GlorpPublisher;
		add: GlorpPublisherEmployee;
		yourself! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpEncyclopedia: aDescriptor 
	| table linkTable |
	table := self tableNamed: 'ENCYC'.
	linkTable := self tableNamed: 'ENCYC_ENTRY_LINK'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id
		to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name
		to: (table fieldNamed: 'NAME').
	(aDescriptor newMapping: BasicDictionaryMapping)
		attributeName: #entries;
		referenceClass: GlorpEncyclopediaEntry;
		keyField: (linkTable fieldNamed: 'REFERENCE_NO');
		linkFields: (Array 
					with: (linkTable fieldNamed: 'ENCYC_ENTRY_ID')).
	(aDescriptor newMapping: DictionaryMapping)
		attributeName: #biographicalEntries;
		referenceClass: GlorpEncyclopediaBioEntry;
		usesLinkTable: false;
		beExclusive;
		keyField: ((self tableNamed: 'ENCYC_BIO') fieldNamed: 'TITLE').! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpEncyclopediaBioEntry: aDescriptor 
	| entryTable |
	entryTable := self tableNamed: 'ENCYC_BIO'.
	aDescriptor table: entryTable.
	"It's important that we have a mapping for the owner here, because we're expecting these entries to be deleted, and we can't delete them without knowing the primary keys."
	(aDescriptor newMapping: OneToOneMapping)
		attributeName: #owner;
		beReadOnly.
	(aDescriptor newMapping: DirectMapping) from: #title
		to: (entryTable fieldNamed: 'TITLE').
	(aDescriptor newMapping: DirectMapping) from: #text
		to: (entryTable fieldNamed: 'ENTRY_TEXT').! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpEncyclopediaEntry: aDescriptor 
	| entryTable |
	entryTable := self tableNamed: 'ENCYC_ENTRY'.
	aDescriptor table: entryTable.
	(aDescriptor newMapping: DirectMapping) from: #id
		to: (entryTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #title
		to: (entryTable fieldNamed: 'TITLE').
	(aDescriptor newMapping: DirectMapping) from: #text
		to: (entryTable fieldNamed: 'ENTRY_TEXT').
	(aDescriptor newMapping: EmbeddedValueOneToOneMapping)
		attributeName: #mediaType;
		referenceClass: GlorpMediaType;
		fieldTranslation: ((Join new)
					addSource: (entryTable fieldNamed: 'MEDIA_TYPE_NAME')
						target: ((self tableNamed: 'IMAGINARY_MEDIA_TYPE') 
								fieldNamed: 'MEDIA_TYPE_NAME');
					addSource: (entryTable fieldNamed: 'MEDIA_TYPE_MIME')
						target: ((self tableNamed: 'IMAGINARY_MEDIA_TYPE') 
								fieldNamed: 'MEDIA_TYPE_MIME');
					yourself).
	^aDescriptor! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpMediaType: aDescriptor 
	| table |
	table := self tableNamed: 'IMAGINARY_MEDIA_TYPE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #name
		to: (table fieldNamed: 'MEDIA_TYPE_NAME').
	(aDescriptor newMapping: DirectMapping) from: #mimeType
		to: (table fieldNamed: 'MEDIA_TYPE_MIME')! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpPublisher: aDescriptor 
	| table titleTable titleStockTable linkTable |
	table := self tableNamed: 'GR_PUBLISHER'.
	titleTable := self tableNamed: 'PUBLISHER_TITLE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id
		to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name
		to: (table fieldNamed: 'NAME').
	(aDescriptor newMapping: DirectToManyMapping)
		attributeName: #titles;
		resultField: (titleTable fieldNamed: 'TITLE');
		writeTheOrderField;
		orderBy: [:each | (each getTable: titleTable) getField: 'POSITION'];
		join: (Join from: (table fieldNamed: 'ID') to: (titleTable fieldNamed: 'PUBLISHER_ID')).
	titleStockTable := self tableNamed: 'PUB_TITLES_STOCK'.
	(aDescriptor newMapping: DictionaryMapping)
		attributeName: #titlesWithNumberInStock;
		usesLinkTable: false;
		keyField: (titleStockTable fieldNamed: 'TITLE');
		valueField: (titleStockTable fieldNamed: 'STOCK');
		join: (Join from: (table fieldNamed: 'ID') to: (titleStockTable fieldNamed: 'PUBLISHER_ID')).
	linkTable := self tableNamed: 'PUB_EMP_LINK'.
	(aDescriptor newMapping: DictionaryMapping)
		attributeName: #employees;
		usesLinkTable: true;
		keyField: ((self tableNamed: 'PUBLISHER_EMP' ) fieldNamed: 'JOBTITLE');
		join: (Join 
			from: (table fieldNamed: 'ID')
			to: (linkTable fieldNamed: 'PUBLISHER_ID')
			from: 'A'
			to: (linkTable fieldNamed: 'FOO')).
	(aDescriptor newMapping: DictionaryMapping)
		attributeName: #differentBunchOfEmployeesById;
		usesLinkTable: true;
		join: (Join 
			from: (table fieldNamed: 'ID')
			to: (linkTable fieldNamed: 'PUBLISHER_ID')
			from: 'B'
			to: (linkTable fieldNamed: 'FOO'));
		keyField: ((self tableNamed: 'PUB_EMP_LINK') fieldNamed: 'EMP_ID').
		
"	titleStockLinkTable := self tableNamed: 'PUB_TITLE_LINK'.
	titleTable2 := self tableNamed: 'PUBLISHER_TITLE2'.
	(aDescriptor newMapping: DictionaryMapping)
		attributeName: #titlesWithNumberInStockLink;
		usesLinkTable: true;
		keyField: (titleTable2 fieldNamed: 'TITLE');
		valueField: (titleStockLinkTable fieldNamed: 'STOCK');
		linkTableJoin: (Join from: (table fieldNamed: 'ID') to: (titleStockLinkTable fieldNamed: 'PUBLISHER_ID'))."! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpPublisherEmployee: aDescriptor 
	| table |
	table := self tableNamed: 'PUBLISHER_EMP'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id
		to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name
		to: (table fieldNamed: 'NAME').
	(aDescriptor newMapping: DirectMapping) from: #jobTitle
		to: (table fieldNamed: 'JOBTITLE').! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleEncyclopedias
	"self new exampleEncyclopedias"
	| encyc1 entry1 entry2 entry3 encyc2 encyc3 |
	encyc1 := GlorpEncyclopedia new id: 3; name: 'The Canadian Encyclopedia'.
	entry1 := self exampleEntry1.
	entry2 := self exampleEntry2.
	entry3 := self exampleEntry3.
	encyc1 articleAt: '12345' put: entry1.
	encyc1 articleAt: 'abcde' put: entry2.
	encyc1 articleAt: 'unique' put: entry3.
	encyc2 := GlorpEncyclopedia new id: 4; name: 'The Plagiarists Compendium'.
	encyc2 articleAt: '54321' put: entry1.
	encyc2 articleAt: 'edcba' put: entry2.
	encyc3 := GlorpEncyclopedia new id: 5; name: 'The Very Frugal Dictionary'.
	^Array with: encyc1 with: encyc2 with: encyc3.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleEntry1
	^GlorpEncyclopediaEntry new 
		id: 'one';
		title: 'All About Ants';
		text: 'Ants are, like, these little tiny creatures. Gross.'.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleEntry2
	^GlorpEncyclopediaEntry new 
		id: 'two';
		title: 'Computer Programming Languages';
		text: 'There are many different computer programming languages. The best one is Smalltalk.'.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleEntry3
	^GlorpEncyclopediaEntry new 
		id: 'three';
		title: 'Vehicular Fish Decay';
		text: 'Do you know what happens if you leave a fish too long in an elevator? Well heres a clue. Fish is biodegradable. That means it rots.'.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePublisher1
	"Put in the copies to avoid an issue with reverting immutable objects."
	^GlorpPublisher new 
		id: 1;
		name: 'Cambridge';
		titles: (Array with: 'Mastering ENVY/Developer' copy with: 'Principia Mathematica' copy with:  'Metadata-Based Persistence for Dummies' copy);
		titlesWithNumberInStock: (Dictionary new
			at:  'Mastering ENVY/Developer' put: 2;
			at: 'Principia Mathematica' put: 2;
			at: 'Metadata-Based Persistence for Dummies' put: 2;
			yourself);
		employees: (Dictionary new
			at: 'CEO' put: (GlorpPublisherEmployee new id: 1; name: 'The Boss'; jobTitle: 'CEO');
			at: 'Editor' put: (GlorpPublisherEmployee new id: 2; name: 'Somebody'; jobTitle: 'Editor');
			yourself);
		differentBunchOfEmployeesById: (Dictionary new
			at: 3 put: (GlorpPublisherEmployee new id: 3; name: 'Contractor 1'; jobTitle: 'Contractor');
			at: 4 put: (GlorpPublisherEmployee new id: 4; name: 'Contractor 2'; jobTitle: 'Contractor');
			yourself);
		yourself.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePublisher2
	^GlorpPublisher new 
		id: 2;
		name: 'Harper Collins';
		titles: (Array with: 'A book' copy);
		titlesWithNumberInStock: (Dictionary new
			at:  'A book' copy put: 200;
			yourself).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePublisher3
	^GlorpPublisher new 
		id: 3;
		name: 'A New Publishing company';
		titles: #().! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'examples' stamp: ''!
examplePublishers
	^Array with: self examplePublisher1 with: self examplePublisher2 with: self examplePublisher3.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForENCYC: aTable 

	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 50).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForENCYC_BIO: aTable 

	| encycId |
	(aTable createFieldNamed: 'TITLE' type: (platform varChar: 50)) bePrimaryKey.
	encycId := aTable createFieldNamed: 'ENCYC_ID' type: platform integer.
	encycId bePrimaryKey.
	aTable addForeignKeyFrom: encycId to: ((self tableNamed: 'ENCYC') fieldNamed: 'ID').
	aTable createFieldNamed: 'ENTRY_TEXT' type: (platform varChar: 50).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForENCYC_ENTRY: aTable 

	(aTable createFieldNamed: 'ID' type: (platform varChar: 50)) bePrimaryKey.
	aTable createFieldNamed: 'TITLE' type: (platform varChar: 50).
	aTable createFieldNamed: 'ENTRY_TEXT' type: (platform varChar: 200).
	aTable createFieldNamed: 'MEDIA_TYPE_NAME' type: (platform varChar: 100).
	aTable createFieldNamed: 'MEDIA_TYPE_MIME' type: (platform varChar: 100).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForENCYC_ENTRY_LINK: aTable 

	| encycId entryId |
	encycId := aTable createFieldNamed: 'ENCYC_ID' type: platform int4.
	encycId bePrimaryKey.
	aTable addForeignKeyFrom: encycId to: ((self tableNamed: 'ENCYC') fieldNamed: 'ID').

	entryId := aTable createFieldNamed: 'ENCYC_ENTRY_ID' type: (platform varChar: 50).
	aTable addForeignKeyFrom: entryId to: ((self tableNamed: 'ENCYC_ENTRY') fieldNamed: 'ID').
	(aTable createFieldNamed: 'REFERENCE_NO' type: (platform varChar: 10)) bePrimaryKey.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGR_PUBLISHER: aTable 

	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 50).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForIMAGINARY_MEDIA_TYPE: aTable 
	aTable isImaginary: true.
  
	aTable createFieldNamed: 'MEDIA_TYPE_NAME' type: (platform varChar: 100).
	aTable createFieldNamed: 'MEDIA_TYPE_MIME' type: (platform varChar: 100).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForPUBLISHER_EMP: aTable 

	| |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 50).	
	aTable createFieldNamed: 'JOBTITLE' type: (platform varChar: 50).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForPUBLISHER_TITLE2: aTable 

	| |
	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	aTable createFieldNamed: 'TITLE' type: (platform varChar: 50).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForPUBLISHER_TITLE: aTable 

	| pubId |
	pubId := aTable createFieldNamed: 'PUBLISHER_ID' type: platform int4.
	aTable addForeignKeyFrom: pubId  to: ((self tableNamed: 'GR_PUBLISHER') fieldNamed: 'ID').
	aTable createFieldNamed: 'POSITION' type: (platform int4).	
	aTable createFieldNamed: 'TITLE' type: (platform varChar: 50).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForPUB_EMP_LINK: aTable 

	| pubId titleId |
	pubId := aTable createFieldNamed: 'PUBLISHER_ID' type: platform int4.
	aTable addForeignKeyFrom: pubId  to: ((self tableNamed: 'GR_PUBLISHER') fieldNamed: 'ID').
	titleId := aTable createFieldNamed: 'EMP_ID' type: platform int4.
	aTable addForeignKeyFrom: titleId  to: ((self tableNamed: 'PUBLISHER_EMP') fieldNamed: 'ID').
	aTable createFieldNamed: 'FOO' type: (platform varchar: 5).! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForPUB_TITLES_STOCK: aTable 

	| pubId |
	pubId := aTable createFieldNamed: 'PUBLISHER_ID' type: platform int4.
	aTable addForeignKeyFrom: pubId  to: ((self tableNamed: 'GR_PUBLISHER') fieldNamed: 'ID').
	aTable createFieldNamed: 'TITLE' type: (platform varChar: 50).
	aTable createFieldNamed: 'STOCK' type: platform int4.! !

!GlorpEncyclopediaDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForPUB_TITLE_LINK: aTable 

	| pubId titleId |
	pubId := aTable createFieldNamed: 'PUBLISHER_ID' type: platform int4.
	aTable addForeignKeyFrom: pubId  to: ((self tableNamed: 'GR_PUBLISHER') fieldNamed: 'ID').
	titleId := aTable createFieldNamed: 'TITLE_ID' type: platform int4.
	aTable addForeignKeyFrom: titleId  to: ((self tableNamed: 'PUBLISHER_TITLE2') fieldNamed: 'ID').
	aTable createFieldNamed: 'STOCK' type: platform int4.! !

GlorpTestDescriptorSystem subclass: #GlorpInheritanceDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpInheritanceDescriptorSystem methodsFor: 'misc' stamp: ''!
allTableNames

	^#('EMPLOYEE' 'OFFICE' 'PERISHABLE_ITEM' 'NONPERISHABLE_ITEM' 'UNASSEMBLED_ITEM' 'WORKING_STIFF' 'POULTRY' 'TAX' 'WAREHOUSE' 'WAREHOUSE_ITEM_LINK'  'GLORP_IMAGE' 'GLORP_TAG' 'GLORP_TAGS').! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
classModelForGlorpImage: aClassModel 
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #location.
	aClassModel newAttributeNamed: #tags collectionOf: GlorpTag.! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
classModelForGlorpTag: aClassModel 
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #key.
	aClassModel newAttributeNamed: #label! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
classModelForGlorpTaggableObject: aClassModel! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
classModelForGlorpWarehouse: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #address.
	aClassModel newAttributeNamed: #currentItem type: GlorpInventoryItem.
	aClassModel newAttributeNamed: #items collectionOf: GlorpInventoryItem.! !

!GlorpInheritanceDescriptorSystem methodsFor: 'misc' stamp: ''!
constructAllClasses
	^(super constructAllClasses)
		add: GlorpOffice;
		add: GlorpEmployee;
		add: GlorpManager;
		add: GlorpRegionalManager;
		add: GlorpLineWorker;
		add: GlorpInventoryItem;
		add: GlorpPerishableItem;
		add: GlorpNonperishableItem;
		add: GlorpUnassembledItem;
		add: GlorpWorkingStiff;
		add: GlorpPoultry;
		add: GlorpTax;
		add: GlorpBasicTax;
		add: GlorpSurcharge;
		add: GlorpWarehouse;
		add: GlorpTag;
		add: GlorpTaggableObject;
		add: GlorpImage;
		yourself! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpEmployee: aDescriptor
	| table |
	table := self tableNamed: 'EMPLOYEE'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	(self typeResolverFor: GlorpEmployee) register: aDescriptor keyedBy: 'E' field: (table fieldNamed: 'EMPLOYEE_TYPE').
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
descriptorForGlorpImage: aDescriptor 
	| table |
	table := self tableNamed: 'GLORP_IMAGE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping)
		from: #id
		to: (table fieldNamed: 'id').
	(aDescriptor newMapping: DirectMapping)
		from: #location
		to: (table fieldNamed: 'location').
	(aDescriptor newMapping: ManyToManyMapping) attributeName: #tags;
		 referenceClass: GlorpTag;
		
		join: (Join
				from: (table fieldNamed: 'id')
				to: ((self tableNamed: 'IMAGETAGS')
						fieldNamed: 'obj_id')).
	(self typeResolverFor: GlorpTaggableObject)
		register: aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpInventoryItem: aDescriptor

	(self typeResolverFor: GlorpInventoryItem) register: aDescriptor abstract: true.
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpLineWorker: aDescriptor
	| table |
	table := self tableNamed: 'EMPLOYEE'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	(self typeResolverFor: GlorpEmployee) register: aDescriptor keyedBy: 'W' field: (table fieldNamed: 'EMPLOYEE_TYPE').
	aDescriptor addMapping: (DirectMapping from: #productionLine to: (table fieldNamed: 'PRODUCTION_LINE')).
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpManager: aDescriptor
	| table |
	table := self tableNamed: 'EMPLOYEE'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping: (DirectMapping from: #branch to: (table fieldNamed: 'BRANCH')).
	(self typeResolverFor: GlorpEmployee) register: aDescriptor keyedBy: 'M' field: (table fieldNamed: 'EMPLOYEE_TYPE').
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpNonperishableItem: aDescriptor

	| table |
	table := self tableNamed: 'NONPERISHABLE_ITEM'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping: (DirectMapping from: #serialNumber to: (table fieldNamed: 'SERIAL_NUMBER')).
	(self typeResolverFor: GlorpInventoryItem) register: aDescriptor.
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpOffice: aDescriptor
	| table |
	table := self tableNamed: 'OFFICE'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (OneToManyMapping new
			attributeName: #employees;
			referenceClass: GlorpEmployee;
			join: (Join 
				from: (table fieldNamed: 'ID')
				to: ((self tableNamed: 'EMPLOYEE') fieldNamed: 'OFFICE_ID'))).
	aDescriptor addMapping: (OneToOneMapping new
		attributeName: #employeeOfMonth;
		referenceClass: GlorpEmployee;
		join: (Join
				from: (table fieldNamed: 'EMPLOYEE_OF_MONTH')
				to: ((self tableNamed: 'EMPLOYEE') fieldNamed: 'ID'))).
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpPerishableItem: aDescriptor

	| table |
	table := self tableNamed: 'PERISHABLE_ITEM'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping: (DirectMapping from: #age to: (table fieldNamed: 'AGE')).
	(self typeResolverFor: GlorpInventoryItem) register: aDescriptor.
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpPoultry: aDescriptor
	"Poultry does not participate in the InventoryItem heirarchy (ie it will not be retrieved when asking for an InventoryItem)"
	| table |
	table := self tableNamed: 'POULTRY'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping: (DirectMapping from: #age to: (table fieldNamed: 'AGE')).
	aDescriptor addMapping: (DirectMapping from: #featherColor to: (table fieldNamed: 'FEATHER_COLOR')).
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpRegionalManager: aDescriptor
	| table |
	table := self tableNamed: 'EMPLOYEE'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping: (DirectMapping from: #branch to: (table fieldNamed: 'BRANCH')).
	aDescriptor addMapping: (DirectMapping from: #region to: (table fieldNamed: 'REGION')).
	(self typeResolverFor: GlorpEmployee) register: aDescriptor keyedBy: 'R' field: (table fieldNamed: 'EMPLOYEE_TYPE').
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
descriptorForGlorpTag: aDescriptor 
	| table |
	table := self tableNamed: 'GLORP_TAG'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping)
		from: #id
		to: (table fieldNamed: 'id').
	(aDescriptor newMapping: DirectMapping)
		from: #key
		to: (table fieldNamed: 'key').
	(aDescriptor newMapping: DirectMapping)
		from: #label
		to: (table fieldNamed: 'label').! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
descriptorForGlorpTaggableObject: aDescriptor 
	(self typeResolverFor: GlorpTaggableObject)
		register: aDescriptor
		abstract: true! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpUnassembledItem: aDescriptor

	| table |
	table := self tableNamed: 'UNASSEMBLED_ITEM'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping: (DirectMapping from: #serialNumber to: (table fieldNamed: 'SERIAL_NUMBER')).
	aDescriptor addMapping: (DirectMapping from: #assemblyCost to: (table fieldNamed: 'ASSEM_COST')).
	(self typeResolverFor: GlorpInventoryItem) register: aDescriptor.
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpWarehouse: aDescriptor
	| table linkTable |
	table := self tableNamed: 'WAREHOUSE'.
	aDescriptor table: table.
	aDescriptor
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor
		addMapping:
			(DirectMapping from: #address to: (table fieldNamed: 'ADDRESS')).
	(aDescriptor newMapping: OneToOneMapping) 
		attributeName: #currentItem;
		join: (self polymorphicJoinToItemFromField: (table fieldNamed: 'CURRENT_ITEM_ID')).
		
	linkTable := self tableNamed: 'WAREHOUSE_ITEM_LINK'.
	(aDescriptor newMapping: ManyToManyMapping)
		attributeName: #items;
		join:
				(Join
						from: (table fieldNamed: 'ID')
						to: (linkTable fieldNamed: 'WAREHOUSE_ID'));
		reverseJoin: (self polymorphicJoinToItemFromField: (linkTable fieldNamed: 'ITEM_ID')).
	^aDescriptor.! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
descriptorForGlorpWorkingStiff: aDescriptor
	"Working stiff does not participate in the Employee type mapping scheme (it uses its own table)"
	| table |
	table := self tableNamed: 'WORKING_STIFF'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	^aDescriptor! !

!GlorpInheritanceDescriptorSystem methodsFor: 'descriptors/employees' stamp: ''!
polymorphicJoinToItemFromField: aField
	"Return a polymorphic join to the GlorpInventoryItem subclasses from the given field."
	| polymorphicJoin |
	polymorphicJoin := VariableJoin new.
	(Array with: GlorpNonperishableItem with: GlorpPerishableItem with: GlorpUnassembledItem)
		do:
			[:eachClass | 
			polymorphicJoin
				addJoinFor: eachClass
				as:
					(Join
						from: aField
						to: ((self descriptorFor: eachClass) primaryTable fieldNamed: 'ID'))].
	^polymorphicJoin! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForEMPLOYEE: aTable
	| officeId |
	aTable name: 'EMPLOYEE'.
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).
	officeId := aTable createFieldNamed: 'OFFICE_ID' type: platform int4.
	aTable addForeignKeyFrom: officeId
		to: ((self tableNamed: 'OFFICE') fieldNamed: 'ID').
	aTable createFieldNamed: 'EMPLOYEE_TYPE' type: (platform varChar: 20).
	aTable createFieldNamed: 'BRANCH' type: (platform varChar: 20).
	aTable createFieldNamed: 'REGION' type: (platform varChar: 20).
	aTable createFieldNamed: 'PRODUCTION_LINE' type: (platform varChar: 20).! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
tableForGLORP_IMAGE: aTable 
	(aTable createFieldNamed: 'id' type: platform sequence) bePrimaryKey.
	aTable
		createFieldNamed: 'location'
		type: (platform varChar: 250).! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
tableForGLORP_TAG: aTable 
	(aTable createFieldNamed: 'id' type: platform sequence) bePrimaryKey.
	aTable
		createFieldNamed: 'key'
		type: (platform varChar: 200).
	aTable
		createFieldNamed: 'label'
		type: (platform varChar: 200).! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
tableForGLORP_TAGS: aTable 
	| tagKey tagObjKey |
	tagKey := aTable createFieldNamed: 'tag_id' type: platform int4.
	aTable
		addForeignKeyFrom: tagKey
		to: ((self tableNamed: 'GLORP_TAG')
				fieldNamed: 'id').
	tagObjKey := aTable createFieldNamed: 'obj_id' type: platform int4.
	aTable
		addForeignKeyFrom: tagObjKey
		to: ((self tableNamed: 'GLORP_IMAGE')
				fieldNamed: 'id')! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
tableForIMAGETAGS: aTable 
	| tagKey tagObjKey |
	tagKey := aTable createFieldNamed: 'tag_id' type: platform int4.
	aTable
		addForeignKeyFrom: tagKey
		to: ((self tableNamed: 'GLORP_TAG')
				fieldNamed: 'id').
	tagObjKey := aTable createFieldNamed: 'obj_id' type: platform int4.
	aTable
		addForeignKeyFrom: tagObjKey
		to: ((self tableNamed: 'GLORP_IMAGE')
				fieldNamed: 'id')! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForNONPERISHABLE_ITEM: aTable
	aTable name: 'NONPERISHABLE_ITEM'.
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).
	aTable createFieldNamed: 'SERIAL_NUMBER' type: platform int4.! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForOFFICE: aTable
	| empOfMonth |
	aTable name: 'OFFICE'.
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	empOfMonth := aTable createFieldNamed: 'EMPLOYEE_OF_MONTH' type: platform int4.
	"Cheat by removing this, because it makes a cycle that we can't really handle yet"
	"aTable addForeignKeyFrom: empOfMonth
		to: ((self tableNamed: 'EMPLOYEE') fieldNamed: 'ID')."! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForPERISHABLE_ITEM: aTable
	aTable name: 'PERISHABLE_ITEM'.
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).
	aTable createFieldNamed: 'AGE' type: platform int4.! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForPOULTRY: aTable
	aTable name: 'POULTRY'.
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).
	aTable createFieldNamed: 'AGE' type: platform int4.
	aTable createFieldNamed: 'FEATHER_COLOR' type: (platform varChar: 20).! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForTAX: aTable
	| superTaxId |
	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: platform text.
	aTable createFieldNamed: 'TYPE' type: (platform varchar: 16).
	superTaxId := aTable createFieldNamed: 'TAX_ID' type: platform int4.
	aTable addForeignKeyFrom: superTaxId to: ((self tableNamed: 'TAX') fieldNamed: 'ID').! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForUNASSEMBLED_ITEM: aTable

	aTable name: 'UNASSEMBLED_ITEM'.
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).
	aTable createFieldNamed: 'SERIAL_NUMBER' type: platform int4.
	aTable createFieldNamed: 'ASSEM_COST' type: platform int4.! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForWAREHOUSE: aTable

	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	aTable createFieldNamed: 'ADDRESS' type: (platform varChar: 50).
	aTable createFieldNamed: 'CURRENT_ITEM_ID' type: platform int4.! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForWAREHOUSE_ITEM_LINK: aTable

	| warehouseId |
	warehouseId := aTable createFieldNamed: 'WAREHOUSE_ID' type: platform int4.
	aTable addForeignKeyFrom: warehouseId to: ((self tableNamed: 'WAREHOUSE') fieldNamed: 'ID').
	aTable createFieldNamed: 'ITEM_ID' type: platform int4.! !

!GlorpInheritanceDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForWORKING_STIFF: aTable
	aTable name: 'WORKING_STIFF'.
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).! !

!GlorpInheritanceDescriptorSystem methodsFor: 'type resolvers' stamp: ''!
typeResolverForGlorpInventoryItem
	^HorizontalTypeResolver forRootClass: GlorpInventoryItem.! !

!GlorpInheritanceDescriptorSystem methodsFor: 'as yet unclassified' stamp: ''!
typeResolverForGlorpTaggableObject
   ^(HorizontalTypeResolver forRootClass: GlorpTaggableObject).! !

GlorpTestDescriptorSystem subclass: #GlorpLockingDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpLockingDescriptorSystem methodsFor: 'other' stamp: ''!
allTableNames

	^#('BOOK' 'BOOK_CUSTOMER' 'CUSTOMER_BOOK_LINK').! !

!GlorpLockingDescriptorSystem methodsFor: 'examples' stamp: ''!
bookExample1
	^(GlorpBook new)
		title: 'Mastering ENVY/Developer';
		author: 'Knight/Pelrine/Cho';
		description:
				'Absolutely the best book written on ENVY/Developer by a set of authors all living on different continents';
		copiesInStock: 2000;
		version: 1.! !

!GlorpLockingDescriptorSystem methodsFor: 'examples' stamp: ''!
bookExample2
	^(GlorpBook new)
		title: 'The Bad Beginning';
		author: 'Lemony Snicket';
		description: 'Very fine description of the EJB standards process';
		copiesInStock: 3;
		version: 5.! !

!GlorpLockingDescriptorSystem methodsFor: 'other' stamp: ''!
constructAllClasses
	^(super constructAllClasses) 
		add: GlorpBook;
		add: GlorpBookstoreCustomer;
		yourself.! !

!GlorpLockingDescriptorSystem methodsFor: 'examples' stamp: ''!
customerExample1
	^(GlorpBookstoreCustomer new)
		accountNumber: '123456789ABCDEF';
		name: 'Aunt Colof'.! !

!GlorpLockingDescriptorSystem methodsFor: 'examples' stamp: ''!
customerExample2
	^(GlorpBookstoreCustomer new)
		accountNumber: '7';
		name: 'Veronica Filbert Davidson'.! !

!GlorpLockingDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpBook: aDescriptor

	| table |
	table := self tableNamed: 'BOOK'.
	aDescriptor table: table.
	aDescriptor addMapping: (	
		DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (	
		DirectMapping from: #title to: (table fieldNamed: 'TITLE')).
	aDescriptor addMapping: (	
		DirectMapping from: #copiesInStock to: (table fieldNamed: 'COPIES_IN_STOCK')).
	aDescriptor addMapping: (	
		DirectMapping from: #version to: (table fieldNamed: 'VERSION')).! !

!GlorpLockingDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpBookstoreCustomer: aDescriptor

	| table |
	table := self tableNamed: 'BOOK_CUSTOMER'.
	aDescriptor table: table.
	aDescriptor addMapping: (	
		DirectMapping from: #accountNumber to: (table fieldNamed: 'ACCOUNT_NUMBER')).
	aDescriptor addMapping: (	
		DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping: (	
		ManyToManyMapping new
			attributeName: #booksOnOrder;
			referenceClass: GlorpBook).! !

!GlorpLockingDescriptorSystem methodsFor: 'examples' stamp: ''!
example1
	"Return an example start set of books, customers and orders."

	| cust1 cust2 book1 book2 |
	cust1 := self customerExample1.
	cust2 := self customerExample2.
	book1 := self bookExample1.
	book2 := self bookExample2.
	^Array
		with: book1
		with: book2
		with: cust1
		with: cust2.! !

!GlorpLockingDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForBOOK: aTable

	(aTable createFieldNamed: 'ID' type: platform serial) bePrimaryKey.
	aTable createFieldNamed: 'TITLE' type: (platform varChar: 100).
	aTable createFieldNamed: 'DESCRIPTION' type: (platform varChar: 255).
	aTable createFieldNamed: 'COPIES_IN_STOCK' type: (platform int4).
	(aTable createFieldNamed: 'VERSION' type: (platform versionFieldFor: platform int4)) beLockKey.! !

!GlorpLockingDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForBOOK_CUSTOMER: aTable

	(aTable createFieldNamed: 'ACCOUNT_NUMBER' type: (platform varChar: 15)) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 100).! !

!GlorpLockingDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForCUSTOMER_BOOK_LINK: aTable

	| custKey bookKey |
	custKey := aTable createFieldNamed: 'CUSTOMER_ACCT_NUM' type: (platform varChar: 15).
	aTable addForeignKeyFrom: custKey
		to: ((self tableNamed: 'BOOK_CUSTOMER') fieldNamed: 'ACCOUNT_NUMBER').
	bookKey := aTable createFieldNamed: 'BOOK_ID' type: (platform int4).
		aTable addForeignKeyFrom: bookKey
		to: ((self tableNamed: 'BOOK') fieldNamed: 'ID').! !

GlorpTestDescriptorSystem subclass: #GlorpVariableTypeTestSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpVariableTypeTestSystem commentStamp: '<historical>' prior: 0!
This is a system for testing variable relationships. That is, where we have a relationship that might be one type of object, or might be another, depending on some criteria. The objects are not related by inheritance. Our example is a relationship to images, where if the images are small we retrieve them directly, but if they are larger we only retrieve minimal information about them.3!

!GlorpVariableTypeTestSystem methodsFor: 'classes' stamp: ''!
classModelForGlorpGallery: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #name.
	aClassModel newAttributeNamed: #images collectionOf: GlorpImageFile.
	"Not a real attribute, but used in our conditional mapping."
	aClassModel newAttributeNamed: #imagesLink collectionOf: GlorpImageLink.! !

!GlorpVariableTypeTestSystem methodsFor: 'classes' stamp: ''!
classModelForGlorpImageFile: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #title.
	aClassModel newAttributeNamed: #bits type: ByteArray.
	(aClassModel newAttributeNamed: #thumbnailBits type: ByteArray) useDirectAccess: false.! !

!GlorpVariableTypeTestSystem methodsFor: 'classes' stamp: ''!
classModelForGlorpImageLink: aClassModel
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #title.
	aClassModel newAttributeNamed: #thumbnailBits type: ByteArray.
	aClassModel newAttributeNamed: #fullImage type: GlorpImageFile.! !

!GlorpVariableTypeTestSystem methodsFor: 'classes' stamp: ''!
descriptorForGlorpGallery: aDescriptor
	
	| table linkTable imagesConditional |
	table := self tableNamed: 'GALLERY'.
	linkTable := self tableNamed: 'GALLERY_LINK'.
	aDescriptor table: table.
	aDescriptor directMappingFor: #id.
	aDescriptor directMappingFor: #name.
	imagesConditional := (aDescriptor newMapping: ConditionalToManyMapping)
		attributeName: #images;
		join: Join new.
	imagesConditional
		forField: (linkTable fieldNamed: 'LARGE')
		attribute: [:object | object isLarge]
		if: [:x | x = true]
		useMapping:
			((imagesConditional newMapping: ToManyMapping)
				attributeName: #imagesLink;
				useLinkTable;
				join:
						(Join
								from: (table fieldNamed: 'ID')
								to: (linkTable fieldNamed: 'GALLERY_ID')
								from: 'T'
								to: (linkTable fieldNamed: 'LARGE'))).
	imagesConditional
		otherwise:
			((imagesConditional newMapping: ToManyMapping)
				attributeName: #images;
				useLinkTable;
				join:
						(Join
								from: (table fieldNamed: 'ID')
								to: (linkTable fieldNamed: 'GALLERY_ID')
								from: 'F'
								to: (linkTable fieldNamed: 'LARGE'))).! !

!GlorpVariableTypeTestSystem methodsFor: 'classes' stamp: ''!
descriptorForGlorpImageFile: aDescriptor
	
	aDescriptor table: 'GLORP_IMAGE_FILE'.
	aDescriptor directMappingFor: #id.
	aDescriptor directMappingFor: #title.
	aDescriptor directMappingFor: #bits.
	aDescriptor directMapping from: #thumbnailBits to: 'THUMBNAIL_BITS'.! !

!GlorpVariableTypeTestSystem methodsFor: 'classes' stamp: ''!
descriptorForGlorpImageLink: aDescriptor
	
	aDescriptor table: 'GLORP_IMAGE_FILE'.
	aDescriptor directMappingFor: #id.
	aDescriptor directMappingFor: #title.
	aDescriptor directMapping from: #thumbnailBits to: 'THUMBNAIL_BITS'.
	"This is really joining to our own table to get the full version of ourselves"
	aDescriptor oneToOneMapping 
		attributeName: #fullImage;
		join: (Join from: (aDescriptor table fieldNamed: 'ID') to: (aDescriptor table fieldNamed: 'ID')).! !

!GlorpVariableTypeTestSystem methodsFor: 'classes' stamp: ''!
tableForGALLERY: aTable
	aTable createFieldNamed: 'ID' type: platform serial.
	aTable createFieldNamed: 'NAME' type: platform text.! !

!GlorpVariableTypeTestSystem methodsFor: 'classes' stamp: ''!
tableForGALLERY_LINK: aTable
	| galleryId imageId |
	galleryId := aTable createFieldNamed: 'GALLERY_ID' type: platform integer.
	aTable addForeignKeyFrom: galleryId to: ((self tableNamed: 'GALLERY') fieldNamed: 'ID').
	imageId := aTable createFieldNamed: 'IMAGE_ID' type: platform integer.
	aTable addForeignKeyFrom: imageId to: ((self tableNamed: 'GLORP_IMAGE_FILE') fieldNamed: 'ID').
	"Avoid booleans here because they aren't portable"
	aTable createFieldNamed: 'LARGE' type: (platform char: 1).! !

!GlorpVariableTypeTestSystem methodsFor: 'classes' stamp: ''!
tableForGLORP_IMAGE_FILE: aTable
	aTable createFieldNamed: 'ID' type: platform serial.
	aTable createFieldNamed: 'TITLE' type: platform text.
	aTable createFieldNamed: 'BITS' type: platform blob.
	aTable createFieldNamed: 'THUMBNAIL_BITS' type: platform blob.! !

GlorpTestDescriptorSystem subclass: #GlorpVideoDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpVideoDescriptorSystem commentStamp: '<historical>' prior: 0!
This descriptor system represents a video rental/sales store which sells both online and in a physical store. The interesting bit is that there is no customer table, but there is a customer object. In the database, orders and credit status are linked directly to the store. We synthesize a customer object out of those relationships.!

!GlorpVideoDescriptorSystem methodsFor: 'accessing' stamp: ''!
allTableNames
	^#('VIDEO_STORE' 'VIDEO_PURCHASE' 'VIDEO_RENTAL' 'VIDEO_PURCHASE_LINK' 'VIDEO_CUSTOMER' 'VIDEO_CREDIT_STATUS').! !

!GlorpVideoDescriptorSystem methodsFor: 'class models' stamp: ''!
classModelForGlorpVideoCreditStatus: aClassModel
	aClassModel newAttributeNamed: #customer type: GlorpVideoCustomer.
	aClassModel newAttributeNamed: #store type: GlorpVideoStore.
	aClassModel newAttributeNamed: #balanceOwing type: Number.
	aClassModel newAttributeNamed: #dateOwing type: Date.
	aClassModel newAttributeNamed: #comments type: String.! !

!GlorpVideoDescriptorSystem methodsFor: 'class models' stamp: ''!
classModelForGlorpVideoCustomer: aClassModel
	aClassModel newAttributeNamed: #name type: String.
	aClassModel newAttributeNamed: #dateOfBirth type: Date.
	aClassModel newAttributeNamed: #onlinePurchases collectionOf: GlorpVideoPurchase.
	aClassModel newAttributeNamed: #inStorePurchases collectionOf: GlorpVideoPurchase.
	aClassModel newAttributeNamed: #inStoreRentals collectionOf: GlorpVideoRental.	
	aClassModel newAttributeNamed: #store type: GlorpVideoStore.
	aClassModel newAttributeNamed: #creditStatus type: GlorpVideoCreditStatus.! !

!GlorpVideoDescriptorSystem methodsFor: 'class models' stamp: ''!
classModelForGlorpVideoPurchase: aClassModel
	aClassModel newAttributeNamed: #id type: Integer.
	aClassModel newAttributeNamed: #title type: String.
	aClassModel newAttributeNamed: #price type: Number.
	aClassModel newAttributeNamed: #inStore type: Boolean.
	aClassModel newAttributeNamed: #customer type: GlorpVideoCustomer.	
	aClassModel newAttributeNamed: #store type: GlorpVideoStore.! !

!GlorpVideoDescriptorSystem methodsFor: 'class models' stamp: ''!
classModelForGlorpVideoRental: aClassModel
	aClassModel newAttributeNamed: #id type: Integer.
	aClassModel newAttributeNamed: #title type: String.
	aClassModel newAttributeNamed: #price type: Number.
	aClassModel newAttributeNamed: #dueDate type: Date.
	aClassModel newAttributeNamed: #customer type: GlorpVideoCustomer.
	aClassModel newAttributeNamed: #store type: GlorpVideoStore.! !

!GlorpVideoDescriptorSystem methodsFor: 'class models' stamp: ''!
classModelForGlorpVideoStore: aClassModel
	aClassModel newAttributeNamed: #name type: String.
	aClassModel newAttributeNamed: #purchases collectionOf: GlorpVideoPurchase.
	aClassModel newAttributeNamed: #customers collectionOf: GlorpVideoCustomer.! !

!GlorpVideoDescriptorSystem methodsFor: 'accessing' stamp: ''!
constructAllClasses
	^(super constructAllClasses)
		add: GlorpVideoStore;
		add: GlorpVideoCreditStatus;
		add: GlorpVideoCustomer;
		add: GlorpVideoPurchase;
		add: GlorpVideoRental;
		yourself! !

!GlorpVideoDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpVideoCreditStatus: aDescriptor

	| table |
	table := self tableNamed: 'VIDEO_CREDIT_STATUS'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #balanceOwing to: (table fieldNamed: 'BALANCE').
	(aDescriptor newMapping: DirectMapping) from: #dateOwing to: (table fieldNamed: 'DATE_OWING').
	(aDescriptor newMapping: DirectMapping) from: #comments to: (table fieldNamed: 'COMMENTS').

	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpVideoCustomer: aDescriptor

	| table linkTable purchaseTable |
	table := self tableNamed: 'VIDEO_CUSTOMER'.
	linkTable := self tableNamed: 'VIDEO_PURCHASE_LINK'.
	purchaseTable := self tableNamed: 'VIDEO_PURCHASE'.

	aDescriptor table: table.

	(aDescriptor newMapping: DirectMapping) from: #name to: (table fieldNamed: 'CUSTOMERNAME').
	(aDescriptor newMapping: DirectMapping) from: #dateOfBirth to: (table fieldNamed: 'BIRTHDATE').

	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #store.
	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #creditStatus.
	(aDescriptor newMapping: ToManyMapping)
		attributeName: #inStoreRentals.
	(aDescriptor newMapping: ToManyMapping)
		useLinkTable;
		attributeName: #inStorePurchases;
		linkTableJoin: (Join 
			from: (table fieldNamed: 'STORE_ID')
			to: (linkTable fieldNamed: 'STORE_ID'));
		targetTableJoin: (Join
			from: (table fieldNamed: 'CUSTOMERNAME')
			to: (purchaseTable fieldNamed: 'CUSTOMERNAME')
			from: (table fieldNamed: 'BIRTHDATE')
			to: (purchaseTable fieldNamed: 'CUSTOMERBIRTHDATE')
			from: 'T'
			to: (purchaseTable fieldNamed: 'IN_STORE'));
			constructRowMapKeyAs: [:customer :purchase | 
					MultipleRowMapKey with: customer with: purchase with: customer videoStore yourself].

	(aDescriptor newMapping: ToManyMapping)
		useLinkTable;
		attributeName: #onlinePurchases;
		linkTableJoin: (Join 
			from: (table fieldNamed: 'STORE_ID')
			to: (linkTable fieldNamed: 'STORE_ID'));
		targetTableJoin: (Join 	
			from: (table fieldNamed: 'CUSTOMERNAME')
			to: (purchaseTable fieldNamed: 'CUSTOMERNAME')
			from: (table fieldNamed: 'BIRTHDATE')
			to: (purchaseTable fieldNamed: 'CUSTOMERBIRTHDATE')
			from: 'F'
			to: (purchaseTable fieldNamed: 'IN_STORE'));
			constructRowMapKeyAs: [:customer :purchase | 
					MultipleRowMapKey with: customer with: purchase with: customer videoStore yourself].

	"A complicated way of saying that the Store plus one other attribute has to be there"
	aDescriptor imaginaryTableIsDefinedByRelationFrom: #store to: #inStorePurchases.
	aDescriptor imaginaryTableIsDefinedByRelationFrom: #store to: #inStoreRentals.
	aDescriptor imaginaryTableIsDefinedByRelationFrom: #store to: #onlinePurchases.
	aDescriptor imaginaryTableIsDefinedByRelationFrom: #store to: #creditStatus.
	aDescriptor buildReplacementTableSelect.! !

!GlorpVideoDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpVideoPurchase: aDescriptor

	| table |
	table := self tableNamed: 'VIDEO_PURCHASE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #title to: (table fieldNamed: 'TITLE').
	(aDescriptor newMapping: DirectMapping) from: #price to: (table fieldNamed: 'PRICE').
	(aDescriptor newMapping: DirectMapping) from: #inStore to: (table fieldNamed: 'IN_STORE').

	(aDescriptor newMapping: OneToOneMapping)
			useLinkTable;
			attributeName: #store;
			constructRowMapKeyAs: [:purchase :store | 
					MultipleRowMapKey with: purchase with: store with: purchase customer yourself].

	(aDescriptor newMapping: OneToOneMapping)
		attributeName: #customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpVideoRental: aDescriptor

	| table |
	table := self tableNamed: 'VIDEO_RENTAL'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #title to: (table fieldNamed: 'TITLE').
	(aDescriptor newMapping: DirectMapping) from: #dueDate to: (table fieldNamed: 'DUE_DATE').
	(aDescriptor newMapping: DirectMapping) from: #price to: (table fieldNamed: 'PRICE').
	(aDescriptor newMapping: DirectMapping) fromPseudoVariable: #customerName to: (table fieldNamed: 'CUSTOMERNAME').

	(aDescriptor newMapping: OneToOneMapping)
		attributeName: #customer.
	(aDescriptor newMapping: OneToOneMapping)
			attributeName: #store.! !

!GlorpVideoDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpVideoStore: aDescriptor

	| table |
	table := self tableNamed: 'VIDEO_STORE'.
	aDescriptor table: table.
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (table fieldNamed: 'NAME').
	(aDescriptor newMapping: ToManyMapping)
			attributeName: #customers;
			join: (Join
				from: (table fieldNamed: 'ID')
				to: ((self tableNamed: 'VIDEO_CUSTOMER') fieldNamed: 'STORE_ID')).
	"The relationship to purchases is different from the one in customer, which distinguishes online from inStore"
	(aDescriptor newMapping: ToManyMapping)
		useLinkTable;
		attributeName: #purchases;
		constructRowMapKeyAs: [:store :purchase | 
					MultipleRowMapKey with: store with: purchase with: purchase customer yourself].
	"No relationship to rentals at all"! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCreditStatus1
	| example |
	example := GlorpVideoCreditStatus new.	
	example balanceOwing: 12.00.
	example dateOwing: (Date today addDays: -1).
	example comments: 'One'.
	^example.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCreditStatus2
	| example |
	example := GlorpVideoCreditStatus new.	
	example balanceOwing: 22.00.
	example dateOwing: (Date today addDays: -1).
	example comments: 'Two'.
	^example.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCreditStatus3
	| example |
	example := GlorpVideoCreditStatus new.	
	example balanceOwing: 33.00.
	example dateOwing: (Date today addDays: -1).
	example comments: 'Three'.
	^example.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomer1
	"A customer with only online purchases"
	| customer |
	customer := GlorpVideoCustomer new.
	customer name: 'Adrian'.
	customer dateOfBirth: (Date today subtractDays: (10 * 365)).
	customer addOnlinePurchase: self exampleOnlinePurchase1.
	customer addOnlinePurchase: self exampleOnlinePurchase2.
	^customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomer2
	"A customer with only inStore purchases"
	| customer |
	customer := GlorpVideoCustomer new.
	customer name: 'Bob'.
	customer dateOfBirth: (Date today subtractDays: (20 * 365)).
	customer addInStorePurchase: self exampleInStorePurchase1.
	^customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomer3
	"A customer with only rentals"
	| customer |
	customer := GlorpVideoCustomer new.
	customer name: 'Carol'.
	customer dateOfBirth: (Date today subtractDays: (30 * 365)).
	customer addRental: self exampleInStoreRental1.
	^customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomer4
	"A customer with only credit"
	| customer |
	customer := GlorpVideoCustomer new.
	customer name: 'David'.
	customer dateOfBirth: (Date today subtractDays: (40 * 365)).
	customer creditStatus: self exampleCreditStatus1.
	^customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomer5
	"A customer with purchases and rentals"

	| customer |
	customer := GlorpVideoCustomer new.
	customer name: 'Eleanor'.
	customer dateOfBirth: (Date today subtractDays: 50 * 365).
	customer addInStorePurchase: self exampleInStorePurchase2.
	customer addInStorePurchase: self exampleInStorePurchase3.
	customer addRental: self exampleInStoreRental2.
	^customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomer6
	"A customer with purchases and credit"

	| customer |
	customer := GlorpVideoCustomer new.
	customer name: 'Francois'.
	customer dateOfBirth: (Date today subtractDays: 60 * 365).
	customer addInStorePurchase: self exampleInStorePurchase4.
	customer addInStorePurchase: self exampleInStorePurchase5.
	customer creditStatus: self exampleCreditStatus2.
	^customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomer7
	"A customer with everything"

	| customer |
	customer := GlorpVideoCustomer new.
	customer name: 'George'.
	customer dateOfBirth: (Date today subtractDays: 70 * 365).
	customer addInStorePurchase: self exampleInStorePurchase6.
	customer addOnlinePurchase: self exampleOnlinePurchase3.
	customer addRental: self exampleInStoreRental3.
	customer addRental: self exampleInStoreRental4.
	customer creditStatus: self exampleCreditStatus3.
	^customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleCustomer8
	"A customer with just a single purchase"

	| customer |
	customer := GlorpVideoCustomer new.
	customer name: 'Hieronymous'.
	customer dateOfBirth: (Date today subtractDays: 80 * 365).
	customer addInStorePurchase: self exampleInStorePurchase7.
	^customer.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStorePurchase1
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 101.
	purchase title: 'An American in Paris'.
	purchase price: 1.00.
	purchase inStore: true.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStorePurchase2
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 102.
	purchase title: 'Brazil'.
	purchase price: 2.00.
	purchase inStore: true.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStorePurchase3
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 103.
	purchase title: 'Cambodia, Swimming To'.
	purchase price: 3.00.
	purchase inStore: true.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStorePurchase4
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 104.
	purchase title: 'Dogma'.
	purchase price: 4.00.
	purchase inStore: true.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStorePurchase5
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 105.
	purchase title: 'ET'.
	purchase price: 5.00.
	purchase inStore: true.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStorePurchase6
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 106.
	purchase title: 'Fargo'.
	purchase price: 6.00.
	purchase inStore: true.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStorePurchase7
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 107.
	purchase title: 'Glengarry GlenRoss'.
	purchase price: 7.00.
	purchase inStore: true.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStoreRental1
	| rental |
	rental := GlorpVideoRental new.
	rental id: 1001.
	rental title: 'Amelie'.
	rental price: 21.00.
	rental dueDate: Date today.
	^rental.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStoreRental2
	| rental |
	rental := GlorpVideoRental new.
	rental id: 1002.
	rental title: 'Best in Show'.
	rental price: 22.00.
	rental dueDate: Date today.
	^rental.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStoreRental3
	| rental |
	rental := GlorpVideoRental new.
	rental id: 1003.
	rental title: 'Charlie Brown Christmas'.
	rental price: 23.00.
	rental dueDate: Date today.
	^rental.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleInStoreRental4
	| rental |
	rental := GlorpVideoRental new.
	rental id: 1004.
	rental title: 'Down By Law'.
	rental price: 24.00.
	rental dueDate: Date today.
	^rental.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleOnlinePurchase1
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 1.
	purchase title: 'All About Eve'.
	purchase price: 1.11.
	purchase inStore: false.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleOnlinePurchase2
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 2.
	purchase title: 'Butch Cassidy and the Sundance Kid'.
	purchase price: 2.22.
	purchase inStore: false.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleOnlinePurchase3
	| purchase |
	purchase := GlorpVideoPurchase new.
	purchase id: 3.
	purchase title: 'Cabaret'.
	purchase price: 3.33.
	purchase inStore: false.
	^purchase.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleStore1
	"self new exampleStore1"
	| example |
	example := GlorpVideoStore new.
	example name: 'Als Video'.
	example id: 1.
	example customers: (Array with: self exampleCustomer1 with: self exampleCustomer2 with: self exampleCustomer3 with: self exampleCustomer4), (Array with: self exampleCustomer5 with: self exampleCustomer6 with: self exampleCustomer7).
	example computeCollections.
	^example.! !

!GlorpVideoDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleStore2
	"self new exampleStore2"
	| example |
	example := GlorpVideoStore new.
	example name: 'Bonzai Video'.
	example id: 2.
	example customers: (Array with: self exampleCustomer8).
	example computeCollections.
	^example.! !

!GlorpVideoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForVIDEO_CREDIT_STATUS: aTable
	| customerTable custName custDate storeId |
	customerTable := self tableNamed: 'VIDEO_CUSTOMER'.
	custName := (aTable createFieldNamed: 'CUSTOMERNAME' type: (platform varchar: 50))
		bePrimaryKey.
	custDate := (aTable createFieldNamed: 'CUSTOMERBIRTHDATE' type: platform date) bePrimaryKey.
	storeId := (aTable createFieldNamed: 'STORE_ID' type: platform int4) bePrimaryKey.
	aTable addForeignKeyFrom: custName
		to: (customerTable fieldNamed: 'CUSTOMERNAME')
		from: custDate
		to: (customerTable fieldNamed: 'BIRTHDATE')
		from: storeId
		to: (customerTable fieldNamed: 'STORE_ID') .
	aTable addForeignKeyFrom: storeId to: ((self tableNamed: 'VIDEO_STORE') fieldNamed: 'ID').

	aTable createFieldNamed: 'BALANCE' type: (platform double).
	aTable createFieldNamed: 'DATE_OWING' type: platform date.
	aTable createFieldNamed: 'COMMENTS' type: (platform varchar: 50).! !

!GlorpVideoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForVIDEO_CUSTOMER: aTable
	"Note: This table is imaginary"
	| storeId |
	aTable isImaginary: true.
	(aTable createFieldNamed: 'CUSTOMERNAME' type: (platform varchar: 50))
		bePrimaryKey.
	(aTable createFieldNamed: 'BIRTHDATE' type: platform date) bePrimaryKey.
	storeId := (aTable createFieldNamed: 'STORE_ID' type: (platform int4)) bePrimaryKey.
	aTable addForeignKeyFrom: storeId
		to: ((self tableNamed: 'VIDEO_STORE') fieldNamed: 'ID')! !

!GlorpVideoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForVIDEO_PURCHASE: aTable
	| custName custDate customerTable |
	(aTable createFieldNamed: 'ID' type: (platform int4)) bePrimaryKey.
	customerTable := self tableNamed: 'VIDEO_CUSTOMER'.
	custName := aTable createFieldNamed: 'CUSTOMERNAME' type: (platform varchar: 50).
	custDate := aTable createFieldNamed: 'CUSTOMERBIRTHDATE' type: platform date.
	aTable addForeignKeyFrom: custName
		to: (customerTable fieldNamed: 'CUSTOMERNAME')
		from: custDate
		to: (customerTable fieldNamed: 'BIRTHDATE').

	aTable createFieldNamed: 'TITLE' type: (platform varchar: 50).	
	aTable createFieldNamed: 'PRICE' type: (platform double).
	aTable createFieldNamed: 'IN_STORE' type: (platform char: 1).! !

!GlorpVideoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForVIDEO_PURCHASE_LINK: aTable
	| storeId purchaseId |
	purchaseId := aTable createFieldNamed: 'PURCHASE_ID' type: platform int4.
	storeId := aTable createFieldNamed: 'STORE_ID' type: (platform int4).
	aTable addForeignKeyFrom: storeId
		to: ((self tableNamed: 'VIDEO_STORE') fieldNamed: 'ID').
	aTable addForeignKeyFrom: purchaseId
		to: ((self tableNamed: 'VIDEO_PURCHASE') fieldNamed: 'ID').! !

!GlorpVideoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForVIDEO_RENTAL: aTable
	| custName custDate storeId customerTable |
	(aTable createFieldNamed: 'ID' type: (platform int4)) bePrimaryKey.
	customerTable := self tableNamed: 'VIDEO_CUSTOMER'.
	custName := aTable createFieldNamed: 'CUSTOMERNAME' type: (platform varchar: 50).
	custDate := aTable createFieldNamed: 'CUSTOMERBIRTHDATE' type: platform date.
	storeId := aTable createFieldNamed: 'STORE_ID' type: (platform int4).
	aTable addForeignKeyFrom: custName
		to: (customerTable fieldNamed: 'CUSTOMERNAME')
		from: custDate
		to: (customerTable fieldNamed: 'BIRTHDATE')
		from: storeId
		to: (customerTable fieldNamed: 'STORE_ID').
	aTable addForeignKeyFrom: storeId to: ((self tableNamed: 'VIDEO_STORE') fieldNamed: 'ID').

	aTable createFieldNamed: 'TITLE' type: (platform varchar: 50).	
	aTable createFieldNamed: 'DUE_DATE' type: platform date.
	aTable createFieldNamed: 'PRICE' type: (platform double).! !

!GlorpVideoDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForVIDEO_STORE: aTable
	(aTable createFieldNamed: 'ID' type: (platform int4)) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varchar: 20).! !

GlorpTestDescriptorSystem subclass: #GlorpWorkerDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpWorkerDescriptorSystem methodsFor: 'other' stamp: ''!
allTableNames

	^#('GLORP_WORKER' 'GLORP_JOB' 'GLORP_WORKER_JOB_LINK').! !

!GlorpWorkerDescriptorSystem methodsFor: 'other' stamp: ''!
constructAllClasses
	^(super constructAllClasses)
		add: GlorpJob;
		add: GlorpWorker;
		yourself! !

!GlorpWorkerDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpJob: aDescriptor 
	"Note that the job table contains a FINISHED field, but the GlorpJob object doesn't. This field is determined only by membership in the finished or pending collections. In this particular case it's not very useful from a domain perspective, but it's interesting to be able to map. Similarly, whether a job is priority or not is not in the domain object, and is stored in the link table defining the relationship"

	| table |
	table := self tableNamed: 'GLORP_JOB'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id
				to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #description
				to: (table fieldNamed: 'DESCRIPTION')).! !

!GlorpWorkerDescriptorSystem methodsFor: 'descriptors' stamp: ''!
descriptorForGlorpWorker: aDescriptor 
	| table linkTable |
	table := self tableNamed: 'GLORP_WORKER'.
	aDescriptor table: table.
	aDescriptor 
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor 
		addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping: ((OneToManyMapping new)
				attributeName: #pendingJobs;
				referenceClass: GlorpJob;
				join: (self workerCriteriaWithConstant: 'N' in: table)).
	aDescriptor addMapping: ((OneToManyMapping new)
				attributeName: #finishedJobs;
				referenceClass: GlorpJob;
				join: (self workerCriteriaWithConstant: 'Y' in: table)).
	linkTable := self tableNamed: 'GLORP_WORKER_JOB_LINK'.

	"Note that priorityJobs may include finished jobs as well, and instances may occur in both this collection and the other two"
	aDescriptor addMapping: ((ManyToManyMapping new)
				attributeName: #priorityJobs;
				referenceClass: GlorpJob;
				join: (Join 
							from: (table fieldNamed: 'ID')
							to: (linkTable fieldNamed: 'WORKER_ID')
							from: 'Y'
							to: (linkTable fieldNamed: 'PRIORITY'))).! !

!GlorpWorkerDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleJobRow: anInteger finished: aBoolean

	| table row |
	table := self tableNamed: 'GLORP_JOB'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'ID') put: anInteger.
	row at: (table fieldNamed: 'DESCRIPTION') put: 'Job ', anInteger printString.
	row at: (table fieldNamed: 'FINISHED') put: (aBoolean ifTrue: ['Y'] ifFalse: ['N']).
	row at: (table fieldNamed: 'OWNER_ID') put: 1234.

	^row.! !

!GlorpWorkerDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleLinkRow1

	| table row |
	table := self tableNamed: 'GLORP_WORKER_JOB_LINK'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'WORKER_ID') put: 1234.
	row at: (table fieldNamed: 'JOB_ID') put: 2.
	row at: (table fieldNamed: 'PRIORITY') put: 'N'.
	^row.! !

!GlorpWorkerDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleLinkRow2

	| table row |
	table := self tableNamed: 'GLORP_WORKER_JOB_LINK'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'WORKER_ID') put: 1234.
	row at: (table fieldNamed: 'JOB_ID') put: 3.
	row at: (table fieldNamed: 'PRIORITY') put: 'Y'.
	^row.! !

!GlorpWorkerDescriptorSystem methodsFor: 'examples' stamp: ''!
exampleWorkerRow

	| table row |
	table := self tableNamed: 'GLORP_WORKER'.
	row := DatabaseRow newForTable: table.
	row at: (table fieldNamed: 'ID') put: 1234.
	row at: (table fieldNamed: 'NAME') put: 'John Worker'.
	^row.! !

!GlorpWorkerDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGLORP_JOB: aTable

	| ownerId |
	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'DESCRIPTION' type: (platform varChar: 40).
	aTable createFieldNamed: 'FINISHED' type: (platform varChar: 1).
	ownerId := aTable createFieldNamed: 'OWNER_ID' type: platform int4.
	aTable addForeignKeyFrom: ownerId
		to: ((self tableNamed: 'GLORP_WORKER') fieldNamed: 'ID').! !

!GlorpWorkerDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGLORP_WORKER: aTable

	(aTable createFieldNamed: 'ID' type: platform int4) bePrimaryKey.
	aTable createFieldNamed: 'NAME' type: (platform varChar: 20).! !

!GlorpWorkerDescriptorSystem methodsFor: 'tables' stamp: ''!
tableForGLORP_WORKER_JOB_LINK: aTable

	| workerId jobId |
	workerId := aTable createFieldNamed: 'WORKER_ID' type: platform int4.
	aTable addForeignKeyFrom: workerId
		to: ((self tableNamed: 'GLORP_WORKER') fieldNamed: 'ID').
	jobId := aTable createFieldNamed: 'JOB_ID' type: platform int4.
	aTable addForeignKeyFrom: jobId
		to: ((self tableNamed: 'GLORP_JOB') fieldNamed: 'ID').
	aTable createFieldNamed: 'PRIORITY' type: (platform varChar: 1).! !

!GlorpWorkerDescriptorSystem methodsFor: 'descriptors' stamp: ''!
workerCriteriaWithConstant: aString in: table 
	^Join 
		from: (table fieldNamed: 'ID')
		to: ((self tableNamed: 'GLORP_JOB') fieldNamed: 'OWNER_ID')
		from: aString
		to: ((self tableNamed: 'GLORP_JOB') fieldNamed: 'FINISHED').! !

!UndefinedObject methodsFor: '*GlorpTests' stamp: ''!
assertEqual: anObject
	GlorpTestModelClass new assert: anObject = nil.! !

Object subclass: #DefaultableThing
	instanceVariableNames: 'id boolean integer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!DefaultableThing methodsFor: 'accessing' stamp: ''!
boolean
	^boolean! !

!DefaultableThing methodsFor: 'accessing' stamp: ''!
boolean: anObject
	boolean := anObject! !

!DefaultableThing methodsFor: 'accessing' stamp: ''!
id
	^id! !

!DefaultableThing methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!DefaultableThing methodsFor: 'accessing' stamp: ''!
integer
	^integer! !

!DefaultableThing methodsFor: 'accessing' stamp: ''!
integer: anObject
	integer := anObject! !

Object subclass: #GlorpAddress
	instanceVariableNames: 'id street number'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpAddress class methodsFor: 'examples' stamp: ''!
example1

	^self new
		id: 1;
		street: 'West 47th Ave';
		number: '2042'.! !

!GlorpAddress class methodsFor: 'examples' stamp: ''!
example1WithChangedAddress

	^self new
		id: 1;
		street: 'Garden of the Gods';
		number: '99999'.! !

!GlorpAddress class methodsFor: 'examples' stamp: ''!
example2

	^self new
		id: 2;
		street: 'Nowhere';
		number: '1000'.! !

!GlorpAddress class methodsFor: 'glorp setup' stamp: ''!
glorpSetupClassModel: aClassModel forSystem: aDescriptorSystem 
	aClassModel newAttributeNamed: #id.
	aClassModel newAttributeNamed: #street.
	aClassModel newAttributeNamed: #number.! !

!GlorpAddress class methodsFor: 'glorp setup' stamp: ''!
glorpSetupDescriptor: aDescriptor forSystem: aDescriptorSystem 
	| table |
	table := aDescriptorSystem tableNamed: 'GR_ADDRESS'.
	aDescriptor table: table.
	aDescriptor
		addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID'));
		addMapping: (DirectMapping from: #street to: (table fieldNamed: 'STREET'));
		addMapping: (DirectMapping from: #number to: (table fieldNamed: 'HOUSE_NUM'))! !

!GlorpAddress methodsFor: 'notification' stamp: ''!
glorpNoticeOfExpiryIn: aSession! !

!GlorpAddress methodsFor: 'accessing' stamp: ''!
id
	"Private - Answer the value of the receiver's ''id'' instance variable."

	^id! !

!GlorpAddress methodsFor: 'accessing' stamp: ''!
id: anObject
	"Private - Set the value of the receiver's ''id'' instance variable to the argument, anObject."

	id := anObject! !

!GlorpAddress methodsFor: 'accessing' stamp: ''!
number
	"Private - Answer the value of the receiver's ''number'' instance variable."

	^number! !

!GlorpAddress methodsFor: 'accessing' stamp: ''!
number: anObject
	"Private - Set the value of the receiver's ''number'' instance variable to the argument, anObject."

	number := anObject! !

!GlorpAddress methodsFor: 'printing' stamp: ''!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '(', id printString, ',', street printString, ',' , number printString, ')'.! !

!GlorpAddress methodsFor: 'accessing' stamp: ''!
street
	"Private - Answer the value of the receiver's ''street'' instance variable."

	^street! !

!GlorpAddress methodsFor: 'accessing' stamp: ''!
street: anObject
	"Private - Set the value of the receiver's ''street'' instance variable to the argument, anObject."

	street := anObject! !

Object subclass: #GlorpAirline
	instanceVariableNames: 'id name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpAirline class methodsFor: 'examples' stamp: ''!
example1
	^self new
		id: 73;
		name: 'Air Canada'.! !

!GlorpAirline class methodsFor: 'examples' stamp: ''!
example2
	^self new
		id: 74;
		name: 'Lufthansa'.! !

!GlorpAirline methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpAirline methodsFor: 'accessing' stamp: ''!
id: anInteger
	id := anInteger! !

!GlorpAirline methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpAirline methodsFor: 'accessing' stamp: ''!
name: aString
	name := aString! !

Object subclass: #GlorpAirlineMeal
	instanceVariableNames: 'id description ingredients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

Object subclass: #GlorpAttachment
	instanceVariableNames: 'index mimeType attachmentBytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpAttachment class methodsFor: 'glorp' stamp: ''!
glorpSetupClassModel: model forSystem: system 
	model
		newAttributeNamed: #index;
		newAttributeNamed: #mimeType;
		newAttributeNamed: #attachmentBytes type: GlorpAttachmentBytes! !

!GlorpAttachment class methodsFor: 'glorp' stamp: ''!
glorpSetupDescriptor: aDesc forSystem: system 
	| t |
	t := system tableNamed: 'attachment'.
	aDesc table: t.
	(aDesc newMapping: DirectMapping) from: #index
		to: (t fieldNamed: 'indexInDocument').
	(aDesc newMapping: DirectMapping) from: #mimeType
		to: (t fieldNamed: 'mimetype').
	(aDesc newMapping: OneToOneMapping) attributeName: #attachmentBytes! !

!GlorpAttachment methodsFor: 'accessing' stamp: ''!
attachmentBytes
	^attachmentBytes! !

!GlorpAttachment methodsFor: 'accessing' stamp: ''!
attachmentBytes: anObject
	attachmentBytes := anObject! !

!GlorpAttachment methodsFor: 'accessing' stamp: ''!
index
	^index! !

!GlorpAttachment methodsFor: 'accessing' stamp: ''!
index: anObject
	index := anObject! !

!GlorpAttachment methodsFor: 'accessing' stamp: ''!
mimeType
	^mimeType! !

!GlorpAttachment methodsFor: 'accessing' stamp: ''!
mimeType: anObject
	mimeType := anObject! !

Object subclass: #GlorpAttachmentBytes
	instanceVariableNames: 'content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpAttachmentBytes class methodsFor: 'glorp' stamp: ''!
glorpSetupClassModel: model forSystem: system 
	model newAttributeNamed: #content! !

!GlorpAttachmentBytes class methodsFor: 'glorp' stamp: ''!
glorpSetupDescriptor: aDesc forSystem: system 
	| t |
	t := system tableNamed: 'attachmentbytes'.
	aDesc table: t.
	(aDesc newMapping: DirectMapping) from: #content
		to: (t fieldNamed: 'content')! !

!GlorpAttachmentBytes methodsFor: 'accessing' stamp: ''!
content
	^content! !

!GlorpAttachmentBytes methodsFor: 'accessing' stamp: ''!
content: anObject
	content := anObject! !

Object subclass: #GlorpBackendFacility
	instanceVariableNames: 'login'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpBackendFacility methodsFor: 'accessing' stamp: 'GP 1/11/2009 23:33'!
login
	^ login! !

!GlorpBackendFacility methodsFor: 'accessing' stamp: 'GP 1/11/2009 23:33'!
login: anObject
	login := anObject! !

Object subclass: #GlorpBankAccount
	instanceVariableNames: 'id accountNumber accountHolders eventsReceived'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpBankAccount class methodsFor: 'examples' stamp: ''!
example1

	^self new 
		id: 123;
		accountNumber: GlorpBankAccountNumber example12345.! !

!GlorpBankAccount class methodsFor: 'instance creation' stamp: ''!
new

	^super new initialize.! !

!GlorpBankAccount methodsFor: 'comparing' stamp: ''!
<= anAccount
	^self accountNumber accountNumber <= anAccount accountNumber accountNumber.! !

!GlorpBankAccount methodsFor: 'accessing' stamp: ''!
accountHolders
	^accountHolders.! !

!GlorpBankAccount methodsFor: 'accessing' stamp: ''!
accountNumber
	^accountNumber! !

!GlorpBankAccount methodsFor: 'accessing' stamp: ''!
accountNumber: anAccountNumber 
	accountNumber := anAccountNumber! !

!GlorpBankAccount methodsFor: 'accessing' stamp: ''!
basicAddHolder: aCustomer

	accountHolders add: aCustomer.! !

!GlorpBankAccount methodsFor: 'accessing' stamp: ''!
basicRemoveHolder: aCustomer

	accountHolders remove: aCustomer.! !

!GlorpBankAccount methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpBankAccount methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpBankAccount methodsFor: 'initialize' stamp: ''!
initialize

	accountHolders := OrderedCollection new.! !

!GlorpBankAccount methodsFor: 'printing' stamp: ''!
printOn: aStream

	super printOn: aStream.
	aStream 
		nextPutAll: '(id=';
		print: id;
		nextPut: $).! !

Object subclass: #GlorpBankAccountNumber
	instanceVariableNames: 'bankCode branchNumber accountNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpBankAccountNumber class methodsFor: 'examples' stamp: ''!
example12345

	^self new
		accountNumber: 12345;
		bankCode: 4;
		branchNumber: 777.! !

!GlorpBankAccountNumber methodsFor: 'accessing' stamp: ''!
accountNumber
	^accountNumber! !

!GlorpBankAccountNumber methodsFor: 'accessing' stamp: ''!
accountNumber: anObject
	accountNumber := anObject! !

!GlorpBankAccountNumber methodsFor: 'accessing' stamp: ''!
bankCode
	^bankCode! !

!GlorpBankAccountNumber methodsFor: 'accessing' stamp: ''!
bankCode: anObject
	bankCode := anObject! !

!GlorpBankAccountNumber methodsFor: 'accessing' stamp: ''!
branchNumber
	^branchNumber! !

!GlorpBankAccountNumber methodsFor: 'accessing' stamp: ''!
branchNumber: anObject
	branchNumber := anObject! !

Object subclass: #GlorpBankBranch
	instanceVariableNames: 'accounts branchNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpBankBranch methodsFor: 'accessing' stamp: ''!
accounts
	^accounts! !

!GlorpBankBranch methodsFor: 'accessing' stamp: ''!
accounts: anObject
	accounts := anObject! !

!GlorpBankBranch methodsFor: 'accessing' stamp: ''!
branchNumber
	^branchNumber.! !

!GlorpBankBranch methodsFor: 'accessing' stamp: ''!
branchNumber: aString
	branchNumber := aString! !

!GlorpBankBranch methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '(', self branchNumber printString, ')'.! !

Object subclass: #GlorpBankTransaction
	instanceVariableNames: 'id owner amount serviceCharge'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpBankTransaction class methodsFor: 'examples' stamp: ''!
example1

	^self new! !

!GlorpBankTransaction class methodsFor: 'examples' stamp: ''!
example2

	^self new! !

!GlorpBankTransaction class methodsFor: 'instance creation' stamp: ''!
new

	^super new initialize.! !

!GlorpBankTransaction methodsFor: 'accessing' stamp: ''!
amount
	^amount! !

!GlorpBankTransaction methodsFor: 'accessing' stamp: ''!
amount: aGlorpMoney 
	amount := aGlorpMoney! !

!GlorpBankTransaction methodsFor: 'accessing' stamp: ''!
id
	"Private - Answer the value of the receiver's ''id'' instance variable."

	^id! !

!GlorpBankTransaction methodsFor: 'accessing' stamp: ''!
id: anObject
	"Private - Set the value of the receiver's ''id'' instance variable to the argument, anObject."

	id := anObject! !

!GlorpBankTransaction methodsFor: 'initialize' stamp: ''!
initialize

	amount := GlorpMoney forAmount: 0.
	serviceCharge := GlorpServiceCharge default.! !

!GlorpBankTransaction methodsFor: 'accessing' stamp: ''!
owner
	"Private - Answer the value of the receiver's ''owner'' instance variable."

	^owner! !

!GlorpBankTransaction methodsFor: 'accessing' stamp: ''!
owner: aCustomer 
	owner := aCustomer! !

!GlorpBankTransaction methodsFor: 'accessing' stamp: ''!
serviceCharge
	^serviceCharge! !

!GlorpBankTransaction methodsFor: 'accessing' stamp: ''!
serviceCharge: aServiceCharge 
	serviceCharge := aServiceCharge! !

Object subclass: #GlorpBook
	instanceVariableNames: 'id description copiesInStock version title author'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpBook methodsFor: 'accessing' stamp: ''!
author
	^author! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
author: aString
	author := aString! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
copiesInStock
	^copiesInStock! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
copiesInStock: anInteger
	copiesInStock := anInteger! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
description
	^description! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
description: aString
	description := aString! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
id: anInteger
	id := anInteger! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
title
	^title! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
title: aString
	title := aString.! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
version
	^version! !

!GlorpBook methodsFor: 'accessing' stamp: ''!
version: anInteger
	version := anInteger! !

Object subclass: #GlorpBookstoreCustomer
	instanceVariableNames: 'name accountNumber booksOnOrder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpBookstoreCustomer class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize.! !

!GlorpBookstoreCustomer methodsFor: 'accessing' stamp: ''!
accountNumber
	^accountNumber! !

!GlorpBookstoreCustomer methodsFor: 'accessing' stamp: ''!
accountNumber: anObject
	accountNumber := anObject! !

!GlorpBookstoreCustomer methodsFor: 'initialize-release' stamp: ''!
initialize
	booksOnOrder := OrderedCollection new.! !

!GlorpBookstoreCustomer methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpBookstoreCustomer methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

Object subclass: #GlorpBreadthFirstTopologicalSortItem
	instanceVariableNames: 'parent id relatedItems status'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpBreadthFirstTopologicalSortItem class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'initialize-release' stamp: ''!
initialize
	relatedItems := OrderedCollection new.! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'accessing' stamp: ''!
parent
	^parent! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'accessing' stamp: ''!
parent: anObject
	parent := anObject! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'printing' stamp: ''!
printOn: aStream
	aStream nextPut: $(.
	id printOn: aStream.
	aStream nextPut: $).! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'relationships' stamp: ''!
relateTo: anItem
	relatedItems add: anItem.! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'accessing' stamp: ''!
relatedItems
	^relatedItems! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'accessing' stamp: ''!
relatedItems: aCollection
	relatedItems := aCollection! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'accessing' stamp: ''!
status
	^status! !

!GlorpBreadthFirstTopologicalSortItem methodsFor: 'accessing' stamp: ''!
status: aSymbol
	status := aSymbol! !

Object subclass: #GlorpCollectionTypeModelsPackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Licence'!

!GlorpCollectionTypeModelsPackage commentStamp: '<historical>' prior: 0!
This class represents package information for the GlorpCollectionTypeModels package, for systems that don't have things like package properties to store things like copyright and license information.

LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!

!GlorpCollectionTypeModelsPackage class methodsFor: 'instance creation' stamp: ''!
LICENSE
"LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"! !

Object subclass: #GlorpCompressedMoney
	instanceVariableNames: 'id array'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpCompressedMoney class methodsFor: 'instance creation' stamp: ''!
currency: aSymbol amount: aNumber

	^super new initialize
		currency: aSymbol;
		amount: aNumber.! !

!GlorpCompressedMoney class methodsFor: 'instance creation' stamp: ''!
defaultCurrency

	^#CDN.! !

!GlorpCompressedMoney class methodsFor: 'instance creation' stamp: ''!
forAmount: aNumber

	^self currency: self defaultCurrency amount: aNumber.! !

!GlorpCompressedMoney class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize.! !

!GlorpCompressedMoney methodsFor: 'accessing' stamp: ''!
amount
	^array at: 2! !

!GlorpCompressedMoney methodsFor: 'accessing' stamp: ''!
amount: anObject
	array at: 2 put: anObject! !

!GlorpCompressedMoney methodsFor: 'accessing' stamp: ''!
currency
	^array at: 1! !

!GlorpCompressedMoney methodsFor: 'accessing' stamp: ''!
currency: anObject
	array at: 1 put: anObject! !

!GlorpCompressedMoney methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpCompressedMoney methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpCompressedMoney methodsFor: 'initialize' stamp: ''!
initialize

	array := Array new: 2.! !

!GlorpCompressedMoney methodsFor: 'printing' stamp: ''!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self amount;
		space;
		nextPutAll: self currency;
		nextPut: $).! !

Object subclass: #GlorpCustomer
	instanceVariableNames: 'id name transactions accounts accountsSortedById accountsSortedByIdDescending eventsReceived seenPostFetch seenPreWrite seenPostWrite seenExpiry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpCustomer class methodsFor: 'examples' stamp: ''!
example1

	^self new
		name: 'Fred Flintstone';
		addTransaction: GlorpBankTransaction example1;
		addTransaction: GlorpBankTransaction example2.! !

!GlorpCustomer class methodsFor: 'examples' stamp: ''!
example2

	^self new
          id: 123;
		name: 'Fred Flintstone';
		addTransaction: GlorpBankTransaction example1;
		addTransaction: GlorpBankTransaction example2.! !

!GlorpCustomer class methodsFor: 'instance creation' stamp: ''!
new

	^super new initialize.! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
accounts

	^accounts.! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
accounts: aCollection

	accounts := aCollection.! !

!GlorpCustomer methodsFor: 'As yet unclassified' stamp: ''!
accountsSortedById
	^accountsSortedById.! !

!GlorpCustomer methodsFor: 'As yet unclassified' stamp: ''!
accountsSortedByIdDescending
	^accountsSortedByIdDescending.! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
addAccount: aBankAccount

	accounts add: aBankAccount.
	aBankAccount basicAddHolder: self.! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
addTransaction: aTransaction

	transactions add: aTransaction.
	aTransaction owner: self.! !

!GlorpCustomer methodsFor: 'glorp/events' stamp: ''!
glorpNoticeOfExpiryIn: aSession
	seenExpiry := true.! !

!GlorpCustomer methodsFor: 'glorp/events' stamp: ''!
glorpPostFetch: aSession
	seenPostFetch := true! !

!GlorpCustomer methodsFor: 'glorp/events' stamp: ''!
glorpPostWrite: aSession
	seenPostWrite := true! !

!GlorpCustomer methodsFor: 'glorp/events' stamp: ''!
glorpPreWrite: aSession
	seenPreWrite := true! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
id
	"Private - Answer the value of the receiver's ''id'' instance variable."

	^id! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
id: anObject
	"Private - Set the value of the receiver's ''id'' instance variable to the argument, anObject."

	id := anObject! !

!GlorpCustomer methodsFor: 'initialize/release' stamp: ''!
initialize

	transactions := OrderedCollection new.
	accounts := OrderedCollection new.
	seenExpiry := false.
	seenPostFetch := false.
	seenPreWrite:= false.
	seenPostWrite := false.! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
name
	"Private - Answer the value of the receiver's ''name'' instance variable."

	^name! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
name: anObject
	"Private - Set the value of the receiver's ''name'' instance variable to the argument, anObject."

	name := anObject! !

!GlorpCustomer methodsFor: 'printing' stamp: ''!
printOn: aStream 
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: id;
		nextPut: $,;
		nextPutAll: (name ifNil: ['']);
		nextPutAll: ')'! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
removeAccount: aBankAccount

	accounts remove: aBankAccount.
	aBankAccount basicRemoveHolder: self.! !

!GlorpCustomer methodsFor: 'glorp/events' stamp: ''!
seenExpiry
	^seenExpiry! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
seenPostFetch
	^seenPostFetch! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
seenPostWrite
	^seenPostWrite! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
seenPreWrite
	^seenPreWrite! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
transactions
	"Private - Answer the value of the receiver's ''transactions'' instance variable."

	^transactions! !

!GlorpCustomer methodsFor: 'accessing' stamp: ''!
transactions: anObject
	"Private - Set the value of the receiver's ''transactions'' instance variable to the argument, anObject."

	transactions := anObject! !

Object subclass: #GlorpDBTestsPackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpDBTestsPackage commentStamp: '<historical>' prior: 0!
This class represents package information for the GlorpDBTests package, for systems that don't have things like package properties to store things like copyright and license information.

LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!

!GlorpDBTestsPackage class methodsFor: 'instance creation' stamp: ''!
LICENSE
"LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"! !

Object subclass: #GlorpDBTypeTestsPackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Licence'!

!GlorpDBTypeTestsPackage commentStamp: '<historical>' prior: 0!
This class represents package information for the GlorpDBTypeTests package, for systems that don't have things like package properties to store things like copyright and license information.

LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!

!GlorpDBTypeTestsPackage class methodsFor: 'instance creation' stamp: ''!
LICENSE
"LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"! !

Object subclass: #GlorpDescriptorSystemWithNamespacePoolDictionaryDeclarer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpDescriptorSystemWithNamespacePoolDictionaryDeclarer class methodsFor: 'VA compatibility' stamp: ''!
_PRAGMA_GlorpTestNamespace
	"%%PRAGMA DECLARE
	(name: GlorpTestNamespace isPool: true isConstant: true)
	(pool: GlorpTestNamespace declarations: (
		))"! !

!GlorpDescriptorSystemWithNamespacePoolDictionaryDeclarer class methodsFor: 'VA compatibility' stamp: ''!
initialize
	"If this is VA, force the pool dictionary/namespace pragma for the test namespace to be handled. That means that subsequent packages on file-in will be able to see it. A lot of jumping through hoops for something that doesn't even get exercised in VA"

	Dialect isVisualAge ifFalse: [^self].
	(Smalltalk at: #EtAbstractMethodsBrowser) new
		rebuildNamespaceIfRequired: (Smalltalk at: #GlorpTestModels)
		askUser: false.! !

Object subclass: #GlorpDocument
	instanceVariableNames: 'id whatever attachments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpDocument class methodsFor: 'glorp' stamp: ''!
glorpSetupClassModel: model forSystem: system 
	model
		newAttributeNamed: #id;
		newAttributeNamed: #whatever;
		newAttributeNamed: #attachments
			collection: OrderedCollection
			of: GlorpAttachment! !

!GlorpDocument class methodsFor: 'glorp' stamp: ''!
glorpSetupDescriptor: aDesc forSystem: system 
	| t |
	t := system tableNamed: 'document'.
	aDesc table: t.
	(aDesc newMapping: DirectMapping) from: #id to: (t fieldNamed: 'id').
	(aDesc newMapping: DirectMapping) from: #whatever to: (t fieldNamed: 'whatever').
	(aDesc newMapping: OneToManyMapping)
		attributeName: #attachments;
		orderBy: [:ea | ea index]! !

!GlorpDocument methodsFor: 'accessing' stamp: ''!
attachments
	^attachments! !

!GlorpDocument methodsFor: 'accessing' stamp: ''!
attachments: anObject
	attachments := anObject! !

!GlorpDocument methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpDocument methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpDocument methodsFor: 'initialize-release' stamp: ''!
initialize
	attachments := OrderedCollection new.
	whatever := 3.! !

Object subclass: #GlorpEmailAddress
	instanceVariableNames: 'id user host'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpEmailAddress methodsFor: 'accessing' stamp: ''!
host
	^host! !

!GlorpEmailAddress methodsFor: 'accessing' stamp: ''!
host: anObject
	host := anObject! !

!GlorpEmailAddress methodsFor: 'accessing' stamp: ''!
id
	^ id! !

!GlorpEmailAddress methodsFor: 'accessing' stamp: ''!
id: anInteger
	 id := anInteger.! !

!GlorpEmailAddress methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: id printString.
	aStream nextPut: $).! !

!GlorpEmailAddress methodsFor: 'accessing' stamp: ''!
user
	^user! !

!GlorpEmailAddress methodsFor: 'accessing' stamp: ''!
user: anObject
	user := anObject! !

Object subclass: #GlorpEncyclopedia
	instanceVariableNames: 'id name entries biographicalEntries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpEncyclopedia class methodsFor: 'examples' stamp: ''!
example1
	| result |
	result := GlorpEncyclopedia new.
	result entries at: 'one' put: GlorpEncyclopediaEntry example1.
	result entries at: 'two' put: GlorpEncyclopediaEntry example2.
	^result.! !

!GlorpEncyclopedia class methodsFor: 'instance creation' stamp: ''!
new

	^super new initialize.! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
articleAt: aString put: anEntry
	entries at: aString put: anEntry.! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
biographicalEntries
	^biographicalEntries! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
biographicalEntries: aDictionary
	biographicalEntries := aDictionary! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
entries
	^entries.! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
entries: aDictionary
	entries := aDictionary! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
id: anInteger
	id := anInteger! !

!GlorpEncyclopedia methodsFor: 'initialize' stamp: ''!
initialize

	entries := Dictionary new.
	biographicalEntries := Dictionary new.! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
name: aString
	name := aString! !

!GlorpEncyclopedia methodsFor: 'accessing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	name isNil ifFalse: [aStream nextPutAll: name].
	aStream nextPut: $).! !

Object subclass: #GlorpEncyclopediaBioEntry
	instanceVariableNames: 'owner title text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpEncyclopediaBioEntry class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize.! !

!GlorpEncyclopediaBioEntry methodsFor: 'initialize-release' stamp: ''!
initialize! !

!GlorpEncyclopediaBioEntry methodsFor: 'accessing' stamp: ''!
text
	^text! !

!GlorpEncyclopediaBioEntry methodsFor: 'accessing' stamp: ''!
text: anObject
	text := anObject! !

!GlorpEncyclopediaBioEntry methodsFor: 'accessing' stamp: ''!
title
	^title! !

!GlorpEncyclopediaBioEntry methodsFor: 'accessing' stamp: ''!
title: anObject
	title := anObject! !

Object subclass: #GlorpEncyclopediaEntry
	instanceVariableNames: 'id text title mediaType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpEncyclopediaEntry class methodsFor: 'examples' stamp: ''!
example1
	^self new
		id: 1;
		name: 'One';
		text: 'The first number (not counting zero)'.! !

!GlorpEncyclopediaEntry class methodsFor: 'examples' stamp: ''!
example2
	^self new
		id: 2;
		name: 'Two';
		text: 'The second number (comes after 1)'.! !

!GlorpEncyclopediaEntry class methodsFor: 'As yet unclassified' stamp: ''!
new
	^super new initialize! !

!GlorpEncyclopediaEntry methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpEncyclopediaEntry methodsFor: 'accessing' stamp: ''!
id: aSmallInteger 
	id := aSmallInteger.! !

!GlorpEncyclopediaEntry methodsFor: 'initialize-release' stamp: ''!
initialize
	mediaType := GlorpMediaType text! !

!GlorpEncyclopediaEntry methodsFor: 'accessing' stamp: ''!
mediaType
	^mediaType! !

!GlorpEncyclopediaEntry methodsFor: 'accessing' stamp: ''!
mediaType: anObject
	mediaType := anObject! !

!GlorpEncyclopediaEntry methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	title isNil ifFalse: [aStream nextPutAll: title].
	aStream nextPut: $).! !

!GlorpEncyclopediaEntry methodsFor: 'accessing' stamp: ''!
text
	^text! !

!GlorpEncyclopediaEntry methodsFor: 'accessing' stamp: ''!
text: aString 
	text := aString.! !

!GlorpEncyclopediaEntry methodsFor: 'accessing' stamp: ''!
title
	^title! !

!GlorpEncyclopediaEntry methodsFor: 'accessing' stamp: ''!
title: aString 
	title := aString! !

Object subclass: #GlorpExampleSystem
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

GlorpExampleSystem subclass: #GlorpBankExampleSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpAddressNumber1: anAddress
	anAddress id: 12.
	anAddress street: 'Paseo Montril'.
	anAddress number: '10185'.! !

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpBankAccountNumber1: anAccount

	anAccount id: 1.
	anAccount accountNumber: (self objectNumber: 1 ofClass: GlorpBankAccountNumber).! !

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpBankAccountNumber2: anAccount

	anAccount id: 2.
	anAccount accountNumber: (self objectNumber: 2 ofClass: GlorpBankAccountNumber).! !

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpBankAccountNumberNumber1: aBankAccountNumber

	aBankAccountNumber bankCode: '004'.
	aBankAccountNumber branchNumber: 0342.
	aBankAccountNumber accountNumber: '12345'.! !

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpBankAccountNumberNumber2: aBankAccountNumber

	aBankAccountNumber bankCode: '004'.
	aBankAccountNumber branchNumber: 0342.
	aBankAccountNumber accountNumber: '01010'.! !

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpBankTransactionNumber1: aTrans

	"Nothing to initialize"! !

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpBankTransactionNumber2: aTrans

	"Nothing to initialize"! !

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpCustomerNumber1: aCustomer

	aCustomer id: 1.
	aCustomer name: 'Fred Flintstone'.
	aCustomer addTransaction: (self objectNumber: 1 ofClass: GlorpBankTransaction).
	aCustomer addTransaction: (self objectNumber: 2 ofClass: GlorpBankTransaction).
	aCustomer addAccount: (self objectNumber: 1 ofClass: GlorpBankAccount).
	aCustomer addAccount: (self objectNumber: 2 ofClass: GlorpBankAccount).! !

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpEmailAddressNumber1: anEmailAddress 
	anEmailAddress id: 2.
	anEmailAddress user: 'foo'.
	anEmailAddress host: 'bar.com'! !

!GlorpBankExampleSystem methodsFor: 'examples' stamp: ''!
exampleGlorpPersonNumber1: aPerson 
	aPerson id: 1.
	aPerson name: 'Barney Rubble'.
	aPerson address: (self objectNumber: 1 ofClass: GlorpAddress).	
	aPerson emailAddress: (self objectNumber: 1 ofClass: GlorpEmailAddress).! !

!GlorpExampleSystem class methodsFor: 'instance creation' stamp: ''!
new

	^super new initialize! !

!GlorpExampleSystem methodsFor: 'initialize' stamp: ''!
initialize

	objects := Dictionary new.! !

!GlorpExampleSystem methodsFor: 'misc' stamp: ''!
lookupObject: aNumber ofClass: aClass ifAbsentPut: absentBlock

	^(objects at: aClass ifAbsentPut: [Dictionary new]) at: aNumber ifAbsentPut: absentBlock.! !

!GlorpExampleSystem methodsFor: 'api' stamp: ''!
objectNumber: aNumber ofClass: aClass

	| symbol instance|
	instance := self lookupObject: aNumber ofClass: aClass ifAbsentPut: [aClass new].
	symbol := ('example', aClass name, 'Number', aNumber printString, ':') asSymbol.
	self perform: symbol with: instance.
	^instance.! !

Object subclass: #GlorpFakeElementBuilder
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpFakeElementBuilder methodsFor: 'accessing' stamp: ''!
value: anObject
	value := anObject.! !

!GlorpFakeElementBuilder methodsFor: 'element builder protocol' stamp: ''!
valueOf: aField
	^value.! !

!GlorpFakeElementBuilder methodsFor: 'element builder protocol' stamp: ''!
valueOfField: aField in: anArray

	^value.! !

Object subclass: #GlorpFolder
	instanceVariableNames: 'user name messages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpFolder class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpFolder methodsFor: 'initialize-release' stamp: ''!
initialize
	user := nil.
	messages := OrderedCollection new.! !

!GlorpFolder methodsFor: 'accessing' stamp: ''!
messages
	^messages! !

!GlorpFolder methodsFor: 'accessing' stamp: ''!
messages: aCollection
	messages := aCollection! !

!GlorpFolder methodsFor: 'accessing' stamp: ''!
name
	^name.! !

!GlorpFolder methodsFor: 'accessing' stamp: ''!
name: aString
	name := aString.! !

!GlorpFolder methodsFor: 'accessing' stamp: ''!
user
	^user! !

!GlorpFolder methodsFor: 'accessing' stamp: ''!
user: aGlorpUser
	user := aGlorpUser! !

!GlorpFolder methodsFor: 'accessing' stamp: ''!
userId
	^self user id.! !

Object subclass: #GlorpGallery
	instanceVariableNames: 'id name images'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpGallery class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize.! !

!GlorpGallery methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpGallery methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpGallery methodsFor: 'accessing' stamp: ''!
images
	^images! !

!GlorpGallery methodsFor: 'accessing' stamp: ''!
images: anObject
	images := anObject! !

!GlorpGallery methodsFor: 'initialize-release' stamp: ''!
initialize
	images := OrderedCollection new.! !

!GlorpGallery methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpGallery methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

Object subclass: #GlorpImageFile
	instanceVariableNames: 'id title bits thumbnailBits'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpImageFile class methodsFor: 'examples' stamp: ''!
example1
	^self new
		title: 'example1';
		bits: #[1 2 3 4 5].! !

!GlorpImageFile class methodsFor: 'examples' stamp: ''!
example2
	^self new
		title: 'example2';
		bits: #[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15].! !

!GlorpImageFile methodsFor: 'accessing' stamp: ''!
bits
	^bits! !

!GlorpImageFile methodsFor: 'accessing' stamp: ''!
bits: anObject
	bits := anObject! !

!GlorpImageFile methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpImageFile methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpImageFile methodsFor: 'testing' stamp: ''!
isLarge
	^bits size > 10.! !

!GlorpImageFile methodsFor: 'testing' stamp: ''!
thumbnailBits
	thumbnailBits isNil ifTrue: [thumbnailBits := bits copyFrom: 1 to: 5].
	^thumbnailBits.! !

!GlorpImageFile methodsFor: 'accessing' stamp: ''!
thumbnailBits: aByteArray
	thumbnailBits := aByteArray! !

!GlorpImageFile methodsFor: 'accessing' stamp: ''!
title
	^title! !

!GlorpImageFile methodsFor: 'accessing' stamp: ''!
title: anObject
	title := anObject! !

Object subclass: #GlorpImageLink
	instanceVariableNames: 'id title thumbnailBits fullImage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpImageLink methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpImageLink methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpImageLink methodsFor: 'accessing' stamp: ''!
image
	^fullImage! !

!GlorpImageLink methodsFor: 'accessing' stamp: ''!
image: anObject
	fullImage := anObject! !

!GlorpImageLink methodsFor: 'accessing' stamp: ''!
thumbnailBits
	^thumbnailBits! !

!GlorpImageLink methodsFor: 'accessing' stamp: ''!
thumbnailBits: anObject
	thumbnailBits := anObject! !

!GlorpImageLink methodsFor: 'accessing' stamp: ''!
title
	^title! !

!GlorpImageLink methodsFor: 'accessing' stamp: ''!
title: anObject
	title := anObject! !

Object subclass: #GlorpInventoryItem
	instanceVariableNames: 'id name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpInventoryItem methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpInventoryItem methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpInventoryItem methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpInventoryItem methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

!GlorpInventoryItem methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: id printString.
	aStream nextPut: $,.
	aStream nextPutAll: name printString.
	aStream nextPut: $).! !

GlorpInventoryItem subclass: #GlorpNonperishableItem
	instanceVariableNames: 'serialNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpNonperishableItem methodsFor: 'accessing' stamp: ''!
serialNumber
	^serialNumber! !

!GlorpNonperishableItem methodsFor: 'accessing' stamp: ''!
serialNumber: anObject
	serialNumber := anObject! !

GlorpNonperishableItem subclass: #GlorpUnassembledItem
	instanceVariableNames: 'assemblyCost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpUnassembledItem methodsFor: 'accessing' stamp: ''!
assemblyCost
	^assemblyCost! !

!GlorpUnassembledItem methodsFor: 'accessing' stamp: ''!
assemblyCost: anObject
	assemblyCost := anObject! !

GlorpInventoryItem subclass: #GlorpPerishableItem
	instanceVariableNames: 'age'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpPerishableItem methodsFor: 'accessing' stamp: ''!
age
	^age! !

!GlorpPerishableItem methodsFor: 'accessing' stamp: ''!
age: anObject
	age := anObject! !

GlorpPerishableItem subclass: #GlorpPoultry
	instanceVariableNames: 'featherColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpPoultry methodsFor: 'accessing' stamp: ''!
featherColor
	^featherColor! !

!GlorpPoultry methodsFor: 'accessing' stamp: ''!
featherColor: anObject
	featherColor := anObject! !

Object subclass: #GlorpItinerary
	instanceVariableNames: 'id reservation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpItinerary commentStamp: '<historical>' prior: 0!
An itinerary holds onto a single reservation. It may not make much sense, but we need to test another layer of indirection.
!

!GlorpItinerary class methodsFor: 'examples' stamp: ''!
example1
	^self new
		reservation: GlorpReservation example1.! !

!GlorpItinerary class methodsFor: 'examples' stamp: ''!
example2
	^self new
		reservation: GlorpReservation example2.! !

!GlorpItinerary methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpItinerary methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpItinerary methodsFor: 'accessing' stamp: ''!
reservation
	^reservation! !

!GlorpItinerary methodsFor: 'accessing' stamp: ''!
reservation: anObject
	reservation := anObject! !

Object subclass: #GlorpJob
	instanceVariableNames: 'id description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpJob methodsFor: 'accessing' stamp: ''!
description
	^description! !

!GlorpJob methodsFor: 'accessing' stamp: ''!
description: anObject
	description := anObject! !

!GlorpJob methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpJob methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

Object subclass: #GlorpMediaType
	instanceVariableNames: 'name mimeType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpMediaType class methodsFor: 'instance creation' stamp: ''!
text
	^self new name: 'text'; mimeType: 'text/plain'! !

!GlorpMediaType methodsFor: 'accessing' stamp: ''!
mimeType
	^mimeType! !

!GlorpMediaType methodsFor: 'accessing' stamp: ''!
mimeType: anObject
	mimeType := anObject! !

!GlorpMediaType methodsFor: 'accessing' stamp: ''!
name 
	^name! !

!GlorpMediaType methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

Object subclass: #GlorpMessage
	instanceVariableNames: 'folder user subject contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpMessage class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
contents
	^contents! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
contents: aString
	contents := aString! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
folder
	^folder! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
folder: aGlorpFolder
	folder := aGlorpFolder! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
folderName
	^self folder name.! !

!GlorpMessage methodsFor: 'initialize-release' stamp: ''!
initialize
	folder := nil.
	user := nil.
	subject := ''.
	contents := ''.! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
subject
	^subject! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
subject: aString
	subject := aString! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
user
	^user! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
user: aGlorpUser
	user := aGlorpUser! !

!GlorpMessage methodsFor: 'accessing' stamp: ''!
userId
	^self user id.! !

Object subclass: #GlorpMoney
	instanceVariableNames: 'currency amount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpMoney class methodsFor: 'instance creation' stamp: ''!
currency: aSymbol amount: aNumber

	^self new
		currency: aSymbol;
		amount: aNumber.! !

!GlorpMoney class methodsFor: 'instance creation' stamp: ''!
defaultCurrency

	^#CDN.! !

!GlorpMoney class methodsFor: 'instance creation' stamp: ''!
forAmount: anAmount

	^self currency: self defaultCurrency amount: anAmount.! !

!GlorpMoney methodsFor: 'accessing' stamp: ''!
amount
	^amount! !

!GlorpMoney methodsFor: 'accessing' stamp: ''!
amount: anInteger
	amount := anInteger! !

!GlorpMoney methodsFor: 'accessing' stamp: ''!
currency
	^currency! !

!GlorpMoney methodsFor: 'accessing' stamp: ''!
currency: aSymbol
	currency := aSymbol! !

Object subclass: #GlorpObjectWithNoAccessors
	instanceVariableNames: 'alue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpObjectWithNoAccessors methodsFor: 'accessing' stamp: ''!
extremelyPrivateValue: aString
	alue := aString! !

Object subclass: #GlorpOffice
	instanceVariableNames: 'id employees street employeeOfMonth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpOffice methodsFor: 'accessing' stamp: ''!
addEmployee: anEmployee
	self employees add: anEmployee.! !

!GlorpOffice methodsFor: 'accessing' stamp: ''!
employeeOfMonth
	^employeeOfMonth! !

!GlorpOffice methodsFor: 'accessing' stamp: ''!
employeeOfMonth: anObject
	employeeOfMonth := anObject! !

!GlorpOffice methodsFor: 'accessing' stamp: ''!
employees
	employees isNil ifTrue: [ employees := OrderedCollection new ].
	^employees! !

!GlorpOffice methodsFor: 'accessing' stamp: ''!
employees: anObject
	employees := anObject! !

!GlorpOffice methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpOffice methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpOffice methodsFor: 'accessing' stamp: ''!
street
	^street! !

!GlorpOffice methodsFor: 'accessing' stamp: ''!
street: aString
	street := aString! !

Object subclass: #GlorpOwner
	instanceVariableNames: 'id slave slaves'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpOwner methodsFor: 'accessing' stamp: ''!
id
	^id.! !

!GlorpOwner methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject.! !

!GlorpOwner methodsFor: 'accessing' stamp: ''!
slave
	^slave.! !

!GlorpOwner methodsFor: 'accessing' stamp: ''!
slave: anObject
	slave := anObject.! !

!GlorpOwner methodsFor: 'accessing' stamp: ''!
slaves
	^slaves.! !

!GlorpOwner methodsFor: 'accessing' stamp: ''!
slaves: anObject
	slaves := anObject.! !

Object subclass: #GlorpPassenger
	instanceVariableNames: 'id name frequentFlyerMiles airline'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpPassenger class methodsFor: 'examples' stamp: ''!
example1

	^self new
		id: 3;
		name: 'Some Passenger';
		frequentFlyerPoints: 10000;
		airline: GlorpAirline example1.! !

!GlorpPassenger class methodsFor: 'examples' stamp: ''!
example2

	^self new
		id: 39;
		name: 'Some Other Passenger';
		frequentFlyerPoints: 7;
		airline: GlorpAirline example2.! !

!GlorpPassenger methodsFor: 'accessing' stamp: ''!
airline
	^airline.! !

!GlorpPassenger methodsFor: 'accessing' stamp: ''!
airline: anAirline
	airline := anAirline.! !

!GlorpPassenger methodsFor: 'accessing' stamp: ''!
frequentFlyerPoints
	^frequentFlyerMiles.! !

!GlorpPassenger methodsFor: 'accessing' stamp: ''!
frequentFlyerPoints: aSmallInteger 
	frequentFlyerMiles := aSmallInteger.! !

!GlorpPassenger methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpPassenger methodsFor: 'accessing' stamp: ''!
id: aSmallInteger 
	id := aSmallInteger.! !

!GlorpPassenger methodsFor: 'accessing' stamp: ''!
name
	^name.! !

!GlorpPassenger methodsFor: 'accessing' stamp: ''!
name: aString 
	name := aString.! !

Object subclass: #GlorpPerson
	instanceVariableNames: 'id name address emailAddresses hasEmail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpPerson class methodsFor: 'examples' stamp: ''!
example1

	^self new
		id: 1;
		name: 'Zaphod Beeblebrox';
		address: GlorpAddress example1.! !

!GlorpPerson class methodsFor: 'examples' stamp: ''!
example1WithChangedAddress

	^self new
		id: 1;
		name: 'Zaphod Beeblebrox';
		address: GlorpAddress example1WithChangedAddress.! !

!GlorpPerson class methodsFor: 'examples' stamp: ''!
example1WithDifferentAddress

	^self new
		id: 1;
		name: 'Zaphod Beeblebrox';
		address: GlorpAddress example2.! !

!GlorpPerson class methodsFor: 'examples' stamp: ''!
example1WithDifferentName

	^self new
		id: 1; 
		name: 'John Doe';
		address: GlorpAddress example1.! !

!GlorpPerson class methodsFor: 'examples' stamp: ''!
example1WithNoAddress

	^self new
		id: 1;
		name: 'Zaphod Beeblebrox';
		address: nil.! !

!GlorpPerson class methodsFor: 'examples' stamp: ''!
example2

	^self new
		id: 2;
		name: 'John Doe';
		address: GlorpAddress example2.! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
address
	"Private - Answer the value of the receiver's ''address'' instance variable."

	^address! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
address: anObject
	"Private - Set the value of the receiver's ''address'' instance variable to the argument, anObject."

	address := anObject! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
emailAddresses
	^emailAddresses! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
emailAddresses: aCollection
	emailAddresses := aCollection! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
hasEmail
	^hasEmail! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
hasEmail: aBoolean
	hasEmail := aBoolean! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
id
	"Private - Answer the value of the receiver's ''id'' instance variable."

	^id! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
id: anObject
	"Private - Set the value of the receiver's ''id'' instance variable to the argument, anObject."

	id := anObject! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
name
	"Private - Answer the value of the receiver's ''name'' instance variable."

	^name! !

!GlorpPerson methodsFor: 'accessing' stamp: ''!
name: anObject
	"Private - Set the value of the receiver's ''name'' instance variable to the argument, anObject."

	name := anObject! !

!GlorpPerson methodsFor: 'printing' stamp: ''!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '('.
	aStream nextPutAll: id printString, ',', (name ifNil: ['']).
	aStream nextPutAll: ')'.! !

Object subclass: #GlorpPublisher
	instanceVariableNames: 'id name titles titlesWithNumberInStock titlesWithNumberInStockLink employees differentBunchOfEmployeesById'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
differentBunchOfEmployeesById
	^differentBunchOfEmployeesById! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
differentBunchOfEmployeesById: aDictionary
	differentBunchOfEmployeesById := aDictionary! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
employees
	^employees.! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
employees: aDictionary
	employees := aDictionary.! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
titles
	^titles! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
titles: aCollectionOfStrings
	titles := aCollectionOfStrings! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
titlesWithNumberInStock
	^titlesWithNumberInStock! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
titlesWithNumberInStock: aDictionary
	titlesWithNumberInStock := aDictionary.
	self titlesWithNumberInStockLink: (Dictionary new addAll: (aDictionary associations collect: [:each | each copy]); yourself).! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
titlesWithNumberInStockLink
	^titlesWithNumberInStockLink! !

!GlorpPublisher methodsFor: 'accessing' stamp: ''!
titlesWithNumberInStockLink: aDictionary
	titlesWithNumberInStockLink := aDictionary! !

Object subclass: #GlorpPublisherEmployee
	instanceVariableNames: 'id name jobTitle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpPublisherEmployee class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpPublisherEmployee methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpPublisherEmployee methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpPublisherEmployee methodsFor: 'initialize-release' stamp: ''!
initialize
	"Initialize a newly created instance. This method must answer the receiver."

	" *** Edit the following to properly initialize instance variables ***"
	id := nil.
	name := nil.
	jobTitle := nil.
	" *** And replace this comment with additional initialization code *** "
	^self! !

!GlorpPublisherEmployee methodsFor: 'accessing' stamp: ''!
jobTitle
	^jobTitle! !

!GlorpPublisherEmployee methodsFor: 'accessing' stamp: ''!
jobTitle: aString
	jobTitle := aString! !

!GlorpPublisherEmployee methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpPublisherEmployee methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

Object subclass: #GlorpRecordWithUpdateTime
	instanceVariableNames: 'id name updateTime updateSomeOtherThing'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpRecordWithUpdateTime commentStamp: '<historical>' prior: 0!
This is a model class that has a timestamp which is updated each time it is written.

Instance Variables:
	id	<Integer>	The primary key
	name	<String>	a name
	updateTime	<Timestamp>	When we were last updated.

!

!GlorpRecordWithUpdateTime methodsFor: 'accessing' stamp: ''!
id
	^id.! !

!GlorpRecordWithUpdateTime methodsFor: 'accessing' stamp: ''!
id: anInteger
	id := anInteger.! !

!GlorpRecordWithUpdateTime methodsFor: 'accessing' stamp: ''!
name
	^name.! !

!GlorpRecordWithUpdateTime methodsFor: 'accessing' stamp: ''!
name: aString
	name := aString.! !

!GlorpRecordWithUpdateTime methodsFor: 'accessing' stamp: ''!
updateSomeOtherThing
	^updateSomeOtherThing! !

!GlorpRecordWithUpdateTime methodsFor: 'accessing' stamp: ''!
updateSomeOtherThing: aString
	updateSomeOtherThing := aString! !

!GlorpRecordWithUpdateTime methodsFor: 'accessing' stamp: ''!
updateTime
	^updateTime.! !

Object subclass: #GlorpReservation
	instanceVariableNames: 'id passenger passengers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpReservation class methodsFor: 'examples' stamp: ''!
example1

	^self new
		id: 2;
		passenger: GlorpPassenger example1.! !

!GlorpReservation class methodsFor: 'examples' stamp: ''!
example2

	^self new
		id: 7;
		passenger: GlorpPassenger example2.! !

!GlorpReservation class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize.! !

!GlorpReservation methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpReservation methodsFor: 'accessing' stamp: ''!
id: anInteger
	id := anInteger! !

!GlorpReservation methodsFor: 'initialize/release' stamp: ''!
initialize
	passengers := OrderedCollection new.! !

!GlorpReservation methodsFor: 'accessing' stamp: ''!
passenger
	^passenger! !

!GlorpReservation methodsFor: 'accessing' stamp: ''!
passenger: aPassenger
	passenger := aPassenger.
	passengers add: aPassenger.! !

!GlorpReservation methodsFor: 'accessing' stamp: ''!
passengers
	^passengers! !

!GlorpReservation methodsFor: 'accessing' stamp: ''!
passengers: aCollection
	passengers := aCollection.! !

Object subclass: #GlorpServiceCharge
	instanceVariableNames: 'description amount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpServiceCharge class methodsFor: 'instance creation' stamp: ''!
default

	^self new
		amount: (GlorpMoney forAmount: 3);
		description: 'additional overcharge'.! !

!GlorpServiceCharge class methodsFor: 'instance creation' stamp: ''!
new

	^super new initialize.! !

!GlorpServiceCharge methodsFor: 'accessing' stamp: ''!
amount
	^amount! !

!GlorpServiceCharge methodsFor: 'accessing' stamp: ''!
amount: anObject
	amount := anObject! !

!GlorpServiceCharge methodsFor: 'accessing' stamp: ''!
description
	^description! !

!GlorpServiceCharge methodsFor: 'accessing' stamp: ''!
description: anObject
	description := anObject! !

!GlorpServiceCharge methodsFor: 'initialize' stamp: ''!
initialize! !

Object subclass: #GlorpSlave
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpSlave methodsFor: 'accessing' stamp: ''!
id
	^id.! !

!GlorpSlave methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject.! !

Object subclass: #GlorpTag
	instanceVariableNames: 'id key label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpTag methodsFor: 'as yet unclassified' stamp: ''!
generateKeyFromLabel
	key := label asLowercase! !

!GlorpTag methodsFor: 'as yet unclassified' stamp: ''!
key
	^ key! !

!GlorpTag methodsFor: 'as yet unclassified' stamp: ''!
key: aKey
	key := aKey! !

!GlorpTag methodsFor: 'as yet unclassified' stamp: ''!
label
	^ label! !

!GlorpTag methodsFor: 'as yet unclassified' stamp: ''!
label: aLabel
	label := aLabel! !

Object subclass: #GlorpTaggableObject
	instanceVariableNames: 'tags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

GlorpTaggableObject subclass: #GlorpImage
	instanceVariableNames: 'id location'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpImage class methodsFor: 'as yet unclassified' stamp: ''!
fromStream: aStream
	^ self new image: aStream! !

!GlorpImage methodsFor: 'as yet unclassified' stamp: ''!
location
	^ location! !

!GlorpImage methodsFor: 'as yet unclassified' stamp: ''!
location: aLocation
	location := aLocation! !

!GlorpTaggableObject class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize.! !

!GlorpTaggableObject methodsFor: 'as yet unclassified' stamp: ''!
initialize
	tags := OrderedCollection new.! !

!GlorpTaggableObject methodsFor: 'as yet unclassified' stamp: ''!
tags
	^ tags! !

!GlorpTaggableObject methodsFor: 'as yet unclassified' stamp: ''!
tags: someTags
	tags := someTags! !

Object subclass: #GlorpTax
	instanceVariableNames: 'id name taxes supertax'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

GlorpTax subclass: #GlorpBasicTax
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpBasicTax class methodsFor: 'glorp' stamp: ''!
glorpSetupDescriptor: aDescriptor forSystem: aSystem
	| table |
	table := aSystem tableNamed: 'TAX'.
	self glorpCommonSetupDescriptor: aDescriptor forSystem: aSystem.
	(aSystem typeResolverFor: GlorpTax)
		register: aDescriptor
		keyedBy: 'T'
		field: (table fieldNamed: 'TYPE').! !

GlorpTax subclass: #GlorpSurcharge
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpSurcharge class methodsFor: 'glorp' stamp: ''!
glorpSetupDescriptor: aDescriptor forSystem: aSystem
	| table |
	table := aSystem tableNamed: 'TAX'.
	self glorpCommonSetupDescriptor: aDescriptor forSystem: aSystem.
	(aSystem typeResolverFor: GlorpTax)
		register: aDescriptor
		keyedBy: 'S'
		field: (table fieldNamed: 'TYPE').! !

!GlorpTax class methodsFor: 'glorp' stamp: ''!
glorpCommonSetupDescriptor: aDescriptor forSystem: aSystem
	| table |
	table := aSystem tableNamed: 'TAX'.
	aDescriptor table: table.
	aDescriptor addMapping: (DirectMapping from: #id to: (table fieldNamed: 'ID')).
	aDescriptor addMapping: (DirectMapping from: #name to: (table fieldNamed: 'NAME')).
	aDescriptor addMapping:
			((OneToManyMapping new)
				attributeName: #taxes;
				referenceClass: GlorpTax;
				join: (Join from: (table fieldNamed: 'ID') to: (table fieldNamed: 'TAX_ID'))).
	aDescriptor addMapping:
			((OneToOneMapping new)
				attributeName: #supertax;
				referenceClass: GlorpTax;
				join: (Join from: (table fieldNamed: 'TAX_ID') to: (table fieldNamed: 'ID'))).! !

!GlorpTax class methodsFor: 'glorp' stamp: ''!
glorpSetupDescriptor: aDescriptor forSystem: aSystem
	| table |
	table := aSystem tableNamed: 'TAX'.
	aDescriptor table: table.

	"(aSystem typeResolverFor: GlorpTax)
		register: aDescriptor
		keyedBy: 'XXX'
		field: (table fieldNamed: 'TYPE')."

	(aSystem typeResolverFor: GlorpTax)
		register: aDescriptor
		abstract: true! !

!GlorpTax class methodsFor: 'glorp' stamp: ''!
glorpTypeResolver
	^FilteredTypeResolver forRootClass: GlorpTax.! !

!GlorpTax methodsFor: 'add/remove' stamp: ''!
addTax: aTax
	self taxes add: aTax.
	aTax supertax: self.! !

!GlorpTax methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpTax methodsFor: 'private' stamp: ''!
id: anObject
	id := anObject.! !

!GlorpTax methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpTax methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject.! !

!GlorpTax methodsFor: 'accessing' stamp: ''!
supertax
	^supertax! !

!GlorpTax methodsFor: 'private' stamp: ''!
supertax: anObject
	supertax := anObject.! !

!GlorpTax methodsFor: 'accessing' stamp: ''!
taxes
	taxes ifNil: [taxes := IdentitySet new].
	^taxes.! !

Object subclass: #GlorpTestClassInNamespace
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

Object subclass: #GlorpTestModelsPackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Licence'!

!GlorpTestModelsPackage commentStamp: '<historical>' prior: 0!
This class represents package information for the GlorpTestModels package, for systems that don't have things like package properties to store things like copyright and license information.

LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!

!GlorpTestModelsPackage class methodsFor: 'instance creation' stamp: ''!
LICENSE
"LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"! !

Object subclass: #GlorpTestsPackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Licence'!

!GlorpTestsPackage commentStamp: '<historical>' prior: 0!
This class represents package information for the GlorpTests package, for systems that don't have things like package properties to store things like copyright and license information.

LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!

!GlorpTestsPackage class methodsFor: 'instance creation' stamp: ''!
LICENSE
"LGPL(S)

The code is Copyright (C) 2000-2010  Alan Knight

This is a Smalltalk library, licensed under the LGPL, and distributed WITHOUT
ANY WARRANTY (see below).  However, it is not obvious how some of
the terms and concepts in the license should be sensibly applied to Smalltalk
The various Smalltalk image models often do not directly correspond to 
the ideas of programs and libraries that are linked to form executables. 
We, the authors, would like to clarify our interpretation of the LGPL as it
applies to Smalltalk, and what we permit you to do with this code. We
will sometimes use the term LGPL(S) to distinguish the use of the LGPL 
with these clarifications.

This code is intended to be usable as a library, without the intention to 
restrict the license of the program that uses it. Thus, you may use this library
the same way you would normally use any other Smalltalk library. That is, 
you may load it into an image or otherwise make it available for use in whatever
ways are appropriate to the Smalltalk implementation you are using (including,
but not limited to file-in, linking a shared library, or loading a binary 
representation such as a parcel,  BOSS file, image segment or image component).
You may write code that uses the library, including subclassing library classes
and having the library call back to your code using blocks, the #perform: 
mechanism, or similar mechanisms. You may use this code,  including creating
and distributing packaged images, libraries, development images and executables
that include the library code. We do not consider these activities sufficient
to make the entire program/image/executable or any portion of it beyond the
library itself be considered a derivative work, and in any case we place no
restrictions on what licenses you may use for code that uses this library in
these or similar manners.

If, however, you modify the class definitions or methods of the library, we do
consider those modified sections a derivative work, and as such they are fully subject
to the restrictions described in the LGPL. Basically, you'll need to release the
source of any modifications.

If you require any further clarification, we'd be happy to provide it. You can contact
the authors c/o Alan Knight, knight@acm.org.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"! !

Object subclass: #GlorpThingOne
	instanceVariableNames: 'id name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-CollectionTypeModels'!

!GlorpThingOne commentStamp: '<historical>' prior: 0!
This just exists to be put in collections.

Instance Variables:
	id	<SmallInteger>	description of id
	name	<String>	description of name

!

!GlorpThingOne class methodsFor: 'instance creation' stamp: ''!
named: aString
	^self new name: aString.! !

!GlorpThingOne methodsFor: 'comparing' stamp: ''!
<= aThingOne
	^self name <= aThingOne name.! !

!GlorpThingOne methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpThingOne methodsFor: 'accessing' stamp: ''!
id: anInteger
	id := anInteger! !

!GlorpThingOne methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpThingOne methodsFor: 'accessing' stamp: ''!
name: aString
	name := aString! !

Object subclass: #GlorpThingWithLotsOfDifferentCollections
	instanceVariableNames: 'id array orderedCollection set bag sortedCollection name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-CollectionTypeModels'!

!GlorpThingWithLotsOfDifferentCollections class methodsFor: 'examples' stamp: ''!
example1
	| example array |
	example := self new.
	example name: 'fred'.
	array := Array with: (GlorpThingOne named: 'array1') with: (GlorpThingOne named: 'array2') with: (GlorpThingOne named: 'array3').
	example array: array.
	example set add: (GlorpThingOne named: 'set1').
	example set add: (GlorpThingOne named: 'set2').	
	example orderedCollection add: (GlorpThingOne named: 'orderedCollection1').
	example orderedCollection add: (GlorpThingOne named: 'orderedCollection2').
	example bag add: (GlorpThingOne named: 'bag1').
	example bag add: (GlorpThingOne named: 'bag2').
	example sortedCollection add: (GlorpThingOne named: 'sorted1').
	example sortedCollection add: (GlorpThingOne named: 'sorted2').
	example sortedCollection add: (GlorpThingOne named: 'sorted3').
	example sortedCollection add: (GlorpThingOne named: 'sorted4').
	^example.! !

!GlorpThingWithLotsOfDifferentCollections class methodsFor: 'examples' stamp: ''!
exampleForOrdering
	| example |
	example := self new.
	example name: 'order'.
	example orderedCollection add: (GlorpThingOne named: 'oc6').
	example orderedCollection add: (GlorpThingOne named: 'oc5').
	example orderedCollection add: (GlorpThingOne named: 'oc4').
	example orderedCollection add: (GlorpThingOne named: 'oc3').
	example orderedCollection add: (GlorpThingOne named: 'oc7').
	example orderedCollection add: (GlorpThingOne named: 'oc8').
	example array: (#('a1' 'a2' 'a3' 'a9' 'a8' 'a7') collect: [:each | GlorpThingOne named: each]).
	^example! !

!GlorpThingWithLotsOfDifferentCollections class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
array
	^array! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
array: anObject
	array := anObject! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
bag
	^bag! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
bag: anObject
	bag := anObject! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'initialize-release' stamp: ''!
initialize
	array := #().
	orderedCollection := OrderedCollection new.
	set := Set new.
	bag := Bag new.
	sortedCollection := #() asSortedCollection.! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
orderedCollection
	^orderedCollection! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
orderedCollection: anObject
	orderedCollection := anObject! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
set
	^set! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
set: anObject
	set := anObject! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
sortedCollection
	^sortedCollection! !

!GlorpThingWithLotsOfDifferentCollections methodsFor: 'accessing' stamp: ''!
sortedCollection: anObject
	sortedCollection := anObject! !

Object subclass: #GlorpTransformedTime
	instanceVariableNames: 'id time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpTransformedTime commentStamp: '<historical>' prior: 0!
This class just holds a time, but that time is transformed into a representation in seconds in the database.

Instance Variables:
	id	<Integer>	The primary key
	time	<Time>	The time

!

!GlorpTransformedTime class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize.! !

!GlorpTransformedTime methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpTransformedTime methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpTransformedTime methodsFor: 'initialize' stamp: ''!
initialize
	time := Time now.! !

!GlorpTransformedTime methodsFor: 'accessing' stamp: ''!
time
	^time! !

!GlorpTransformedTime methodsFor: 'accessing' stamp: ''!
time: anObject
	time := anObject! !

Object subclass: #GlorpTravelAgency
	instanceVariableNames: 'name agents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpTravelAgency commentStamp: '<historical>' prior: 0!
This represents a travel agency. The interesting persistence feature of these is that they are embedded in the agent table, but also treated as unique objects with their own relationships.

Instance Variables:
	agents	<Collection of: GlorpTravelAgent>	description of agents
	name	<String>	description of name

!

!GlorpTravelAgency methodsFor: 'comparing' stamp: ''!
= aGlorpTravelAgency
	^self name = aGlorpTravelAgency name.! !

!GlorpTravelAgency methodsFor: 'accessing' stamp: ''!
agents
	^agents! !

!GlorpTravelAgency methodsFor: 'accessing' stamp: ''!
agents: anObject
	agents := anObject! !

!GlorpTravelAgency methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpTravelAgency methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

Object subclass: #GlorpTravelAgent
	instanceVariableNames: 'id name agency'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpTravelAgent commentStamp: '<historical>' prior: 0!
This represents a travel agent. Travel agents are very simple, but have their agency embedded in their table, even though we expect the agencies to be unique instances, and to have multiple agents.

Instance Variables:
	agency	<GlorpTravelAgency>	description of agency
	id	<Integer>	description of id
	name	<String>	description of name

!

!GlorpTravelAgent methodsFor: 'accessing' stamp: ''!
agency
	^agency! !

!GlorpTravelAgent methodsFor: 'accessing' stamp: ''!
agency: anObject
	agency := anObject! !

!GlorpTravelAgent methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpTravelAgent methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpTravelAgent methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpTravelAgent methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

Object subclass: #GlorpTreeNode
	instanceVariableNames: 'id children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpTreeNode class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize.! !

!GlorpTreeNode methodsFor: 'As yet unclassified' stamp: ''!
children
	^children.! !

!GlorpTreeNode methodsFor: 'As yet unclassified' stamp: ''!
children: aCollection
	children := aCollection.! !

!GlorpTreeNode methodsFor: 'As yet unclassified' stamp: ''!
id
	^id.! !

!GlorpTreeNode methodsFor: 'As yet unclassified' stamp: ''!
id: anInteger
	id := anInteger.! !

!GlorpTreeNode methodsFor: 'As yet unclassified' stamp: ''!
initialize
	children := OrderedCollection new.! !

!GlorpTreeNode methodsFor: 'printing' stamp: ''!
printOn: aStream
	aStream nextPutAll: 'Node('.
	id printOn: aStream.
	aStream nextPut: $).! !

Object subclass: #GlorpTypeTestsModelClass
	instanceVariableNames: 'id test'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Database'!

!GlorpTypeTestsModelClass methodsFor: 'As yet unclassified' stamp: ''!
test
	^test.! !

!GlorpTypeTestsModelClass methodsFor: 'As yet unclassified' stamp: ''!
test: anObject
	test:= anObject.! !

Object subclass: #GlorpUser
	instanceVariableNames: 'id name folders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpUser class methodsFor: 'instance creation' stamp: ''!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!GlorpUser methodsFor: 'accessing' stamp: ''!
folders
	^folders! !

!GlorpUser methodsFor: 'accessing' stamp: ''!
folders: aCollection
	folders := aCollection! !

!GlorpUser methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpUser methodsFor: 'accessing' stamp: ''!
id: anInteger
	id := anInteger! !

!GlorpUser methodsFor: 'initialize-release' stamp: ''!
initialize

	id := nil.
	folders := OrderedCollection new.! !

!GlorpUser methodsFor: 'accessing' stamp: ''!
name
	^name.! !

!GlorpUser methodsFor: 'accessing' stamp: ''!
name: aString
	name := aString.! !

Object subclass: #GlorpWarehouse
	instanceVariableNames: 'id address currentItem items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpWarehouse methodsFor: 'accessing' stamp: ''!
address
	^address! !

!GlorpWarehouse methodsFor: 'accessing' stamp: ''!
address: anObject
	address := anObject! !

!GlorpWarehouse methodsFor: 'accessing' stamp: ''!
currentItem
	^currentItem! !

!GlorpWarehouse methodsFor: 'accessing' stamp: ''!
currentItem: anInventoryItem
	currentItem := anInventoryItem! !

!GlorpWarehouse methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpWarehouse methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpWarehouse methodsFor: 'accessing' stamp: ''!
items
	^items! !

!GlorpWarehouse methodsFor: 'accessing' stamp: ''!
items: aCollection
	items := aCollection! !

Object subclass: #GlorpWorker
	instanceVariableNames: 'id name pendingJobs finishedJobs priorityJobs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpWorker class methodsFor: 'instance creation' stamp: ''!
new

	^super new initialize.! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
finishedJobs
	^finishedJobs! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
finishedJobs: anObject
	finishedJobs := anObject! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpWorker methodsFor: 'initialize/release' stamp: ''!
initialize

	pendingJobs := OrderedCollection new.
	finishedJobs := OrderedCollection new.
	priorityJobs := OrderedCollection new.! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
pendingJobs
	^pendingJobs! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
pendingJobs: anObject
	pendingJobs := anObject! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
priorityJobs
	^priorityJobs! !

!GlorpWorker methodsFor: 'accessing' stamp: ''!
priorityJobs: anObject
	priorityJobs := anObject! !

Object subclass: #GlorpWorkingStiff
	instanceVariableNames: 'id name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

GlorpWorkingStiff subclass: #GlorpEmployee
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpEmployee class methodsFor: 'glorp setup' stamp: ''!
glorpTypeResolver
	^FilteredTypeResolver forRootClass: GlorpEmployee! !

GlorpEmployee subclass: #GlorpLineWorker
	instanceVariableNames: 'productionLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpLineWorker methodsFor: 'comparing' stamp: ''!
= aLineWorker
	^super = aLineWorker and: [productionLine = aLineWorker productionLine].! !

!GlorpLineWorker methodsFor: 'accessing' stamp: ''!
productionLine
	^productionLine! !

!GlorpLineWorker methodsFor: 'accessing' stamp: ''!
productionLine: anObject
	productionLine := anObject! !

GlorpEmployee subclass: #GlorpManager
	instanceVariableNames: 'branch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpManager methodsFor: 'comparing' stamp: ''!
= aManager
	^super = aManager and: [branch = aManager branch].! !

!GlorpManager methodsFor: 'accessing' stamp: ''!
branch
	^branch! !

!GlorpManager methodsFor: 'accessing' stamp: ''!
branch: anObject
	branch := anObject! !

GlorpManager subclass: #GlorpRegionalManager
	instanceVariableNames: 'region'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpRegionalManager methodsFor: 'comparing' stamp: ''!
= aRegionalManager
	^super = aRegionalManager and: [region = aRegionalManager region].! !

!GlorpRegionalManager methodsFor: 'accessing' stamp: ''!
region
	^region! !

!GlorpRegionalManager methodsFor: 'accessing' stamp: ''!
region: anObject
	region := anObject! !

GlorpEmployee subclass: #GlorpTransientEmployee
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests-Models'!

!GlorpTransientEmployee commentStamp: '<historical>' prior: 0!
GlorpTransientEmployee is used to test that not all subclasses need to be persistent in inheritance mapping.!

!GlorpWorkingStiff methodsFor: 'comparing' stamp: ''!
= aWorkingStiff
	^self class = aWorkingStiff class and: [id = aWorkingStiff id and: [name = aWorkingStiff name]].! !

!GlorpWorkingStiff methodsFor: 'accessing' stamp: ''!
id
	^id! !

!GlorpWorkingStiff methodsFor: 'accessing' stamp: ''!
id: anObject
	id := anObject! !

!GlorpWorkingStiff methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GlorpWorkingStiff methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

Object subclass: #IdentityHashPrintingObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!IdentityHashPrintingObject commentStamp: '<historical>' prior: 0!
This is just an object we can easily identify by identity, used for transaction test debugging. We can put these into a transaction and easily distinguish them from their copies.
!

!IdentityHashPrintingObject methodsFor: 'printing' stamp: ''!
printOn: aStream
	aStream nextPutAll: 'thing with hash '.
	self identityHash printOn: aStream.! !

DatabaseCommand subclass: #GlorpNullCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpNullCommand commentStamp: '<historical>' prior: 0!
This represents a command with no additional syntax, basically just a stream. Useful for testing the generation of chunks of SQL.!

!GlorpNullCommand class methodsFor: 'As yet unclassified' stamp: ''!
useBinding: aBoolean session: aSession
	^(self new)
		useBinding: aBoolean;
		session: aSession;
		yourself.! !

!GlorpNullCommand methodsFor: 'accessing' stamp: ''!
sqlString
	^stream contents.! !

!GlorpNullCommand methodsFor: 'testing' stamp: ''!
succeeded
	^true.! !

TestResource subclass: #GlorpDatabaseLoginResource
	instanceVariableNames: 'accessor login'
	classVariableNames: 'DefaultLogin'
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDatabaseLoginResource class methodsFor: 'accessing' stamp: ''!
constructEventsTriggered
	^(super constructEventsTriggered)
		add: #changedDefaultLogin;
		yourself! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: ''!
defaultAccessLogin
	"To set the default database login for Access, execute the following statement."
	"self defaultAccessLogin."

	^DefaultLogin := (Login new)
		database: AccessPlatform new;
		username: 'whatever';
		password: '';
		connectString: 'access'! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: ''!
defaultFirebirdLogin
	"To set the default database login to Firebird, execute the following statement."
	"self defaultFirebirdLogin."

	^DefaultLogin := (Login new)
		database: FirebirdPlatform new;
		username: 'BaseSystem';
		password: 'BaseSyst';
		connectString: 'firebirdGlorpTestDB'! !

!GlorpDatabaseLoginResource class methodsFor: 'accessing' stamp: ''!
defaultLogin
	"Return the default Login."

	DefaultLogin isNil ifTrue: [^DefaultLogin := self defaultPostgreSQLLoginForVPN].
	DefaultLogin database useBinding: DatabasePlatform useBindingIfSupported.
	^DefaultLogin! !

!GlorpDatabaseLoginResource class methodsFor: 'accessing' stamp: ''!
defaultLogin: aLogin 
	DefaultLogin := aLogin.
	self triggerEvent: #changedDefaultLogin! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/28/2011 06:23'!
defaultMysqlLogin
	"To set the default database login to MySQL, execute the following statement."
	"DefaultLogin := self defaultMysqlLogin."

	^(Login new)
		database: MySQLPlatform new;
		username: 'root';
		password: 'root';
		host: 'localhost';
       port: '8889';
		connectString: 'localhost_test'.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/28/2011 06:23'!
defaultOracleLiteLogin
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultOracleLiteLogin."

		^(Login new)
		database: OraclePlatform new;
		username: 'sodbxtest';
		password: 'sodbxtest';
		connectString: '127.0.0.1_sodbxtest'.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/28/2011 06:23'!
defaultOracleLogin
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultOracleLogin."

		^(Login new)
		database: OraclePlatform new;
		username: 'sodbxtest';
		password: 'sodbxtest';
		connectString: '127.0.0.1_sodbxtest'.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/28/2011 06:24'!
defaultOracleLogin2
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultOracleLogin2."

		^(Login new)
		database: OraclePlatform new;
		username: 'sodbxtest';
		password: 'sodbxtest';
		connectString: '127.0.0.1_sodbxtest'.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/28/2011 06:24'!
defaultOracleODBCLogin
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultOracleODBCLogin."

	^(Login new)
		database: OracleODBCPlatform new;
		username: 'System';
		password: 'manager';
		connectString: 'oracle'! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/28/2011 06:24'!
defaultPersonalOracleLogin
	"To set the default database login to Oracle, execute the following statement."
	"DefaultLogin := self defaultPersonalOracleLogin."

	^(Login new)
		database: OraclePlatform new;
		username: 'system';
		password: 'manager';
		connectString: ''! !

!GlorpDatabaseLoginResource class methodsFor: 'accessing' stamp: ''!
defaultPlatform
	^self defaultLogin database.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/2/2011 07:56'!
defaultPostgreSQLInternetLogin
	"To set the default database login to PostgreSQL, execute the following statement."
	"self defaultPostgreSQLInternetLogin."

	^DefaultLogin := (Login new)
		database: PostgreSQLPlatform new;
		username: 'sodbxtest';
		password: 'sodbxtest';
		connectString: '127.0.0.1:5432_sodbxtest'.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/2/2011 07:56'!
defaultPostgreSQLLocalLogin
	"To set the default database login to PostgreSQL, execute the following statement."
	"self defaultPostgreSQLLocalLogin."

	^DefaultLogin := (Login new)
		database: PostgreSQLPlatform new;
		username: 'sodbxtest';
		password: 'sodbxtest';
		connectString: '127.0.0.1:5432_sodbxtest'.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/2/2011 07:56'!
defaultPostgreSQLLogin
	"To set the default database login to PostgreSQL, execute the following statement."
	"self defaultPostgreSQLLogin."

	^DefaultLogin := (Login new)
		database: PostgreSQLPlatform new;
		username: 'sodbxtest';
		password: 'sodbxtest';
		connectString: '127.0.0.1:5432_sodbxtest'.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/7/2011 23:53'!
defaultPostgreSQLLoginForGlorpStore
	"To set the default database login to PostgreSQL, execute the following statement."
	"self defaultPostgreSQLLoginForGlorpStore."

	^DefaultLogin := (Login new)
		database: PostgreSQLPlatform new;
		username: 'sodbxtest';
		password: 'sodbxtest';
		connectString: '127.0.0.1:5432_sodbxtest'.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: 'GuillermoPolito 5/2/2011 07:54'!
defaultPostgreSQLLoginForVPN
	"To set the default database login to PostgreSQL, execute the following statement."
	"self defaultPostgreSQLLoginForVPN."

	^DefaultLogin := (Login new)
		database: PostgreSQLPlatform new;
		username: 'sodbxtest';
		password: 'sodbxtest';
		connectString: '127.0.0.1:5432_sodbxtest'.! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: ''!
defaultSQLServerLogin
	"To set the default database login for Dolphin, execute the following statement."
	"self defaultSQLServerLogin."

	^DefaultLogin := (Login new)
		database: SQLServerPlatform new;
		username: 'glorptest';
		password: 'password';
		connectString: 'glorptest'! !

!GlorpDatabaseLoginResource class methodsFor: 'data' stamp: ''!
defaultSQLiteLocalLogin
	"To set the default database login to SQLite, change the connectString to an appropriate path and execute the following statement."
	"self defaultSQLiteLocalLogin."

	^DefaultLogin := (Login new)
		database: SQLite3Platform new;
		username: '';
		password: '';
		connectString: 'C:\MyPath\StoreData\glorpTestSQLite.db'.! !

!GlorpDatabaseLoginResource methodsFor: 'accessing' stamp: ''!
accessor
	^accessor! !

!GlorpDatabaseLoginResource methodsFor: 'accessing' stamp: ''!
accessor: anObject
	accessor := anObject! !

!GlorpDatabaseLoginResource methodsFor: 'accessing' stamp: ''!
login
	^login! !

!GlorpDatabaseLoginResource methodsFor: 'accessing' stamp: ''!
login: anObject
	"We don't attempt to be secure on logins being used for tests. Make sure we don't surprise someone by modifying the original instance."

	login := anObject copy.
	login secure: false.! !

!GlorpDatabaseLoginResource methodsFor: 'accessing' stamp: ''!
platform
	^login database! !

!GlorpDatabaseLoginResource methodsFor: 'initialize/release' stamp: ''!
setUp
	Transcript show: self class name asString, ' setUp'; cr.
	super setUp.
	self login: self class defaultLogin.
	accessor := DatabaseAccessor forLogin: login.
	accessor login.! !

!GlorpDatabaseLoginResource methodsFor: 'initialize/release' stamp: ''!
tearDown

	Transcript show: self class name asString, ' tearDown'; cr.
	accessor notNil ifTrue: [[accessor logout] on: Dialect error do: [:ex | ]].! !

TestResource subclass: #GlorpDemoTablePopulatorResource
	instanceVariableNames: 'login'
	classVariableNames: 'NeedsSetup'
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpDemoTablePopulatorResource class methodsFor: 'setup' stamp: ''!
invalidateSetup
	"GlorpDemoTablePopulatorResource invalidateSetup"

	NeedsSetup := true.
	self reset.! !

!GlorpDemoTablePopulatorResource class methodsFor: 'setup' stamp: ''!
needsSetup
	NeedsSetup isNil ifTrue: [NeedsSetup := true].
	^NeedsSetup! !

!GlorpDemoTablePopulatorResource class methodsFor: 'setup' stamp: ''!
needsSetup: aBoolean

	NeedsSetup := aBoolean.! !

!GlorpDemoTablePopulatorResource class methodsFor: 'setup' stamp: ''!
resources

	^Array with: GlorpDatabaseLoginResource.! !

!GlorpDemoTablePopulatorResource methodsFor: 'setup' stamp: ''!
populateStuffTable
 
	login accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (12,''abc'')'.
	login accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (13, ''hey nonny nonny'')'.
	login accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (42, ''yabba dabba doo'')'.
	login accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (9625, ''the band played on'')'.
	login accessor executeSQLStringNoResult: 'INSERT INTO STUFF VALUES (113141, ''Smalltalk'')'.! !

!GlorpDemoTablePopulatorResource methodsFor: 'setup' stamp: 'GuillermoPolito 4/25/2011 06:15'!
setUp
	
	super setUp.
	login := GlorpDatabaseLoginResource current.
	self class needsSetup ifFalse: [^self].
	login accessor doDDLOperation:
			[GlorpTestDescriptorSystem allSubclasses
				do: [:eachSystemClass | 
					self setUpSystem: (eachSystemClass forPlatform: login platform) setUpDefaults].
			self populateStuffTable].! !

!GlorpDemoTablePopulatorResource methodsFor: 'setup' stamp: 'GuillermoPolito 5/28/2011 06:29'!
setUpSystem: system 
	| errorBlock |
	login accessor dropTables: ((TableSorter for: system allTables) sort reverse).
	system platform areSequencesExplicitlyCreated ifTrue: [
		login accessor dropSequences: system allSequences.
		system allSequences do: [:each |
			login accessor createSequence: each ifError: errorBlock]].
	system allTables do: [:each | 
			login accessor createTable: each
				ifError: errorBlock].
	system allTables do: [:each |
			login accessor createTableIndexes: each ifError: errorBlock]. 
	"SQLite3 doesn't support adding/dropping constraints."
	system platform supportsConstraints ifTrue: [
		system allTables do: [:each | 
				login accessor createTableFKConstraints: each ifError: errorBlock]].

	self class needsSetup: false! !

TestResource subclass: #GlorpSessionResource
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GlorpTests'!

!GlorpSessionResource class methodsFor: 'resources' stamp: ''!
resources

	^Array with: GlorpDatabaseLoginResource with: GlorpDemoTablePopulatorResource.! !

!GlorpSessionResource methodsFor: 'setup' stamp: ''!
glorpTestsSystem
	"We want to prep the GlorpDemoTablePopulatorResource but subclasses using different systems may not;  put it here where it will be overridden."

	GlorpDemoTablePopulatorResource current.
	^GlorpDemoDescriptorSystem forPlatform: GlorpDatabaseLoginResource defaultLogin database! !

!GlorpSessionResource methodsFor: 'accessing' stamp: ''!
newSession

	self setUp.
	^self session.! !

!GlorpSessionResource methodsFor: 'accessing' stamp: ''!
session

	^session.! !

!GlorpSessionResource methodsFor: 'setup' stamp: ''!
setAccessor
	
	session accessor: GlorpDatabaseLoginResource current accessor.! !

!GlorpSessionResource methodsFor: 'setup' stamp: ''!
setUp
	
	super setUp.
	session := self glorpTestsSystem
		sessionForLogin: GlorpDatabaseLoginResource current login.
	self setAccessor.! !

!GlorpSessionResource methodsFor: 'setup' stamp: 'GuillermoPolito 4/25/2011 06:33'!
tearDown
	super tearDown.	
	session ifNotNil: [ :s | s logout ].! !
GlorpDescriptorSystemWithNamespacePoolDictionaryDeclarer initialize!
